
    processor 6803

; SET TABS to 4 if text appears jumbled
;  ____________________________________________________________________________
; |                                                                            \
; |** BLUEBERRY 60 **                                                          |
; |   Mopar ECM                                                                |
; |   1987 MP Turbo II                                                         |
; |                                                                            |
; | Code and original design and data Copyright  1985 by Chrysler Corporation |
; |       This relocateable source is Copyright  2005-2007 by Geoff Allan     |
; | Modifications for 3-bar and larger injectors, and additional code modules  |
; |                               are Copyright  2005-2007 by Geoff Allan     |
; \____________________________________________________________________________|

; Beginning with version 50, Ladybug parallel development.

    chem2 title Blueberry 60
    chem2 description 1987 Chrysler Turbo II Logic Module calibration Baseline
    chem2 author Geoff Allan
    chem2 boost 20 psi
    chem2 inj 35 pph

; NOTE: the above information is strictly informative. Changing the numbers
; above will NOT change anything in the source. You must use ChEM to
; change the injectors and desired boost.

; ***********************************************************************
; The following variables can be changed, then the source recompiled.
; ***********************************************************************

; ***********************************************************************
; MAP SENSOR
; ***********************************************************************
;
; Select which MAP sensor range you are using here:
;
Map2Bar = 0
Map3Bar = 1
;
; One or the other of the MAP values MUST be set. Not none, Not both. One.

    .if Map2Bar > 0
        chem2 map 2 Bar
    .endif
    .if Map3Bar > 0
        chem2 map 3 Bar
    .endif

;
; ***********************************************************************
; Assembler quirks:
;   Since this is a custom assembler, there are a few quirks that should be
;   noted.
; ***********************************************************************
; Bit functions: (derived from Ladybug development)
; Many bytes in the RAM contain status bits. For each of these the address value
; was separated into bit numbers, for example BitFlags_7f has b7f_bit7 through b7f_bit0.
; As the function of the bits is decoded, a global replace is performed to replace
; the bit name with a meaningful name, like:
;
;       b7f_SparkEngineNotRunning   = %10000000                 ;  -10000000-
;                                     ^ NOTE: no leading $
;
; When working with bits, you can add several together using the "or" operator | like this:
;
;       [b50_SparkEngineNotRunning | b50_BaroSolEnergized]      ;  -10100000-
;       ^  NOTE: Square brackets, not round              ^
;
; The inverse function ~ sets all bits except the ones specified, so:
;
;       <~b50_SparkEngineNotRunning                              ;  -01111111-
;       ^ NOTE: MUST INCLUDE < to convert the internal 32-bit value to required 8-bit
;
; You can combine the inverse ~ with the or | like this:
;
;       <~[b50_SparkEngineNotRunning | b50_BaroSolEnergized]     ;  -01011111-
;
; The intent here is to make the logical functionality of the code easier to follow.
;
; ***********************************************************************
;
; ***********************************************************************
; FUEL INJECTORS
; ***********************************************************************
; Engine Type and Flow Rate (at 55psi)
; 1985 - 1988 2.2L Turbo I                     - 27 lbs/hr
; 1987 - 1988 2.2L Turbo II                    - 33 lbs/hr
; 1988 - 1993 2.5L Turbo I                     - 34.85 lbs/hr
; 1989 - 1990 2.2L Turbo II                    - 34.85 lbs/hr
; 1991 - 1993 2.2L Turbo III                   - 34.85 lbs/hr
; 1989 - 1990 2.2L Turbo IV                    - 34.85 lbs/hr
; Mopar Performance "803s" (P4452803)          - 27 lbs/hr
; Mopar Performance "804s" (P4452804)          - 34.85 lbs/hr
; Mopar Performance +20% (P4529495)            - 42 lbs/hr
; Mopar Performance +40% "Super 60" (P5249452) - 52 lbs/hr
;
; The decision was made to hard code MAP changes and make injector changes
; via software because a MAP change is usually done once, upward. Injectors,
; however, may be changed frequently and adjusted for variations in flow or
; fuel composition (ethanol blend fuels require slightly more volume). Also,
; there are significantly more MAP changes than injector.
;
; ***********************************************************************

; *****************************************
; Overboost Value  (don't change this)
OverboostValue              = $aa

; *****************************************
; High speed error code flash mod: (currently disabled, don't change)
ucErrorCodeNumberWait       = $fa     ; stock=$fa ; mod=$50
ucErrorCodeNibbleWait       = $7d     ; stock=$7d ; mod=$3e
ucErrorCodeDigitFlash       = $1f     ; stock=$1f ; mod=$10

; *****************************************
; Transmission selection
AutomaticTransmission       = 0       ; 0=Manual, 1=Automatic

; *****************************************
; Detonation detector variables
UseDetonationMod            = 1
DetonationModMinTemp        = $a8     ; minimum temp, otherwise false positives on cold motor

; *****************************************
; The EGR/Purge solenoid is not used in my car (no EGR for T2 spec,
; cannister removed) therefore I plan to use the solenoid output for
; "something else".
UseEGRPurge                 = 0       ; set to 1 to use stock EGR/Purge function

; *****************************************
; Staging Rev Limiter Mod variables
UseStagingMod               = 1
; When stopped (speed < 3), rev limit to 3500 RPM.
; This allows staging with full throttle, then dumping the clutch on a green light.
; It is also handy to prevent some dumb mechanic from jumping your timing belt by revving.

;  ______________________
; |                      \
; |   CPU                |
; \______________________|

; Control Registers:

ioPort1ddr                              = $0000
ioPort2ddr                              = $0001

ioPort1data                             = $0002
; Pin   Mask    Bit Function            LM Pin
; 20    x80     7   knock kill (int)
; 19    x40     6   injector (int)
; 18    x20     5   power loss light    R4
; 17    x10     4   voltage reg
; 16    x08     3   cts bias (int)      R5
; 15    x04     2   injector 3-4        R3
; 14    x02     1   injector 1-2        R2
; 13    x01     0   speed sensor        R9

ioPort2data                             = $0003
; Pin   Mask    Bit Function            LM Pin
;       x80     7
;       x40     6
;       x20     5
; 12    x10     4   sci tx              R11
; 11    x08     3   sci rx              R13
; 10    x04     2   serial clock?
; 9     x02     1   anti-dwell          R6      anti-dwell on = spark, off = no spark
; 8     x01     0   dist reference      R10

ioPort3ddr                              = $0004
ioPort4ddr                              = $0005
ioPort3data                             = $0006
ioPort4data                             = $0007
timerCSR                                = $0008
counterHi                               = $0009
counterLo                               = $000a
ocrHi                                   = $000b
ocrLo                                   = $000c
icrHi                                   = $000d   ; used for distributor sync timer
icrLo                                   = $000e
ioPort3CSR                              = $000f
sciModeControl                          = $0010
sciTransceiveCtrl                       = $0011
sciRxData                               = $0012
sciTxData                               = $0013
ramControl                              = $0014
counterAltHi                            = $0015
counterAltLo                            = $0016
timerControl1                           = $0017
timerControl2                           = $0018
timerStatus                             = $0019
ocr2Hi                                  = $001a
ocr2Lo                                  = $001b
ocr3Hi                                  = $001c
ocr3Lo                                  = $001d
icr2Hi                                  = $001e   ; used for speed/distance pulse timer
icr2Lo                                  = $001f

;  ______________________
; |                      \
; |   Memory             |
; \______________________|

; Battery-Backed RAM:

LowestSessionTPS                        = $0040
L0041                                   = $0041
L0042                                   = $0042
L0043                                   = $0043
L0044                                   = $0044
LowestHistoricalTPS                     = $0045

FaultCode1                              = $0046
;7 $80 = Battery Temperature, Code 44
;6 $40 = Overboost, Code 45
;5 $20 = Battery Volts High, Code 46
;4 $10 = Battery Volts Low
;3 $08 = Lean condition, Code 51
;2 $04 = Rich Condition, Code 52
;1 $02 = rom Checksum, Code 53
;0 $01 = Distributor Sync

FaultCode2                              = $0047
;7 $80 = AC Cutout Relay
;6 $40 = Speed Ctl Solenoid, Code 34
;5 $20 = Radiator Fan Relay
;4 $10 = Wastegate Solenoid, Code 36
;3 $08 = Barometric Read Solenoid
;2 $04 = Charging System ASD volts too low, Code 41
;1 $02 = Auto Shutdown Relay
;0 $01 = Ignition Coil Control

FaultCode3                              = $0048
;7 $80 = Coolant Temp Sensor, Code 22
;6 $40 = Charge Temp Sensor, Code 23
;5 $20 = Throttle Position, Code 24
;4 $10 = AIS Motor
;3 $08 = Injector Current Limit Not Achieved
;2 $04 = Injector Driver interface
;1 $02 = Purge Solenoid
;0 $01 = Nothing

FaultCode4                              = $0049
;7 $80 = Engine Not Cranked Since Battery Disconnected, Cpde 11
;6 $40 = Memory Standby Power Lost, Code 12
;5 $20 = MAP Pneumatic, Code 14
;4 $10 = MAP Electrical, Code 13
;3 $08 = Vehicle Distance Sensor, Code 15
;2 $04 = Battery Voltage Sense, Code 16
;1 $02 = Engine Running Too Cool, Code 17?
;0 $01 = Oxygen Sensor, Code 21?

ErrorCodes1                             = $004a
ErrorCodes2                             = $004b
ErrorCodes3                             = $004c
ErrorCodes4                             = $004d
L004E                                   = $004e
L004F                                   = $004f

AutoCalCell0                            = $0050
AutoCalCell1                            = $0051
AutoCalCell2                            = $0052
AutoCalCell3                            = $0053
AutoCalCell4                            = $0054
AutoCalCell5                            = $0055
AutoCalCell6                            = $0056
AutoCalCell7                            = $0057
AutoCalCell8                            = $0058
AutoCalCell9                            = $0059
AutoCalCell10                           = $005a
AutoCalCell11                           = $005b
AutoCalCell12                           = $005c   ; idle cell
AutoCalCellEnd                          = $005d

ChargeTempVolts                         = $005d
Delta_MAP                               = $005e
AISMotorPositionStorage                 = $005f

; Volatile RAM:

; in progress: replace bits with meaningful names to make code self-descriptive

L0060                                   = $0060   ; bit flags to do with AIS and idle
b60_bit7                    = %10000000    ;
b60_bit6                    = %01000000    ;
b60_bit5                    = %00100000    ;
b60_bit4                    = %00010000    ;
b60_bit3                    = %00001000    ;
b60_bit2                    = %00000100    ;
b60_bit1                    = %00000010    ;
b60_bit0                    = %00000001    ;

Z83_PortA_copy                          = $0061
PortA_CruiseVent            = %10000000    ; cruise vent         B6
PortA_PurgeEGR              = %01000000    ; purge / egr         B5
PortA_CruiseVac             = %00100000    ; cruise vac          B4
PortA_ACclutch              = %00010000    ; ac clutch out       B3
PortA_FAN                   = %00001000    ; fan                 R21
PortA_Wastegate             = %00000100    ; wastegate           R19
PortA_ASD                   = %00000010    ; asd relay           R17
PortA_BaroRead              = %00000001    ; baro read solenoid  R15
; NOTE: For above, 1 is OFF, 0 is ON.

CruiseStatus                            = $0062
b62_bit7                    = %10000000    ; Cruise not enabled
b62_bit6                    = %01000000    ; Resuming
b62_bit5                    = %00100000    ;
b62_bit4                    = %00010000    ; Cruise Engaged
b62_bit3                    = %00001000    ;
b62_bit2                    = %00000100    ;
b62_bit1                    = %00000010    ;
b62_bit0                    = %00000001    ;

L0063                                   = $0063
Something_W                             = $0064
L0066                                   = $0066
L0067                                   = $0067
VehicleSpeed_W                          = $0069
Timer_CruiseDecelerate                  = $006b   ; used in cruise control
CruiseSetSpeed                          = $006c   ; used in cruise control
L006D                                   = $006d   ; used in cruise control
CruiseStatus2                           = $006e   ; used in cruise control
L006F                                   = $006f   ; used in cruise control
L0070                                   = $0070   ; used for stashing stack pointer
OverallAdvance                          = $0072
BarometricPressure                      = $0073
ErrorTimer                              = $0074   ; possibly cold load enrichment timer
Timer_OutOfClosedLoopMode               = $0076

; Inverse RPM is actually the time between pulses, so smaller numbers are higher RPMs
; 24 bit value
InverseRPM_B                            = $0077
InverseRPM_W                            = $0078

Counter_DistTimerOverflow               = $007a ; number of overflows for dist sync
FuelMonitorOutputTimer_W                = $007b
ElapsedTimeSinceLastDistSync_W          = $007d ; in microseconds

BitFlags7f                              = $007f
b7f_bit7                    = %10000000    ; Engine has knocked
b7f_MapOutOfRange                    = %01000000    ; MAP error ?
b7f_bit5                    = %00100000    ;
b7f_bit4                    = %00010000    ; Boost Above 5psi
b7f_bit3                    = %00001000    ; Dist Sync Error?
b7f_bit2                    = %00000100    ; Wastegate Control
b7f_bit1                    = %00000010    ; injector circuit 3-4
b7f_bit0                    = %00000001    ; injector circuit 1-2
; or, the bottom two bits are the current cylinder as 1-4-2-3

BitFlags80                              = $0080
b80_BadMAP                  = %10000000    ; Bad Map over/under Code 14/13
b80_BadTPS                  = %01000000    ; Bad TPS sensor Code
b80_bit5                    = %00100000    ; Distributor sync Code 54?
b80_WastegateLimpinMode     = %00010000    ; Wastegate error Code 36
b80_bit3                    = %00001000    ; Battery voltage too high Code 46
b80_bit2                    = %00000100    ;   ?
b80_BadCoolantTemp          = %00000010    ; Bad coolant temp Code 22
b80_BadChargeTemp           = %00000001    ; Bad air temp sensor Code 23

BitFlags81                              = $0081
b81_OffIdle                 = %10000000    ; Throttle Pressed (off idle)
b81_LowTpsUpdated           = %01000000    ; LowestHistoricalTPS has been updated
b81_bit5                    = %00100000    ;   ? Oxygen Sensor Stuck?
b81_bit4                    = %00010000    ; Oxygen Sensor Has Been Outside .75V to .29V Range (Thus is returning a signal)
b81_bit3                    = %00001000    ; Throttle below half
b81_O2Rich                  = %00000100    ; Oxygen Sensor data currently not valid (just after starting)
b81_bit1                    = %00000010    ;   ? WHEN SET, DOESNT DO CLOSED LOOP STUFF? DOES CHECK RAM0077
b81_FullThrottle            = %00000001    ; Throttle Floored (full throttle)

BitFlags82                              = $0082
b82_EngineNotRunning        = %10000000    ; ENGINE RUNNING = 0, Engine not running = 1
b82_bit6                    = %01000000    ;
b82_FuelRunMode             = %00100000    ; Fuel Mode: 1 = Running, 0 = Starting
b82_bit4                    = %00010000    ; baro read processing done
b82_DecayingFuelStartToRun  = %00001000    ; 1 if still decaying fuel from start to run
b82_bit2                    = %00000100    ;   ? Currently in Open Loop mode
b82_SparkStartMode          = %00000010    ; Spark Mode: 1 = Starting, 0 = Running
b82_bit0                    = %00000001    ;   ? O2 sensor known bad

BitFlags83                              = $0083
b83_bit7                    = %10000000    ;
b83_bit6                    = %01000000    ;
b83_bit5                    = %00100000    ;
b83_bit4                    = %00010000    ;   ? ASD circuit set (code 42)
b83_bit3                    = %00001000    ;   ? Injector 2 circuit failure (code 27)
b83_bit2                    = %00000100    ;   ? Injector 1 circuit failure (code 26)
b83_bit1                    = %00000010    ; Cruise Control Enabled
b83_bit0                    = %00000001    ;

BitFlags84                              = $0084
b84_bit7                    = %10000000    ; Baud Rate Signal or TX Enable?
b84_bit6                    = %01000000    ; Car Speed Too High and Engine Speed Too Low and Idling
b84_bit5                    = %00100000    ; TransientEnrich1  ? Engine normal temp
b84_bit4                    = %00010000    ;   ? Dont Try Closed Loop
b84_bit3                    = %00001000    ; Fuel is shut off (overboost, overrev, StagingMod)
b84_bit2                    = %00000100    ; Diagnostics Mode
b84_bit1                    = %00000010    ; Cold Load Enrichment  ? Something with reg 74 ('89 Cal)
b84_bit0                    = %00000001    ;   ? Oxygen Sensor Bad, fan status?

BitFlags85                              = $0085
b85_bit7                    = %10000000    ;   ? Something todo with setting baro.
b85_bit6                    = %01000000    ;
b85_bit5                    = %00100000    ;
b85_bit4                    = %00010000    ;
b85_bit3                    = %00001000    ;
b85_bit2                    = %00000100    ;
b85_bit1                    = %00000010    ;
b85_bit0                    = %00000001    ;

anotherPreviousMAP                      = $0086
TargetIdleSpeed                         = $0087
Counter_EngineRunningTime               = $0089
Counter_MainLoop1                       = $008a
Counter_MainLoop2                       = $008b
Counter_MainLoop3                       = $008c
Pulsewidth_InjectorSolution             = $008d
RPM_W                                   = $008f
L0091                                   = $0091

Block_MAP                               = $0092
MapAverageValue                         = $0092    ; 0
MapBlockScratchValue                    = $0093    ; 1
MapValue                                = $0094    ; 2
Factor_LeanoutOrEnrichTime_MAP          = $0095    ; 3

Block_TPS                               = $0096
TpsAverageValue                         = $0096    ; 0
TpsBlockScratchValue                    = $0097    ; 1
TpsVolts                                = $0098    ; 2
Factor_LeanoutOrEnrichTime_TPS          = $0099    ; 3

CoolantTemp                             = $009a
RawBattTemp                             = $009b
RawO2Sensor                             = $009c
RawAirTemp                              = $009d
RawCoolantTemp                          = $009e
RawTPS                                  = $009f
RawKnockSensor                          = $00a0
RawBattVoltage                          = $00a1
TargetBattVoltage                       = $00a2
ColdLoadEnrichmentValue                 = $00a3
LeanoutFactor_W                         = $00a4   ; word
Timer_Leanout                           = $00a6
BoostGoal                               = $00a7

; An unused byte, for our own mods:
ChEMreserved                            = $00a8
;5 $20 = Detonation Detection triggered

BoostTimer                              = $00a9
WastegateControlDutyCycle               = $00aa   ; max is c8 = 200 = % * 2
SecondaryO2ControllerKicks_Signed       = $00ab
CurrentAutoCalCell                      = $00ac
Timer_AISDelay                          = $00ad
L00AE                                   = $00ae
L00AF                                   = $00af
Timer_CLTIME_ClosedLoopTimer            = $00b0
L00B1                                   = $00b1
Timer_AutoCalInhibit                    = $00b2
Counter_L00B3                           = $00b3
EPPsSinceCalChange                      = $00b4
L00B5                                   = $00b5
Scratch_b6                              = $00b6
Scratch_b7                              = $00b7
Scratch_b8                              = $00b8
Scratch_b9                              = $00b9
Scratch_ba                              = $00ba
Scratch_bb                              = $00bb   ; used in mul16bit
Scratch_bc                              = $00bc   ; also used in mul16bit
FuelPulsewidth_scratch1                 = $00bd
L00BE                                   = $00be
FuelBaselineResult                      = $00bf
L00C0                                   = $00c0
L00C1                                   = $00c1
L00C2                                   = $00c2
Pulsewidth_PartThrottleEnrichment       = $00c3
L00C5                                   = $00c5
Timer_Over5PSIBoost                     = $00c6
ADScanIndex                             = $00c7
L00C8                                   = $00c8
L00C9                                   = $00c9
L00CA                                   = $00ca
L00CB                                   = $00cb
Counter_EPPs                            = $00cc
L00CD                                   = $00cd

; If the 4 CylinderRetard locations are moved they must stay together.
CylinderRetard1                         = $00ce
CylinderRetard2                         = $00cf
CylinderRetard3                         = $00d0
CylinderRetard4                         = $00d1
MaxRetard                               = $00d2

Timer_VoltageRegSignal                  = $00d3
FuelPulsewidthAddedForBatteryVoltage    = $00d4
Counter_BaroReadTime                    = $00d5
L00D6                                   = $00d6
Counter_ConsecutiveEPPs                 = $00d7
Timer_NavigatorFuelPulsewidth           = $00d8
Timer_FuelShutoff                       = $00d9
L00DA                                   = $00da
Counter_WastegateControl                = $00db
VerySlowCounter2point6seconds           = $00dc
KnockLookup                             = $00dd
BaroReadTimeEngineWithinReadRange       = $00de
L00DF                                   = $00df
FanOnCount                              = $00e0
FuelSubtractedFromStartToRun            = $00e1
L00E2                                   = $00e2
L00E3                                   = $00e3
L00E4                                   = $00e4
L00E5                                   = $00e5
Bits_WhichInjector                      = $00e6    ; bits 1 and 0 show which injectors are firing
L00E7                                   = $00e7
L00E8                                   = $00e8
L00E9                                   = $00e9

;  ______________________
; |                      \
; |   Hardware           |
; \______________________|

; ***********************************************************************************
; ONLY Change here if you are remapping the hardware addresses, otherwise DON'T TOUCH
; ***********************************************************************************
Z83Mapping                  = $2000   ; address for Z83 I/O port mapping
Z22Mapping                  = $4000   ; address for Z22 Timer mapping
Z85Mapping                  = $8000   ; address for Z85 A/D converter mapping
Z51Mapping                  = $c000   ; address for ROM begin mapping


; ***********
; Z83 IO Port
; ***********
Z83_IOPortADataCA1          = Z83Mapping

Z83_PortAdata               = Z83Mapping + 2
; Pin   Mask    Bit Function            LM Pin
; 11    x80     7   cruise vent         B6
; 10    x40     6   egr                 B5
; 9     x20     5   cruise solenoid     B4
; 8     x10     4   ac clutch out       B3
; 7     x08     3   fan                 R21
; 6     x04     2   wastegate           R19
; 5     x02     1   asd relay           R17
; 4     x01     0   baro read solenoid  R15
; NOTE: for above, 1 is OFF, 0 is ON

Z83_PortBdata               = Z83Mapping + 3
; Pin   Mask    Bit Function            LM Pin
; 27    x80     7   a/d conversion  (int)
; 28    x40     6   fuel monitor        R14
; 29    x20     5   tach out            R12
; 30    x10     4   alt fault       (int)
; 31    x08     3   ais                 R22
; 32    x04     2   ais                 R16
; 33    x02     1   ais                 R18
; 34    x01     0   ais                 R20

Z83_PortCdata               = Z83Mapping + 4
; Pin   Mask    Bit Function            LM Pin
; 35    x80     7   dist sync           B17
; 36    x40     6   cruise enable       B15
; 37    x20     5                       B14
; 38    x10     4   brake               B13
; 39    x08     3   park/neutral        B12
; 1     x04     2   ac clutch in        B11
; 2     x02     1   cruise resume       B10
; 3     x01     0   cruise set          B9

Z83_PortADataDirection      = Z83Mapping + 6
Z83_PortBDataDirection      = Z83Mapping + 7
Z83_PortCDataDirection      = Z83Mapping + 8
Z83_PortAcontrol            = Z83Mapping + 9
Z83_PortBcontrol            = Z83Mapping + $a
Z83_PortCcontrol            = Z83Mapping + $b

; *********
; Z22 Timer
; *********
Z22_Timer6840               = Z22Mapping
Z22_Timer6840_1             = Z22Mapping + 1
Z22_Timer6840_4             = Z22Mapping + 4

; *****************
; Z85 A/D Converter
; *****************
; Pin   Addr    Function            LM Pin
; 26    8000    MAP                 B19
; 27    8001    batt temp           B20
; 28    8002    O2                  B18
; 1     8003    air temp            B25
; 2     8004    coolant temp        B23
; 3     8005    tps                 B21
; 4     8006    knock               B24
; 5     8007    batt voltage        B22
Z85_ADConverter1            = Z85Mapping
Z85_ADConverter2            = Z85Mapping + 1
Z85_ADConverter3            = Z85Mapping + 2
Z85_ADConverter4            = Z85Mapping + 3
Z85_ADConverter5            = Z85Mapping + 4
Z85_ADConverterCoolantTemp  = Z85Mapping + 5
Z85_ADConverter7            = Z85Mapping + 6  ; MAP
Z85_ADConverterKnockSensor  = Z85Mapping + 7

;  ______________________
; |                      \
; |   ROM                |
; \______________________|

    seg ROM16k
    org Z51Mapping
    chem2 rom

RomBegin:
    db  $a6
ChecksumFixup:
    db  $66      ; NOT USED FOR CHECKSUM FIXUP. A666 is id code. Avoid changing.
HunstvilleTesterCodeBits:
    db  $00, $89, $e8, $44
FaultMasks:
    db  $e9, $3f, $55, $aa
TRANSM_Transmission:
    chem 6 n Transmission_Manual00_AutomaticFF X 0 256 in Y 0 256 out TRANSM Type_of_transmission_in_car._Used_for_idle_speed_and_boost. Manual 0 Automatic ff Unused 0 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
    chem2 byte TRANSM choice Automatic ff choice Manual 00 "Type of transmission in car. Used for idle speed and boost. Use 00 for Manual, ff for Automatic."
    .if AutomaticTransmission > 0
    db  $ff
    .else
    db  $00
    .endif

;  ______________________
; |                      \
; |   Spark Advance      |
; \______________________|
;
; Note: all curves are referenced to zero degrees and do not include mechanical advance
    chem2 group Timing
; *****************************************
; CLDMAP: Curve for spark advance vs. MAP when engine temp < TMPSWP
;         input is MAP
;         output is Spark Advance * 2 degrees
CLDMAP_AdvanceFromMapCold:
    .if Map2Bar > 0
      chem 2 n AdvanceFromMapCold2bar MAP -14.7 14.7 psi Spark_Advance -64 64 Degrees*2 CLDMAP Curve_for_Spark_Advance_from_MAP_when_Engine_Temp_is_below_TMPSWP.
      chem2 table signed CLDMAP simple use MAP -14.7 14.7 Psi use "Spark Advance" -64 64 Degrees "Curve for Spark Advance from MAP when Engine Temp is below TMPSWP."
      hex  03
      hex  5e f4 0139
      hex  70 0a ffd5
      hex  b8 fe
    .endif

    .if Map3Bar > 0
      chem 2 n AdvanceFromMapCold3bar MAP -14.7 29.4 psi Spark_Advance -64 64 degrees*2 CLDMAP Curve_for_Spark_Advance_from_MAP_when_Engine_Temp_is_below_TMPSWP.
      chem2 table signed CLDMAP simple use MAP -14.7 29.4 Psi use "Spark Advance" -64 64 Degrees "Curve for Spark Advance from MAP when Engine Temp is below TMPSWP."
      hex  03
      hex  33 0a 00e8
      hex  3e 14 ffde
      hex  9e 07
    .endif

    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
CLDLMT_LimitForColdMAPAtClosedThrottle:
    chem 3 n Advance_ColdestInForCLDMAP Coolant_Temp -200 260 Degrees Coolant_Temp -200 260 Degrees CLDLMT Limit_for_Cold_MAP_at_closed_throttle._Used_when_engine_temp_less_than_TMPSWP
    chem2 byte CLDLMT use "Coolant Temp" -200 260 Degrees "Coldest possible input for CLDMAP at closed throttle. Used when engine temp less than TMPSWP."
    db  $0a

; *****************************************
; HOTMAP: Curve for spark advance vs. MAP when engine temp > TMPSWP
;         input is MAP
;         output is Spark Advance * 2 degrees
;         output is signed (-127 to +127)
HOTMAP_AdvanceFromMapWarmPart:
    .if Map2Bar > 0
      chem 2 n AdvanceFromMapWarmPart2bar MAP -14.7 14.7 psi Spark_Advance -64 64 degrees HOTMAP Curve_for_Spark_Advance_from_MAP_when_Engine_Temp_is_above_TMPSWP_and_at_Full_Throttle.
      chem2 table signed HOTMAP simple use MAP -14.7 14.7 Psi use "Spark Advance" -64 64 Degrees "Curve for Spark Advance from MAP when Engine Temp is above TMPSWP and not at Full Throttle."
      hex 07
      hex 44 18 FF1C
      hex 4D 10 0000
      hex 54 10 FF33
      hex 5E 08 0000
      hex 77 08 FF73
      hex AA EC FF92
      hex B8 E6
    .endif
    .if Map3Bar > 0
      chem 2 n AdvanceFromMapWarmPart3bar MAP -14.7 29.4 psi Spark_Advance -64 64 degrees HOTMAP Curve_for_Spark_Advance_from_MAP_when_Engine_Temp_is_above_TMPSWP_and_at_Full_Throttle.
      chem2 table signed HOTMAP simple use MAP -14.7 29.4 Psi use "Spark Advance" -64 64 Degrees "Curve for Spark Advance from MAP when Engine Temp is above TMPSWP and not at Full Throttle."
    ; AdvanceFromMapWarmPart (HOTMAP)
    hex 07
    hex 2d 14 feab
    hex 33 0c 0000
    hex 37 0c fedc
    hex 3e 04 ffa6
    hex 4f fe ff88
    hex 71 ee ffad
    hex c7 d2
    .endif

    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
; WOTMAP: Curve for spark advance vs. MAP when WOT fuel and engine temp > TMPSWP
;         input is MAP
;         output is Spark Advance * 2 degrees
;         output is signed (-127 to +127)
WOTMAP_AdvanceFromMapWarmFull:
    .if Map2Bar > 0
      chem 2 n AdvanceFromMapWarmFull2bar MAP -14.7 14.7 psi Spark_Advance -64 64 degrees WOTMAP Curve_for_Spark_Advance_from_MAP_when_Engine_Temp_is_above_TMPSWP_and_not_at_Full_Throttle.
      chem2 table signed WOTMAP simple use MAP -14.7 14.7 Psi use "Spark Advance" -64 64 Degrees "Curve for Spark Advance from MAP when Engine Temp is above TMPSWP and at Full Throttle."
      hex 04
      hex 66 14 fe00
      hex 70 00 ff78
      hex 92 ee ffc7
      hex b6 e6
    .endif
    .if Map3Bar > 0
      chem 2 n AdvanceFromMapWarmFull3bar MAP -14.7 29.4 psi Spark_Advance -64 64 degrees WOTMAP Curve_for_Spark_Advance_from_MAP_when_Engine_Temp_is_above_TMPSWP_and_not_at_Full_Throttle.
      chem2 table signed WOTMAP simple use MAP -14.7 29.4 Psi use "Spark Advance" -64 64 Degrees "Curve for Spark Advance from MAP when Engine Temp is above TMPSWP and at Full Throttle."
    ; AdvanceFromMapWarmFull (WOTMAP)
    hex 04
    hex 2d 14 ff50
    hex 4a 00 ff45
    hex 73 e2 ffcb
    hex c9 d0
    .endif
    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
aMaxMap1:
    db  $1f
; *****************************************
aFurtherAdvanceCalcIfLT80:
;    chem2 byte COMMON bit 7 "Further Calc" "If Further Calc is set, advance calculations are refined."
    db  $ff

; *****************************************
TMPSWP_AdvanceEngineTempSwPoint:
    chem 3 n Advance_EngineTempSwitchPoint Coolant_Temp -200 260 Degrees Coolant_Temp -200 260 Degrees TMPSWP Engine_temperature_switch_point_for_hot_and_cold_map_advance_tables
    chem2 byte TMPSWP simple use "Coolant Temp" -200 260 Degrees "Engine temperature switch point for hot and cold Advance tables."
    db  $b1

; *****************************************
zAdvanceFromRpmsWarmAlways:
    chem 0 n AdvanceFromRpmsWarmAlways RPMs 0 8192 rpm Advance 0 128 Degrees name Unknown_table
    chem2 table COMMON use RPMs 0 8192 RPM use "Spark Advance" 0 128 Degrees "Unknown"
    hex 02
    hex 3f 08 0155
    hex 4b 18

    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
RetardFactorFromBarometricPressure:
    .if Map2Bar > 0
      chem 0 n RetardFactorFromBarometricPressure2bar Barometric_Pressure_from_MAP -14.7 14.7 psi Retard_Multiplier 0 256 out name Description
      chem2 table signed COMMON use MAP -14.7 14.7 Psi "Unknown, but seems to be a retard multiplier."
      hex 02
      hex 6D 04 FFA3
      hex 78 00
    .endif
    .if Map3Bar > 0
      chem 0 n RetardFactorFromBarometricPressure3bar Barometric_Pressure_from_MAP -14.7 29.4 psi Retard_Multiplier 0 256 out name Description
      chem2 table signed COMMON use MAP -14.7 29.4 Psi "Unknown, but seems to be a retard multiplier."
      hex 02
      hex 48 04 FF6e
      hex 4f 00
    .endif
    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
zRetardFromRpmsForAltitude:
    chem 2 n * RPMs 0 8192 rpm Retard_Amount -128 128 out name Unknown_for_certain
    chem2 table COMMON use RPMs 0 8192 RPM "Unknown, but seems to be a retard multiplier."
    hex 02
    hex 38 ff ec62
    hex 45 00

    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
MinimumAdvance:
    chem 3 n AdvanceMinimum Advance 0 128 Degrees Advance 0 128 Degrees MINADV Minimum_allowed_spark_advance._Default_is_$06
    chem2 byte MINADV simple use "Spark Advance" 0 128 Degrees "Minimum allowed Spark Advance."
    db  $06

; *****************************************
RetardFromTPS:
    chem 0 n * TPS_Delta 0 256 in Retard 0 256 out name When_you_lift_off_throttle,_this_retards_spark
    chem2 table signed COMMON use "TPS Delta" 0 256 units "Unknown, but seems to retard spark when throttle lifted."
    hex 03
    hex 05 00 0133
    hex 1e 1e 0072
    hex 4b 32
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
RetardFromTPSAboveThisMAP:
    .if Map2Bar > 0
      chem 3 n RetardFromTPSAboveThisMAP2bar MAP -14.7 14.7 psi MAP -14.7 14.7 psi NAME During_the_advance_calculations,_if_the_MAP_is_above_this_value_the_RetardFromTPS_table_will_be_applied
      chem2 byte COMMON use MAP -14.7 14.7 psi "If MAP is above this value, RetardFromTPS will be used."
      db  $93
    .endif
    .if Map3Bar > 0
      chem 3 n RetardFromTPSAboveThisMAP3bar MAP -14.7 29.4 psi MAP -14.7 29.4 psi NAME During_the_advance_calculations,_if_the_MAP_is_above_this_value_the_RetardFromTPS_table_will_be_applied
      chem2 byte COMMON use MAP -14.7 29.4 psi "If MAP is above this value, RetardFromTPS will be used."
      db  $41
    .endif

; *****************************************
RetardFromTPSAboveThisRPM:
    chem 3 n * RPMs 0 8192 RPM RPMs 0 8192 RPM NAME Use_TPS_retard_table_above_this_RPM._Default_is_$27
    chem2 byte COMMON use RPMs 0 8192 RPM "If RPM is above this value, RetardFromTPS will be used."
    db  $27

; *****************************************
RetardFromTPSAboveThisTPS:
    chem 3 n * TPS 0 5 Volts TPS 0 5 Volts NAME During_the_advance_calculations,_if_the_TPS_is_above_this_value_the_RetardFromTPS_table_will_be_applied
    chem2 byte COMMON use TPS 0 5 Volts "If TPS is above this value, RetardFromTPS will be used."
    db  $50

; *****************************************
; GOVNER: Spark Advance vs. RPM
;         input is RPMs
;         output is Spark Advance * 2
GOVNER_AdvanceFromRpms:
    chem 0 n AdvanceFromRpms RPMs 0 8192 RPM Spark_Advance 0 128 degrees GOVNER Simulates_the_mechanical_advance_function._Spark_Advance_is_added_from_this_table_from_RPMs_at_all_engine_temperatures.
    chem2 table GOVNER simple use RPMs 0 8192 RPM use "Spark Advance" 0 128 Degrees "Simulates the mechanical advance function. Spark Advance is added from this table at all engine temperatures and MAP levels."
    ; AdvanceFromRpms (GOVNER)
    hex 04
    hex 20 2e 0063
    hex 32 35 0017
    hex 9d 3f 0080
    hex c7 54

    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
; Start/Run transition values:
; *****************************************

TransitionRPMsImmediateSwitchToRun:
    chem 4 n StartingRpmsImmediateSwitchToRun Speed 0 65535 something Y 0 256 out SKAPRD Immediate_Switch_To_Run_Mode_when_Inverse_RPMs_above_this_value._Stock_is_$0092,_should_be_814_RPM
    chem2 word SKAPRD "Immediate switch to Fuel Run mode when RPMs are above this value. This is inverse RPMs, lower numbers are higher RPMs. Stock is 0092 which is about 814 RPM."
    dw  $0092   ; should be 814 RPM

; Spark transfer points:
; Number of consecutive EPPs needed to be above Start/Run:
SRTIME_StartRunHotNumConsecEPPs:
    chem 3 n StartingConsecEppsForAboveStartRunHot X 0 256 in Y 0 256 out SRTIME Number_of_consecutive_engine_turns_before_allowing_start/run_transition_-_Hot
    chem2 byte SRTIME "Number of consecutive engine turns before allowing start to run transition when engine above SRTMTP"
    db  $04
SRTIME2_StartRunColdNumConsecEPPs
    chem 3 n StartingConsecEppsForAboveStartRunCold X 0 256 in Y 0 256 out SRTIME2 Number_of_consecutive_engine_turns_before_allowing_start/run_transition_-_Cold
    chem2 byte SRTIME2 "Number of consecutive engine turns before allowing start to run transition when engine below SRTMTP"
    db  $04
; This is the temperature switch point for "hot" and "cold" start/run values above
SRTMTP_StartRunTemperatureSwitchPoint:
    chem 3 n StartingTempSwitchPointForSRTIME X 0 256 in Y 0 256 out SRTMTP Temperature_switch_point_for_SRTIME_Start/run_transition
    chem2 byte SRTMTP use "Coolant Temp" -200 260 Degrees "Engine temperature switch point for start run transfer values SRTIME."
    db  $b0
; *****************************************

Table09_FromCoolantTemp:
    db  $ff

; Spark Start/Run transition RPMs:
; Apparently this was designed to vary based on temperature, but the 87
; code was just set to a single number.
;
; Note: These numbers represent the time between EPPs, the inverse of RPMs.
;       Therefore, lower numbers are higher RPMs.

;       mod, GLA, remove lookup, replace with just the return values

SparkTransitionStartToRun:
    chem 4 n StartingSparkTransitionStartToRun Speed 8192 0 Inverse_RPMs Y 0 256 out STRNTR Switch_Spark_To_Run_Mode_when_Inverse_RPMs_below_this_value._Stock_is_$00c3_should_be_about_600_RPM
    chem2 word STRNTR "Immediate switch to Spark Run mode when RPMs are above this value. This is inverse RPMs, lower numbers are higher RPMs. Stock is 00c3 which is about 600 RPM."
    dw  $00c3       ; should be about 600 RPM


    hex ff ff ffff
    hex ff ff ffff

Table10_FromCoolantTemp:
    db  $ff

SparkTransitionRunToStart:
    chem 4 n StartingSparkTransitionRunToStart Speed 8192 0 Inverse_RPMs Y 0 256 out RMSTTR Switch_Back_To_Spark_Start_Mode_if_Inverse_RPMs_increase_above_this_value._Stock_is_$011b_should_be_about_410_RPM
    chem2 word RMSTTR "Immediate switch to Spark Start mode when RPMs are below this value. This is inverse RPMs, lower numbers are higher RPMs. Stock is 011b which is about 410 RPM."
    dw  $011b       ; should be about 400-450 RPM

    hex ff ff ff

; Original tables:
; Table09_FromCoolantTemp:
;   db  $ff
;   dw  $00c3, $ffff
;   db  $ff, $ff
;   db  $ff, $ff, $ff, $ff
;
; Table10_FromCoolantTemp:
;   db  $ff
;   dw  $011b, $ffff
;   db  $ff

; *****************************************
; SPKLMT: Total Spark Advance Allowed vs. Engine RPM
;         input RPM
;         output Maximum Advance * 2
SPKLMT_AdvanceAllowedFromRpms:
    chem 0 y AdvanceAllowedFromRpms RPM 0 8192 in Maximum_Advance 0 128 degrees SPKLMT This_is_the_absolute_maximum_amount_of_Advance_that_can_be_added_based_on_RPMs._If_the_other_calculations_exceed_this_value,_this_value_will_be_used_instead.
    chem2 table SPKLMT simple use RPMs 0 8192 RPM use "Spark Advance" 0 128 Degrees "Absolute maximum amount of advance allowed per RPM. If other calculations exceed this value, this value will be used instead."
    ; AdvanceAllowedFromRpms (SPKLMT)
    hex 03
    hex 64 64 fe77
    hex 71 50 ffe8
    hex db 46

    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
; PNIDLE: Spark Advance subtracted when closed throttle in Park/Neutral
;         and RPM < PNRPM
;         input Engine Temp
;         output Spark Advance * 2
PNIDLE_AdvanceParkNeutralIdle:
    chem2 table PNIDLE use RPMs 0 8192 RPM use "Spark Advance" 0 128 Degrees "Advance subtracted when closed throttle in Park/Neutral. Increasing this value could make a smoother idle."
    hex 01
    hex ff 00
    hex ff ff ffff
PNRPM_AdvanceParkNeutralIdleRPMSwitch:
    chem2 byte PNRPM "RPM detection value for Park/Neutral Idle advance reduction (to smooth idle)."
    db  $01

; *****************************************
; Knock Constants:
    chem2 group "Knock Retard"

; *****************************************
KNKRPM_KnockRetardMinRPM:
    chem 3 n KnockRetardMinimumRPMs 0 256 in Y 0 256 out KNKRPM Minimum_RPMs_for_Knock_to_cause_Retard_-_x10_=_1024_rpm
    chem2 byte KNKRPM use "RPMs" 0 16384 RPM "Minimum RPMs for Knock event to cause Retard."
    db  $10

; *****************************************
; KNKOFS: Delta above Baro up to which RTRDRT is used and KNKCNT and KNKCOR incremented
KNKOFS_KnockRetardOffset:
    chem 6 n KnockRetardOffset X 0 256 in Y 0 256 out KNKOFS Delta_above_Baro_up_to_which_RTRDRT_is_used_and_KNKCNT_and_KNKCOR_incremented._5.4_psi 2-bar 2f 3-bar 1f Unused 0 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
    .if Map2Bar > 0
      chem2 byte KNKOFS use MAP 0 29.4 psi "Delta above Baro up to which RTRDRT (Retard Rate) is used and KNKCNT and KNKCOR are incremented. Stock is 5.4 psi"
      db  $2f
    .endif
    .if Map3Bar > 0
      chem2 byte KNKOFS use MAP 0 44.1 psi "Delta above Baro up to which RTRDRT (Retard Rate) is used and KNKCNT and KNKCOR are incremented. Stock is 5.4 psi"
      db  $1f
    .endif
; *****************************************
; RTRDRT: Retard Rate for knock events, not in boost
RTRDRT_KnockRetardRateNoBoost:
    chem 3 n KnockRetardRateNoBoost X 0 256 in Y 0 256 out RTRDRT Knock_Retard_Rate_-_no_boost._Amount_of_retard_applied_when_knock_detected_when_not_in_boost._x06_=_3_degrees
    chem2 byte RTRDRT use Retard 0 128 Degrees "Knock Retard Rate, no boost. Amount of retard applied when not in boost. Stock is 3 degrees."
    db  $09
; RTRDRT2: Retard Rate for knock events, in boost
RTRDRT2_KnockRetardRateInBoost:
    chem 3 n KnockRetardRateInBoost X 0 256 in Y 0 256 out RTRDRT2 Knock_Retard_Rate_in_boost._x03_=_1.5_degrees
    chem2 byte RTRDRT2 use Retard 0 128 Degrees "Knock Retard Rate, in boost. Amount of retard applied when in boost. Stock is 1.5 degrees."
    db  $04
; *****************************************
; INCSRT: Increase rate - 0.5 Engine Degrees every (INCSRT+1) * 11ms
;         Use ONLY $00, $01, $03, $07, $0f, $1f, $3f, $7f, $ff
INCSRT_IncreaseRate:
    chem 3 n KnockRetardIncreaseRate X 0 256 in Y 0 256 out INCSRT Knock_Retard_increase_rate._x1f=1.4_degrees/sec_-_use_ONLY_x00,_x01,_x03,_x07,_x0f,_x1f,_x3f,_x7f,_xff
;    chem2 byte INCSRT use "Increase Rate" 0 128 Degrees "Knock Retard Rate, no boost. Amount of retard applied when not in boost. Stock is 3 degrees."
    db  $1f

RETMAX_KnockRetardMaxRetardFromMAP:
    .if Map2Bar > 0
      chem 0 n KnockRetardMaxRetardFromMap2bar MAP -14.7 14.7 psi Maximum_Retard 0 128 Degrees RETMAX Maximum_Retard_From_MAP.
      chem2 table RETMAX use MAP -14.7 14.7 psi use "Retard" 0 128 Degrees "Maximum Knock Retard from MAP. "
      hex 02
      hex 56 04 00a7
      hex 70 15
    .endif
    .if Map3Bar > 0
      chem 0 n KnockRetardMaxRetardFromMap3bar MAP -14.7 29.4 psi Maximum_Retard 0 128 Degrees RETMAX Maximum_Retard_From_MAP.
      chem2 table RETMAX use MAP -14.7 29.4 psi use "Retard" 0 128 Degrees "Maximum Knock Retard from MAP."
    ; KnockRetardMaxRetardFromMAP (RETMAX)
    hex 02
    hex 39 06 015a
    hex 4a 1d
    .endif
    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
; MAXRET - KNKLIM if THR > THRKNK and RPM < KLMRPM
KNKLIM_KnockRetardLimitDegrees:
    chem 3 n KnockRetardLimit X 0 256 in Y 0 256 out KNKLIM Knock_Limit_in_degrees_times_2
    chem2 byte KNKLIM use Retard 0 128 Degrees "Knock retart limit. Stock is 2 degrees."
    db  $06
THRKNK_KnockRetardThrottleLimit:
    chem 3 n KnockRetardThrottleLimit X 0 256 in Y 0 256 out THRKNK Delta_above_MINTHR._Maximum_Retard_is_KNKLIM_if_Throttle_above_THRKNK_and_RPM_<_KLMRPM_-_x53_equals_60_percent
    chem2 byte THRKNK use TPS 0 5 Volts "Delta above Min throttle above which Knock Retard is used. Stock is $53."
    db  $53
KLMRPM_KnockRetardMaxRPMforRate:
    chem 3 n KnockRetardRpmLimit X 0 256 in Y 0 256 out KLMRPM Knock_limit_RPM_-_H-E/16_ROM_-_x10_=_1024_RPM
    chem2 byte KLMRPM use RPMs 0 16384 RPM "Maximum RPM for using Knock Retard Rate. Stock is 1024"
    db  $10

; *****************************************
DLTKNK_KnockSensorVoltageFromKnock:
    chem 0 n KnockSensorVoltageFromKnock RPMs 0 16384 RPM Volts 0 5 Volts DLTKNK Knock_Sensor_Voltage_From_Knock._Raw_sensor_voltage_above_this_level_is_detected_as_Knock
    chem2 table DLTKNK use MAP 0 16384 RPM use "Threshold" 0 5 Volts "Knock sensor voltages above the threshold are treated as a Knock Event."
    ; KnockSensorVoltageFromKnock (DLTKNK)
    hex 05
    hex 1f 1a 0140
    hex 2f 2e 02c0
    hex 3f 5a 0570
    hex 4f b1 01e3
    hex 61 d3
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
SubtractedFromRPMDuringIdleRoutine:
    dw  $001e
AddedBackToRPMDuringIdleRoutine:
    db  $1e
IdleBytes1:
    db  $16,$bc,$00
IdleBytes2:
    db  $0b,$5e,$00
aByteUsedDuringIdleRoutine:
    db  $01
ComparedWithInverseRPM_BDuringIdleRoutine:
    dw  $00d0

;  ______________________
; |                      \
; |   Starting Fuel      |
; \______________________|

    chem2 group "Starting Fuel"

STRTFL_StartingFuelFromEngineTemp:
    chem 0 n StartFuelFromEngineTemp Coolant_Temp -200 260 in sqrtPW 0 1024 uSec STRTFL Starting_Fuel_-_actual_Y_axis_(output)_is_Sqrt(PulseWidth_in_uSec/4)._SCALE_to_FUEL_using_SQRT
    chem2 table inj sqrt STRTFL simple use "Coolant Temp" -200 260 Degrees use "Pulsewidth" 0 256 uSec "Starting Fuel. Actual starting fuel Pulsewidth is (STRTFL * STRTFL * (1 - STFBAR * STFTMP) * EPPMUL + BATOFF * 4) * 4"
;    .inj table sqrt y
    ; StartingFuelFromEngineTemp (STRTFL)
    hex 09
    hex 58 b1 ffe9
    hex 63 b0 fd8c
    hex 6e 95 fd8c
    hex 79 7a fea3
    hex 84 6b fe95
    hex 90 5a fde9
    hex 9b 43 ffe9
    hex b1 41 ff39
    hex cc 2c
    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
; This determines the minimum input to the above table, STRTFL_StartingFuelFromEngineTemp,
; if no throttle input during startup
STRLMT_StartingFuelColdestTemp:
    chem 3 n StartFuelColdestTemp temperature -200 260 degrees_F Y 0 256 out STRLMT Coldest_temp_used_for_STRTFL_at_closed_throttle._x69_=_-10F
    chem2 byte STRLMT use "Coolant Temp" -200 260 Degrees "Coldest Temperature allowed for input to STRTFL"
    db  $69

; *****************************************
EPPMUL_StartFuelPerEPP:
    chem 0 n StartFuelMultiplierPerEPP EPPs 0 256 in Multiplier 0.0 1.0 out EPPMUL Start_Fuel_multiplier_per_engine_pulse._This_lowers_the_amount_of_fuel_for_starting_the_longer_you_crank_the_engine.
    chem2 table EPPMUL use EPPs 0 256 units use "Multiplier" 0.0 1.0 out "Starting Fuel Multiplier per Engine Pulse. Actual starting fuel Pulsewidth is (STRTFL * STRTFL * (1 - STFBAR * STFTMP) * EPPMUL + BATOFF * 4) * 4"
    ; StartFuelPerEPP (EPPMUL)
    hex 02
    hex 0a ff fde7
    hex 28 c0

; *****************************************
STFTMP_StartFuelTemperature:
    chem 0 n StartFuelBaroCompTemp Coolant_Temp -200 260 degreesF Inverse_Multiplier 0 1 out STFTMP Baro_comp_multiplier._Used_for_starting_fuel_calculations._Not_used_by_default_in_87.
    chem2 table STFTMP use "Coolant Temp" -200 260 Degrees use "Inverse Multiplier" 0 1 out "Baro Compensation Multiplier for Starting Fuel (not used by default in 87). Actual starting fuel Pulsewidth is (STRTFL * STRTFL * (1 - STFBAR * STFTMP) * EPPMUL + BATOFF * 4) * 4"
    hex 01
    hex ff ff
    hex ff ff ffff
    hex ff ff ffff

; *****************************************
STFBAR_StartFuelBaro:
    .if Map2Bar > 0
      chem 0 n StartFuelBaroCompBaro2bar Barometric_Pressure_From_MAP -14.7 14.7 Baro Inverse_Multiplier 0 1 out STFBAR Barometric_compensation_for_starting.
      chem2 table STFBAR use "Barometric Pressure" -14.7 14.7 psi use "Inverse Multiplier" 0 1 out "Baro Compensation Multiplier for Starting Fuel. Actual starting fuel Pulsewidth is (STRTFL * STRTFL * (1 - STFBAR * STFTMP) * EPPMUL + BATOFF * 4) * 4"
      hex 02
      hex 60 59 fc94
      hex 7a 00
    .endif
    .if Map3Bar > 0
      chem 0 n StartFuelBaroCompBaro3bar Barometric_Pressure_From_MAP -14.7 29.4 Baro Inverse_Multiplier 0 1 out STFBAR Barometric_compensation_for_starting.
      chem2 table STFBAR use "Barometric Pressure" -14.7 29.4 psi use "Inverse Multiplier" 0 1 out "Baro Compensation Multiplier for Starting Fuel. Actual starting fuel Pulsewidth is (STRTFL * STRTFL * (1 - STFBAR * STFTMP) * EPPMUL + BATOFF * 4) * 4"
    ; StartFuelBaro (STFBAR)
    hex 02
    hex 3f 4d fbb9
    hex 51 00
    .endif
    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
SFDCAY_StartDecayIntoRun:
    chem 0 n StartFuelDecayIntoRunFuel Coolant_Temp -200 260 degreesF PW_Subtracted 0 256 ms/rev SFDCAY Start_fuel_decay_into_run_fuel._SCALE_to_FUEL
    chem2 table inj SFDCAY use "Coolant Temp" -200 260 Degrees use "Pulsewidth Removed" 0 256 "uSec per EPP" "Start fuel decay into run fuel. This is how much starting fuel to remove per engine rotation while cross fading from start to run"
;    .inj table scale y
    ; StartDecayIntoRun (SFDCAY)
    hex 03
    hex 63 1b ff94
    hex 90 08 ffef
    hex cc 04

    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

; *****************************************
PRSHOT_StartPrimeFuelShot:
    chem 0 n StartFuelPrimeFuelShot Coolant_Temp -200 260 degreesF Pulsewidth 0 256 ms PRSHOT Prime_Fuel_Shot._SCALE_to_FUEL
    chem2 table inj PRSHOT simple use "Coolant Temp" -200 260 Degrees use "Pulsewidth" 0 256 ms "Fuel injector open time between key on and cranking."
;    .inj table scale y
    ; StartPrimeFuelShot (PRSHOT)
    hex 03
    hex 69 7d fc0a
    hex 84 12 ffdf
    hex 9b 0f

    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

;  ______________________
; |                      \
; |   O2 Sensor          |
; \______________________|

    chem2 group "O2 Sensor"

; *****************************************
O2TIME_O2DelayTimeToClosedLoop:
    .chem 3 n DelayTimeToClosedLoop in_desc 0 65535 in Y 0 256 out O2TIME Detailed_Description3
    chem2 byte O2TIME use "Delay Time" 0 721 Seconds "Time delay before switching to Closed Loop, O2 must still have left the middle range."
    db  $07
CLTEMP_O2TempToStartClosedLoopTimer:
    .chem 3 n TempToStartCLTIMETimer in_desc 0 65535 in Y 0 256 out CLTEMP Temperature_to_start_CLTIME_Timer_-_$94_=_70degreesF
    chem2 byte CLTEMP use "Coolant Temp" -200 260 Degrees "Temperature before switching to Closed Loop, O2 must still have left the middle range."
    db  $94

; *****************************************
; For detecting toggles
; *****************************************
O2HSWP_O2HighLimitForRampSwitching:
    chem 3 n O2HighLimitForRampSwitching O2 0 256 none Y 0 256 out O2HSWP Compared_with_raw_O2_sensor_signal_to_determine_toggle._$19_stock._O2_sensor_range_is_$00_to_$2f
    chem2 byte O2HSWP "Compared with Raw O2 signal for switching toggles. This is the high limit"
    db  $19
O2LSWP_O2LowLimitForRampSwitching:
    chem 3 n O2LowLimitForRampSwitching O2 0 256 none Y 0 256 out O2LSWP Compared_with_raw_O2_sensor_signal_to_determine_toggle._$18_stock._O2_sensor_range_is_$00_to_$2f
    chem2 byte O2LSWP "Compared with Raw O2 signal for switching toggles. This is the low limit"
    db  $18
; *****************************************
O2TLMH_O2HighLimitDelayedClosedLoop:
    chem 3 n O2HighLimitDelayedClosedLoop O2 0 256 none Y 0 256 out O2TLMH O2_Sensor_High_Limit_for_delayed_switch_to_closed_loop._$1f_stock._O2_sensor_range_is_$00_to_$2f
    chem2 byte O2TLMH "Compared with Raw O2 signal for Delayed switch to Closed Loop"
    db  $1f
O2TLML_O2LowLimitDelayedClosedLoop:
    chem 3 n O2LowLimitDelayedClosedLoop O2 0 256 none Y 0 256 out O2TLML O2_Sensor_Low_Limit_for_delayed_switch_to_closed_loop._$0f_stock._O2_sensor_range_is_$00_to_$2f
    chem2 byte O2TLML "Compared with Raw O2 signal for Delayed switch to Closed Loop"
    db  $0f
; *****************************************
O2MIDH_O2HighLimitForMidLevelDetermination:
    chem 3 n O2HighLimitForMidLevelDetermination O2 0 256 none Y 0 256 out O2MIDH O2_Sensor_High_Limit_for_In-Middle_determination._$1c_stock._O2_sensor_range_is_$00_to_$2f
    chem2 byte O2MIDH "Compared with Raw O2 signal for In-Middle determination"
    db  $1c
O2MIDL_O2LowLimitForMidLevelDetermination:
    chem 3 n O2LowLimitForMidLevelDetermination O2 0 256 none Y 0 256 out O2MIDL O2_Sensor_Low_Limit_for_In-Middle_determination._$12_stock._O2_sensor_range_is_$00_to_$2f
    chem2 byte O2MIDL "Compared with Raw O2 signal for In-Middle determination"
    db  $12

; Note:
; There was no Immediate switch to closed loop in 87, that was added for 88
; Adding immediate switch might improve cold emissions.

; *****************************************
;      O2 Controller Ramp Calibration
; *****************************************
; How it works:
;   The O2 sensor is monitored but ONLY for "rich/lean". Even though
;   the O2 sensor value is from 0 to 15, the only thing that counts
;   for this system is whether it is "rich" or "lean". A change from one
;   to the other is a "toggle". The system constantly toggles between
;   rich and lean so that the overall average is around stoichiometric.
;
; Background information (from the net:)
;     An "ideal" fuel/air mixture in which both the fuel and the oxygen in the air are
;     completely consumed is called the "stoichiometric" mixture. The stoichiometric mixture
;     is usually specified as the ratio of the mass of air to the mass of a particular fuel.
;     For example, if a fuel required 10 kg of air to precisely burn 1 kg of the fuel (that is,
;     all the fuel burned and there was no oxygen left in the air afterwards) then the
;     stoichiometric ratio for that fuel would be 10:1.
;
;     Chevron give the stoichiometric ratio of air mass to fuel mass for gasoline as 14.7:1,
;     but don't specify the composition of this hypothetical gasoline.
;
;     From http://www.faqs.org/faqs/autos/gasoline-faq/part2/ comes the following table
;     of typical stoichiometric air-fuel ratios:
;
;            Fuel   | Stoichiometric air-fuel ratio
;        -----------+------------------------------
;          methanol | 6.4
;           ethanol | 9.0
;              MTBE | 11.7
;        ETBE, TAME | 12.1
;          gasoline | 14.6
;
;     An oxygenated gasoline with 89% gasoline and 11% MTBE would then have a stoichiometric
;     ratio of 14.281 (note that this would require a Wide Band O2 sensor to adjust, since the
;     O2 sensor in our cars is designed for gasoline only, and is a very narrow range. In case
;     you were wondering, this is one reason some people are interested in Wide Band O2 sensors.)
;
; *****************************************
; This system divides a range of RPMs and MAP readings into "cells". The number of
; "kicks", or increases/decreases in fuel required to toggle the O2 reading, is different
; for each of these cells. This allows some fairly powerful flexibility in tuning. The only time
; you might want to adjust these values is for different injector flow rates, or to speed or slow
; the adjustment being performed.
; *****************************************
; Note to tuners: DO NOT AIM FOR STOICHIOMETRY! Although it might be fine for idle or
; low speed operation, at higher RPMs and under boost you DEFINITELY want a RICH mixture. This
; is because stoichiometric combustion generates the most heat. No current engine would survive
; very long with the temperatures this would generate, hot enough to melt the engine.
; *****************************************
; Each cell has a kick value. This is the amount of change applied to the running fuel
; every eleven milliseconds until a rich/lean toggle occurs. The net result is the
; familiar rich/lean flashing that your O2 sensor gauge shows you.
; *****************************************
;
; (one kick per 11ms)
; Fudge factor: if an O2 toggle occurs before the primary ramp
; finishes, then the "first" step in the new direction is limited
; to the accumulated steps before the toggle.
;
;               Cells:
;
; HIRPM  +-----+-----+-----+
;        |  2  |  5  |  8  |
; RPM#2 >+-----+-----+-----+
;        |  1  |  4  |  7  |
; RPM#1 >+-----+-----+-----+
;        |  0  |  3  |  6  |        Idle Cell: 9
;        +-----+-----+-----+
;              ^     ^
;            Map#1 MAP#2 HIMAP
;

; *****************************************
PRIRMP_PrimaryO2Ramp:
; *****************************************
    chem2 ramp PRIRMP

    chem 3 n O2PrimaryRampLowerRpm X 0 256 in Y 0 256 out PRIRMP Primary_Ramp_lower_Rpm_boundary_in_RPM/64
    db  $13
    chem 3 n O2PrimaryRampHigherRpm X 0 256 in Y 0 256 out PRIRMP Primary_Ramp_higher_Rpm_boundary_in_RPM/64
    db  $19
    chem 3 n O2PrimaryRampLowerMap X 0 256 in Y 0 256 out PRIRMP Primary_Ramp_lower_Map_boundary
    .if Map2Bar > 0
      db  $33
    .endif
    .if Map3Bar > 0
      db  $22
    .endif
    chem 3 n O2PrimaryRampHigherMap X 0 256 in Y 0 256 out PRIRMP Primary_Ramp_higher_Map_boundary
    .if Map2Bar > 0
      db  $55
    .endif
    .if Map3Bar > 0
      db  $39
    .endif
; *****************************************
; For each cell, the first number is "rich", the second is "lean".
; If the O2 sensor is returning a lean value, "rich" kicks are added.
; If the O2 sensor is returning a rich value, "lean" kicks are added.
;
;     KICK - (factor * 512) - 512  (unsigned number)
; eg: $19  -   1.05  * 512  - 512 - 5%
; *****************************************
;          KICKS
;       Rich, Lean
    chem 4 n O2PrimaryRampKickCell0 x 0 65535 y Y 0 256 out PRIRMP Kick_Cell0_PrimaryRamp
    db  $1a,  $04       ; Cell 0
    chem 4 n O2PrimaryRampKickCell1 x 0 65535 y Y 0 256 out PRIRMP Kick_Cell1_PrimaryRamp
    db  $1a,  $05       ; Cell 1
    chem 4 n O2PrimaryRampKickCell2 x 0 65535 y Y 0 256 out PRIRMP Kick_Cell2_PrimaryRamp
    db  $1a,  $05       ; Cell 2
    chem 4 n O2PrimaryRampKickCell3 x 0 65535 y Y 0 256 out PRIRMP Kick_Cell3_PrimaryRamp
    db  $1a,  $04       ; Cell 3
    chem 4 n O2PrimaryRampKickCell4 x 0 65535 y Y 0 256 out PRIRMP Kick_Cell4_PrimaryRamp
    db  $1a,  $05       ; Cell 4
    chem 4 n O2PrimaryRampKickCell5 x 0 65535 y Y 0 256 out PRIRMP Kick_Cell5_PrimaryRamp
    db  $1a,  $05       ; Cell 5
    chem 4 n O2PrimaryRampKickCell6 x 0 65535 y Y 0 256 out PRIRMP Kick_Cell6_PrimaryRamp
    db  $1a,  $04       ; Cell 6
    chem 4 n O2PrimaryRampKickCell7 x 0 65535 y Y 0 256 out PRIRMP Kick_Cell7_PrimaryRamp
    db  $1a,  $05       ; Cell 7
    chem 4 n O2PrimaryRampKickCell8 x 0 65535 y Y 0 256 out PRIRMP Kick_Cell8_PrimaryRamp
    db  $1a,  $05       ; Cell 8
    chem 4 n O2PrimaryRampKickCellIdle x 0 65535 y Y 0 256 out PRIRMP Kick_CellIdle_PrimaryRamp
    db  $01,  $01       ; Idle Cell
; *****************************************
; The Limits are the maximum number of kicks that may be applied to each
; cell. If the O2 sensor fails to toggle after reaching the limit for more
; than a few seconds, the AutoCal cell gets changed.
; *****************************************
;     LIMITS
;       Rich, Lean
    chem 4 n O2PrimaryRampLimitCell0 x 0 65535 y Y 0 256 out PRIRMP Limit_Cell0_PrimaryRamp
    db  $0f,  $0f       ; Cell 0
    chem 4 n O2PrimaryRampLimitCell1 x 0 65535 y Y 0 256 out PRIRMP Limit_Cell1_PrimaryRamp
    db  $15,  $15       ; Cell 1
    chem 4 n O2PrimaryRampLimitCell2 x 0 65535 y Y 0 256 out PRIRMP Limit_Cell2_PrimaryRamp
    db  $15,  $15       ; Cell 2
    chem 4 n O2PrimaryRampLimitCell3 x 0 65535 y Y 0 256 out PRIRMP Limit_Cell3_PrimaryRamp
    db  $0f,  $0f       ; Cell 3
    chem 4 n O2PrimaryRampLimitCell4 x 0 65535 y Y 0 256 out PRIRMP Limit_Cell4_PrimaryRamp
    db  $15,  $15       ; Cell 4
    chem 4 n O2PrimaryRampLimitCell5 x 0 65535 y Y 0 256 out PRIRMP Limit_Cell5_PrimaryRamp
    db  $15,  $15       ; Cell 5
    chem 4 n O2PrimaryRampLimitCell6 x 0 65535 y Y 0 256 out PRIRMP Limit_Cell6_PrimaryRamp
    db  $0f,  $0f       ; Cell 6
    chem 4 n O2PrimaryRampLimitCell7 x 0 65535 y Y 0 256 out PRIRMP Limit_Cell7_PrimaryRamp
    db  $15,  $15       ; Cell 7
    chem 4 n O2PrimaryRampLimitCell8 x 0 65535 y Y 0 256 out PRIRMP Limit_Cell8_PrimaryRamp
    db  $15,  $15       ; Cell 8
    chem 4 n O2PrimaryRampLimitCellIdle x 0 65535 y Y 0 256 out PRIRMP Limit_CellIdle_PrimaryRamp
    db  $01,  $01       ; Idle Cell

; *****************************************
SECRMP_SecondaryO2Ramp:
; *****************************************
    chem2 ramp SECRMP
    chem 3 n O2SecondaryRampLowerRpm X 0 256 in Y 0 256 out SECRMP Secondary_Ramp_lower_Rpm_boundary_in_RPM/64
    db  $13
    chem 3 n O2SecondaryRampHigherRpm X 0 256 in Y 0 256 out SECRMP Secondary_Ramp_higher_Rpm_boundary_in_RPM/64
    db  $1b
    chem 3 n O2SecondaryRampLowerMap X 0 256 in Y 0 256 out SECRMP Secondary_Ramp_lower_Map_boundary
    .if Map2Bar > 0
      db  $2b
    .endif
    .if Map3Bar > 0
      db  $1d
    .endif
    chem 3 n O2SecondaryRampHigherMap X 0 256 in Y 0 256 out SECRMP Secondary_Ramp_higher_Map_boundary
    .if Map2Bar > 0
      db  $50
    .endif
    .if Map3Bar > 0
      db  $35
    .endif

; *****************************************
; (note that these are signed numbers, so $fd means -3)
;          KICKS
;       Rich, Lean
    chem 4 n O2SecondaryRampKickCell0 x 0 65535 y Y 0 256 out SECRMP Kick_Cell0_SecondaryRamp
    db  $03,  $fd       ; Cell 0
    chem 4 n O2SecondaryRampKickCell1 x 0 65535 y Y 0 256 out SECRMP Kick_Cell1_SecondaryRamp
    db  $01,  $ff       ; Cell 1
    chem 4 n O2SecondaryRampKickCell2 x 0 65535 y Y 0 256 out SECRMP Kick_Cell2_SecondaryRamp
    db  $01,  $ff       ; Cell 2
    chem 4 n O2SecondaryRampKickCell3 x 0 65535 y Y 0 256 out SECRMP Kick_Cell3_SecondaryRamp
    db  $03,  $fd       ; Cell 3
    chem 4 n O2SecondaryRampKickCell4 x 0 65535 y Y 0 256 out SECRMP Kick_Cell4_SecondaryRamp
    db  $03,  $fd       ; Cell 4
    chem 4 n O2SecondaryRampKickCell5 x 0 65535 y Y 0 256 out SECRMP Kick_Cell5_SecondaryRamp
    db  $03,  $fd       ; Cell 5
    chem 4 n O2SecondaryRampKickCell6 x 0 65535 y Y 0 256 out SECRMP Kick_Cell6_SecondaryRamp
    db  $03,  $fd       ; Cell 6
    chem 4 n O2SecondaryRampKickCell7 x 0 65535 y Y 0 256 out SECRMP Kick_Cell7_SecondaryRamp
    db  $03,  $fd       ; Cell 7
    chem 4 n O2SecondaryRampKickCell8 x 0 65535 y Y 0 256 out SECRMP Kick_Cell8_SecondaryRamp
    db  $03,  $fd       ; Cell 8
    chem 4 n O2SecondaryRampKickCellIdle x 0 65535 y Y 0 256 out SECRMP Kick_CellIdle_SecondaryRamp
    db  $03,  $fd       ; Idle Cell
; *****************************************
;     LIMITS
;       Rich, Lean
    chem 4 n O2SecondaryRampLimitCell0 x 0 65535 y Y 0 256 out SECRMP Limit_Cell0_SecondaryRamp
    db  $07,  $08       ; Cell 0
    chem 4 n O2SecondaryRampLimitCell1 x 0 65535 y Y 0 256 out SECRMP Limit_Cell1_SecondaryRamp
    db  $07,  $08       ; Cell 1
    chem 4 n O2SecondaryRampLimitCell2 x 0 65535 y Y 0 256 out SECRMP Limit_Cell2_SecondaryRamp
    db  $07,  $08       ; Cell 2
    chem 4 n O2SecondaryRampLimitCell3 x 0 65535 y Y 0 256 out SECRMP Limit_Cell3_SecondaryRamp
    db  $06,  $08       ; Cell 3
    chem 4 n O2SecondaryRampLimitCell4 x 0 65535 y Y 0 256 out SECRMP Limit_Cell4_SecondaryRamp
    db  $06,  $08       ; Cell 4
    chem 4 n O2SecondaryRampLimitCell5 x 0 65535 y Y 0 256 out SECRMP Limit_Cell5_SecondaryRamp
    db  $06,  $08       ; Cell 5
    chem 4 n O2SecondaryRampLimitCell6 x 0 65535 y Y 0 256 out SECRMP Limit_Cell6_SecondaryRamp
    db  $06,  $08       ; Cell 6
    chem 4 n O2SecondaryRampLimitCell7 x 0 65535 y Y 0 256 out SECRMP Limit_Cell7_SecondaryRamp
    db  $06,  $08       ; Cell 7
    chem 4 n O2SecondaryRampLimitCell8 x 0 65535 y Y 0 256 out SECRMP Limit_Cell8_SecondaryRamp
    db  $05,  $08       ; Cell 8
    chem 4 n O2SecondaryRampLimitCellIdle x 0 65535 y Y 0 256 out SECRMP Limit_CellIdle_SecondaryRamp
    db  $07,  $08       ; Idle Cell
; *****************************************
; O2 Controller Range of Authority
; *****************************************
; Range of Authority is a positive number for max addition, and a
; negative number for max subtraction.

; 1 + O2LIMH / 512 - Max. Multiplier
O2LIMH_O2RangeLimitHigh:
    chem 3 n O2RangeOfAuthorityHigh X 0 256 in Y 0 256 out O2LIMH O2_Controller_Range_of_authority_High_limit._1+O2LIMH/512=Max.Multiplier_-_cannot_exceed_$7f
    chem2 byte O2LIMH "Max multiplier that the O2 sensor adjustments make to fueling. High Range. Multiplier = 1 + O2LIMH / 512 (Cannot exceed $7f)"
    db  $7f     ; (cannot exceed $7f)
; .5 + O2LIML / 512 - Min. Multiplier
O2LIML_O2RangeLimitLow:
    chem 3 n O2RangeOfAuthorityLow X 0 256 in Y 0 256 out O2LIML O2_Controller_Range_of_authority_Low_limit._.5+O2LIML/512=Max.Multiplier_-_cannot_be_below_$80
    chem2 byte O2LIML "Max multiplier that the O2 sensor adjustments make to fueling. Low Range. Multiplier = 0.5 + O2LIML / 512 (Cannot be below $80)"
    db  $b3     ; (cannot be below $80)
    ; Note: this is a signed number, and needs to be negative. It is the maximum
    ;       amount of fuel that can be removed by the O2 feedback

; *****************************************
; Auto Cal Factors Table
; *****************************************

    chem2 group AutoCal

;
;  closed throttle = cell 12, otherwise:
;  $ff
;        +-----+-----+-----+-----+-----+-----+
;        |  1  |  3  |  5  |  7  |  9  |  11 |
;    R1 >+-----+-----+-----+-----+-----+-----+
;        |  0  |  2  |  4  |  6  |  8  |  10 |
;  rpm^  +-----+-----+-----+-----+-----+-----+
;              ^     ^     ^     ^     ^
;  map>       MP1   MP2   MP3   MP4   MP5     $ff
;
; *****************************************
; mp1-mp5 should be scaled for 3-bar
MP1_AutocalMapCellBoundary1:
    chem 3 n AutoCalMapCellBoundary1 X 0 256 in Y 0 256 out MP1 AutoCal_Map_Cell_Boundary_1
    .if Map2Bar > 0
      chem2 byte MP1 use "MAP" -14.7 14.7 psi "AutoCal Map Cell Boundary 1"
      db  $24
    .endif
    .if Map3Bar > 0
      chem2 byte MP1 use "MAP" -14.7 29.4 psi "AutoCal Map Cell Boundary 1"
      db  $1b
    .endif

MP2_AutocalMapCellBoundary2:
    chem 3 n AutoCalMapCellBoundary2 X 0 256 in Y 0 256 out MP2 AutoCal_Map_Cell_Boundary_2
    .if Map2Bar > 0
      chem2 byte MP2 use "MAP" -14.7 14.7 psi "AutoCal Map Cell Boundary 2"
      db  $37
    .endif
    .if Map3Bar > 0
      chem2 byte MP2 use "MAP" -14.7 29.4 psi "AutoCal Map Cell Boundary 2"
      db  $25
    .endif
MP3_AutocalMapCellBoundary3:
    chem 3 n AutoCalMapCellBoundary3 X 0 256 in Y 0 256 out MP3 AutoCal_Map_Cell_Boundary_3
    .if Map2Bar > 0
      chem2 byte MP3 use "MAP" -14.7 14.7 psi "AutoCal Map Cell Boundary 3"
      db  $4b
    .endif
    .if Map3Bar > 0
      chem2 byte MP3 use "MAP" -14.7 29.4 psi "AutoCal Map Cell Boundary 3"
      db  $32
    .endif
MP4_AutocalMapCellBoundary4:
    chem 3 n AutoCalMapCellBoundary4 X 0 256 in Y 0 256 out MP4 AutoCal_Map_Cell_Boundary_4
    .if Map2Bar > 0
      chem2 byte MP4 use "MAP" -14.7 14.7 psi "AutoCal Map Cell Boundary 4"
      db  $61
    .endif
    .if Map3Bar > 0
      chem2 byte MP4 use "MAP" -14.7 29.4 psi "AutoCal Map Cell Boundary 4"
      db  $41
    .endif
MP5_AutocalMapCellBoundary5:
    chem 3 n AutoCalMapCellBoundary5 X 0 256 in Y 0 256 out MP5 AutoCal_Map_Cell_Boundary_5
    .if Map2Bar > 0
      chem2 byte MP5 use "MAP" -14.7 14.7 psi "AutoCal Map Cell Boundary 5"
      db  $78
    .endif
    .if Map3Bar > 0
      chem2 byte MP5 use "MAP" -14.7 29.4 psi "AutoCal Map Cell Boundary 5"
      db  $50
    .endif
MapCellBoundaryEnd:
DLTMAP_AutocalMapHysteresisAtMAPCellBoundaries:
    chem 3 n AutoCalMapHysterisis X 0 256 in Y 0 256 out DLTMAP AutoCal_Map_Cell_Hysteresis_at_cell_boundaries
    chem2 byte DLTMAP "MAP Hysteresis at map cell boundaries"
    db  $01
R1_AutocalRPMCellBoundary:
    chem 3 n AutoCalRpmCellBoundary5 X 0 256 in Y 0 256 out R1 AutoCal_Rpm_Cell_Boundary
    chem2 byte R1 use "RPM" 0 16384 RPM "RPM Cell Boundary"
    db  $1d
DLTRPM_AutocalDeltaRPMatRPMBoundaries:
    chem 3 n AutoCalDeltaRpmAtRpmBoundaries X 0 256 in Y 0 256 out DLTRPM AutoCal_Rpm_Cell_Delta
    chem2 byte DLTRPM "RPM Hysteresis at RPM cell boundaries"
    db  $01
IDLOFS_AutocalRPMDeltaForIdleRPMforIdleCell:
    chem 3 n AutoCalRpmDeltaForIdle X 0 256 in Y 0 256 out IDLOFS AutoCal_Rpm_Cell_Delta
    chem2 byte IDLOFS "MAP Delta for Idle cell boundary"
    db  $01
HYTRPM_AutocalHysterisisForIDLOFS:
    chem 3 n AutoCalIDLOFSHysterisis X 0 256 in Y 0 256 out HYTRPM AutoCal_Idle_Rpm_Hysteresis
    chem2 byte HYTRPM "MAP Hysteresis at Idle cell boundary"
    db  $02
CALTHR_AutocalO2CtrlDeadbandLimitForUpdatingCells:
    chem 3 n AutoCalDeadbandLimit X 0 256 in Y 0 256 out CALTHR AutoCal_Deadband_limit_for_updating_cells
    chem2 byte CALTHR "Autocal deadband limit for updating cells"
    db  $09
RPMLIM_AutocalNoCellUpdateAboveThisRPM:
    chem 3 n AutoCalRpmLimit X 0 256 in Y 0 256 out RPMLIM AutoCal_Rpm_Limit._No_Cell_update_above_this_RPM
    chem2 byte RPMLIM use "RPM" 0 16384 RPM "No AutoCal cell update above this RPM"
    db  $2b

; *****************************************
MAPLIM_AutocalNoCellUpdateAboveThisMAP:
    chem 3 n AutoCalMapLimit X 0 256 in Y 0 256 out MAPLIM AutoCal_Map_Limit._No_Cell_update_above_this_MAP
    .if Map2Bar > 0
      chem2 byte MAPLIM use "MAP" -14.7 14.7 psi "No AutoCal cell update above this Map"
      db  $5c ; $8a
    .endif
    .if Map3Bar > 0
      chem2 byte MAPLIM use "MAP" -14.7 29.4 psi "No AutoCal cell update above this Map"
      db  $52
    .endif

; SCALE_for_FUEL
PWMIN_AutocalNoCellUpdateBelowThisValue:
    chem 4 n * PW 0 65535 something Y 0 256 out PWMIN Don't_update_cells_below_this_Pulsewidth._SCALE_for_FUEL
;    .inj word scale
    chem2 word inj PWMIN "No AutoCal cell update below this injector Pulsewidth."
    dw  1200   ; don't update cells below this pulsewidth: SCALE for FUEL

CELCHG_AutocalBitsChangePerUpdateInCell:
    chem 3 n AutoCalBitsChangePerUpdate X 0 256 in Y 0 256 out CELCHG AutoCal_Number_of_bits_change_per_update_in_Cell
    chem2 byte CELCHG "AutoCal number of bits change per update in Cell"
    db  $01
AMTEMP_AutocalStartingTempToSelectHotOrColdAMtimer:
    chem 3 n AutoCalStartingTemp X 0 256 in Y 0 256 out AMTEMP AutoCal_starting_temperature_to_select_hot_or_cold_AM_timer
    chem2 byte AMTEMP use "Coolant Temp" -200 260 Degrees "AutoCal starting temp to select hot or cold AM timer"
    db  $cd
AMTIMH_AutocalAMInhibitTimeHot:
    chem 3 n AutoInhibitTimeHot X 0 256 in Y 0 256 out AMTIMH AutoCal_inhibit_time_when_engine_temp_is_above_AMTEMP_times_2.8_seconds
    chem2 byte AMTIMH use Time 0 714 Seconds "AutoCal AM Inhibit Time Hot"
    db  $40
AMTIMC_AutocalAMInhibitTimeCold:
    chem 3 n AutoInhibitTimeCold X 0 256 in Y 0 256 out AMTIMC AutoCal_inhibit_time_when_engine_temp_is_below_AMTEMP_times_2.8_seconds
    chem2 byte AMTIMC use Time 0 714 Seconds "AutoCal AM Inhibit Time Cold"
    db  $0c
AMTPSW_AutocalTempPointToTurnOnSelectedAMTimer:
    chem 3 n AutoCalTempPoint X 0 256 in Y 0 256 out AMTPSW AutoCal_temperature_to_turn_on_selected_AM_timer
    chem2 byte AMTPSW use "Coolant Temp" -200 260 Degrees "AutoCal Temperature to turn on selected AM timer"
    db  $cd
AutoCalEppsBetweenCalChanges:
    chem 3 n * X 0 256 in Y 0 256 out AMUPDT AutoCal_EPPs_between_cal_call_updates
    chem2 byte AMUPDT "Autocal number of EPPs between cal change updates"
    db  $1a
; 1 + AMLIMH / 512 - Max. Multiplier
AMLIMH_AutocalRangeOfAuthorityHigh:
    chem 3 n AutoCalRangeOfAuthorityHigh X 0 256 in Y 0 256 out AMLIMH AutoCal_Range_of_authority_High_limit._1+AMLIMH/512=Max.Multiplier_-_cannot_exceed_$7f
    chem2 byte AMLIMH "AutoCal Range of Authority High. Must be 7f or below"
    db  $7f     ; (cannot exceed $7f)
; .5 + AMLIML / 512 - Min. Multiplier
AMLIML_AutocalRangeOfAuthorityLow:
    chem 3 n AutoCalRangeOfAuthorityLow X 0 256 in Y 0 256 out AMLIML AutoCal_Range_of_authority_Low_limit._.5+AMLIML/512=Max.Multiplier_-_cannot_be_below_$80
    chem2 byte signed AMLIML "AutoCal Range of Authority Low. Must be $80 or higher since it's a negative number"
    db  $b3     ; (cannot be below $80)
    ; Note: this is a signed number, and needs to be negative. It is the maximum
    ;       amount of fuel that can be removed by the AutoCal feedback

L1364:
    db  $00

;  ______________________
; |                      \
; |   Running Fuel       |
; \______________________|
;
    chem2 group "Running Fuel"

; *****************************************
MAPTBL_FuelPartThrottle:
    .if Map2Bar > 0
      chem 1 n FuelPartThrottle_FromMap2bar MAP -14.7 14.7 psi Pulsewidth 0 24576 uSec MAPTBL Fuel_Pulsewidth_from_MAP_with_part_throttle._SCALE_to_FUEL_and_MAP
      chem2 table inj fuel MAPTBL simple use MAP -14.7 14.7 psi use "Pulsewidth" 0 24576 uSec "Fuel Pulsewidth from MAP, part throttle."
;      .inj table5 scale y
      hex 05 0000 1E5D
      hex 3C 0686 1E0F
      hex 4D 0885 185A
      hex 5E 0A23 1C8A
      hex 6B 0B96 3133
      hex 7F 0F6E 1D12
      hex AA 1450 3BC4
    .endif
    .if Map3Bar > 0
      chem 1 n FuelPartThrottle_FromMap3bar MAP -14.7 29.4 psi Pulsewidth 0 24576 uSec MAPTBL Fuel_Pulsewidth_from_MAP_with_part_throttle._SCALE_to_FUEL_and_MAP
      chem2 table inj fuel MAPTBL simple use MAP -14.7 29.4 psi use "Pulsewidth" 0 24576 uSec "Fuel Pulsewidth from MAP, part throttle."
;      .inj table5 scale y
    ; FuelPartThrottle (MAPTBL)
    hex 04 0000 25ed
    hex 20 0426 2df0
    hex 30 0705 346d
    hex 3e 09e3 3855
    hex 47 0bde 40f0
    hex 58 102e 50eb
    hex 97 2418 5dc2
    hex ff
    .endif

    hex  ff ffff ffff
    hex  ff ffff ffff
    hex  ff ffff ffff
    hex  ff ffff ffff

    hex  ff
; *****************************************
WOTTBL_FuelFullThrottle:
    .if Map2Bar > 0
      chem 1 n FuelFullThrottle_FromMap2bar MAP -14.7 14.7 psi Pulsewidth 0 24576 uSec WOTTBL Fuel_Pulsewidth_from_MAP,_full_throttle_and_maximum_allowed_at_part_throttle._SCALE_to_FUEL_and_MAP
      chem2 table inj fuel WOTTBL simple use MAP -14.7 14.7 psi use "Pulsewidth" 0 24576 uSec "Fuel Pulsewidth from MAP, full throttle and upper limit for part throttle."
;    .inj table5 scale y
      hex 00 0000 25DA
      hex 6B 0FD2 362B
      hex B3 1F0E 27C1
      hex D8 24CD 2355
    .endif
    .if Map3Bar > 0
      chem 1 n FuelFullThrottle_FromMap3bar MAP -14.7 29.4 psi Pulsewidth 0 24576 uSec WOTTBL Fuel_Pulsewidth_from_MAP,_full_throttle_and_maximum_allowed_at_part_throttle._SCALE_to_FUEL_and_MAP
      chem2 table inj fuel WOTTBL simple use MAP -14.7 29.4 psi use "Pulsewidth" 0 24576 uSec "Fuel Pulsewidth from MAP, full throttle and upper limit for part throttle."
;    .inj table5 scale y
    ; FuelFullThrottle (WOTTBL)
    hex 02 0000 3d43
    hex 59 14d2 5755
    hex 77 1f0e 42a0
    hex 8f 254d 4fd6
    hex ff
    .endif
    hex  ff ffff ffff
    hex  ff ffff ffff
    hex  ff ffff ffff
    hex  ff ffff ffff

    hex  ff
; *****************************************
MAPIDL_FuelNoThrottle:
    .if Map2Bar > 0
      chem 1 n FuelNoThrottle_FromMap2bar MAP -14.7 14.7 psi Pulsewidth 0 24576 uSec MAPIDL Fuel_Pulsewidth_from_MAP,_at_idle._SCALE_to_FUEL_and_MAP
      chem2 table inj fuel MAPIDL simple use MAP -14.7 14.7 psi use "Pulsewidth" 0 24576 uSec "Fuel Pulsewidth from MAP, off throttle and lower limit for part throttle."
;    .inj table5 scale y
      hex 0C 0000 224C
      hex 27 039E 172B
      hex 33 04B4 2808
      hex 74 0EDE 1042
    .endif
    .if Map3Bar > 0
      chem 1 n FuelNoThrottle_FromMap3bar MAP -14.7 29.4 psi Pulsewidth 0 24576 uSec MAPIDL Fuel_Pulsewidth_from_MAP,_at_idle._SCALE_to_FUEL_and_MAP
      chem2 table inj fuel MAPIDL simple use MAP -14.7 29.4 psi use "Pulsewidth" 0 24576 uSec "Fuel Pulsewidth from MAP, off throttle and lower limit for part throttle."
;    .inj table5 scale y
    ; FuelNoThrottle (MAPIDL)
    hex 0e 0008 3300
    hex 18 0206 36e3
    hex 21 03f4 3209
    hex 55 0e1e 2a80
    hex ff
    .endif
    hex ff ffff ffff
    hex ff ffff ffff
    hex ff ffff ffff
    hex ff ffff ffff
    hex ff

; *****************************************
AIRTMP_FuelChargeTempFactor:
    chem 0 n FuelChargeTempFactor Charge_Temp 245 -45 degreesF Factor 1 2 mult AIRTMP Charge_Temp_factor_-_note_temperature_range_is_not_completely_linear,_this_is_an_approximation
    chem2 table AIRTMP use "Charge Temp" 245 -45 Degrees use "Factor" 1 2 out "Charge temp sensor factor. Temperature range is not linear, this is an approximation."
    hex 05
    hex 1a 00 0029
    hex 59 0a 002f
    hex 90 14 004b
    hex e9 2e 0100
    hex fb 40

    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff

; *****************************************
FuelLeanoutFactorOnThrottle:
    chem 2 n * RPMs 0 8192 rpm output 0.5 1.5 Factor name Leanout_Factor_when_on_throttle_(should_be_1.0_-_DO_NOT_CHANGE)
    chem2 table signed LOTONT use RPMs 0 8192 RPM use "Factor" 0.5 1.5 out "Leanout Factor when on throttle. Should be 1.0, change is not recommended."
    hex 01
    hex ff 00
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff

; *****************************************
FuelLeanoutFactorOffThrottle:
    chem 2 n * RPMs 0 8192 rpm output 0.5 1.5 Factor name Leanout_Factor_used_during_deceleration_off_throttle._Lower_this_to_reduce_deceleration_rich_condition
    chem2 table signed LOTOFT use RPMs 0 8192 RPM use "Factor" 0.5 1.5 out "Leanout Factor when OFF throttle. Adjust this when deceleration fuel is pig rich after scaling."
    ; FuelLeanoutFactorOffThrottle (LOTOFT)
    hex 04
    hex 1b f5 00d5
    hex 2d 04 00d8
    hex 3a 0f 0040
    hex 46 12
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff

; *****************************************
PWADD_FuelAddBaroComp:
    chem 0 n FuelAddBaroComp Barometric_Pressure_From_MAP -14.7 29.4 psi output 0 256 out PWADD Additive_Baro_Fuel_Compensation_(Additional_Fuel_-_PWADD_x_IGNPRD_/_256)
	chem2 table PWADD "Additive Baro Fuel Compensation (Additional Fuel = PWADD x IGNPRD / 256)"
    hex 01
    hex ff 00
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff

; *****************************************
L1367:
    db  $1a   ; scaled something if coolant temp bad?

; *****************************************
; PEFTBL_PumpingEfficiency:
PEFTBL_PumpingEfficiency:
    chem 0 y PumpingEfficiency RPM 0 8192 rpm Factor 0.0 1.0 out PEFTBL Pumping_Efficiency_(Volumetric)_-_should_be_changed_to_compensate_when_an_engine's_flow_characteristics_are_changed._This_is_for_the_2-piece_intake.
; stock 87 T2 MP
    chem2 table PEFTBL simple use RPMs 0 8192 RPM use "Factor" 0.0 1.0 out "Volumetric Efficiency. Fuel pulsewidth is scaled against this value. Probably should only change this when modifications have altered the flow characteristics, such as intake porting or valve work. Stock 87 T2 with 2-piece intake and slider cam."
    ; PumpingEfficiency (PEFTBL)
    hex 05
    hex 4b e7 ffdc
    hex 67 e3 0068
    hex 7d ec fff4
    hex 91 eb ff49
    hex b8 cf

    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff

; my hacked version
;    chem2 table PEFTBL use RPMs 0 8192 RPM use "Factor" 0.0 1.0 out "Volumetric Efficiency. Fuel pulsewidth is scaled against this value. Probably should only change this when modifications have altered the flow characteristics, such as intake porting or valve work. Hacked version."
;      hex 0a
;      hex 0e de 000a
;      hex 26 df 0033
;      hex 53 e8 002a
;      hex 65 eb 000d
;      hex 78 ec 0012
;      hex 86 ed 0000
;      hex 99 ed ff6e
;      hex ae e1 ff5d
;      hex c7 d1 ffc3
;      hex e0 cb


;   directly from the 89 T2 MP
;    chem2 table PEFTBL use RPMs 0 8192 RPM use "Factor" 0.0 1.0 out "Volumetric Efficiency. Fuel pulsewidth is scaled against this value. Probably should only change this when modifications have altered the flow characteristics, such as intake porting or valve work. Stock 89 T2 with 1-piece intake and roller cam."
;    hex  0a				; number of points
;    hex  10 d7 0000		; point 1
;    hex  1b d7 0000		; point 2
;    hex  3f d7 0095		; point 3
;    hex  4b de ffeb		; point 4
;    hex  57 dd 0051		; point 5
;    hex  7d e9 ffa4		; point 6
;    hex  96 e0 ff27		; point 7
;    hex  a3 d5 fef5		; point 8
;    hex  bb bc ff80		; point 9
;    hex  cf b2			; point 10

; *****************************************
BATOFF_FuelBatteryOffset:
    chem 0 n FuelBatteryOffset Battery_Voltage 0 16.2 volts Pulsewidth_Added 0 4096 uSec BATOFF Adds_fuel_for_lower_battery_voltage_to_account_for_slower_injector_opening_time_from_lower_voltage_during_cranking._SCALE_FOR_FUEL
    chem2 table BATOFF simple use "Battery Voltage" 0 16.2 Volts use "Pulsewidth Added" 0 2048 uSec "Additional pulsewidth added to account for injector opening/closing time, which is affected by battery voltage. You might want to try actually INCREASING this value for larger injectors, but DO NOT scale downward."
    ; FuelBatteryOffset (BATOFF)
    hex 04
    hex 59 d5 fdc8
    hex 82 7a ff20
    hex c2 42 ff90
    hex fb 29
    hex ff ff ffff
    hex ff ff ffff

; *******************************
; Fuel Start/Run transition RPMs:
;
; Note: These numbers represent the time between EPPs, the inverse of RPMs.
;       Therefore, lower numbers are higher RPMs.
RNFPRD_FuelTransitionStartToRun:
    chem 4 n FuelTransitionStartToRun Speed 8192 0 Inverse_RPMs Y 0 256 out RNFPRD Switch_To_Fuel_Run_Mode_when_Inverse_RPMs_below_this_value._Stock_is_$00a7_should_be_about_610_RPM
    chem2 word RNFPRD "Immediate switch to Fuel Run mode when RPMs are above this value. This is inverse RPMs, lower numbers are higher RPMs. Stock is 00a7 which is about 610 RPM."
    dw  $00a7
STFPRD_FuelTransitionRunToStart:
    chem 4 n FuelTransitionRunToStart Speed 8192 0 Inverse_RPMs Y 0 256 out STFPRD Switch_To_Fuel_Start_Mode_when_Inverse_RPMs_below_this_value._Stock_is_$0104_should_be_about_450_RPM
    chem2 word STFPRD "Immediate switch to Fuel Start mode when RPMs are above this value. This is inverse RPMs, lower numbers are higher RPMs. Stock is 0104 which is about 450 RPM."
    dw  $0104

; *******************************
; Load enrichment
;
    chem2 group "Load Enrichment"

HIGHMP_LoadEnrichment:
    chem 0 n LoadEnrichment Coolant_Temp -200 260 degreesF percent_enrich_per_torr 0 100 percent HIGHMP Load_enrichment_calculation_-_Cold_load_factor_-_(HIGHMP)_*_(HMPMUL)/32
    chem2 table HIGHMP use "Coolant Temp" -200 260 Degrees use "Enrichment" 0 100 Percent "Load enrichment calculation. Cold Load Factor = HIGHMP * HMPMUL / 32"
    ; LoadEnrichment (HIGHMP)
    hex 03
    hex 84 6f e9cd
    hex 89 00 0000
    hex cd 00
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
;
HMPMUL_LoadEnrichment2:
    .if Map2Bar > 0
      chem 0 n LoadEnrichment2_2bar MAP -14.7 14.7 psi output 0 100 map HMPMUL More_of_the_load_enrichment_calculations._SCALE_to_MAP
      chem2 table HMPMUL use MAP -14.7 14.7 psi use "Factor" 0 100 out "Load enrichment calculation. Cold Load Factor = HIGHMP * HMPMUL / 32"
      hex 04
      hex 2E 00 003B
      hex 55 09 008B
      hex 83 22 FEB3
      hex 97 08
    .endif
    .if Map3Bar > 0
      chem 0 n LoadEnrichment2_3bar MAP -14.7 29.4 psi output 0 100 map HMPMUL More_of_the_load_enrichment_calculations._SCALE_to_MAP
      chem2 table HMPMUL use MAP -14.7 29.4 psi use "Factor" 0 100 out "Load enrichment calculation. Cold Load Factor = HIGHMP * HMPMUL / 32"
    ; LoadEnrichment2 (HMPMUL)
    hex 04
    hex 2d 00 00d1
    hex 38 09 0025
    hex 5a 0e 0000
    hex 64 0e
    .endif
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
CURVEB_ColdEnrichmentFuelCurveB:
    chem 0 n ColdEnrichmentFuelCurveB Coolant_Temp -200 260 degreesF Factor 0.0 330 out CURVEB Cold_Enrichment_Factor_CNRICH_=_1+((CURVEA_*_ACVDCY/256_+_CURVEC_*_ACVDCY/256_+_CURVEB_*_FCRCVB/256)_*_2_+_(HMPMUL*HIGHMP)/32)_*_BAROCP/256
    chem2 table CURVEB use "Coolant Temp" -200 260 Degrees use "Factor" 0 330 out "Cold Enrichment Factor CNRICH = 1+((CURVEA * ACVDCY/256 + CURVEC * ACVDCY/256 + CURVEB * FCRCVB/256) * 2 + (HMPMUL*HIGHMP)/32) * BAROCP/256"
    ; ColdEnrichmentFuelCurveB (CURVEB)
    hex 06
    hex 58 70 fe37
    hex 74 3e ff1f
    hex a6 12 ff48
    hex bf 00 0000
    hex ee 00 00e8
    hex f9 0a
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
CURVEA_ColdEnrichmentFuelCurveA:
    chem 0 n ColdEnrichmentFuelCurveA CoolantTemp -200 260 degreesF Factor 1.0 2.0 mult CURVEA Cold_Enrichment_Factor_CNRICH_=_1+((CURVEA_*_ACVDCY/256_+_CURVEC_*_ACVDCY/256_+_CURVEB_*_FCRCVB/256)_*_2_+_(HMPMUL*HIGHMP)/32)_*_BAROCP/256
    chem2 table CURVEA use "Coolant Temp" -200 260 Degrees use "Factor" 1 2 out "Cold Enrichment Factor CNRICH = 1+((CURVEA * ACVDCY/256 + CURVEC * ACVDCY/256 + CURVEB * FCRCVB/256) * 2 + (HMPMUL*HIGHMP)/32) * BAROCP/256"
    ; ColdEnrichmentFuelCurveA (CURVEA)
    hex 05
    hex 58 ff faca
    hex 74 6d fd80
    hex 84 45 fd80
    hex 92 22 ffd1
    hex e3 13
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
CURVEC_ColdEnrichmentFuelCurveC:
    chem 0 n ColdEnrichmentFuelCurveC Coolant_Temp -200 260 degreesF Factor 1.0 3.0 mult CURVEC Cold_Enrichment_Factor_CNRICH_=_1+((CURVEA_*_ACVDCY/256_+_CURVEC_*_ACVDCY/256_+_CURVEB_*_FCRCVB/256)_*_2_+_(HMPMUL*HIGHMP)/32)_*_BAROCP/256
    chem2 table CURVEC use "Coolant Temp" -200 260 Degrees use "Factor" 1 3 out "Cold Enrichment Factor CNRICH = 1+((CURVEA * ACVDCY/256 + CURVEC * ACVDCY/256 + CURVEB * FCRCVB/256) * 2 + (HMPMUL*HIGHMP)/32) * BAROCP/256"
    ; ColdEnrichmentFuelCurveC (CURVEC)
    hex 05
    hex 79 3a ff75
    hex 84 34 ff1c
    hex a9 13 0000
    hex ee 13 0280
    hex f6 27
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
DCYTMC_CurveAColdDecayTime:
    chem 3 n ColdEnrichmentCurveAColdDecayTime X 0 256 in Y 0 256 out DCYTMC Curve_A_Cold_decay_time_as_89.76/this_value_seconds._$4=22.4_seconds
	chem2 byte DCYTMC "Curve A Cold decay time as 89.76 / this value seconds. $4 = 22.4 seconds"
    db  $04
DCYTMH_CurveAHotDecayTime:
    chem 3 n ColdEnrichmentCurveAHotDecayTime X 0 256 in Y 0 256 out DCYTMH Curve_A_Hot_decay_time_as_89.76/this_value_seconds._$9=10_seconds
	chem2 byte DCYTMH "Curve A Hot decay time as 89.76 / this value seconds. $9 = 10 seconds"
    db  $09
CLDHLD_CurveAColdHoldTime:
    chem 3 n ColdEnrichmentCurveAColdHoldTime X 0 256 in Y 0 256 out CLDHLD Curve_A_Cold_hold_time_as_this_value_*_.7_seconds
	chem2 byte CLDHLD use Time 0 180 Seconds "Curve A Cold hold time as this value * .7 seconds"
    db  $66
HOTHLD_CurveAHotHoldTime:
    chem 3 n ColdEnrichmentCurveAHotHoldTime X 0 256 in Y 0 256 out HOTHLD Curve_A_Hot_hold_time_as_this_value_*_.7_seconds
	chem2 byte HOTHLD use Time 0 180 Seconds "Curve A Hot hold time as this value * .7 seconds"
    db  $15
TMPHLD_TempSwitchPointForCurveAholdAndDecayTimes:
    chem 3 n ColdEnrichmentTempSwitchForCurveA X 0 256 in Y 0 256 out TMPHLD Curve_A_Hot/Cold_switch_point
	chem2 byte TMPHLD use "Coolant Temp" -200 260 Degrees "Engine temperature switch point for Curve A hold and decay times"
    db  $b2
CCURV_HoldTimeForCurveC:
    chem 3 n ColdEnrichmentCurveCHoldTime X 0 256 in Y 0 256 out CCURV Curve_C_hold_time_as_this_value_*_.7_seconds
	chem2 byte CCURV use Time 0 180 Seconds "Curve C hold time as this value * .7 seconds"
    db  $0a
DCYCC_DecayRateForCurveC:
    chem 3 n ColdEnrichmentCurveCDecayTime X 0 256 in Y 0 256 out DCYCC Curve_C_decay_time_as_89.76/this_value_seconds._$d=7_seconds
	chem2 byte DCYCC "Curve C Decay time as 89.76 / this value seconds. $d = 7 seconds"
    db  $0d
BAROCP_BaroCompWarmup:
    .if Map2Bar > 0
      chem 0 n BaroCompensationOfWarmupFactor2bar Barometric_Pressure_From_MAP -14.7 14.7 psi factor 0 1 out BAROCP Baro_Compensation_of_Warmup_factor._-_SCALE_for_MAP
      chem2 table BAROCP use "Baro Pressure" -14.7 14.7 psi use "Factor" 0 1 out "Cold Enrichment Factor CNRICH = 1+((CURVEA * ACVDCY/256 + CURVEC * ACVDCY/256 + CURVEB * FCRCVB/256) * 2 + (HMPMUL*HIGHMP)/32) * BAROCP/256"
      hex 02
      hex 5f f3 0100
      hex 6b ff
    .endif
    .if Map3Bar > 0
      chem 0 n BaroCompensationOfWarmupFactor3bar Barometric_Pressure_From_MAP -14.7 29.4 psi factor 0 1 out BAROCP Baro_Compensation_of_Warmup_factor._-_SCALE_for_MAP
      chem2 table BAROCP use "Baro Pressure" -14.7 29.4 psi use "Factor" 0 1 out "Cold Enrichment Factor CNRICH = 1+((CURVEA * ACVDCY/256 + CURVEC * ACVDCY/256 + CURVEB * FCRCVB/256) * 2 + (HMPMUL*HIGHMP)/32) * BAROCP/256"
      hex 02
      hex 3f f3 0180
      hex 47 ff
    .endif
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
RUNTMP_RuntmpModifierEngineTemp:
    chem 0 n RuntmpModifierEngineTemp Time_after_engine_stalls 0 720 seconds delta_added_to_engine_temp 0 512 out RUNTMP Engine_temp_modification._This_adds_temperature_to_the_detected_temp_based_on_how_long_the_engine_has_been_stalled_without_ignition_key_reset._Also_uses_RUNLMT
    chem2 table RUNTMP use "Time after stall" 0 720 Seconds use "Delta added to temp" 0 512 out "Temperature added to Coolant Temp Sensor value after an engine stall."
    ; RuntmpModifierEngineTemp (RUNTMP)
    hex 03
    hex 00 00 03ab
    hex 06 16 0900
    hex 0b 43
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
RUNLMT_MaxAllowableRunTemp:
    chem 3 n RuntmpMaxAllowableRunTemp Temperature -200 260 degreesF Y 0 256 out RUNLMT Max_allowable_temp_(ENGTMP_+_MODIFIER)_(modifier_is_derived_from_RUNTMP_table)
    chem2 byte RUNLMT use "Coolant Temp" -200 260 Degrees "Max allowable (ENGTMP + RUNTMP value) for load enrichment."
    db  $95

; *****************************************
; Transient (accelerator pump function) values:
; *****************************************

    chem2 group Transients

MAPTRG_TransientMapTrigger:
    .if Map2Bar > 0
      chem 3 n TransientMapTrigger Delta_Map_trigger_level -14.7 14.7 psi Y 0 256 out MAPTRG Transient_enrichment_MAP_Delta_trigger_level_-_used_with_MAPMAX_and_AESLOP
      chem2 byte MAPTRG use Delta_Map_trigger_level 0 29.4 psi "Transient enrichment MAP Delta trigger level - used with MAPMAX and AESLOP"
      db  $05
    .endif
    .if Map3Bar > 0
      chem 3 n TransientMapTrigger Delta_Map_trigger_level -14.7 29.4 psi Y 0 256 out MAPTRG Transient_enrichment_MAP_Delta_trigger_level_-_used_with_MAPMAX_and_AESLOP
      chem2 byte MAPTRG use Delta_Map_trigger_level 0 44.1 psi "Transient enrichment MAP Delta trigger level - used with MAPMAX and AESLOP"
      db  $03
    .endif
MAPMAX_TransientMaxDeltaMap:
    chem 3 n TransientMaxDeltaMap MaxDelta_allowed -14.7 14.7 psi Y 0 256 out MAPMAX Maximum_delta_map_allowed._This_is_part_of_the_MAP_transient_enrichment_calculation_that_also_uses_MAPTRG_and_AESLOP
    .if Map2Bar > 0
    chem2 byte MAPMAX use "Max Delta Map" -14.7 14.7 psi "Transient enrichment max MAP Delta"
    .endif
    .if Map3Bar > 0
    chem2 byte MAPMAX use "Max Delta Map" -14.7 29.4 psi "Transient enrichment max MAP Delta"
	.endif
    db  $ff
AESLOP_TransientEnrichMap:
    chem 0 n TransientEnrichMap Engine_Temp -200 260 degreesF Slope 0 256 out AESLOP Map_transient_enrich_PW_addition._Additional_PW(uSec)_=_MAP-AVMAP-MAPTRG_*_AESLOP
	chem2 table AESLOP use "Coolant Temp" -200 260 Degrees use Slope 0 256 out "Map transient enrich PW addition. Additional PW(uSec) = MAP - AVMAP - MAPTRG * AESLOP"
;    .inj table scale y
    ; TransientEnrichMap (AESLOP)
    hex 03
    hex 58 23 ffc9
    hex 95 16 ffa9
    hex cd 03
    hex ff ff ffff
AETIME_TransientMapEnrichTimeConstant:
    chem 0 n TransientMapEnrichTimeConstant Engine_Temp -200 260 degreesF factor 0 256 out AETIME MAPTIM_=_factor*256,_AVMAP(16bit)_and_MAP(8bit),_AVMAP(new)=MAP_INST*(MAPTIM/256)+AVMAP(old)*(1-(MAPTIM/256))
	chem2 table AETIME use "Coolant Temp" -200 260 Degrees use Factor 0 256 out "MAPTIM = factor*256, AVMAP(16bit) and MAP(8bit), AVMAP(new)=MAP_INST*(MAPTIM/256)+AVMAP(old)*(1-(MAPTIM/256))"
    hex 01
    hex ff 14
    hex ff ff ffff
POSTRG_TransientThrottleTriggerLevel:
    chem 3 n TransientThrottleTriggerLevel Trigger_Voltage 0 256 volts Y 0 256 out POSTRG Positive_Transient_trigger_level
    chem2 byte POSTRG use "Trigger" 0 5 Volts "Positive transient enrichment trigger level"
    db  $04
THRMAX_TransientMaxDeltaThrottle:
    chem 3 n TransientMaxDeltaThrottle Max_Delta_Throttle 0 256 volts Y 0 256 out THRMAX Max_delta_thr_(THR-AVTHR-POSTRG)
    chem2 byte THRMAX use "Max Delta Thr" 0 5 Volts "Maximum Delta Throttle (THR - AVTHR - POSTRG)"
    db  $26
POSSLP_TransientThrottleEnrich:
    chem 0 n TransientThrottleEnrich Engine_Temperature -200 260 degreesF Additional_Pulsewidth 0 64 out POSSLP Throttle_transient_enrich_PW_addition_-_accelerator_pump_equivalent?_Additional_PW_in_uSec_=_THR_-_AVTHR_-_POSTRG(in_volts)_*_POSSLP
	chem2 table inj POSSLP use "Coolant Temp" -200 260 Degrees use "Additional PW" 0 64 out "Throttle transient enrich PW addition - accelerator pump equivalent. Additional PW in uSec = THR - AVTHR - POSTRG(in volts) * POSSLP"
;    .inj table scale y
    ; TransientThrottleEnrich (POSSLP)
    hex 03
    hex 74 1b ff6e
    hex 90 0b ffda
    hex b1 06
    hex ff ff ffff
    hex ff ff ffff
THRTCN_TransientThrottleEnrichTime:
    chem 0 n TransientThrottleEnrichTime Engine_Temp -200 260 degreesF Fuel_Enrich_(see_desc) 0 256 out THRTCN Throttle_transient_enrich_time_constant._AVTHR=16_bits,_THR=8_bits,_X=engine_temp,_Y=THRTIM(8bits)_-_Factor*256_when_THR>AVTHR,_then_AVTHR=THR_INST*(THRTIM/256)+AVTHR(OLD)*(1-(THRTIM/256))
	chem2 table THRTCN use "Coolant Temp" -200 260 Degrees use "Fuel Enrich" 0 256 out "Throttle transient enrich time constant. AVTHR=16 bits, THR=8 bits, X=engine temp, Y=THRTIM(8bits) - Factor*256 when THR>AVTHR, then AVTHR=THR_INST*(THRTIM/256)+AVTHR(OLD)*(1-(THRTIM/256))"
    ; TransientThrottleEnrichTime (THRTCN)
    hex 02
    hex 74 18 0017
    hex cd 20
    hex ff ff ffff
    hex ff ff ffff
L1063:
    db  $05
MinCoolantTempForTransient:
	chem2 byte MINCTT use "Coolant Temp" -200 260 Degrees "Minimum Coolant temp for Transient (otherwise cold enrichment is used)"
    db  $b1
MinTimeForTransient:
    db  $02
MinRPMforTransient:
	chem2 byte MINCTR use RPMs 0 8192 RPM "Minimum RPM for Transient"
    db  $23
L1483:
    db  $14
L1484:
    db  $02
L1486:
    db  $02
L1487:
    db  $03
L1489:
    db  $08
L1488:
    db  $ed
L1067:
    db  $28
TestedAgainstMAPDuringTPSProcessing:
    .if Map2Bar > 0
      chem 6 n * X 0 256 in Y 0 256 out NAME Tested_Against_MAP_Pressure_during_Transient_enrichment_calculations 2-Bar a 3-Bar 7 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
      db  $0a ; $0a
    .endif
    .if Map3Bar > 0
      chem 6 n * X 0 256 in Y 0 256 out NAME Tested_Against_MAP_Pressure_during_Transient_enrichment_calculations 2-Bar a 3-Bar 7 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
      db  $07 ; $0a
    .endif

; These values are related to O2 feedback:

OverboostConstant1:
    chem 6 n * X 0 256 in Y 0 256 out NAME This_value_is_compared_against_the_MAP_Pressure 2-Bar aa 3-Bar 71 Disable ff Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
    db  $aa
SomeFFs:
    db  $ff,$ff,$ff,$ff,$ff

FuelPulsewidthFunctionUnknown:
    dw  $1eb2   ; stock
; *****************************************
L1660:
    db  $64
L1663:
    db  $3c
; *****************************************
L1254:
    db  $2b
; *****************************************
L1393:
    db  $3a
RPMMAP_FuelPartThrottleEnrichment:
    .if Map2Bar > 0
      chem 0 n FuelPartThrottleEnrichment2bar RPMs 0 8192 in MAP_Compare -14.7 14.7 psi RPMMAP Part_Throttle_Enrichment_-_Above_this_MAP_Count_down_per_EPP,_below_this_MAP_count_up_per_EPP._SCALE_for_MAP
      chem2 table RPMMAP use RPMs 0 8192 RPM use "MAP Compare" -14.7 14.7 psi "Part Throttle Enrichment - Above this MAP Count down per EPP, below this MAP count up per EPP."
      hex 04
      hex 4b b3 fea4
      hex 64 91 ff40
      hex 74 85 fb80
      hex 7a 6a
    .endif
    .if Map3Bar > 0
      chem 0 n FuelPartThrottleEnrichment3bar RPMs 0 8192 in MAP_Compare -14.7 29.4 psi RPMMAP Part_Throttle_Enrichment_-_Above_this_MAP_Count_down_per_EPP,_below_this_MAP_count_up_per_EPP._SCALE_for_MAP
      chem2 table RPMMAP use RPMs 0 8192 RPM use "MAP Compare" -14.7 29.4 psi "Part Throttle Enrichment - Above this MAP Count down per EPP, below this MAP count up per EPP."
    ; FuelPartThrottleEnrichment (RPMMAP)
    hex 04
    hex 4b 4f ff5d
    hex 64 3f ffb0
    hex 74 3a fe00
    hex 7a 2e
    .endif
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
L1658:
    db  3

;  ______________________
; |                      \
; |   Idle Control       |
; \______________________|

    chem2 group "Idle Control"

AISTRT_AisStartPosition:
    chem 0 n AisStartPosition Engine_Temp -200 260 degreesF AIS_Open_steps_from_idle_position 0 256 out AISTRT AIS_Control_start_position._Determines_where_the_AIS_motor_should_be_positioned_prior_to_a_startup_based_on_engine_temperature
	chem2 table AISTRT use "Coolant Temp" -200 260 Degrees use "AIS Open" 0 256 Steps "AIS Control start position. Determines where the AIS motor should be positioned prior to a startup based on engine temperature"
    ; AisStartPosition (AISTRT)
    hex 05
    hex 6e 99 fc0e
    hex 81 4e ff25
    hex cf 0b 0045
    hex f4 15 0000
    hex fa 15
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff

DRIDHT_AdditionalStartupIdle:
    chem 3 n AisAdditionalStartupIdle X 0 256 in Y 0 256 out DRIDHT Additional_startup_idle_speed._x0D_=_104rpm
	chem2 byte DRIDHT use RPMs 0 2048 RPM "Additional startup idle speed"
    db  $0d
STTIME_AdditionalStartupIdleTime:
    chem 3 n AisAdditionalStartupIdleTime X 0 256 in Y 0 256 out STTIME Time_that_DRIDHT_is_used_-_x17_=_.65_seconds
	chem2 byte STTIME use Time 0 7.2 Seconds "Additional startup idle time"
    db  $12
AISPN_AisIdleSpeed:
    chem 0 n AisIdleSpeed Engine_Temp -200 260 degreesF RPM 0 2048 out AISPN Determines_idle_speed_based_on_the_engine_temperature_for_manual_car_or_automatic_in_park/neutral
    chem2 table AISPN simple use "Coolant Temp" -200 260 Degrees use RPM 0 2048 out "Idle speed based on Engine temperature for manual car or automatic in park/neutral"
    hex 02
    hex 7f af ff2f
    hex cc 70
    hex ff ff ffff
AISCLD_AisIdleSpeedDrive:
    chem 0 n AisIdleSpeedDrive Engine_Temp -200 260 degreesF RPM 0 2048 out AISCLD Determines_idle_speed_based_on_engine_temperature_when_automatic_car_is_in_Drive_(load)
    chem2 table AISCLD simple use "Coolant Temp" -200 260 Degrees use RPM 0 2048 out "Idle speed based on Engine temperature for automatic in drive"
    hex 02
    hex 7f 96 ff5a
    hex cc 64
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff

CLDPOS_AisOffIdlePositionCold:
    chem 0 n AisOffIdlePositionCold Engine_Temp -200 260 degreesF AIS_Open_Steps 0 256 out CLDPOS Off_idle_positioning_of_AIS_motor._This_changes_immediately._Actual_AIS_position_is_largest_of_CLDPOS,_RPMPOS,_and_THRPOS
    chem2 table CLDPOS use "Coolant Temp" -200 260 Degrees use "AIS Open" 0 2048 Steps "Off idle positioning of AIS motor. This changes immediately. Actual AIS position is largest of CLDPOS, RPMPOS, and THRPOS"
    ; AisOffIdlePositionCold (CLDPOS)
    hex 03
    hex 58 54 ff96
    hex b1 2f fe56
    hex cc 02
    hex ff ff ffff
BARAIS_AisBaroCompensation:
    .if Map2Bar > 0
      chem 0 n AisBaroCompensation2bar Barometric_Pressure_From_MAP -14.7 14.7 psi Factor_(FCT-1+Y/256) 0.0 1.0 out BARAIS Barometric_compensation_factor_for_AIS_control_-_SCALE_for_MAP
      chem2 table BARAIS use Baro -14.7 14.7 psi Factor_(FCT-1+Y/256) 0.0 1.0 out "Barometric compensation factor for AIS control"
      hex 02
      hex 5e 80 fa2f
      hex 69 40
    .endif
    .if Map3Bar > 0
      chem 0 n AisBaroCompensation3bar Barometric_Pressure_From_MAP -14.7 29.4 psi Factor_(FCT-1+Y/256) 0.0 1.0 out BARAIS Barometric_compensation_factor_for_AIS_control_-_SCALE_for_MAP
      chem2 table BARAIS use Baro -14.7 29.4 psi Factor_(FCT-1+Y/256) 0.0 1.0 out "Barometric compensation factor for AIS control"
    ; AisBaroCompensation (BARAIS)
    hex 02
    hex 3e 68 f6dc
    hex 45 28
      hex 02
      hex 3e 80 f7dc
      hex 45 40
    .endif
    hex ff ff ffff
    hex ff ff ffff
THRPOS_AisOffIdlePositionThrottle:
    chem 0 n AisOffIdlePositionThrottle Throttle_Position_-_MinThrottle 0 256 in AIS_Open_Steps 0 256 out THRPOS Off_idle_positioning_of_AIS_motor._This_changes_immediately._Actual_AIS_position_is_largest_of_CLDPOS,_RPMPOS,_and_THRPOS._X_=_(THR_-_MINTHR)*51
	chem2 table THRPOS use TPS 0 5 Volts "AIS off idle position from throttle"
    hex 02
    hex 02 00 009e
    hex 0f 08
    hex ff ff ffff
    hex ff ff ffff
RPMPOS_AisOffIdlePositionRpm:
    chem 0 n AisOffIdlePositionRpm RPMs 0 8192 in AIS_Open_Steps 0 256 out RPMPOS Off_idle_positioning_of_AIS_motor._This_changes_immediately._Actual_AIS_position_is_largest_of_CLDPOS,_RPMPOS,_and_THRPOS
	chem2 table RPMPOS use RPM 0 8192 rpm "AIS position from RPM"
    hex 03
    hex 1f 00 0030
    hex 2f 03 0021
    hex 46 06
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
PNPOS_ParkNeutralPosition:
    chem 3 n AisParkNeutralPosition X 0 256 in Y 0 256 out PNPOS Park/Neutral_position_(can_be_pos_or_neg..._signed_number)._This_is_used_for_throttle_positioning.
	chem2 byte signed PNPOS "Park / Neutral AIS position (signed number)"
    db  $00
ACPOS_AcPosition:
    chem 3 n AisAcPosition X 0 256 in Y 0 256 out ACPOS AC_Position_(clutch)_when_Temp_to_Cold
	chem2 byte ACPOS "AC Position when Temp to Cold"
    db  $17
ACTMP_AisTempSwap:
    chem 3 n AisTempSwap X 0 256 in Y 0 256 out ACTMP Use_ACCTBL_When_ENGTMP_>_ACTMP,_Otherwise_use_ACPOS
    db  $a6
CALPOS_AisPositionGuess:
    chem 3 n AisPositionGuess X 0 256 in Y 0 256 out CALPOS Position_that_electronics_thinks_AIS_motor_is_(at_idle).
	chem2 byte CALPOS "Position AIS motor is at Idle"
    db  $1a
CALRAT_AisCalibrateRate:
    chem 3 n AisPositionGuess X 0 256 in Y 0 256 out CALRAT AIS_Update_rate_when_calibrating._Possible_values:_x00,_x01,_x03,_x07,_x0f,_x1f,_x3f,_x7f,_xff_(PROBABLY_NOT)
    db  $0e
BRKRPM_AisBrakeOverride:
    chem 0 y AisBrakeOverride Throttle_Position 0 256 in RPM 0 8192 out BRKRPM Brake_override_for_AIS_adjustments
    hex  01
    hex  ff 3f
    hex  ff ff ffff
    hex  ff ff ffff
FWDAIS_AisPeriodForward:
    chem 2 y AisPeriodForward Delta_RPM 0 256 in Number_of_Steps -128 128 out FWDAIS For_Computing_AIS_adjustments._Note:_Slope_is_always_zero..._don't_recalc_this_table!
    hex  04
    hex  14 01 0000
    hex  28 02 0000
    hex  3c 04 0000
    hex  50 06
    hex  ff ff ffff
    hex  ff ff ffff
REVAIS_AisPeriodReverse:
    chem 2 y AisPeriodReverse Delta_RPM 0 256 in Number_of_Steps -128 128 out REVAIS For_Computing_AIS_adjustments._Note:_Slope_is_always_zero..._don't_recalc_this_table!
    hex  03
    hex  14 ff 0000
    hex  28 fe 0000
    hex  50 fc
    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff

	chem2 group "Decel AIS"

TIMEDC_AisDecelTime:
    chem 0 y AisDecelTime Time_(2.81sec_units) 0 716 seconds Time_Term 0 256 out TIMEDC Time_factor_for_handling_Ais_during_deceleration
	chem2 table TIMEDC use Time 0 720 seconds "Time factor for AIS base airflow curve offset"
    hex  02
    hex  07 08 ff1c
    hex  10 00
    hex ff ff ffff
    hex ff ff ffff
TEMPDC_AisDecelTemp:
    chem 0 n AisDecelTemp Engine_Temp -200 260 degreesF Temp_Term_for_Decel_AIS_control 0 256 out TEMPDC For_adjusting_AIS_during_Decel
	chem2 table TEMPDC use "Coolant Temp" -200 260 Degrees "Temp factor for AIS base airflow curve offset"
    hex  02
    hex  58 25 ffb4
    hex  d2 01
    hex  ff ff ffff
    hex  ff ff ffff
    hex  ff ff ffff
DAISOP_AisDecelOpenSteps:
    chem 2 n AisDecelOpenSteps Delta_MAP 0 44.1 psi AIS_Steps_(signed) -128 128 out DAISOP Deceleration_AIS_Open_Steps_vs._-Delta_MAP
	chem2 table signed DAISOP use "Delta Map" 0 44.1 psi "Decel AIS open steps from -Delta MAP"
    hex  02
    hex  01 00 00ba
    hex  0c 08
    hex  ff ff ffff
    hex  ff ff ffff
DAISCL_AisDecelCloseSteps:
    chem 2 n AisDecelCloseSteps Delta_MAP 0 44.1 psi AIS_Steps -128 128 out DAISCL Deceleration_AIS_Open_Steps_vs._+Delta_MAP
	chem2 table signed DAISCL use "Delta Map" 0 44.1 psi "Decel AIS open steps from +Delta MAP"
    hex  02
    hex  02 00 ffab
    hex  11 fb
    hex  ff ff ffff
    hex  ff ff ffff
MAPFLR_AisAdditionToControlMap:
    chem 0 y AisAdditionToControlMap RPM 0 8192 in Offset_in_scaled_MAP_units 0 256 out MAPFLR Determines_the_addition_to_control_MAP_vs._RPM
	chem2 table MAPFLR use RPM 0 8192 rpm use MAP 0 256 A2D "AID Decel Addition to control MAP vs. RPM"
	; TODO: scale for MAP
    hex  03
    hex  5e 00 0029
    hex  7d 05 0010
    hex  9c 07
    hex  ff ff ffff
    hex ff ff ffff
BARFLR_AisBarometricFloor:
    .if Map2Bar > 0
      chem 0 n AisBarometricFloor2bar Barometric_Pressure_From_MAP -14.7 14.7 psi Offset -14.7 14.7 psi BARFLR MAPFLR_correction_due_to_barometric_effect
      chem2 table BARFLR "AIS Decel Offset correction due to baro effect"
	  ; TODO: scale for MAP
      hex  01
      hex  ff 80
    .endif
    .if Map3Bar > 0
      chem 0 n AisBarometricFloor3bar Barometric_Pressure_From_MAP -14.7 29.4 psi Offset -14.7 29.4 psi BARFLR MAPFLR_correction_due_to_barometric_effect
      hex  01
      hex  ff 55
    .endif
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
MPTEMP_AisTempTermForMapOffset:
    chem 0 n AisTempTermForMapOffset Engine_Temp -200 260 degreesF Factor 0.0 1.0 out MPTEMP Temperature_Term_for_MAP_Offset
	chem2 table MPTEMP use "Coolant Temp" -200 260 Degrees use Factor 0 1 Factor "AIS Decel Temperature Term"
    hex  02
    hex  90 80 024f
    hex  c7 ff
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
DCLFRQ_AisDecelPulseTime:
    chem 3 n AisDecelPulseTime X 0 256 in Y 0 256 out DCLFRQ Time_between_AIS_pulses_for_decel_AIS
	chem2 byte DCLFRQ use Time 0 2.8 Seconds "Time between AIS pulses for Decel AIS"
    db  $3f
DATHOT_AisCalEngineTemp:
    chem 3 n AisCalEngineTemp X 0 256 in Y 0 256 out DATHOT Engine_temp_to_start_calibrating_DACAL
	chem2 byte DATHOT use "Coolant Temp" -200 260 Degrees "Engine temp to start calibrating Decel AIS"
    db  $cd
DADRPM_AisDeltaRpmFromIdle:
    chem 3 n AisDeltaRpmFromIdle X 0 256 in Y 0 256 out DADRPM Delta_RPM_from_idle_speed_for_DACAL_update
    db  $02
DAACF_AisAcFactor:
    chem 3 n AisAcFactor X 0 256 in Y 0 256 out DAACF A/C_factor_delta_from_base
    db  $09
DANOM_AisBase:
    chem 3 n AisBase X 0 256 in Y 0 256 out DANOM Decel_AIS_nominal_base_(set_nominal_cal_to_about_$80)_stock_87_is_$10
    db  $10
DCALTM_AisDacalInhibitAfterStart:
    chem 3 n AisDacalInhibitAfterStart X 0 256 in Y 0 256 out DCALTM DACAL_inhibit_time_after_startup_(2.816_sec/bit)
    db  $15
L1329:
    db  $7e
L1337:
    db  $00
L1336:
    db  $cd
DCLMN2_NoAISControlIfMapBelowThis:
    chem 6 n AisMinimumMapForControl X 0 256 in Y 0 256 out DCLMN2 Compared_with_MAP_during_AIS_calculations 2-Bar 14 3-Bar d Unused 0 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
    .if Map2Bar > 0
      db $14  ; $14
    .endif
    .if Map3Bar > 0
      db $0d
    .endif

; if vehicle speed is above the value in MPHREF_SwitchPointBetweenDecelIdle, $06, use the first value $17,
; otherwise use the 2nd value, $2d. This value is then stored in Timer_AISDelay
DLYTIM_AisDelayAfterThrottleCloses_moving_stopped:
DLYTIM_AisDelayAfterThrottleCloses_moving:
    chem 3 n AisDelayAfterThrottleCloses_moving X 0 256 in Y 0 256 out DLYTIM AIS_Delay_Time_When_Moving
	chem2 byte DLYTIM use Time 0 2.8 Seconds "Decel AIS Delay after throttle closes when moving"
    db  $17
DLYTM2_AisDelayAfterThrottleCloses_stopped:
    chem 3 n AisDelayAfterThrottleCloses_stopped X 0 256 in Y 0 256 out DLYTM2 AIS_Delay_Time_When_Stopped
	chem2 byte DLYTM2 use Time 0 2.8 Seconds "Decel AIS Delay after throttle closes when stopped"
    db  $2d

DRPNK_AisParkNeutralKick:
    chem 2 n AisParkNeutralKick Engine_Temp -200 260 degreesF Kick_(steps) -128 128 out DRPNK Number_of_AIS_steps_to_kick_when_changing_from_drive_to_park/neutral
    db  1
    db  $ff, $00
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
PNDRK_AisNeutralDriveKick:
    chem 2 n AisNeutralDriveKick Engine_Temp -200 260 degreesF Kick_(steps) 0 256 out PNDRK Number_of_AIS_kicks_when_shifting_from_Park/Neutral_to_Drive
    db  1
    db  $ff, $00
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
PNDLT_DeltaRPMEngineChangesToInitPNDriveKicks:
    dw  $00c8
ACCTBL_AisACOffIdleDelta:
    chem 0 n AisACOffIdleDelta Vehicle_Speed 0 128 in Kick 0 256 steps ACCTBL Number_of_kicks_to_AIS_when_AC_goes_on_when_off_idle
    db  3
    db  $32, $1e, $00, $70
    db  $64, $34, $00, $a6
    db  $78, $41
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
L1502:
    db  $f0
L1505:
    db  $f8
L1510:
    db  $12
L1516:
    db  $07
L1508:
    db  $16
L1540:
    db  $80

BTVAIS_AisMinVoltageToControlIdle:
    chem 3 n AisMinVoltageToControlIdle X 0 256 in Y 0 256 out BTVAIS Minimum_voltage_to_control_AIS
	chem2 byte BTVAIS simple use Battery 0 16.19 Volts "Minimum voltage for AIS to be active"
    db  $6f

    db  $0d,$0d,$ac
L1529:
    db  $3f
L1542:
    db  $dc
L1497:
    db  $0c
MPHREF_SwitchPointBetweenDecelIdle:
	chem2 byte MPHREF use Speed 0 128 mph "Switch point between Decel or Idle AIS control. Note there is also a 1.5 mph hysteresis"
    db  $06

;  ______________________
; |                      \
; |   Boost Control      |
; \______________________|

    chem2 group "Boost Control"

BoostAllowedFromRPM_WSGRPM:
	.if Map2Bar > 0
	  chem 0 n * RPM 0 8192 in Boost_Allowed_above_Baro 0 29.4 psi WSGRPM Amount_of_boost_(MAP_reading_above_Barometric_Pressure)_allowed_based_on_RPMs
	  chem2 table WSGRPM simple use RPM 0 8192 rpm use Boost 0 29.4 psi "Boost Allowed from RPMs. Use this to back off boost above a certain RPM to keep within your maximum available fuel"
      db  2
      db  $45, $8c, $00, $00
      db  $6d, $8c
      hex ff ff ffff
      hex ff ff ffff
    .endif
	.if Map3Bar > 0
	  chem 0 n * RPM 0 8192 in Boost_Allowed_above_Baro 0 44.1 psi WSGRPM Amount_of_boost_(MAP_reading_above_Barometric_Pressure)_allowed_based_on_RPMs
	  chem2 table WSGRPM simple use RPM 0 8192 rpm use Boost 0 44.1 psi "Boost Allowed from RPMs. Use this to back off boost above a certain RPM to keep within your maximum available fuel"
    ; BoostAllowedFromRPM (WSGRPM)
    hex 04
    hex 57 74 0000
    hex 64 74 0000
    hex a3 74 fedc
    hex bf 54
    .endif
    hex ff ff ffff
    hex ff ff ffff

WSGCLT_BoostAllowedFromCoolantTemp:
    .if Map2Bar > 0
      chem 0 n BoostAllowedFromCoolantTemp2bar Engine_Temp -200 260 degreesF Boost_Allowed_above_Baro 0 29.4 psi WSGCLT Amount_of_boost_(MAP_reading_above_Barometric_Pressure)_allowed_based_on_Coolant_Temp
	  chem2 table WSGCLT simple use "Coolant Temp" -200 260 Degrees use "Boost Allowed" 0 29.4 psi "Boost allowed above Baro based on Engine Temperature. (cannot exceed 14.7 psi)"
      db  2
      db  $81, $23, $03, $38
      db  $9c, $7a
    .endif
    .if Map3Bar > 0
      chem 0 n BoostAllowedFromCoolantTemp3bar Engine_Temp -200 260 degreesF Boost_Allowed_above_Baro 0 44.1 psi WSGCLT Amount_of_boost_(MAP_reading_above_Barometric_Pressure)_allowed_based_on_Coolant_Temp
	  chem2 table WSGCLT simple use "Coolant Temp" -200 260 Degrees use "Boost Allowed" 0 44.1 psi "Boost allowed above Baro based on Engine Temperature. (cannot exceed 29.4 psi)"
    ; BoostAllowedFromCoolantTemp (WSGCLT)
    hex 02
    hex 9a 23 0261
    hex bc 74
    .endif
    hex ff ff ffff
    hex ff ff ffff

BoostAllowedFromSpeed_MXSBST:
    .if Map2Bar > 0
      chem 0 n * Vehicle_Speed 0 128 mph Boost_Allowed_above_Baro 0 29.4 psi MXSBST Amount_of_boost_(MAP_reading_above_Barometric_Pressure)_allowed_based_on_Vehicle_Speed._Primarily_designed_to_protect_automatic_transmissions.
	  chem2 table MXSBST simple use "Vehicle Speed" 0 128 mph use Boost 0 29.4 psi "Boost Allowed from Speed. Use this to avoid wheelspin at launch, most useful for automatics or people who slip their clutches a lot."
      db  2
      db  $01, $1d, $05, $00
      db  $10, $74
      db  $ff, $ff, $ff, $ff
      db  $ff, $ff, $ff, $ff
    .endif
    .if Map3Bar > 0
      chem 0 n * Vehicle_Speed 0 128 mph Boost_Allowed_above_Baro 0 44.1 psi MXSBST Amount_of_boost_(MAP_reading_above_Barometric_Pressure)_allowed_based_on_Vehicle_Speed._Primarily_designed_to_protect_automatic_transmissions.
	  chem2 table MXSBST simple use "Vehicle Speed" 0 128 mph use Boost 0 44.1 psi "Boost Allowed from Speed. Use this to avoid wheelspin at launch, most useful for automatics or people who slip their clutches a lot."
    ; BoostAllowedFromSpeed (MXSBST)
    hex 02
    hex 01 1d 07e8
    hex 0c 74
      db  $ff, $ff, $ff, $ff
      db  $ff, $ff, $ff, $ff
    .endif

BoostAllowedMaxPartThrottle:
    .if Map2Bar > 0
      chem 3 n * X 0 256 in Boost 0 29.4 psi BSTPT Max_boost_allowed_at_part_throttle._(cannot_exceed_14.7_psi)
      chem2 byte BSTPT simple use "Boost" 0 29.4 psi "Boost allowed at part throttle. (cannot exceed 14.7 psi)"
      db  $65			; about 12 psi
    .endif
    .if Map3Bar > 0
      chem 3 n * X 0 256 in Boost 0 44.1 psi BSTPT Max_boost_allowed_at_part_throttle._(cannot_exceed_29.4_psi)
      chem2 byte BSTPT simple use "Boost" 0 44.1 psi "Boost allowed at part throttle. (cannot exceed 29.4 psi)"
      db  $45
    .endif

RESFAC_BoostMapCorrectionResponseFactor:
    chem 0 n BoostMapCorrectionResponseFactor BoostGoal_-_MAP_+_$80 0 256 in Response_Factor 0.0 1.0 factor RESFAC Map_Correction_Response_Factor_for_controlling_wastegate
    db  2
    db  $91, $06, $ff, $a6
    db  $a2, $00
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
RATLIM_BoostOverboostPrevention:
    chem 2 y BoostOverboostPrevention RPMs 0 8192 in POS_Change_in_Map -128 128 out RATLIM Overboost_Prevention._Forces_100%_duty_cycle_in_Wastegate_Solenoid_if_+delta_MAP_>_RATLIM
	chem2 table RATLIM use RPM 0 8192 rpm "Delta Map" 0 29.4 psi "Rate Limit: overboost prevention. Forces 100% wastegate duty cycle if +Delta Map greater than this. This is to prevent Boost Creep"
    db  2
    db  $0f, $14, $ff, $e0	; increased slightly since we want
    db  $4e, $0c			; boost to build quicker

;    db  $10, $0c, $ff, $e7
;    db  $4e, $06
    hex ff ff ffff
    hex ff ff ffff

DCWOT_BoostWotDutyCycle:
    .if Map2Bar > 0
        chem 0 y BoostWotDutyCycle Boost 0 29.4 psi Duty_Cycle 0 128 Percent DCWOT Wastegate_duty_cycle_at_a_given_boost,_at_WOT.
        chem2 table DCWOT use Boost 0 29.4 psi use "Duty Cycle" 0 128 Percent "Wastegate duty cycle at a given boost, at WOT."
        db   5                       ; number of points
        db   $2e, $c8, $f3, $9a  ; point 1
        db   $33, $8a, $fe, $5f  ; point 2
        db   $4e, $5e, $fe, $be  ; point 3
        db   $69, $3c, $ff, $ad  ; point 4
        db   $88, $32            ; point 5

    .endif
    .if Map3Bar > 0
        chem 0 y BoostWotDutyCycle Boost 0 44.1 psi Duty_Cycle 0 128 Percent DCWOT Wastegate_duty_cycle_at_a_given_boost,_at_WOT.
        chem2 table DCWOT use Boost 0 44.1 psi use "Duty Cycle" 0 128 Percent "Wastegate duty cycle at a given boost, at WOT."
        ; BoostWotDutyCycle (DCWOT)
        hex 05
        hex 1e c8 eb56
        hex 21 8a fd8f
        hex 33 5e fe1d
        hex 45 3c ff87
        hex 5a 32
    .endif
    hex ff ff ffff

DCWADJ_BoostWotDutyCycleAdj:
    chem 0 y BoostWotDutyCycleAdj RPM 0 8192 in Duty_Cycle 0 256 out DCWADJ Adjusts_duty_cycle_required_to_control_Wastegate_Solenoid_based_on_RPM._Zero_is_no_adjustment,_higher_numbers_decrease_duty_cycle.
    db  1
    db  $ff, $00
    hex ff ff ffff
    hex ff ff ffff
MAPOFS_BoostMapOffsetForDCWOT:
    chem 3 n BoostMapOffsetForDCWOT X 0 256 in Y 0 256 out MAPOFS Offset_in_MAP_above_baro_for_DCWOT
    db  $10  ; $17
KNOKMP_BoostKNKCNTinc:
    chem 3 n BoostKNKCNTinc X 0 256 in Y 0 256 out KNOKMP KNKCNT_inc_when_MAP_>_PBARO+KNKMP
    db  $10  ; $17
BoostMaxKNKBeforeReset_KNKMAX:
    chem 3 n BoostMaxKNKBeforeReset X 0 256 in Y 0 256 out KNKMAX Max_number_of_knocks/2.8_seconds_before_TIMKNK_reset
    db  $10
RPMBST_NoBOOSTIincWhenBelowThis:
    db  $26
PRTMAX_MaxDCForPartThrottleInc:
    db  $c8    ; $c8=100%
MPDEL_HysteresisForBoostIncreasingCondition:
    db  $00
UPRLIM_UpperLimitOnMPCORR:
    chem 6 n BoostPullbackUpperLimitOnMPCORR IN 0 255 X BOOST 0 29.4 psi UPRLIM Boost_1.8_psi_constant 2-Bar 10 3-Bar b Unused 0 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
    db  $0b
LWRLIM_LowerLimitOnMPCORR:
    chem 6 n BoostPullbackLowerLimitOnMPCORR IN 0 255 X BOOST 0 29.4 psi LWRLIM Boost_3.7_psi_constant 2-Bar 20 3-Bar 15 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
    db  $15
KDCINC_DCincreaseStepWhenKnockPresent:
    db  4    ; 4 = 2%
KDCMAX_MaxDCwhenKnockPresent:
    db  $50  ; $50 = 40%
KDCMIN_MinDCWhenIncreasingDueToKnock:
    db  $0c
BSOLCO_BoostSolenoidCutout:
    db  $07
DECTIM_TimeScaleForDCDECK:
    db  $03  ; (DECTIM+1 * 11ms), $03=44ms
DCDECK_DCdecrementWhenSparkStillDerated:
    db  $02
; ***********
MAPPT_MapPointForWastegateControlCells:
    chem 6 n BoostMapPointForWastegateControlCells X 0 256 in Y 0 256 out MAPPT MAP_Point_for_Wastegate_Control_Cells 2-bar 62 3-bar 41 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
    .if Map2Bar > 0
	  chem2 byte MAPPT use Boost 0 29.4 psi "Wastegate Duty Cycle Adjustment Map Switch Point"
      db  $62  ; $62
    .endif
    .if Map3Bar > 0
	  chem2 byte MAPPT use Boost 0 44.1 psi "Wastegate Duty Cycle Adjustment Map Switch Point"
      db  $41
    .endif
; ***********
MPDEL1_IncCounterWhenDeltaMAPLessThanThis:
    db  $03
INCPT_AdaptiveIncAfterThisTime:
    db  $0a  ; time is this value+1 * 11ms
MPROOF_NoAdaptiveWhenMapIsThisCloseToGoal:
    db  $02
INCLIM_UpperLimitOnDCCORX:
    db  $b0
L1812:
    db  $02
L1813:
    db  $0a
L1814:
    db  $00
L1815:
    db  $62
L1810:
    db  $2f
L1811:
    db  $ff
; *****************************************
MPSHOF_OverboostShutoff:
    chem 3 n OverboostShutoff X 0 256 in Y 0 256 out MPSHOF When_MAP_+_BARO_exceeds_MPSHOF,_wait_SHFDLT,_then_kill_engine_for_TRNTM._$ff_to_disable_overboost_shutoff
	.if Map2Bar > 0
	  chem2 byte MPSHOF simple use Boost 0 29.4 psi "If MAP + BARO exceeds MPSHOF, wait SHFDLT, then kill engine for TRNTM. $ff to disable overboost shutoff"
	.endif
	.if Map3Bar > 0
	  chem2 byte MPSHOF simple use Boost 0 44.1 psi "If MAP + BARO exceeds MPSHOF, wait SHFDLT, then kill engine for TRNTM. $ff to disable overboost shutoff"
	.endif
    db  $8c  ; $76, stock, was 13.4psi
SHFDLT_OverboostShutoffDelayTime:
    chem 3 n OverboostShutoffDelayTime X 0 256 in Y 0 256 out SHFDLT When_MAP_+_BARO_exceeds_MPSHOF,_wait_SHFDLT,_then_kill_engine_for_TRNTM._$1a_is_about_4_seconds,_$35_is_about_6_seconds
	chem2 byte SHFDLT simple use Time 0 40 Seconds "When MAP + BARO exceeds MPSHOF, wait SHFDLT, then kill engine for TRNTM. $1a is about 4 seconds, $35 is about 6 seconds"
    db  $1a  ; time above MPSHOF before shutting off
TRNTM_OverboostShutoffTime:
    chem 3 n OverboostShutoffTime X 0 256 in Y 0 256 out TRNTM When_MAP_+_BARO_exceeds_MPSHOF,_wait_SHFDLT,_then_kill_engine_for_TRNTM._$2d=.5_seconds
	chem2 byte TRNTM simple use Time 0 2.8 Seconds "When MAP + BARO exceeds MPSHOF, wait SHFDLT, then kill engine for TRNTM. $2d is about .5 seconds"
    db  $2e  ; time to kill engine
LMPSHF_OverboostShutoffDuringLimpinMode:
    chem 3 n OverboostShutoffBadWastegate X 0 256 in Y 0 256 out LMPSHF Overboost_Value_when_in_wastegate_limpin_mode._This_really_should_be_kept.
	.if Map2Bar > 0
  	  chem2 byte LMPSHF use Boost 0 29.4 psi "Overboost Shutoff when Wastegate problem has been detected (should not disable this)"
	.endif
	.if Map3Bar > 0
	  chem2 byte LMPSHF use Boost 0 44.1 psi "Overboost Shutoff when Wastegate problem has been detected (should not disable this)"
	.endif
    db  $73  ; $73 = 13.1 psi in 2-bar
; factory tables:
;BoostAllowedFromRPM_WSGRPM:
;    chem 0 n * RPM 0 8192 in Boost_Allowed_above_Baro 0 44.1 psi WSGRPM Amount_of_boost_(MAP_reading_above_Barometric_Pressure)_allowed_based_on_RPMs
;   db  2
;   db  $45, $8c, $00, $00
;   db  $6d, $8c
;   db  $ff, $ff, $ff, $ff
;   db  $ff, $ff, $ff, $ff
;
;BoostAllowedFromTime_WSGTIM:
;    chem 0 n * Time_in_Boost 0 716 seconds Boost_Allowed_above_Baro 0 44.1 psi WSGTIM Amount_of_boost_(MAP_reading_above_Barometric_Pressure)_allowed_based_on_Time_in_milliseconds_/_352
;   db  1
;   db  $ff, $8c
;   db  $ff, $ff, $ff, $ff
;   db  $ff, $ff, $ff, $ff

; WSGCLT_BoostAllowedFromCoolantTemp:
;    chem 0 n * Engine_Temp -200 260 degreesF Boost_Allowed_above_Baro 0 44.1 psi WSGCLT Amount_of_boost_(MAP_reading_above_Barometric_Pressure)_allowed_based_on_Coolant_Temp
;   db  2
;   db  $7f, $30, $03, $2c
;   db  $9c, $8c
;   db  $ff, $ff, $ff, $ff
;   db  $ff, $ff, $ff, $ff
;
;BoostAllowedFromChargeTemp_WSGCHG:
;    chem 0 n * Charge_Temp 245 -45 degreesF Boost_Allowed_above_Baro 0 44.1 psi WSGCHG Amount_of_boost_(MAP_reading_above_Barometric_Pressure)_allowed_based_on_Charge_Temp
;   db  1
;   db  $ff, $8c
;   db  $ff, $ff, $ff, $ff
;   db  $ff, $ff, $ff, $ff
;BoostTimeSinceLastKnock_WSGKNK:
;    chem 0 y * Time_Since_Last_Knock_Subtractive_Term 0 716 seconds Subtractive_Term 0 256 out WSGKNK Controls_how_quickly_boost_is_allowed_to_come_back_up_after_a_Knock_event_has_backed_it_down._Each_X_is_2.8_seconds,_each_Y_is_one_MAP_resolution_unit.
;   db  2
;   db  $02, $12, $fb, $80
;   db  $06, $00
;   db  $ff, $ff, $ff, $ff
;BoostAllowedFromSpeed_MXSBST:
;    chem 0 n * Vehicle_Speed 0 128 mph Boost_Allowed_above_Baro 0 44.1 psi MXSBST Amount_of_boost_(MAP_reading_above_Barometric_Pressure)_allowed_based_on_Vehicle_Speed._Primarily_designed_to_protect_automatic_transmissions.
;   db  1
;   db  $ff, $68
;   db  $ff, $ff, $ff, $ff
;

;  ______________________
; |                      \
; |   Fan Control        |
; \______________________|
    chem2 group Fan

FANTMP_FanOnOver45MPH:
    chem 3 n FanOnOver45MPH Temperature -200 260 degreesF Y 0 256 out FANTMP Temperature_that_fan_turns_on_when_vehicle_speed_above_45_mph
    chem2 byte FANTMP simple use "Coolant Temp" -200 260 Degrees "Temperature to turn on fan when vehicle speed above 45 mph"
    db  $ee
FANTMP2_FanOffOver45MPH:
    chem 3 n FanOffOver45MPH Temperature -200 260 degreesF Y 0 256 out FANTMP2 Temperature_that_fan_turns_off_when_vehicle_speed_above_45_mph
    chem2 byte FANTMP2 simple use "Coolant Temp" -200 260 Degrees "Temperature to turn off fan when vehicle speed above 45 mph"
    db  $e8
FANTMP3_FanOnBelow45MPH:
    chem 3 n FanOnBelow45MPH Temperature -200 260 degreesF Y 0 256 out FANTMP3 Temperature_that_fan_turns_on_when_vehicle_speed_below_45_mph
    chem2 byte FANTMP3 simple use "Coolant Temp" -200 260 Degrees "Temperature to turn on fan when vehicle speed below 45 mph"
    db  $e3
FANTMP4_FanOffBelow45MPH:
    chem 3 n FanOffBelow45MPH Temperature -200 260 degreesF Y 0 256 out FANTMP4 Temperature_that_fan_turns_off_when_vehicle_speed_below_45_mph
    chem2 byte FANTMP4 simple use "Coolant Temp" -200 260 Degrees "Temperature to turn off fan when vehicle speed below 45 mph"
    db  $dd
FANMAP_FanMap:
    chem 6 n FanMap MAP -14.7 29.4 psi Y 0 256 out FANMAP Turn_on_fan_when_MAP_>_PBARO_+_FANMAP_and_Battery_Temp_>_FANBAT_(increases_intercooler_efficiency_in_boost) 2-Bar 1a 3-Bar 11 Disabled 7f Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
	.if Map2Bar > 0
      chem2 byte FANMAP use "Boost" 0 29.4 psi "Turn on fan when this far into Boost and Battery Temperature above FANBAT (to increase intercooler efficiency in boost)"
    .endif
	.if Map3Bar > 0
      chem2 byte FANMAP use "Boost" 0 44.1 psi "Turn on fan when this far into Boost and Battery Temperature above FANBAT (to increase intercooler efficiency in boost)"
    .endif
    db  $11
FANBAT_FanBat:
    chem 3 n FanBat Temperature -200 260 degreesF Y 0 256 out FANBAT Turn_on_fan_when_MAP_>_PBARO_+_FANMAP_and_Battery_Temp_>_FANBAT_(increases_intercooler_efficiency_in_boost)
    chem2 byte FANBAT use "Battery Temp" -200 260 Degrees "Turn on fan when FANMAP into Boost and Battery Temperature above this (to increase intercooler efficiency in boost)"
    db  $9a

;  ______________________
; |                      \
; |   Baro Read          |
; \______________________|

    chem2 group "Baro Read"

BRPMLO_BaroReadRPMLow:
    chem 4 n BaroReadRPMLow X 0 256 in Y 0 256 out BRPMLO RPM_above_which_Baro_is_read_(x1c20_=_1800_rpm)
    chem2 word BRPMLO "RPM above which Baro is read. This is actual RPM * 2. Stock is 1c20 which is about 1800 RPM."
    dw  $1c20
BRPMHI_BaroReadRPMHigh:
    chem 4 n BaroReadRPMHigh X 0 256 in Y 0 256 out BRPMHI RPM_below_which_Baro_is_read_(x3e80_=_6650_rpm)
    chem2 word BRPMHI "RPM below which Baro is read. This is actual RPM * 2. Stock is 3e80 which is about 6650 RPM."
    dw  $3e80
BRDLAY_BaroReadThrottleDelay:
    chem 3 n BaroReadThrottleDelay X 0 256 in Y 0 256 out BRDLAY Baro_Read_Delay_after_throttle_closed
	chem2 byte BRDLAY use Time 0 2.8 Seconds "Delay after throttle closes before Baro Read is performed"
    db  $2d
BARTIM_BaroReadTime:
    chem 3 n BaroReadTime X 0 720 in Y 0 256 out BARTIM Baro_Read_Time_(seconds/2.8)_(x40_=_179_seconds)
	chem2 byte BARTIM use Time 0 720 Seconds "Time between Baro Reads"
    db  $40
BROPTM_BaroReadOpenTime:
    chem 3 n BaroReadOpenTime X 0 256 in Y 0 256 out BROPTM Delay_of_MAP_value_after_solenoid_closed_(msec/11)
	chem2 byte BROPTM use Time 0 2.8 Seconds "Time between Baro Read solenoid opening and valid Baro data available"
    db  $05
BRCLTM_BaroReadCloseTime:
    chem 3 n BaroReadCloseTime X 0 256 in Y 0 256 out BRCLTM Delay_of_baro_read_after_solenoid_open_(msec/11)
	chem2 byte BRCLTM use Time 0 2.8 Seconds "Time between Baro Read solenoid closing and valid Map data available"
    db  $05
BAROHI_BaroReadMaxValid:
    chem 6 n BaroReadMaxValid X 0 256 in Y 0 256 out BAROHI Baro_Read_Highest_valid_amount 2-Bar 80 3-Bar 56 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
    .if Map2Bar > 0
	  chem2 byte BAROHI use MAP -14.7 14.7 psi "Highest value from MAP sensor accepted as Baro"
      db  $80
    .endif
    .if Map3Bar > 0
	  chem2 byte BAROHI use MAP -14.7 29.4 psi "Highest value from MAP sensor accepted as Baro"
      db  $56
    .endif
BAROLO_BaroReadMinValid:
    chem 6 n BaroReadMinValid X 0 256 in Y 0 256 out BAROLO Baro_Read_Lowest_valid_amount 2-Bar 45 3-Bar 2e Unused 0 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
    .if Map2Bar > 0
	  chem2 byte BAROLO use MAP -14.7 14.7 psi "Lowest value from MAP sensor accepted as Baro"
      db  $45
    .endif
    .if Map3Bar > 0
	  chem2 byte BAROLO use MAP -14.7 29.4 psi "Lowest value from MAP sensor accepted as Baro"
      db  $2e
    .endif
BARDFT_BaroReadDefault:
    chem 6 n BaroReadDefault X 0 256 in Y 0 256 out BARDFT Baro_Read_Default_Value 2-Bar 7f 3-Bar 54 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
    .if Map2Bar > 0
	  chem2 byte BARDFT use MAP -14.7 14.7 psi "Default value for Baro in case of Baro Read problems"
      db  $7f
    .endif
    .if Map3Bar > 0
	  chem2 byte BARDFT use MAP -14.7 29.4 psi "Default value for Baro in case of Baro Read problems"
      db  $54
    .endif
L1459:
    db  $6e
L1466:
    db  $6d
L1370:
    db  $1a

;  ______________________
; |                      \
; |   Errors             |
; \______________________|

    chem2 group Errors

ErrorCodeBitMask1:
    chem 4 n * X 0 256 in Y 0 256 out NAME Error_Code_Flag_Bit_Mask_-_stock_is_ffbf
    dw  $ffbf
ErrorCodeBitMask2:
    chem 4 n * X 0 256 in Y 0 256 out NAME Error_Code_Flag_Bit_Mask_-_stock_is_feff
    dw  $feff

L1274:
    db  $fe
L1588:
    db  $63
L1590:
    db  $e4
L1018:
    db  $c7
L1021:
    db  $ae
AHighMAPvalueDuringTPSProcessing:
    db  $5e
L1734:
    db  $18
L1575:
    db  $5c
L1582:
    db  3
L1736:
    db  $04
ValueAboveLowestTPSToConsiderOffIdle:
    db  $01
L1721:
    db  $03
L1720:
    ; ?
    db  $10

;  ______________________
; |                      \
; |   Rev Limiter        |
; \______________________|

    chem2 group "Rev Limits"

; **********************
; Several values here, indexed. Note that these are DistSync timer pulses, not RPMs
; so smaller numbers are higher RPMs

REVLMT_OverRevFuelShutoff:
    chem 4 n OverRevFuelShutoffMapLimpinLower X 0 256 in Y 0 256 out REVLMT Map_limpin_and_throttle_above_25%_-_$3f00=1850rpm
    chem2 word REVLMT "Overrev fuel shutoff. This is inverse RPM. Stock is 3f00 which is about 1850 RPM. To determine this number, divide 3,000,000 by the desired RPM. This is used when MAP is marked as bad and throttle is above 25%"
    ; Map Limp-In and throttle above 25% (lower limit)
    dw  $3f00  ; 1850 rpm
REVLMT2_OverRevFuelShutoff:
    chem 4 n OverRevFuelShutoffMapLimpinUpper X 0 256 in Y 0 256 out REVLMT2 Map_limpin_and_throttle_above_25%_-_$3b00=2000rpm
    chem2 word REVLMT2 "Overrev fuel shutoff. This is inverse RPM. Stock is 3b00 which is about 2000 RPM. To determine this number, divide 3,000,000 by the desired RPM. This is used when MAP is marked as bad and throttle is above 25%"
    ; Map Limp-In and throttle above 25% (upper limit)
    dw  $3b00  ; 2000 rpm
REVLMT3_OverRevFuelShutoff:
    chem 4 n OverRevFuelShutoffTpsLimpinLower X 0 256 in Y 0 256 out REVLMT3 Map_>_75%_of_Baro_and_IdleSwitch_Closed_OR_Map_>_75%_of_Baro_and_Throttle_Limp-in_&_speed_<_RVLSPD_-_$3b00=2000rpm
    chem2 word REVLMT3 "Overrev fuel shutoff. This is inverse RPM. Stock is 3b00 which is about 2000 RPM. To determine this number, divide 3,000,000 by the desired RPM. This is used when TPS is marked as bad and MAP is above 75% of Baro"
    ; Map > 75% of Baro and IdleSwitch Closed OR
    ; Map > 75% of Baro and Throttle Limp-in & speed < RVLSPD
    dw  $3b00  ; 2000 rpm
REVLMT4_OverRevFuelShutoff:
    chem 4 n OverRevFuelShutoffTpsLimpinUpper X 0 256 in Y 0 256 out REVLMT4 Map_>_75%_of_Baro_and_IdleSwitch_Closed_OR_Map_>_75%_of_Baro_and_Throttle_Limp-in_&_speed_<_RVLSPD_-_$2700=3000rpm
    chem2 word REVLMT4 "Overrev fuel shutoff. This is inverse RPM. Stock is 2700 which is about 3000 RPM. To determine this number, divide 3,000,000 by the desired RPM. This is used when TPS is marked as bad and MAP is above 75% of Baro"
    ; UpperLimit of above
    dw  $2700  ; 3000 rpm
REVLMT5_OverRevFuelShutoff:
    chem 4 n OverRevFuelShutoffNormalLower X 0 256 in Y 0 256 out REVLMT5 Lower_limit_of_normal_operation_-_$1300=6168rpm
    chem2 word REVLMT5 "Overrev fuel shutoff. This is inverse RPM. Stock is 1300 which is about 6168 RPM. To determine this number, divide 3,000,000 by the desired RPM. This is the normal operational Rev Limiter"
    ; Lower Limit of Normal Operation
    dw  $1300  ; 6168 rpm
REVLMT6_OverRevFuelShutoff:
    chem 4 n OverRevFuelShutoffNormalUpper X 0 256 in Y 0 256 out REVLMT6 Upper_limit_of_normal_operation_-_$119f=6650rpm
    chem2 word REVLMT6 "Overrev fuel shutoff. This is inverse RPM. Stock is 119f which is about 6649 RPM. To determine this number, divide 3,000,000 by the desired RPM. This is the normal operational Rev Limiter"
    ; Upper Limit of Normal Operation
    dw  $119f  ; 6649 rpm

THRTHR_25pctValueForThrForOverRevShutoff:
    db  $21
RVLSPD_VehicleSpeedForRevLimit:
    db  $14  ; MPH/2, $14=10mph

    .if UseStagingMod > 0
    ; Speed below 5, to allow easier staging.
RVLSPD_Staging:
StagingModSpeed:
    chem 3 n StagingModSpeed X 0 128 MPH Y 0 128 MPH RVLSPD Vehicle_speed_for_Staging_rev_limiter_to_disable._Use_0_to_disable_Staging_mod,_recommended_if_you_have_a_failing_or_marginal_speed_sensor.
    chem2 byte RVLSPD simple use Speed 0 128 MPH "Vehicle speed for Staging rev limiter to disable. Use 0 to disable Staging mod, recommended if you have a failing or marginal speed sensor."
    db  $04     ; 2 mph
REVLMTX_StagingModFuelShutoff:
    chem 4 n OverRevFuelShutoffStagingLower X 0 256 in Y 0 256 out REVLMTX Staging_mod_shutoff_-_$2300=3300rpm
    chem2 word REVLMTX simple "Overrev fuel shutoff. This is inverse RPM. To determine this number, divide 3,000,000 by the desired RPM. This is the Staging Mod"
    dw	$271d	; 3000 rpm @ 2700
REVLMTX2_StagingModFuelShutoff:
    chem 4 n OverRevFuelShutoffStagingUpper X 0 256 in Y 0 256 out REVLMTX2 Staging_mod_shutoff2_-_$2100=3500rpm
    chem2 word REVLMTX2 simple "Overrev fuel shutoff. This is inverse RPM. To determine this number, divide 3,000,000 by the desired RPM. This is the Staging Mod"
    dw	$26f8   ; 3300 rpm? @ 2500
    .endif


; *************************************
ACLRPM_DisableACBelowThisRPM:
    db  $a2
ACOUT_ThrottleForACCutout:
    db  $85
ACCTIM_ACDelayFromStartRun:
    db  $04
ACCRPM_DontEngageACAboveThisRPM:
    dw  $3e80

PURMAP_Purge:
    db  $05  ; Obviously, don't want to open the purge when there is boost.
PURHYS_PurgeHysteresis:
    db  $02
L1246:
    db  $85  ; something to do with TPS - full throttle detection
L1250:
    db  2    ; hysteresis for the TPS

; *****************************************
; Mod: removed temperature dependency for voltage target
VoltageRegulatorTargetVoltage:
    chem 3 n * X 0 256 in Y 0 256 out NAME Target_Charge_Voltage._Should_be_around_e8
    db  $e8

;  ______________________
; |                      \
; |   Limp-In Constants  |
; \______________________|

    chem2 group "Limp-In Constants"

TMPLOW:
    db  $1a  ; out of range (hot - low A/D)
TMPHIG:
    db  $fd  ; out of range (cold - high A/D)
DEFTMP:
    db  $95  ; engine default temp

MinValidAirTemp:
    db  $03
MaxValidAirTemp:
    db  $fe
DefaultAirTemp:
    db  $ad

MinValidBattTemp:
    db  $02
MaxValidBattTemp:
    db  $fd
RPMsForBattTempChecking:
    db  $17

L1043:
    db  $cd
L1045:
    db  $ff
L1040:
    db  $0c
L1041:
    db  $12
L1039:
    db  $80

THRLOW_MinValidTPS:
    db  $08
THRHIG_MaxValidTPS:
    db  $f0
MPTHR_MapSwitchPointForWOTorIdle:
    db  $68 ; used when simulating TPS in case of a bad TPS sensor

L1834:
    db  $10
L1830:
    db  $10
TimeAfterStartupToStopCheckingThrottle:
    dw  $002d
L1840:
    db  $02
MinimumAllowableMAP:
    db  $01
MaximumAllowableMAP:
    db  $fc
L1634:
    db  $07,$d0,$0f
L1236:
    db  $db

FuelMonitorScaling:
    chem 4 n * X 0 256 in Y 0 256 out FUELTR This_is_the_scaling_factor_for_the_navigator._It_must_be_scaled_with_injector_size_so_that_the_fuel_used_and_fuel_economy_numbers_work_out_properly._Stock_T2:_$311,_+20:_$28e
    chem2 word inj FUELTR "Scaling factor for the Navigator. Scale with injector changes"
;    .inj word scale
    dw  $0311

ADTableOffsets83:
    db  $05,$07,$04,$07,$01,$07,$02,$07
    db  $03,$07,$07,$07
ScaleCoolantTempCold_TMPTL1:
    chem 0 y * Raw_Sensor 0 256 in Engine_Temp 0 256 out TMPTL1 Used_to_convert_the_raw_coolant_temp_sensor_data_to_actual_temperature
    db  3
    db  $00, $c9, $ff, $a2
    db  $d7, $7a, $ff, $0d
    db  $ff, $54
ScaleCoolantTempHot_TMPTL2:
    chem 0 y * Raw_Sensor 0 256 in Engine_Temp 0 256 out TMPTL2 Used_to_convert_the_raw_coolant_temp_sensor_data_to_actual_temperature
    db  2
    db  $40, $ff, $ff, $72
    db  $ff, $95
ApproxCoolantTempFromAirTemp_CHGTL1:
    chem 0 n * Air_Temp 0 256 in Coolant_Temp 0 256 out CHGTL1 Charge_Temp_to_Coolant_Temp_conversion_used_when_Coolant_Temp_sensor_appears_bad._Designed_for_limp-in_mode.
    db  4
    db  $0e, $ff, $fe, $43
    db  $29, $d0, $ff, $8d
    db  $e3, $7c, $fe, $70
    db  $fa, $58
Table75_FromTPS:
    chem 1 y * Throttle_Position 0 256 in output 0 12288 out name Description
    db  $00
    dw  $0000, $0250
    db  $10
    dw  $0025, $01db
    db  $17
    dw  $0032, $015d
    db  $22
    dw  $0041, $00f9
    db  $45
    dw  $0063, $0000
    db  $ff
L1942:
    db  $20
L1058:
    db  $07
Unknownword:
    dw  $0361

;  ______________________
; |                      \
; |   Cruise Control     |
; \______________________|

    chem2 group "Cruise Control"

CruiseRegulator_VACREG:
    chem 0 n CruiseRegulator SetSpeed_-_Car_Speed 0 128 in output -128 128 out VACREG Cruise_control_table
	chem2 table VACREG use "Set Speed - Car Speed" 0 -4 mph use AccRate 0 5 "mph/sec" "Vac - Regular Gain Curve"
    db  5
    db  $00, $00, $00, $33
    db  $05, $01, $00, $0d
    db  $40, $04, $00, $08
    db  $80, $06, $00, $14
    db  $ff, $10
    hex ff ff ffff
    hex ff ff ffff
CruiseAcc_VACACC:
    chem 0 n CruiseAcc SetSpeed_-_Car_Speed 0 128 in output 0 256 out VACACC Cruise_control
	chem2 table VACACC use "Set Speed - Car Speed" 0 -4 mph use AccRate 0 5 "mph/sec" "Vac - Accelerated Gain Curve"
    db  2
    db  $00, $02, $00, $2e
    db  $ff, $30
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
CruiseVent_VENTRG:
    chem 0 n CruiseVent Car_speed_-_Set_speed 0 128 in output -128 128 out VENTRG Cruise_control_vent_stuff
	chem2 table signed VENTRG use "Car Speed - Set Speed" 0 4 mph use AccRate 0 5 "mph/sec" "Vent - Regular Gain Curve"
    db  3
    db  $00, $00, $ff, $fb
    db  $66, $fe, $ff, $ef
    db  $ff, $f4
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
CruiseSetPW_SETPW:
    chem 0 n CruiseSetPW Car_Speed 0 128 in Vac_Pulse_Width/11ms 0 256 out SETPW Cruise_Control_stuff
	chem2 table SETPW use "Car Speed" 0 128 mph use Vacuum 0 2.8 Pulsewidth "Set up pulse when Set is pressed"
    db  3
    db  $46, $24, $00, $40
    db  $82, $33, $00, $50
    db  $ff, $5a
    hex ff ff ffff
    hex ff ff ffff
CruiseRestbl_RESTBL:
    chem 0 n CruiseRestbl SetSpeed_-_Car_Speed 0 128 in Vac_Pulse_Width/11ms 0 256 out RESTBL Cruise_Control_stuff
	chem2 table RESTBL use "Set Speed - Car Speed" 0 128 mph use Vacuum 0 2.8 Pulsewidth "Added to SETPW when Resume is pressed"
    db  3
    db  $00, $00, $00, $80
    db  $14, $0a, $00, $1c
    db  $6e, $14
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
CruiseThrottleGain_THGAIN:
    chem 0 n CruiseThrottleGain Throttle_Position 0 256 in THGAIN 0 256 out THGAIN Cruise_Control_stuff
    db  2
    db  $00, $08, $00, $00
    db  $11, $20
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
CruiseSpeedGain_SPGAIN:
    chem 0 n CruiseSpeedGain Vehicle_Speed 0 128 in Speed_Gain 0 256 out SPGAIN Cruise_Control_stuff
    db  3
    db  $3c, $14, $ff, $9a
    db  $6e, $00, $00, $9a
    db  $aa, $24
    hex ff ff ffff
    hex ff ff ffff
    hex ff ff ffff
VNTAB1_LimitToAMaxVent:
    db  $ff
VNTAB2_LimitToHalfMaxVent:
    db  $80
SETIME:
    db  $37      ; no set up pulse when SETIME < DecelTimer < SETIM1
SETIM1:
    db  $52
ACLRST_DeltaSpeedToResetAccFlag:
    db  $30
CLOTHR_DontVentIfAlreadyClosed:
    db  $03  ; This is to take up slack in the cruise control linkage
MAXTHR_DontVacIfAlreadyFullThrottle:
    db  $50
CruiseMaxSetSpeed_MAXSET:
    chem 6 n CruiseMaxSetSpeed Speed 0 128 mph Y 0 256 out MAXSET Maximum_cruise_control_set_speed_(mph*2)._Stock_value_is_xAA,_which_is_170,_divided_by_2_is_85_mph. 85_mph_(stock) aa 90_mph b4 95_mph be 100_mph c8 110_mph dc 127_mph fe Unused 0 Unused 0
    chem2 byte MAXSET simple use "Vehicle Speed" 0 128 MPH choice "85 MPH" aa choice "90 MPH" b4 choice "95 MPH" be choice "100 MPH" c8 choice "110 MPH" dc choice "127 MPH" fe "Maximum Cruise Control set speed. Stock is 85 MPH."
    db  $f0
CruiseMinSetSpeed_MINSET:
    chem 6 n CruiseMinSetSpeed Speed 0 128 mph Y 0 256 out MINSET Minimum_cruise_control_set_speed._Stock_value_x3c_which_is_60,_divided_by_2_is_30mph. 30_mph_(stock) 3c 20_mph 28 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0 Unused 0
    chem2 byte MINSET use "Vehicle Speed" 0 128 MPH choice "30 MPH" 3c choice "20 MPH" 28 "Minimum Cruise Control set speed"
    db  $3c

; SCNV values are to cancel cruise if you press the clutch
SCNVHI_RevChangeToSetBrakeFlag:
    db  $f1
SCNVLO_RevChangeToSetBrakeFlag:
    db  $52

CruiseMinimumCruiseSpeed_LOSPCO:
    chem 4 n CruiseMinimumCruiseSpeed Speed 0 65535 something Y 0 256 out LOSPCO Cruise_Control_low_speed_cutout_in_MPH_*_64._Stock_is_0640
    chem2 word LOSPCO "Cruise Control minimum set speed in MPH * 64"
    dw  $0640

CopyrightInfo:
    db  "COPR. 1985 CHRYSLER CORP."
    db  0
    db  "MODS AND RELOC COPR. 2007 GEOFF ALLAN."

        ; Begin code:
        org Z51Mapping + $900   ; this org is NOT required.

ResetStartup subroutine
        ldd  #$83ff         ; load d (a&b) with value -10000011 11111111-
        ; a = $83 : Engine Not Running, fuel starting, spark starting
        ; a = $83 : b82_EngineNotRunning | b82_SparkStartMode | b82_bit0
        sta  BitFlags82     ; store a into memory
        stb  Z83_PortAdata  ; store b into memory
        stb  Z83_PortA_copy ; store b into memory
        ldd  #$6963         ; load d (a&b) with value -01101001 01100011-
        std  timerControl1  ; store d (a&b) into memory

        ; turn on power loss light and knock kill, turn off injectors
        lda  #$a0           ; load a with value -10100000-
        ora  ioPort1data    ; OR a with memory contents
        ; leave inj 3-4 on if already on?
        anda #$fb           ; AND a with value -11111011-
        sta  ioPort1data    ; store a into memory

        lda  timerStatus    ; load a with memory contents
        ldd  counterHi      ; load d (a&b) with memory contents
        inca                ; increment a
        std  ocr3Hi         ; store d (a&b) into memory
        cli                 ; enable interrupts (clear)
        ldd  #$060a         ; * 06 = 976.6 baud, 0a = enable Tx & Rx
        std  sciModeControl ; store d (a&b) into memory
        lda  sciTransceiveCtrl    ; load a with memory contents
        lda  #$88           ; load a with value -10001000-
        sta  sciTxData      ; store a into memory

;        ldx  #RomBegin      ; load index with value
;        clrb                ; b = 0
;ChecksumLoop:
;        addb $00,x          ; * Add bytes to calc checksum
;        inx                 ; increment index (x=x+1)
;        bne  ChecksumLoop   ; branch if not equal (not zero)
;        subb ChecksumFixup  ; subtract memory contents from b (data is 41)
;        bra  ChecksumFinished    ; branch
;LC692:
; UNKNOWN from $c692 to $c699
; Starts with:  ce 00 46 c6 02 bd c9 93 ;c692 "N.FF.=I."
;
;ChecksumFinished:

        ldd  ErrorCodeBitMask1    ;  (data is ffbf)
        anda ErrorCodes1          ; logical and a with memory contents
        andb ErrorCodes2
        std  ErrorCodes1          ; store d (a&b) into memory
        ldd  ErrorCodeBitMask2    ;  (data is feff)
        anda ErrorCodes3          ; logical and a with memory contents
        andb ErrorCodes4
        std  ErrorCodes3          ; store d (a&b) into memory
        ldx  #LowestHistoricalTPS         ; load index with value
        ldd  #$5500         ; load d (a&b) with value -01010101 00000000-
        psha                ; push a onto stack
        deca                ; decrement a
        sec                 ; set carry
L1004:  rolb                ; rotate b left through carry
        inx                 ; increment index (x=x+1)
        cpx  #ErrorCodes1   ;  -00000000 01001010-
        beq  L1000          ; branch if equal (zero)
L1005:  bitb $04,x
        bne  L1001          ; branch if not equal (not zero)
        bra  L1002          ; branch

L1001:  psha                ; push a onto stack
L1002:  deca                ; decrement a
        bita #$0f           ;  -00001111-
        bne  L1003          ; branch if not equal (not zero)
        suba #$09           ;  -00001001-
L1003:  lslb
        bcs  L1004          ; branch if lower
        bra  L1005          ; branch

L1000:  tsx                 ; transfer stack to index (x=sp+1)
        stx  Scratch_bb     ; store index into memory
        dex                 ; decrement index (x=x-1)
L1008:  inx                 ; increment index (x=x+1)
        lda  L00C2          ; load a with memory contents
        adda #ucErrorCodeNumberWait     ;
        jsr  L1006          ; call subroutine
        lda  $00,x          ; load a with memory at index + value
        lsra
        lsra
        lsra
        lsra
        jsr  L1007          ; call subroutine
        lda  L00C2          ; load a with memory contents
        adda #ucErrorCodeNibbleWait     ; pause between digits
        jsr  L1006          ; call subroutine
        lda  $00,x          ; load a with memory at index + value
        anda #$0f           ; AND a with value -00001111-
        jsr  L1007          ; call subroutine
        lda  $00,x          ; load a with memory at index + value
        cmpa #$55           ; compare a with value -01010101-
        bne  L1008          ; branch if not equal (not zero)
L1012:  sei                 ; disable interrupts (set)
        lda  BitFlags81     ; load a with memory contents
        ora  #b81_LowTpsUpdated      ;  -01000000-
        sta  BitFlags81     ; store a into memory
        cli                 ; enable interrupts (clear)
WaitForInterrupt:
        wai                 ; wait for interrupt
        bra  WaitForInterrupt    ; branch

L1007:  sta  L00C0          ; store a into memory
L1010:  lda  #$20           ; load a with value -00100000-
        sei                 ; disable interrupts (set)
        ora  ioPort1data    ; OR a with memory contents
        sta  ioPort1data    ; * turn power loss lamp ON
        cli                 ; enable interrupts (clear)
        jsr  L1009          ; call subroutine
        lda  #$df           ; load a with value -11011111-
        sei                 ; disable interrupts (set)
        anda ioPort1data    ; logical and a with memory contents
        sta  ioPort1data    ; * turn power loss lamp OFF
        cli                 ; enable interrupts (clear)
        jsr  L1009          ; call subroutine
        dec  L00C0          ; decrement memory contents
        bne  L1010          ; branch if not equal (not zero)
        rts                 ; return from subroutine

L1009:  lda  L00C2          ; load a with memory contents
        adda #ucErrorCodeDigitFlash     ; wait between numbers - per digit
L1006:  cmpa L00C2          ; compare a with memory contents
        beq  L1011          ; branch if equal (zero)
        ldb  BitFlags81     ; load b with memory contents
        bitb #b81_bit5      ;  -00100000-
        beq  L1006          ; branch if equal (zero)
        lds  #$00ff         ;  -00000000 11111111-
        bra  L1012          ; branch

L1011:  rts                 ; return from subroutine
;

MM_IncrementSlowCounter subroutine
        lda  Counter_MainLoop1   ; load a with memory contents
        bne  L1013          ; branch if not equal (not zero)
        ldx  #ErrorTimer    ; load index with value
        ldd  ErrorTimer     ; load d (a&b) with memory contents
L1016:  tsta                ; test a
        beq  L1014          ; branch if equal (zero)
        suba #$20           ;  -00100000-
        bita #$e0           ;  -11100000-
        bne  L1015          ; branch if not equal (not zero)
        clra                ; a = 0
L1015:  sta  $00,x
L1014:  inx                 ; increment index (x=x+1)
        tba                 ; a = b
        cpx  #ErrorTimer + 2
        bne  L1016          ; branch if not equal (not zero)
L1013:  rts                 ; return from subroutine

LC74C   subroutine
        ldb  L004E          ; load b with memory contents
        cmpb #$14           ;  -00010100-
        blo  L1826          ; branch if lower
        ldx  #FaultCode1    ; load index with value
        clrb                ; b = 0
L1827:  lda  $00,x          ; load a with memory at index + value
        anda $04,x
        sta  $04,x
        stb  $00,x
        inx                 ; increment index (x=x+1)
        cpx  #ErrorCodes1   ;  -00000000 01001010-
        bne  L1827          ; branch if not equal (not zero)
L1826:  incb
        stb  L004E          ; store b into memory
        rts                 ; return from subroutine

MM_TestCoolantTemp subroutine
        lda  BitFlags82     ; load a with memory contents
        bmi  L1017          ; branch if b82_EngineNotRunning set - Engine Not Running (hi bit 1)
        lda  BitFlags80     ; load a with memory contents
        bita #b80_BadCoolantTemp      ;  -00000010-
        bne  L1017          ; branch if not equal (not zero)
        lda  FanOnCount     ; load a with memory contents
        beq  L1017          ; branch if equal (zero)
        ldb  Counter_MainLoop1    ; load b with memory contents
        andb #$fc           ;  -11111100-
        bne  L1017          ; branch if not equal (not zero)
        ldb  CoolantTemp    ; load b with memory contents
        cmpb L1018          ; compare b with memory contents (data is c7)
        bhs  L1019          ; branch if greater or equal
        ldb  Counter_EngineRunningTime ; load b with memory contents
        incb
        bne  L1017          ; branch if not equal (not zero)
        ldb  VehicleSpeed_W ; load b with memory contents
        cmpb #$07           ;  -00000111-
        blo  L1017          ; branch if lower
        inca                ; increment a
        beq  L1020          ; branch if equal (zero)
        cmpa L1021          ; compare a with memory contents (data is ae)
        blo  L1022          ; branch if lower
L1020:  ldx  #FaultCode4    ; load index with value
        ldb  #$02           ; load b with value -00000010-
        jsr  SetNonCriticalFaultCode    ; call subroutine
L1019:  clra                ; a = 0
L1022:  sta  FanOnCount     ; store a into memory
L1017:  rts                 ; return from subroutine
;

LC7A2 subroutine
        ldb  RPM_W          ; load b with memory contents
        cmpb #$3f           ;  -00111111-
        lda  #$00           ; load a with value -00000000-
        bhs  L1940          ; branch if greater or equal
        subb #$08           ;  -00001000-
        bcc  L1941          ; branch if greater or equal
        clrb                ; b = 0
L1941:  lda  #$29           ; load a with value -00101001-
        mul                 ; multiply (d=a*b)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        lslb
        adca LowestSessionTPS
        tab                 ; b = a
        lda  TpsVolts            ; load a with memory contents
        adda #$0b           ;  -00001011-
        sba                 ; subtract (a=a-b)
        bcc  L1940          ; branch if greater or equal
        clra                ; a = 0
L1940:  ldx  #Table75_FromTPS    ; load index with value
        jsr  Lookup5ByteTable    ; call subroutine
        tba                 ; a = b
        adda L1942          ;  (data is 20)
        rts                 ; return from subroutine

MM_DetermineChargeTempVolts_v    subroutine
        ldb  DefaultAirTemp          ; load b with memory contents (data is ad)
        lda  BitFlags80     ; load a with memory contents
        bita #b80_BadChargeTemp      ;  -00000001-
        bne  L1024          ; branch if not equal (not zero)
        lda  Counter_MainLoop1    ; load a with memory contents
        anda #$07           ; AND a with value -00000111-
        cmpa #$06           ; compare a with value -00000110-
        bne  L1025          ; branch if not equal (not zero)
        lda  RawAirTemp     ; load a with memory contents
        ldb  ChargeTempVolts    ; load b with memory contents
        cmpa MinValidAirTemp    ; compare a with memory contents (data is 03)
        blo  InvalidChargeTemp  ; branch if lower
        cmpa MaxValidAirTemp    ; compare a with memory contents (data is fe)
        bhs  InvalidChargeTemp  ; branch if greater or equal
        sba                 ; subtract (a=a-b)
        bhs  L1026          ; branch if greater or equal
        decb
        bra  L1024          ; branch

L1026:  cmpa #$03           ; compare a with value -00000011-
        blo  L1025          ; branch if lower
        incb
L1024:  stb  ChargeTempVolts    ; store b into memory
L1025:  rts                 ; return from subroutine

InvalidChargeTemp:
        stb  ChargeTempVolts    ; store b into memory
        lda  #$17           ; load a with value -00010111-
        jsr  SetFaultCode   ; call subroutine
        bcc  L1027          ; branch if greater or equal
        lda  #b80_BadChargeTemp      ; load a with value -00000001-
        sei                 ; disable interrupts (set)
        ora  BitFlags80     ; OR a with memory contents
        sta  BitFlags80     ; store a into memory
        cli                 ; enable interrupts (clear)
L1027:  rts                 ; return from subroutine

MM_DetermineCoolantTemp_v    subroutine
        ldb  DEFTMP                     ; load b with memory contents (data is 95)
        lda  BitFlags80                 ; load a with memory contents
        anda #b80_BadChargeTemp | b80_BadCoolantTemp ; AND a with value -00000011-
        cmpa #b80_BadChargeTemp | b80_BadCoolantTemp ; compare a with value -00000011-
        beq  SaveCoolantTemp            ; branch if equal (zero)
        cmpa #b80_BadCoolantTemp        ; compare a with value -00000010-
        bne  CoolantTempValueGood       ; branch if not equal (not zero)
        ; bad coolant temp sensor, approximate with air temp sensor
        lda  RawAirTemp                 ; load a with memory contents
        ldx  #ApproxCoolantTempFromAirTemp_CHGTL1 ; load index with value
        jsr  LookupTable                ; call subroutine
        bra  SaveCoolantTemp            ; branch

CoolantTempValueGood:
        lda  Counter_MainLoop1 ; load a with memory contents
        anda #$07            ; AND a with value -00000111-
        cmpa #$02            ; compare a with value -00000010-
        bne  .Return         ; branch if not equal (not zero)
        lda  RawCoolantTemp  ; load a with memory contents
        cmpa TMPLOW          ; compare a with memory contents (data is 1a)
        blo  CoolantTempInvalid    ; branch if lower
        cmpa TMPHIG          ; compare a with memory contents (data is fd)
        bhs  CoolantTempInvalid    ; branch if greater or equal
GetCoolantTempQuick:
        ldx  #ScaleCoolantTempHot_TMPTL2    ; load index with value
        ldb  ioPort1data     ; load b with memory contents
        bitb #$08            ; * Check Coolant Temp Signal bit (Red 5)
        beq  L1032           ; branch if equal (zero)
        ldx  #ScaleCoolantTempCold_TMPTL1    ; load index with value
        cmpa #$40           ; compare a with value -01000000-
        bhs  L1033          ; branch if greater or equal
ToggleCTSbias:
        sei                 ; disable interrupts (set)
        lda  ioPort1data    ; load a with memory contents
        xora #$08           ; XOR a with value -00001000-
        sta  ioPort1data    ; store a into memory
        cli                 ; enable interrupts (clear)
        rts                 ; return from subroutine

L1032:  cmpa #$dc           ; compare a with value -11011100-
        bhs  ToggleCTSbias  ; branch if greater or equal
L1033:  jsr  LookupTable    ; call subroutine
        ldb  CoolantTemp    ; load b with memory contents
        sba                 ; subtract (a=a-b)
        bcc  L1034          ; branch if greater or equal
        decb
        bra  SaveCoolantTemp    ; branch

L1034:  cmpa #$03           ; compare a with value -00000011-
        blo  .Return        ; branch if lower
        incb
SaveCoolantTemp:
        stb  CoolantTemp    ; store b into memory
.Return rts                 ; return from subroutine

CoolantTempInvalid:
        lda  #$18           ; load a with value -00011000-
        jsr  SetFaultCode   ; call subroutine
        bcc  .Return        ; branch if greater or equal
        lda  #b80_BadCoolantTemp      ; load a with value -00000010-
        sei                 ; disable interrupts (set)
        ora  BitFlags80     ; OR a with memory contents
        sta  BitFlags80     ; store a into memory
        cli                 ; enable interrupts (clear)
        rts                 ; return from subroutine

MM_TestIfBatteryChargingProperly subroutine
        lda  RawBattVoltage ; load a with memory contents
        cmpa #$41           ; * low battery voltage
        blo  .1             ; branch if lower
        cmpa #$7c           ; * goal battery voltage
        blo  .Return        ; branch if lower
        cmpa #$95           ; * high battery voltage
        bhs  .Return        ; branch if greater or equal
.1      lda  BitFlags82     ; load a with memory contents
        bmi  .Return        ; branch if b82_EngineNotRunning set - Engine Not Running (hi bit 1)
        lda  L004F          ; load a with memory contents
        bita #$20           ;  -00100000-
        bne  .Return        ; branch if not equal (not zero)
        ldd  Counter_EngineRunningTime          ; load d (a&b) with memory contents
        cmpa #$15           ; compare a with value -00010101-
        blo  .Return        ; branch if lower
        bitb #$7c           ;  -01111100-
        bne  .Return        ; branch if not equal (not zero)
        lda  #$1b           ; load a with value -00011011-
        jsr  SetFaultCode   ; * low battery voltage
        bcc  .Return        ; branch if greater or equal
        lda  L004F          ; load a with memory contents
        ora  #$20           ;  -00100000-
        sta  L004F          ; store a into memory
.Return
        rts                 ; return from subroutine
;

MM_CheckIfFanIsNeeded subroutine
        lda  Counter_MainLoop1    ; load a with memory contents
        anda #$3f           ; AND a with value -00111111-
        cmpa #$38           ; compare a with value -00111000-
        beq  L1037          ; branch if equal (zero)
        cmpa #$18           ; compare a with value -00011000-
        beq  DetermineBattTemp    ; branch if equal (zero)
        cmpa #$08           ; compare a with value -00001000-
        bne  L1038          ; branch if not equal (not zero)
        lda  BitFlags80     ; load a with memory contents
        lsla                ; logical shift left a
        ora  BitFlags81     ; OR a with memory contents
        bmi  L1038          ; * branch if off idle
        lda  Counter_EngineRunningTime          ; load a with memory contents
        cmpa L1039          ; compare a with memory contents (data is 80)
        blo  L1038          ; branch if lower
        lda  TargetIdleSpeed    ; load a with memory contents
        adda L1040          ;  (data is 0c)
        cmpa RPM_W          ; compare a with memory contents
        bhs  L1038          ; branch if greater or equal
        ldd  VehicleSpeed_W    ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        cmpa L1041          ; compare a with memory contents (data is 12)
        lda  #$1c           ; load a with value -00011100-
        bcs  L1042          ; branch if lower
L1038:  rts                 ; return from subroutine

DetermineBattTemp:
        lda  #$08           ; load a with value -00001000-
        ldb  RawBattTemp    ; load b with memory contents
        cmpb MinValidBattTemp    ; compare b with memory contents (data is 02)
        bls  L1042          ; branch if lower or same
        cmpb MaxValidBattTemp    ; compare b with memory contents (data is fd)
        bhi  L1042          ; branch if higher
        rts                 ; return from subroutine

L1037   subroutine
        lda  CoolantTemp    ; load a with memory contents
        cmpa L1043          ; compare a with memory contents (data is cd)
        blo  L1038          ; branch if lower
        lda  RPM_W          ; load a with memory contents
        cmpa RPMsForBattTempChecking    ; compare a with memory contents (data is 17)
        blo  L1038          ; branch if lower
        lda  Counter_EngineRunningTime  ; load a with memory contents
        cmpa L1045          ; compare a with memory contents (data is ff)
        blo  L1038          ; branch if lower
        lda  Counter_MainLoop1    ; load a with memory contents
        bita #$40           ;  -01000000-
        bne  L1046          ; branch if not equal (not zero)
        lda  BitFlags82     ; load a with memory contents
        bita #b82_DecayingFuelStartToRun      ;  -00001000-
        bne  L1047          ; branch if start-run fuel decay in progress (not zero)
        lda  BitFlags81     ; load a with memory contents
        bita #b81_bit4      ;  -00010000-
        bne  L1048          ; branch if not equal (not zero)
        lda  RawO2Sensor    ; load a with memory contents
        cmpa O2MIDL_O2LowLimitForMidLevelDetermination    ; compare a with memory contents (data is 12)
        blo  L1048          ; branch if lower
        cmpa O2MIDH_O2HighLimitForMidLevelDetermination    ; compare a with memory contents (data is 1c)
        bhs  L1048          ; branch if greater or equal
        ldb  BitFlags80     ; load b with memory contents
        bitb #b80_bit5      ;  -00100000-
        bne  L1048          ; branch if not equal (not zero)
        lda  FuelSubtractedFromStartToRun          ; load a with memory contents
        inca                ; increment a
        beq  L1048          ; branch if equal (zero)
        sta  FuelSubtractedFromStartToRun          ; store a into memory
        cmpa #$56           ; compare a with value -01010110-
        blo  L1047          ; branch if lower
        ldb  #$01           ; load b with value -00000001-
        ldx  #FaultCode4    ; load index with value
        jsr  SetNonCriticalFaultCode    ; call subroutine
        lda  #b80_bit5      ; load a with value -00100000-
        sei                 ; disable interrupts (set)
        ora  BitFlags80     ; OR a with memory contents
        sta  BitFlags80     ; store a into memory
        cli                 ; enable interrupts (clear)
L1048:  clra                ; a = 0
        sta  FuelSubtractedFromStartToRun          ; store a into memory
L1047:  rts                 ; return from subroutine

L1046:  lda  #$05           ; load a with value -00000101-
        ldb  TargetBattVoltage    ; load b with memory contents
        subb RawBattVoltage ; b = b-memory contents
        bcs  L1047          ; branch if lower
        cmpb #$10           ;  -00010000-
        blo  L1047          ; branch if lower
L1042:  sta  Scratch_ba     ; store a into memory
        jsr  L1049          ; call subroutine
        ldb  $00,x          ; load b with memory at index + value
L1050:  lsrb                ; logical shift right b
        deca                ; decrement a
        bne  L1050          ; branch if not equal (not zero)
        bcc  SetFaultCode2  ; branch if greater or equal
        clc                 ; clear carry
        rts                 ; return from subroutine

; For setting fault code:
;   $16 = bad TPS
;   $17 = charge temp volts
;   $18 = bad coolant temp

SetFaultCode subroutine
        sta  Scratch_ba     ; store a into memory
SetFaultCode2:
        ldx  #ErrorTimer    ; load index with value
        ldd  $00,x
        anda #$1f           ; AND a with value -00011111-
        cmpa Scratch_ba     ; compare a with memory contents
        beq  L1052          ; branch if equal (zero)
        inx                 ; increment index (x=x+1)
        andb #$1f           ;  -00011111-
        cmpb Scratch_ba
        beq  L1052          ; branch if equal (zero)
        tstb
        beq  L1052          ; branch if equal (zero)
        dex                 ; decrement index (x=x-1)
        tsta                ; test a
        bne  L1053          ; branch if not equal (not zero)
L1052:  ldb  $00,x          ; load b with memory at index + value
        addb #$20           ;  -00100000-
        bcs  L1054          ; branch if lower
        orb  Scratch_ba
        stb  $00,x
L1053:  clc                 ; clear carry
        rts                 ; return from subroutine

L1054:  clr  $00,x
        jsr  L1049          ; call subroutine
        ldb  Scratch_ba     ; load b with memory contents
        cmpb #$1f           ;  -00011111-
        bhi  L1055          ; branch if higher
        clrb                ; b = 0
        sec                 ; set carry
L1056:  rolb                ; rotate b left through carry
        deca                ; decrement a
        bne  L1056          ; branch if not equal (not zero)
SetNonCriticalFaultCode:
        orb  $00,x
        stb  $00,x
        orb  $04,x
        stb  $04,x
L1055:  sec                 ; set carry
        rts                 ; return from subroutine

L1049:  ldx  #L0041 - 1     ; load index with value
        lda  Scratch_ba     ; load a with memory contents
L1057:  inx                 ; increment index (x=x+1)
        suba #$08           ;  -00001000-
        bhi  L1057          ; branch if higher
        adda #$08           ;  -00001000-
        rts                 ; return from subroutine

MM_TestForFaultyMAP   subroutine
        lda  BitFlags7f             ; load a with memory contents
        bita #b7f_MapOutOfRange     ;  -01000000-
        beq  L1828                  ; branch if equal (zero)
        lda  #$1d                   ; load a with value -00011101-
        jsr  SetFaultCode           ; call subroutine
        sei                         ; disable interrupts (set)
        lda  BitFlags80             ; load a with memory contents
        bcc  L1829                  ; branch if greater or equal
        ora  #b80_BadMAP            ;  -10000000-
        sta  BitFlags80             ; store a into memory
L1829:  lda  BitFlags7f             ; load a with memory contents
        anda #<~b7f_MapOutOfRange   ; AND a with value -10111111-
        sta  BitFlags7f             ; store a into memory
        cli                         ; enable interrupts (clear)
L1828:  rts                         ; return from subroutine


CheckMapSensor subroutine
        lda  Z85_ADConverter7    ; MAP Sensor
        ldb  LowestSessionTPS    ; load b with memory contents
        addb L1830          ;  (data is 10)
        cmpb TpsVolts
        blo  L1831          ; branch if lower
        cmpa MaximumAllowableMAP    ; compare a with memory contents (data is fc)
        bhs  MapAppearsToBeFaulty    ; branch if greater or equal
L1831:  ldx  RPM_W
        cpx  #$1770         ;  -00010111 01110000-
        bhs  L1832          ; branch if greater or equal
        cmpa MinimumAllowableMAP ; compare a with memory contents (data is 01)
        bhs  L1832          ; branch if greater or equal
MapAppearsToBeFaulty:
        ldb  BitFlags7f     ; load b with memory contents
        orb  #b7f_MapOutOfRange      ;  -01000000-  Bad Map?
        stb  BitFlags7f     ; store b into memory
L1832:  sta  Scratch_bb     ; store a into memory
        ldb  L1058          ; load b with memory contents (data is 07)
        bpl  L1059          ; branch if plus (hi bit 0)
        clr  Scratch_bb     ; zero byte at memory location
L1059:  mul                 ; multiply (d=a*b)
        adda Scratch_bb
        bcc  L1060          ; branch if greater or equal
        lda  #$ff           ; load a with value -11111111-
L1060:  addd Unknownword    ;  (data is 03)
        bcc  L1061          ; branch if greater or equal
        lda  #$ff           ; load a with value -11111111-
L1061:  rts                 ; return from subroutine


MM_CheckThrottle_v   subroutine
    ; 89: THROTTLE_CHECK_ROUTINE
        lda  Counter_DistTimerOverflow ; load a with memory contents
        cmpa L1834          ; compare a with memory contents (data is 10)
        bhs  .return        ; branch if greater or equal (very very low RPMs)
        lda  BitFlags82     ; load a with memory contents
        bmi  L1836          ; branch if b82_EngineNotRunning set - Engine Not Running (hi bit 1)
        ldx  Counter_EngineRunningTime
        cpx  TimeAfterStartupToStopCheckingThrottle          ;  (data is 002d)
        bhs  .return        ; branch if greater or equal
L1836:  lda  LowestSessionTPS    ; load a with memory contents
        adda L1830          ;  (data is 10)
        cmpa TpsVolts            ; compare a with memory contents
        blo  L1838          ; branch if lower
        lda  L0070          ; load a with memory contents
        suba L0091
        bcc  L1839          ; branch if greater or equal
        nega                ; negate a (set high bit)
L1839:  cmpa L1840          ; compare a with memory contents (data is 02)
        blo  .return        ; branch if lower
L1838:  sei                 ; disable interrupts (set)
        lda  BitFlags80     ; load a with memory contents
        anda #<~b80_BadMAP  ; clear bit
        sta  BitFlags80     ; store a into memory
        cli                 ; enable interrupts (clear)
.return rts                 ; return from subroutine

MM_DetermineTransient1 subroutine
        lda  anotherPreviousMAP  ; load a with memory contents
        ldb  MapValue            ; load b with memory contents
        stb  anotherPreviousMAP    ; store b into memory
        sba                 ; subtract (a=a-b)
        ldb  BitFlags84     ; load b with memory contents
        andb #<~b84_bit5    ;  -11011111-
        bcs  L1062          ; branch if lower
        cmpa L1063          ; compare a with memory contents (data is 05)
        blo  L1062          ; branch if lower
        lda  BitFlags82     ; load a with memory contents
        ora  BitFlags81     ; OR a with memory contents
        bmi  L1062          ; branch if b82_EngineNotRunning AND b81_bit7 
        lda  CoolantTemp    ; load a with memory contents
        cmpa MinCoolantTempForTransient    ; compare a with memory contents (data is b1)
        blo  L1062          ; branch if lower
        lda  Counter_EngineRunningTime          ; load a with memory contents
        cmpa MinTimeForTransient    ; compare a with memory contents (data is 02)
        blo  L1062          ; branch if lower
        lda  RPM_W          ; load a with memory contents
        cmpa MinRPMforTransient          ; compare a with memory contents (data is 23)
        blo  L1062          ; branch if lower
        orb  #b84_bit5      ;  -00100000-
L1062:  stb  BitFlags84     ; store b into memory
        rts                 ; return from subroutine

CalculateHeatSoakAdjustment subroutine
        lda  CoolantTemp    ; load a with memory contents
        cmpa RUNLMT_MaxAllowableRunTemp  ; compare a with memory contents (data is 95)
        bhs  L1065          ; branch if greater or equal
        lda  VerySlowCounter2point6seconds      ; load a with memory contents
        ldx  #RUNTMP_RuntmpModifierEngineTemp   ; load index with value
        jsr  LookupTable    ; call subroutine
        lda  CoolantTemp    ; load a with memory contents
        aba                 ; a=a+b
        ldb  RUNLMT_MaxAllowableRunTemp    ; load b with memory contents (data is 95)
        bcs  L1066          ; branch if lower
        cba
        bcs  L1065          ; branch if lower
L1066:  tba                 ; a = b
L1065:  rts                 ; return from subroutine

MM_DetermineTransient3 subroutine
        lda  L1067          ; load a with memory contents (data is 28)
        ldb  MapValue            ; load b with memory contents
        cmpb MapAverageValue
        blo  L1068          ; branch if lower
        ldx  #AETIME_TransientMapEnrichTimeConstant    ; load index with value
        lda  CoolantTemp    ; load a with memory contents
        jsr  LookupTable    ; call subroutine
L1068:  sta  Factor_LeanoutOrEnrichTime_MAP   ; store a into memory
        lda  CoolantTemp    ; load a with memory contents
        ldx  #THRTCN_TransientThrottleEnrichTime    ; load index with value
        jsr  LookupTable    ; call subroutine
        sta  Factor_LeanoutOrEnrichTime_TPS  ; store a into memory
        rts                 ; return from subroutine

;  ______________________
; |                      \
; |   Voltage Regulation |
; \______________________|

MM_ControlBatteryVoltage subroutine
        ldb  RawBattTemp    ; Get the battery temp
        lda  #$e3           ; Load a default target voltage

        cmpb MinValidBattTemp    ; compare b with memory contents (data is 02)
        bcs  .3                  ; branch if lower
        cmpb MaxValidBattTemp    ; compare b with memory contents (data is fd)
        bhi  .3                  ; branch if higher

        lda  #$20           ; load a with value -00100000-
        mul                 ; multiply (d=a*b)
        adca #$d1           ;  -11010001-

        cmpa #$ec           ; compare a with Maximum allowed target voltage
        bcs  .2             ; branch if lower
        lda  #$ec           ; use Maximum allowed target voltage

.2      cmpa #$d8           ; compare a with Minimum allowed target voltage
        bcc  .3             ; branch if greater or equal
        lda  #$d8           ; use Minimum allowed target voltage

.3      sta  TargetBattVoltage  ; store a into memory

        lda  Timer_VoltageRegSignal ; load a with memory contents
        cmpa #$f0           ; compare a with value -11110000-
        blo  L1070          ; branch if lower
        ldb  Counter_MainLoop1   ; load b with memory contents
        bitb #$7c           ;  -01111100-
        bne  L1070          ; branch if not equal (not zero)
        suba #$60           ;  -01100000-
        sta  Timer_VoltageRegSignal ; store a into memory
        ldb  BitFlags82     ; load b with memory contents
        bmi  .return        ; branch if b82_EngineNotRunning set
        lda  Counter_EngineRunningTime  ; load a with memory contents
        cmpa #$10           ; compare a with value -00010000-
        blo  L1070          ; branch if lower
        lda  #$0b           ; load a with value -00001011-
        jsr  L1042          ; call subroutine
L1070:  clrb                ; b = 0
        lda  RawBattVoltage ; load a with memory contents
        suba TargetBattVoltage
        bcs  L1072          ; branch if lower
        cmpa #$10           ; compare a with value -00010000-
        blo  L1072          ; branch if lower
        lda  BitFlags80     ; load a with memory contents
        bita #b80_bit3      ;  -00001000-
        bne  .return        ; branch if not equal (not zero)
        lda  Counter_MainLoop1   ; load a with memory contents
        bita #$7c           ;  -01111100-
        bne  .return        ; branch if not equal (not zero)
        lda  #$06           ; load a with value -00000110-
        jsr  SetFaultCode   ; call subroutine
        bcc  .return        ; branch if greater or equal
        rorb
L1072:  sei                 ; disable interrupts (set)
        lda  BitFlags80     ; load a with memory contents
        ora  #b80_bit3      ;  -00001000-
        tstb
        bmi  L1073          ; branch if minus(hi bit 1)
        anda #<~b80_bit3    ; AND a with value -11110111-
L1073:  sta  BitFlags80     ; store a into memory
        cli                 ; enable interrupts (clear)
.return rts                 ; return from subroutine

DetermineVoltage    subroutine
    ; we get here from the frequently executed sensor scan routine
    ; a contains the raw battery voltage
        cmpa #$41           ; compare a with value -01000001-
        blo  L1074          ; branch if lower
        ldb  L004F          ; load b with memory contents
        cmpa #$7c           ; compare a with value -01111100-
        blo  L1075          ; branch if lower
        cmpa #$95           ; compare a with value -10010101-
        blo  L1076          ; branch if lower
L1075:  andb #$df           ;  -11011111-
        stb  L004F          ; store b into memory
        bra  L1074          ; branch

L1076:  bitb #$20           ;  -00100000-
        beq  L1074          ; branch if equal (zero)
        clra                ; a = 0
L1074:  sta  $00,x
        ldb  BitFlags82     ; load b with memory contents
        bmi  L1077          ; branch if b82_EngineNotRunning set
        cmpa #$41           ; compare a with value -01000001-
        bhs  L1078          ; branch if greater or equal
        ldb  Counter_MainLoop1    ; load b with memory contents
        bitb #$06           ;  -00000110-
        bne  L1077          ; branch if not equal (not zero)
L1078:  ldb  ioPort1data    ; load b with memory contents
        bitb #$10           ; * check voltage reg signal
        beq  L1079          ; branch if equal (zero)
        adda #$01           ;  -00000001-
        bcs  L1077          ; branch if lower
L1079:  cmpa TargetBattVoltage    ; compare a with memory contents
        bhi  L1077          ; branch if higher
        ldb  #$80           ; load b with value -10000000-
        lda  Timer_VoltageRegSignal          ; load a with memory contents
        cmpa #$f0           ; compare a with value -11110000-
        bhs  L1080          ; branch if greater or equal
        anda #$0f           ; AND a with value -00001111-
        cmpa #$01           ; compare a with value -00000001-
        bne  L1081          ; branch if not equal (not zero)
        dec  Timer_VoltageRegSignal          ; decrement memory contents
        clrb                ; b = 0
        bra  L1080          ; branch

L1081:  lda  ioPort1data    ; load a with memory contents
        bita #$10           ; * check voltage reg signal
        beq  L1080          ; branch if equal (zero)
        lda  Z83_PortBdata  ; load a with memory contents
        bita #$10           ; * Test for alternator fault
        beq  L1082          ; branch if equal (zero)
        lda  TargetBattVoltage    ; load a with memory contents
        suba RawBattVoltage
        cmpa #$04           ; compare a with value -00000100-
        blo  L1082          ; branch if lower
        lda  Timer_VoltageRegSignal          ; load a with memory contents
        bita #$0f           ;  -00001111-
        bne  L1083          ; branch if not equal (not zero)
        adda #$10           ;  -00010000-
        anda #$f0           ; AND a with value -11110000-
        ora  #$03           ;  -00000011-
L1083:  deca                ; decrement a
        sta  Timer_VoltageRegSignal          ; store a into memory
        bra  L1080          ; branch

L1077:  clrb                ; b = 0
L1082:  clra                ; a = 0
        sta  Timer_VoltageRegSignal          ; store a into memory
L1080:  lda  #$10           ; load a with value -00010000-
        sei                 ; disable interrupts (set)
        ora  ioPort1data    ; turn on voltage reg bit
        tstb
        bmi  L1084          ; branch if minus(hi bit 1)
        anda #$ef           ; AND a with value -11101111- (turn OFF voltage reg bit)
L1084:  sta  ioPort1data    ; store a into memory
        cli                 ; enable interrupts (clear)
        rts                 ; return from subroutine

RegulateVoltage subroutine
        ldb  L004F          ; load b with memory contents
        bitb #$20           ;  -00100000-
        beq  .1             ; branch if equal (zero)
        clra                ; a = 0
.1      sta  $00,x
        ldb  BitFlags82     ; load b with memory contents
        bmi  VoltageRegSignalOff    ; branch if b82_EngineNotRunning set
        cmpa #$41           ; compare a with value -01000001-
        bhs  .2             ; branch if greater or equal
        ldb  Counter_MainLoop1 ; load b with memory contents
        bitb #$06           ;  -00000110-
        bne  VoltageRegSignalOff    ; branch if not equal (not zero)
.2      ldb  ioPort1data    ; load b with memory contents
        bitb #$10           ; * check voltage reg signal
        beq  .3             ; branch if equal (zero)
        adda #$01           ;  -00000001-
        bcs  VoltageRegSignalOff  ; branch if lower
.3      cmpa TargetBattVoltage    ; compare a with memory contents
        bls  VoltageRegSignalOn   ; branch if lower or same

VoltageRegSignalOff:
        clra                ; a = 0
        sta  Timer_VoltageRegSignal   ; store a into memory
        lda  ioPort1data    ; load a with memory contents
        anda #$ef           ; AND a with value -11101111-
        sta  ioPort1data    ; * Turn off Voltage Reg Signal
        rts                 ; return from subroutine

VoltageRegSignalOn:
        lda  ioPort1data    ; load a with memory contents
        ora  #$10           ;  -00010000-
        sta  ioPort1data    ; * Turn on Voltage Reg Signal
        rts                 ; return from subroutine

SetDelayedFaultCode subroutine
        jsr  SetFaultCode   ; call subroutine
        bcs  .1             ; branch if lower
        lda  InverseRPM_B   ; load a with memory contents
        cmpa #$02           ; compare a with value -00000010-
        blo  .Return        ; branch if lower
        jsr  SetFaultCode2  ; call subroutine
        bcc  .Return        ; branch if greater or equal
.1      lda  BitFlags81     ; load a with memory contents
        ora  #b81_bit5      ;  -00100000-
        sta  BitFlags81     ; store a into memory
.Return rts                 ; return from subroutine

MM_ConfirmO2GoodAndAdjustKnockRetard subroutine
        ldb  BitFlags81     ; load b with memory contents
        bitb #b81_bit5      ;  -00100000-
        ; check if O2 sensor is stuck
        bne  L1090          ; branch if not equal (not zero)
        ldb  L004F          ; load b with memory contents
        bitb #$04           ;  -00000100-
        bne  L1090          ; branch if not equal (not zero)
        ldb  BitFlags83     ; load b with memory contents
        bitb #b83_bit2      ;  -00000100-
        ; check if O2 sensor is still too cold to return valid data
        beq  L1091          ; branch if equal (zero)
        ; otherwise, trigger the O2 low fault code?
        lda  #$14           ; load a with value -00010100-
        jsr  SetDelayedFaultCode    ; call subroutine
L1091:  ldb  BitFlags83     ; load b with memory contents
        bitb #b83_bit3      ;  -00001000-
        ; ? see if we have some throttle
        beq  L1090          ; branch if equal (zero)
        ; otherwise trigger the O2 high fault code?
        lda  #$13           ; load a with value -00010011-
        jsr  SetDelayedFaultCode    ; call subroutine

L1090:  ldb  BitFlags83     ; load b with memory contents
        bitb #b83_bit4      ;  -00010000-
        beq  L1092          ; branch if equal (zero)
        lda  #$09           ; load a with value -00001001-
        jsr  L1042          ; call subroutine
L1092:  lda  BitFlags82     ; load a with memory contents
        bita #b82_bit2      ;  test bit -00000100-
        bne  L1093          ; branch if b82_bit2 set
        lda  BitFlags7f     ; load a with memory contents
        bita #b7f_bit3      ;  -00001000-
        beq  L1093          ; branch if equal (zero)
        lda  #$01           ; load a with value -00000001-
        jsr  L1042          ; call subroutine
L1093:  lda  #$f7           ; load a with value -11110111-
        ldb  #$e3           ; load b with value -11100011-
        sei                 ; disable interrupts (set)
        andb BitFlags83     ; turn off b83_bit4 b83_bit3 b83_bit2
        stb  BitFlags83     ; store b into memory
        anda BitFlags7f     ; turn off b7f_bit3
        sta  BitFlags7f     ; store a into memory
        cli                 ; enable interrupts (clear)
        lda  Counter_MainLoop1    ; load a with memory contents
        bita INCSRT_IncreaseRate    ;  (data is 1f)
        bne  L1094          ; branch if not equal (not zero)
        ldx  #CylinderRetard1         ; load index with value
L1096:  sei                 ; disable interrupts (set)
        lda  $00,x          ; load a with memory at index + value
        tab                 ; b = a
        anda #$c0           ; AND a with value -11000000-
        andb #$3f           ;  -00111111-
        beq  L1095          ; branch if equal (zero)
        decb
L1095:  aba                 ; a=a+b
        sta  $00,x
        cli                 ; enable interrupts (clear)
        inx                 ; increment index (x=x+1)
        cpx  #CylinderRetard4 ;  -00000000 11010001-
        bls  L1096          ; branch if lower or same
L1094:  rts                 ; return from subroutine

;  ______________________
; |                      \
; |   Timer Interrupt    |
; \______________________|

; Timer 1 is used for Injectors?
; Timer 3 is used for "diagnostics" and serial comm

IRQtOut subroutine
        lda  timerStatus                ; load a with memory contents $19
        bita #$08                       ;  -00001000- Output Compare Flag 1 - OCreg1 matches free-running counter
        bne  .timer1done                ; branch if set (timer one has completed)

        ldb  BitFlags82                 ; load b with memory contents
        lsrb                            ; logical shift right b
        bcc  .1                         ; branch if b82_bit0 is clear  (O2 sensor bad?)

        bita #$20                       ;  -00100000- Output Compare Flag 3 - OCreg3 matches free-running counter
        beq  Injector34Off              ; branch if not set (zero)
        jsr  HandleOtherTimerInterrupts ; call subroutine   call if set.
        rti                             ; return from interrupt

; **********
Injector34Off:
        lda  ioPort1data                ; load a with memory contents
        anda #$fb                       ; AND a with value -11111011-
        sta  ioPort1data                ; * Turn off Injector 3-4

        ; Turn OFF Output Compare Interrupt 2
        lda  timerControl2              ; load a with memory contents
        anda #$ef                       ; AND a with value -11101111-
        sta  timerControl2              ; store a into memory

        lda  ocr2Hi                     ; load a with memory contents
        sta  ocr2Hi                     ; store a into memory
        rti                             ; return from interrupt

; **********
.1      ldx  #ocr3Hi                    ; Get Output Compare Register 3
        ldb  #$fb                       ; load b with value -11111011-
        bita #$20                       ;  -00100000- a is still TimerStatus, bit 5 is Output Compare Flag 3
        bne  L1100                      ; branch if not equal (not zero) if OCF3 is set

        ldx  #ocr2Hi                    ; load index with value
        ldb  #$fd                       ; load b with value -11111101-

; **********
; This is called from CalculateInjectorPulsewidthForEngineRunning as a subroutine:
L1100:  lda  $01,x                      ; load a with memory at index + value
        sta  $01,x
        cmpb #$fd                       ;  -11111101-
        beq  .Isfd                      ; branch if equal (zero)
        ldx  #Bits_WhichInjector        ; load index with value
        bra  .Notfd                     ; branch

.Isfd   ldx  #BitFlags7f                ; load index with value
.Notfd  lda  $00,x                      ; load a with memory at index + value
        anda #b7f_bit0 | b7f_bit1       ; AND a with value -00000011-
        beq  L1103                      ; branch if equal (zero)
        deca                            ; decrement a
        beq  L1104                      ; branch if equal (zero)
        ldb  $00,x                      ; load b with memory at index + value
        andb #%11111100                 ; all except bottom 2 bits -11111100-
        aba                             ; a=a+b  add bottom 2 bits
        sta  $00,x
        rti                             ; return from interrupt

L1104:  lda  $00,x                      ; load a with memory at index + value
        anda #<~[b7f_bit0 | b7f_bit1]   ; zero bottom 2 bits -11111100-
        sta  $00,x
L1103:  andb timerControl1
        stb  timerControl1              ; store b into memory
        rti                             ; return from interrupt

; **********
; Ouput Compare 1 - timer1 has completed
.timer1done
        lda  BitFlags82     ; load a with memory contents
        bita #b82_SparkStartMode      ;  -00000010-
        bne  L1105          ; branch if b82_SparkStartMode set

        bita #b82_bit2      ;  -00000100-
        beq  L1105          ; branch if equal (zero)
        ldb  InverseRPM_B   ; load b with memory contents
        bne  L1105          ; branch if not equal (not zero) Low RPMs
        lda  timerCSR       ; load a with memory contents
        xora #%00000001     ; XOR a with value -00000001-
        sta  timerCSR       ; store a into memory
        lsra
        bcs  L1106          ; branch if lower
        ldd  L00C1          ; load d (a&b) with memory contents
        bra  L1107          ; branch

L1106:  lsra
        bcs  L1108          ; branch if lower
        lda  timerCSR       ; load a with memory contents
        anda #%11111110     ; AND a with value -11111110-
        sta  timerCSR       ; store a into memory
L1108:  ldd  InverseRPM_W          ; load d (a&b) with memory contents
        subd L00C1          ; d = d - memory contents
        bra  L1107          ; branch

L1105:  ldd  #$7000         ; load d (a&b) with value -01110000 00000000-
L1107:  addd ocrHi          ; d = d + memory contents
        std  ocrHi          ; store d (a&b) into memory
        rti                 ; return from interrupt

     .if UseEGRPurge > 0
MM_HandleCanisterPurge_v   subroutine
        ldb  BitFlags81             ; load b with memory contents
        lsrb                        ; logical shift right b
        bcs  L1841                  ; branch if b81_FullThrottle is set

        ldb  L00CA                  ; load b with memory contents
        cmpb L1254                  ; compare b with memory contents (data is 2b)
        bhs  L1841                  ; branch if greater or equal
        ldb  BarometricPressure     ; load b with memory contents
        lda  Z83_PortA_copy         ; load a with memory contents
        bita #PortA_PurgeEGR        ;  -01000000-
        bne  L1842                  ; branch if not equal (not zero)
        subb PURHYS_PurgeHysteresis ; subtract memory contents from b (data is 02)
L1842:  subb PURMAP_Purge           ; subtract memory contents from b (data is 05)
        cmpb MapValue
        blo  L1841                  ; branch if lower
        ldb  BitFlags81             ; load b with memory contents
        bitb #b81_bit4              ;  -00010000-
        bne  L1845                  ; branch if not equal (not zero)
        ldb  Timer_CLTIME_ClosedLoopTimer ; load b with memory contents
        cmpb O2TIME_O2DelayTimeToClosedLoop ; compare b with memory contents (data is 15)
        blo  L1841                  ; branch if lower
L1845:  lda  Z83_PortA_copy         ; load a with memory contents
        ora  #PortA_PurgeEGR        ;  -01000000-
        bra  L1846                  ; branch

L1841:  lda  Z83_PortA_copy         ; load a with memory contents
        anda #<~PortA_PurgeEGR      ; AND a with value -10111111-
L1846:  sta  Z83_PortA_copy         ; store a into memory
        rts                         ; return from subroutine
     .else
     ; Not using cannister purge, using speed sensitive
     ; chin spoiler dropdown. This modification uses two
     ; spare Headlight door motors to raise and lower the
     ; "chin spoiler" which always scrapes. Only lower it above
     ; 20MPH since it's not doing anything below that speed anyway.
     ; Actual numbers: raise below 18 mph, lower above 22 mph to avoid
     ; excessive toggling
     ; For the control bit: 1 = lowered, 0 = raised.
     ; specifically, the DEFAULT position at idle or startup is RAISED
MM_SpeedSensitiveAero subroutine
        lda  Z83_PortA_copy ; load a with memory contents
        bita #PortA_PurgeEGR     ;  -01000000-
        bne  .alreadydown   ; branch if not equal (not zero)
        ; if we are here, the spoiler is up. Check if we need to lower it
        ldd  VehicleSpeed_W
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        cmpa $16            ; 22mph
        bhi  .lowerspoiler  ; branch if higher
        bra  .goback        ; otherwise exit routine
.alreadydown
        ; if we are here, the spoiler is down. Check if we need to raise it
        ldd  VehicleSpeed_W
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        cmpa $12            ; 18mph
        blo  .raisespoiler  ; branch if lower
        bra  .goback        ; otherwise exit routine
.raisespoiler
        lda  Z83_PortA_copy ; load a with memory contents
        anda #<~PortA_PurgeEGR
        bra  .changeit
.lowerspoiler
        lda  Z83_PortA_copy ; load a with memory contents
        ora  #PortA_PurgeEGR     ;  -01000000-
.changeit
        sta  Z83_PortA_copy ; store a into memory
.goback
        rts
     .endif

MM_DetermineBaroRead subroutine
        lda  BitFlags82                 ; load a with memory contents
        bita #b81_bit3                  ; test bit -00001000-
        beq  L1109                      ; branch if beyond start-run fuel decay time (zero)
        lda  Z83_PortA_copy             ; load a with memory contents
        ora  #PortA_BaroRead            ;  -00000001-
        sta  Z83_PortA_copy             ; store a into memory
        rts                             ; return from subroutine

L1109:  ldb  BitFlags85                 ; load b with memory contents
        bmi  L1110                      ; branch if b85_bit7 is set
        lda  Counter_BaroReadTime       ; load a with memory contents
        tst  Counter_MainLoop1
        bne  L1110                      ; branch if not equal (not zero)
        inca                            ; increment a
        beq  L1110                      ; branch if equal (zero)
        sta  Counter_BaroReadTime       ; store a into memory
L1110:  lda  BitFlags81                 ; load a with memory contents
        bpl  L1111                      ; branch if b81_OffIdle is clear
        lda  Z83_PortA_copy             ; load a with memory contents
        ora  #PortA_BaroRead            ;  -00000001-
        sta  Z83_PortA_copy             ; store a into memory
        clra                            ; a = 0
        sta  BaroReadTimeEngineWithinReadRange    ; store a into memory
        lda  #<~[b85_bit7 | b85_bit6]   ; load a with value -00111111-
        anda BitFlags85                 ; logical and a with memory contents
        sta  BitFlags85                 ; store a into memory
        lda  #<~b82_bit4                ; load a with value -11101111-
        sei                             ; disable interrupts (set)
        anda BitFlags82                 ; logical and a with memory contents
        sta  BitFlags82                 ; store a into memory
        cli                             ; enable interrupts (clear)
        rts                             ; return from subroutine

L1111:  tstb
        bmi  L1112                      ; branch if b81_OffIdle is set
        ldx  RPM_W
        cpx  BRPMHI_BaroReadRPMHigh     ;  (data is 3e80)
        bhi  .Return                    ; branch if higher
        cpx  BRPMLO_BaroReadRPMLow      ;  (data is 1c20)
        blo  .Return                    ; branch if lower
        lda  BaroReadTimeEngineWithinReadRange    ; load a with memory contents
        inca                            ; increment a
        beq  L1114                      ; branch if equal (zero)
        sta  BaroReadTimeEngineWithinReadRange    ; store a into memory
        cmpa BRDLAY_BaroReadThrottleDelay         ; compare a with memory contents (data is 2d)
        blo  .Return                    ; branch if lower

L1114:  lda  Counter_BaroReadTime       ; load a with memory contents
        cmpa BARTIM_BaroReadTime        ; compare a with memory contents (data is 40)
        blo  .Return                    ; branch if lower
        bitb #b81_LowTpsUpdated         ;  -01000000-
        bne  .Return                    ; branch if not equal (not zero)
        lda  Z83_PortA_copy             ; load a with memory contents
        anda #<~PortA_BaroRead          ; AND a with value -11111110- Energize Solenoid
        sta  Z83_PortA_copy             ; store a into memory
        ldb  #b85_bit7                  ; load b with value -10000000-
        bra  L1115                      ; branch

L1112:  inc  Counter_BaroReadTime
        bitb #b85_bit6                  ;  -01000000-
        bne  L1116                      ; branch if not equal (not zero)
        lda  Counter_BaroReadTime       ; load a with memory contents
        cmpa BROPTM_BaroReadOpenTime    ; compare a with memory contents (data is 05)
        blo  .Return                    ; branch if lower
        ldb  #b85_bit6                  ; load b with value -01000000-
        lda  #b82_bit4                  ; load a with value -00010000-
        sei                             ; disable interrupts (set)
        ora  BitFlags82                 ; OR a with memory contents
        sta  BitFlags82                 ; store a into memory
        cli                             ; enable interrupts (clear)
L1115:  orb  BitFlags85
        bra  L1117                      ; branch

L1116:  lda  BitFlags82                 ; load a with memory contents
        bita #b82_bit4                  ;  -00010000-
        bne  L1118                      ; branch if b82_bit4 set 
        lda  Z83_PortA_copy             ; load a with memory contents
        ora  #PortA_BaroRead            ;  -00000001-
        sta  Z83_PortA_copy             ; store a into memory Deenergize Relay
        lda  Counter_BaroReadTime       ; load a with memory contents
        cmpa BRCLTM_BaroReadCloseTime   ; compare a with memory contents (data is 05)
        blo  .Return                    ; branch if lower
        ldb  #<~b85_bit7                ; load b with value -01111111-
        andb BitFlags85
L1117:  stb  BitFlags85                 ; store b into memory
L1118:  clrb                            ; b = 0
        stb  Counter_BaroReadTime       ; store b into memory
.Return:
        rts                             ; return from subroutine

MM_DealWith_AC_Cutout subroutine
        lda  ACLRPM_DisableACBelowThisRPM                      ; load a with memory contents (data is a2)
        sei                             ; disable interrupts (set)
        sta  Scratch_bb                 ; store a into memory
        ldd  TargetIdleSpeed            ; load d (a&b) with memory contents
        jsr  mul16bit                   ; call subroutine
        cli                             ; enable interrupts (clear)
        std  Scratch_b6                 ; store d (a&b) into memory
        lda  Z83_PortA_copy             ; load a with memory contents
        ldb  Counter_EngineRunningTime  ; load b with memory contents
        cmpb ACCTIM_ACDelayFromStartRun                      ; compare b with memory contents (data is 04)
        blo  L1121                      ; branch if lower
        ldb  BitFlags85                 ; load b with memory contents
        bitb #b85_bit3                  ; test bit -00001000-
        bne  L1121                      ; branch if not equal (not zero)
        ldb  TpsVolts                   ; load b with memory contents
        bita #PortA_ACclutch            ; test bit -00010000- Turn Off Relay
        beq  L1122                      ; branch if equal (zero)
        addb #$03                       ;  -00000011-
L1122:  anda #<~PortA_ACclutch          ; turn off bit -11101111- Turn On Relay
        subb LowestSessionTPS           ; b = b-memory contents
        bcs  L1123                      ; branch if lower
        cmpb ACOUT_ThrottleForACCutout              ; compare b with memory contents (data is 85)
        bhs  L1121                      ; branch if greater or equal

    ; matches location C999 in 89 code:
L1123:  ldx  RPM_W
        cpx  ACCRPM_DontEngageACAboveThisRPM         ;  (data is 3e80)
        bhi  L1125                      ; branch if higher
        cpx  TargetIdleSpeed
        bhs  L1126                      ; branch if greater or equal
        cpx  Scratch_b6
        bhs  L1125                      ; branch if greater or equal
L1121:  ora  #PortA_ACclutch            ; turn on bit -00010000-
L1126:  sta  Z83_PortA_copy             ; store a into memory  - A/C clutch
L1125:  rts                             ; return from subroutine

MM_DealWith_ASD_Relay   subroutine
        lda  Z83_PortA_copy ; load a with memory contents
        ldb  BitFlags81     ; load b with memory contents
        bitb #b81_bit5      ;  -00100000-
        bne  L1847          ; branch if not equal (not zero)
        ldx  #$002d         ; load index with value
        lda  CoolantTemp    ; load a with memory contents
        cmpa #$8f           ; compare a with value -10001111-
        bhs  L1848          ; branch if greater or equal
        ldx  #$005b         ; load index with value
L1848:  ldb  #$10           ; load b with value -00010000-
        lda  BitFlags81     ; load a with memory contents
        bita #b81_bit1      ;  -00000010-
        bne  L1849          ; branch if not equal (not zero)
        lda  RawBattVoltage ; load a with memory contents
        cmpa #$b2           ; compare a with value -10110010-
        bhs  L1850          ; branch if greater or equal
        cpx  #$005b         ;  -00000000 01011011-
        bne  L1850          ; branch if not equal (not zero)
        ldb  #$2e           ; load b with value -00101110-
        bra  L1850          ; branch

L1849:  lda  RawBattVoltage ; load a with memory contents
        cmpa #$b2           ; compare a with value -10110010-
        blo  L1850          ; branch if lower
        ldb  #$0a           ; load b with value -00001010-
L1850:  lda  Z83_PortA_copy ; load a with memory contents
        anda #<~PortA_ASD   ; turn off bit -11111101-
        cmpb Counter_DistTimerOverflow
        bhi  L1851          ; branch if higher
        cpx  Counter_EngineRunningTime
        bhi  L1851          ; branch if higher
L1847:  ora  #PortA_ASD     ; turn on bit -00000010-
L1851:  sta  Z83_PortA_copy ; store a into memory
        lda  BitFlags81     ; load a with memory contents
        ldb  BitFlags82     ; load b with memory contents
        bmi  L1852          ; branch if b82_EngineNotRunning set
        ldb  Counter_EngineRunningTime  ; load b with memory contents
        cmpb #$04           ;  -00000100-
        blo  L1852          ; branch if lower
        ora  #b81_bit1      ;  -00000010-
        sta  BitFlags81     ; store a into memory
L1852:  rts                 ; return from subroutine

;  ______________________
; |                      \
; |   Vehicle Speed      |  INTERRUPT HANDLER
; \______________________|

HandleSpeedSensorInterrupt   subroutine
    ; save the speed/distance sensor time
        ldd  icr2Hi         ; load d (a&b) with memory contents
        std  Scratch_bb     ; store d (a&b) into memory

        lda  ioPort1data    ; load a with memory contents
        lsra
        bcc  L1853          ; branch if NOT a speed/distance sensor pulse

        ; since speed/distance sensor is a reed switch,
        ; it might "bounce". This small delay is a debounce
        ldb  #$05           ; load b with value -00000101-
.debounce
        decb
        bne  .debounce      ; branch if not equal (not zero)

        lda  ioPort1data    ; load a with memory contents
        lsra                ; * Check speed/distance pulse
        ; if the speed/distance sensor is not still on, jump
        bcs  L1855          ; branch if lower

L1853:  ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        subd #$0640         ;  -00000110 01000000-
        lda  BitFlags83     ; load a with memory contents
        bcc  L1856          ; branch if b83_bit7 not set
        bita #b83_bit1      ;  -00000010-
        bne  L1857          ; branch if BitFlags83 bit 1 is set
        ora  #b83_bit1      ;  -00000010-
        bra  L1858          ; set BitFlags83 bit 2, branch

L1857:  anda #%00011111     ; AND a with value -00011111-
        ora  #b83_bit0      ;  -00000001-
        sta  BitFlags83     ; store a into memory
        clra                ; a = 0
        sta  L00DF          ; store a into memory
        sta  L00AF          ; store a into memory
        sta  L0063          ; store a into memory
        ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        addd #$0053         ;  -00000000 01010011-
        std  VehicleSpeed_W ; store d (a&b) into memory
        bra  L1855          ; branch

L1856:  bita #b83_bit1      ; test bit -00000010-
        bne  L1859          ; branch if not equal (not zero)
L1858:  adda #b83_bit5      ;  -00100000-
        bcs  L1860          ; branch if lower
        anda #<~b83_bit1    ; AND a with value -11111101-
        sta  BitFlags83     ; turn off b83_bit1
        bra  L1855          ; branch

L1860:  sta  BitFlags83     ; store a into memory
        ldb  L00DF          ; load b with memory contents
        beq  L1861          ; branch if equal (zero)
        bita #b83_bit0      ;  -00000001-
        bne  L1861          ; branch if not equal (not zero)
        lda  #$3a           ; load a with value -00111010-
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
        cmpa #$03           ; compare a with value -00000011-
        bhs  L1862          ; branch if greater or equal
        lda  #$03           ; load a with value -00000011-
L1862:  tab                 ; b = a
        lda  L00AF          ; load a with memory contents
        suba L00DF
        bcc  L1863          ; branch if greater or equal
        nega                ; negate a (set high bit)
L1863:  sba                 ; subtract (a=a-b)
        bcs  L1864          ; branch if lower
        lda  BitFlags83     ; load a with memory contents
        ora  #b83_bit0      ; set bit -00000001-
        sta  BitFlags83     ; store a into memory
L1861:  ldb  L00AF          ; load b with memory contents
        stb  L00DF          ; store b into memory
L1864:  clra                ; a = 0
        sta  L00AF          ; store a into memory
        ldd  Scratch_bb     ; load d (a&b) with memory contents
        jsr  MM_L1602          ; call subroutine
        lda  L0063          ; load a with memory contents
        lsra
        sta  L0066          ; store a into memory
        clra                ; a = 0
        sta  L0063          ; store a into memory
        ldd  Scratch_bb     ; load d (a&b) with memory contents
        subd Something_W    ; d = d - memory contents
        std  L0067          ; store d (a&b) into memory
L1865:  ldd  Scratch_bb     ; load d (a&b) with memory contents
        std  Something_W    ; store d (a&b) into memory
L1855:  lda  timerStatus    ; load a with memory contents
        lda  icr2Hi         ; load a with memory contents
        rti                 ; return from interrupt

L1859   subroutine
        anda #<~b83_bit1    ; clear bit -11111101-
        sta  BitFlags83     ; store a into memory
        clra                ; a = 0
        sta  L0063          ; store a into memory
        lda  #$02           ; load a with value -00000010-
        sta  L0066          ; store a into memory
        ldd  #$3280         ; load d (a&b) with value -00110010 10000000-
        std  L0067          ; store d (a&b) into memory
        bra  L1865          ; branch

MM_L1602    subroutine
    ; on entry, d is loaded with counter
        subd Something_W    ; d = d - memory contents
        sta  FuelPulsewidth_scratch1    ; store a into memory
        lda  L0063          ; load a with memory contents
        lsrd                ; shift right (d=d/2)
        xorb FuelPulsewidth_scratch1
        bpl  L1866          ; branch if plus (hi bit 0)
        lda  L0063          ; load a with memory contents
        inca                ; increment a
        beq  L1866          ; branch if equal (zero)
        sta  L0063          ; store a into memory
L1866:  lda  L0063          ; load a with memory contents
        lsra
        cmpa L0066          ; compare a with memory contents
        blo  L1867          ; branch if lower
        sta  L0066          ; store a into memory
L1867:  rts                 ; return from subroutine

;  ______________________
; |                      \
; |   Cruise Control     |
; \______________________|

MM_HandleCruiseControl_v subroutine
        lda  Z83_PortCdata              ; load a with memory contents
        bita #$40                       ; * check if cruise switched On
        bne  CruiseIsSwitchedOn         ; * branch if cruise switched On
        lda  CruiseStatus               ; load a with memory contents
        anda #$8b                       ; AND a with value -10001011-
        clrb                            ; b = 0
        stb  CruiseSetSpeed             ; store b into memory
        stb  CruiseStatus2              ; store b into memory
        jmp  L1127

CruiseIsSwitchedOn:
        lda  CruiseStatus               ; load a with memory contents
        bita #$80                       ;  -10000000-
        beq  L1128                      ; branch if equal (zero)
        ldb  Z83_PortCdata              ; load b with memory contents
        bitb #$02                       ; * check for cruise resume pressed
        bne  L1128                      ; * branch if cruise resume pressed
        anda #$ef                       ; AND a with value -11101111-
        sta  CruiseStatus               ; store a into memory
L1128:  ldd  VehicleSpeed_W             ; load d (a&b) with memory contents
        asld                            ; shift left (d=d*2)
        asld                            ; shift left (d=d*2)
        asld                            ; shift left (d=d*2)
        lslb
        adca #$00                       ;  -00000000-
        sta  Scratch_b6                 ; store a into memory
        ldd  RPM_W                      ; load d (a&b) with memory contents
        subd #$4330                     ;  -01000011 00110000-
        bcs  L1129                      ; branch if lower
        subd #$0af0                     ;  -00001010 11110000-
        bcc  DropOutOfCruise            ; branch if greater or equal
        lda  Scratch_b6                 ; load a with memory contents
        suba CruiseSetSpeed
        bcs  DropOutOfCruise            ; branch if lower
        suba #$08                       ;  -00001000-
        bcs  DropOutOfCruise            ; branch if lower
L1129:  ldx  VehicleSpeed_W
        cpx  CruiseMinimumCruiseSpeed_LOSPCO  ;  (data is 0640)
        blo  DropOutOfCruise                  ; branch if lower
        lda  TRANSM_Transmission      ; load a with memory contents (data is 00)
        beq  L1130                      ; branch if equal (zero)
        lda  Z83_PortCdata              ; load a with memory contents
        bita #$08                       ; * check if park/neutral
        beq  DropOutOfCruise            ; branch if equal (zero)
        bra  L1131                      ; branch

L1130:  lda  InverseRPM_B               ; load a with memory contents
        bne  L1132                      ; branch if not equal (not zero)
        lda  #$11                       ; load a with value -00010001-
        ldb  Scratch_b6                 ; load b with memory contents
        sei                             ; disable interrupts (set)
        sta  Scratch_bb                 ; store a into memory
        lda  InverseRPM_W               ; load a with memory contents
        mul                             ; multiply (d=a*b)
        lsrd                            ; shift right (d=d/2)
        jsr  mul16bit                   ; call subroutine
        cli                             ; enable interrupts (clear)
        tsta                            ; test a
        beq  L1133                      ; branch if equal (zero)
        ldb  #$ff                       ; load b with value -11111111-
L1133:  cmpb SCNVHI_RevChangeToSetBrakeFlag    ; compare b with memory contents (data is f1)
        bhs  DropOutOfCruise            ; branch if greater or equal
        cmpb SCNVLO_RevChangeToSetBrakeFlag    ; compare b with memory contents (data is 52)
        blo  DropOutOfCruise            ; branch if lower
L1132:  lda  CruiseStatus               ; load a with memory contents
        bita #$80                       ;  -10000000-
        bne  L1131                      ; branch if not zero (cruise disengaged)
        lda  BitFlags83                 ; load a with memory contents
        bita #$01                       ;  -00000001-
        bne  DropOutOfCruise            ; branch if not equal (not zero)
L1131:  lda  Z83_PortCdata              ; load a with memory contents
        bita #$10                       ; * check if brake pressed
        beq  L1134                      ; branch if equal (zero)
DropOutOfCruise:
        clrb                            ; b = 0
        lda  CruiseStatus               ; load a with memory contents
        anda #$93                       ; AND a with value -10010011-
L1127:  stb  Timer_CruiseDecelerate     ; store b into memory
        bra  L1135                      ; branch

L1134:  lda  Z83_PortA_copy             ; load a with memory contents
        tab                             ; b = a
        xora Z83_PortAdata
        anda #PortA_CruiseVent | PortA_CruiseVac ; AND a with value -10100000-
        beq  L1136                      ; branch if equal (zero)
        cmpa #PortA_CruiseVent          ; compare a with value -10000000-
        bhs  L1137                      ; branch if greater or equal
        lslb
        lslb
L1137:  lda  #$01                       ; load a with value -00000001-
        tstb
        bpl  L1138                      ; branch if plus (hi bit 0)
        lda  CruiseStatus               ; load a with memory contents
        bita #$80                       ;  -10000000-
        bne  L1139                      ; branch if not equal (not zero)
        lda  #$0f                       ; load a with value -00001111-
        jsr  SetFaultCode               ; call subroutine
        cmpb #$ef                       ;  -11101111-
        bne  L1136                      ; branch if not equal (not zero)
        clr  $00,x
        bra  DropOutOfCruise            ; branch

L1139:  lda  #$0f                       ; load a with value -00001111-
        ldb  Z83_PortAdata              ; load b with memory contents
        andb #PortA_CruiseVent | PortA_CruiseVac ;  -10100000-
        bne  L1138                      ; branch if not equal (not zero)
        lda  #$7f                       ; load a with value -01111111-
L1138:  bita Counter_MainLoop1
        bne  L1136                      ; branch if not equal (not zero)
        lda  #$0f                       ; load a with value -00001111-
        jsr  L1042                      ; call subroutine
L1136:  lda  CruiseStatus               ; load a with memory contents
        ldb  Z83_PortCdata              ; load b with memory contents
        bitb #$01                       ; * check if cruise set pressed
        bne  CruiseControlSet           ; branch if not equal (not zero)
        ora  #$20                       ;  -00100000-
        ldb  Timer_CruiseDecelerate     ; load b with memory contents
        incb
        beq  L1140                      ; branch if equal (zero)
        stb  Timer_CruiseDecelerate     ; store b into memory
L1140:  ldb  Timer_CruiseDecelerate     ; load b with memory contents
        cmpb SETIME                     ; compare b with memory contents (data is 37)
        blo  L1141                      ; branch if lower
        cmpb SETIM1                     ; compare b with memory contents (data is 5b)
        bhs  L1142                      ; branch if greater or equal
        bita #$80                       ;  -10000000-
        bne  L1141                      ; branch if not equal (not zero)
        ora  #$04                       ;  -00000100-
        bra  L1135                      ; branch

L1142:  anda #$fb                       ; AND a with value -11111011-
L1135:  ora  #$80                       ;  -10000000-
L1141:  sta  CruiseStatus               ; store a into memory
L1145:  jmp  L1143

CruiseControlSet:
        bita #$20                       ;  -00100000-
        beq  L1144                      ; branch if equal (zero)
        anda #$df                       ; AND a with value -11011111-
        sta  CruiseStatus               ; store a into memory
        ldb  Timer_CruiseDecelerate     ; load b with memory contents
        clr  Timer_CruiseDecelerate     ; zero byte at memory location
        cmpb #$03                       ;  -00000011-
        blo  L1145                      ; branch if lower
        bita #$80                       ;  -10000000-
        bne  L1146                      ; branch if not equal (not zero)
        lda  Scratch_b6                 ; load a with memory contents
        suba CruiseSetSpeed
        bcs  L1147          ; branch if lower
        cmpa #$04           ; compare a with value -00000100-
        bls  L1147          ; branch if lower or same
L1146:  lda  #$fe           ; load a with value -11111110-
        sei                 ; disable interrupts (set)
        anda BitFlags83     ; logical and a with memory contents
        sta  BitFlags83     ; store a into memory
        cli                 ; enable interrupts (clear)
        lda  Scratch_b6     ; load a with memory contents
        sta  CruiseSetSpeed ; store a into memory
        clra                ; a = 0
        ldb  CruiseStatus   ; load b with memory contents
        bitb #$04           ;  -00000100-
        bne  L1148          ; branch if not equal (not zero)
        lda  CruiseSetSpeed ; load a with memory contents
        ldx  #CruiseSetPW_SETPW    ; load index with value
        jsr  LookupTable    ; call subroutine
L1148:  ldb  CruiseStatus   ; load b with memory contents
        orb  #$10           ;  -00010000-
        andb #$fb           ;  -11111011-
        bra  L1149          ; branch

L1147:  lda  CruiseSetSpeed ; load a with memory contents
        adda #$04           ;  -00000100-
        sta  CruiseSetSpeed ; store a into memory
        lda  #$02           ; load a with value -00000010-
        ldb  CruiseStatus   ; load b with memory contents
        orb  #$40           ;  -01000000-
L1149:  andb #$7f           ;  -01111111-
        stb  CruiseStatus   ; store b into memory
        ldb  CruiseSetSpeed ; load b with memory contents
        cmpb CruiseMaxSetSpeed_MAXSET    ; compare b with memory contents (data is fe)
        blo  L1150          ; branch if lower
        ldb  CruiseMaxSetSpeed_MAXSET    ; load b with memory contents (data is fe)
        bra  L1151          ; branch

L1150:  cmpb CruiseMinSetSpeed_MINSET    ; compare b with memory contents (data is 3c)
        bhs  L1152          ; branch if greater or equal
        clrb                ; b = 0
L1151:  stb  CruiseSetSpeed ; store b into memory
        bra  L1152          ; branch

L1144:  bitb #$02           ;  -00000010-
        beq  L1143          ; branch if equal (zero)
        ldb  #$fe           ; load b with value -11111110-
        sei                 ; disable interrupts (set)
        andb BitFlags83
        stb  BitFlags83     ; store b into memory
        cli                 ; enable interrupts (clear)
        bita #$50           ;  -01010000-
        bne  L1143          ; branch if not equal (not zero)
        anda #$7f           ; AND a with value -01111111-
        ora  #$50           ;  -01010000-
        sta  CruiseStatus   ; store a into memory
        ldx  #CruiseSetPW_SETPW    ; load index with value
        lda  CruiseSetSpeed ; load a with memory contents
        jsr  LookupTable    ; call subroutine
        sta  L006F          ; store a into memory
        lda  CruiseSetSpeed ; load a with memory contents
        suba Scratch_b6
        bcs  L1153          ; branch if lower
        ldx  #CruiseRestbl_RESTBL    ; load index with value
        jsr  LookupTable    ; call subroutine
        adda L006F
        cmpa #$7f           ; compare a with value -01111111-
        bls  L1152          ; branch if lower or same
        lda  #$7f           ; load a with value -01111111-
L1152:  sta  L006F          ; store a into memory
L1153:  lda  CruiseStatus   ; load a with memory contents
        ora  #$08           ;  -00001000-
        sta  CruiseStatus   ; store a into memory
L1143:  lda  Counter_MainLoop1    ; load a with memory contents
        cmpa #$f3           ; compare a with value -11110011-
        bhs  L1154          ; branch if greater or equal
        suba #$84           ;  -10000100-
        bcc  L1155          ; branch if greater or equal
        lda  Counter_MainLoop1    ; load a with memory contents
L1155:  tab                 ; b = a
        suba #$42           ;  -01000010-
        bcc  L1156          ; branch if greater or equal
        tba                 ; a = b
L1156:  suba #$0b           ;  -00001011-
        bcc  L1156          ; branch if greater or equal
        adda #$0b           ;  -00001011-
        beq  L1157          ; branch if equal (zero)
L1154:  jmp  L1158

L1157:  clrb                ; b = 0
        lda  BitFlags83     ; load a with memory contents
        bita #$02           ;  -00000010-
        bne  L1159          ; branch if not equal (not zero)
        ldd  #$ddf2         ; load d (a&b) with value -11011101 11110010-
        std  Scratch_b6     ; store d (a&b) into memory
        ldx  L0067
        lda  #$0d           ; load a with value -00001101-
        ldb  L0066          ; load b with memory contents
        beq  L1160          ; branch if equal (zero)
        cmpb #$7f           ;  -01111111-
        blo  L1161          ; branch if lower
        clra                ; a = 0
        clrb                ; b = 0
        bra  L1162          ; branch

L1161:  ldx  L0066
        lda  #$05           ; load a with value -00000101-
L1160:  jsr  L1163          ; call subroutine
        addd VehicleSpeed_W ; d = d + memory contents
        rora
        rorb
L1162:  ldx  VehicleSpeed_W
        stx  Scratch_b6     ; store index into memory
        std  VehicleSpeed_W ; store d (a&b) into memory
        ldd  Scratch_b6     ; load d (a&b) with memory contents
        subd VehicleSpeed_W ; d = d - memory contents
        bne  L1164          ; branch if not equal (not zero)
        clr  L006D          ; zero byte at memory location
L1164:  std  Scratch_b6     ; store d (a&b) into memory
        clra                ; a = 0
        ldb  L006D          ; load b with memory contents
        bpl  L1166          ; branch if plus (hi bit 0)
        deca                ; decrement a
L1166:  addd Scratch_b6     ; d = d + memory contents
        asra
        rorb
        asld                ; shift left (d=d*2)
        bcs  L1167          ; branch if lower
        tsta                ; test a
        beq  L1168          ; branch if equal (zero)
        ldb  #$7f           ; load b with value -01111111-
        bra  L1159          ; branch

L1167:  cmpa #$ff           ; compare a with value -11111111-
        beq  L1169          ; branch if equal (zero)
        clrb                ; b = 0
L1169:  sec                 ; set carry
L1168:  rorb
L1159:  stb  L006D          ; store b into memory
        lda  CruiseStatus   ; load a with memory contents
        bita #$08           ;  -00001000-
        bne  L1170          ; branch if not equal (not zero)
        bita #$80           ;  -10000000-
        bne  L1171          ; branch if not equal (not zero)
        lda  Timer_CruiseDecelerate ; load a with memory contents
        cmpa SETIME         ; compare a with memory contents (data is 37)
        blo  L1172          ; branch if lower
L1171:  lda  #$0f           ; load a with value -00001111-
L1178:  ora  #$80           ;  -10000000-
        sta  L006F          ; store a into memory
L1170:  jmp  L1158

L1172:  ldx  #CruiseRegulator_VACREG    ; load index with value
        clrb                ; b = 0
        lda  CruiseSetSpeed ; load a with memory contents
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        subd VehicleSpeed_W ; d = d - memory contents
        bcc  Cruise_NeedToAccel ; branch if need to accelerate
        ; need to decel:
        ldx  #CruiseVent_VENTRG    ; load index with value
        coma                ; complement (toggle all bits)
        comb                ; complement (toggle all bits)
        addd #ioPort2ddr    ;  -00000000 00000001-
        stb  Scratch_b6     ; store DeltaSpeed
        ldb  CruiseStatus   ; load b with memory contents
        andb #$bf           ;  -10111111-
        stb  CruiseStatus   ; store b into memory
        ldb  Scratch_b6     ; load b with memory contents
        tsta                ; test a
        beq  L1174          ; branch if equal (zero)
        clr  Scratch_b6     ; zero byte at memory location
        dec  Scratch_b6     ; decrement memory contents
L1174:  lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        tsta                ; test a
        bne  L1171          ; branch if not equal (not zero)
        cmpb VNTAB1_LimitToAMaxVent    ; compare b with memory contents (data is 70)
        bhs  L1171          ; branch if greater or equal
        tba                 ; a = b
        ldb  Scratch_b6     ; load b with memory contents
        cmpa VNTAB2_LimitToHalfMaxVent ; compare a with memory contents (data is 50)
        blo  L1177          ; branch if lower
        lda  #$05           ; load a with value -00000101-
        bra  L1178          ; branch

Cruise_NeedToAccel:
        tsta                ; test a
        beq  L1179          ; branch if equal (zero)
        ldb  #$ff           ; load b with value -11111111-
L1179:  lda  CruiseStatus   ; load a with memory contents
        bita #$40           ;  -01000000-
        beq  L1177          ; branch if equal (zero)
        ldx  #CruiseAcc_VACACC    ; load index with value
        cmpb ACLRST_DeltaSpeedToResetAccFlag          ; compare b with memory contents (data is 05)
        bhs  L1177          ; branch if greater or equal
        anda #$bf           ; AND a with value -10111111-
        sta  CruiseStatus   ; store a into memory
L1177:  tba                 ; a = b
        jsr  LookupTable    ; call subroutine
        clra                ; a = 0
        tstb
        bpl  L1181          ; branch if plus (hi bit 0)
        deca                ; decrement a
L1181:  std  Scratch_b6     ; store d (a&b) into memory
        ldx  #CruiseThrottleGain_THGAIN    ; load index with value
        lda  TpsVolts            ; load a with memory contents
        suba LowestSessionTPS
        bcc  L1182          ; branch if greater or equal
        clra                ; a = 0
L1182:  jsr  LookupTable    ; call subroutine
        sta  Scratch_ba     ; store a into memory
        ldx  #CruiseSpeedGain_SPGAIN    ; load index with value
        ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        jsr  LookupTable    ; call subroutine
        adda Scratch_ba
        sta  Scratch_b9     ; store a into memory
        clra                ; a = 0
        sta  Scratch_b8     ; store a into memory
        ldb  CruiseStatus2  ; load b with memory contents
        bpl  L1183          ; branch if plus (hi bit 0)
        deca                ; decrement a
L1183:  addd Scratch_b6     ; d = d + memory contents
        std  Scratch_b6     ; store d (a&b) into memory
        clra                ; a = 0
        ldb  L006D          ; load b with memory contents
        bpl  L1184          ; branch if plus (hi bit 0)
        deca                ; decrement a
L1184:  asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        addd Scratch_b6     ; d = d + memory contents
        sta  Scratch_b6     ; store a into memory
        bpl  L1185          ; branch if plus (hi bit 0)
        coma                ; complement (toggle all bits)
        comb                ; complement (toggle all bits)
        addd #ioPort2ddr    ;  -00000000 00000001-
L1185:  subd Scratch_b8     ; d = d - memory contents
        bcc  L1186          ; branch if greater or equal
        addb Scratch_b9     ; b=b+memory contents
        lda  Scratch_b6     ; load a with memory contents
        bpl  L1187          ; branch if plus (hi bit 0)
        negb
L1187:  stb  CruiseStatus2  ; store b into memory
        bra  L1158          ; branch

L1186:  clra                ; a = 0
        sta  CruiseStatus2  ; store a into memory
        lda  Scratch_b6     ; load a with memory contents
        anda #$80           ; AND a with value -10000000-
        ora  #$01           ;  -00000001-
        ldb  L006F          ; load b with memory contents
        beq  L1188          ; branch if equal (zero)
        bpl  L1158          ; branch if plus (hi bit 0)
        tab                 ; b = a
        xorb L006F
        bpl  L1158          ; branch if plus (hi bit 0)
L1188:  ldb  TpsVolts            ; load b with memory contents
        subb LowestSessionTPS    ; b = b-memory contents
        bcc  L1189          ; branch if greater or equal
        clrb                ; b = 0
L1189:  tsta                ; test a
        bpl  L1190          ; branch if plus (hi bit 0)
        cmpb CLOTHR_DontVentIfAlreadyClosed ; compare b with memory contents (data is 03)
        blo  L1192          ; branch if lower
        inca                ; increment a
        bra  L1193          ; branch

L1190:  cmpb MAXTHR_DontVacIfAlreadyFullThrottle ; compare b with memory contents (data is 50)
        blo  L1193          ; branch if lower
L1192:  clra                ; a = 0
L1193:  sta  L006F          ; store a into memory
L1158:  lda  Z83_PortA_copy ; load a with memory contents
        anda #<~[PortA_CruiseVent | PortA_CruiseVac]   ; $5f ; AND a with value -01011111-
        ldb  CruiseStatus   ; load b with memory contents
        andb #$f7           ;  -11110111-
        stb  CruiseStatus   ; store b into memory
        bmi  L1195          ; branch if minus(hi bit 1)
        ldb  L006F          ; load b with memory contents
        beq  L1196          ; branch if equal (zero)
        bpl  L1197          ; branch if plus (hi bit 0)
L1195:  ora  #PortA_CruiseVent | PortA_CruiseVac  ; $a0 set bits -10100000-
L1196:  ora  #PortA_CruiseVac                     ; $20 set bit  -00100000-
L1197:  sta  Z83_PortA_copy ; store a into memory
        ldb  L006F          ; load b with memory contents
        beq  L1198          ; branch if equal (zero)
        asld                ; shift left (d=d*2)
        subb #$02           ;  -00000010-
        beq  L1199          ; branch if equal (zero)
        lsrd                ; shift right (d=d/2)
L1199:  stb  L006F          ; store b into memory
L1198:  rts                 ; return from subroutine

; *****************************************

MM_SubtractOne32ndFromSpeed_v subroutine
        lda  BitFlags83     ; load a with memory contents
        bita #$02           ; * check if cruise control enabled
        beq  .Return        ; branch if equal (zero)
        sei                 ; disable interrupts (set)
        ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        std  Scratch_b6     ; store d (a&b) into memory
        ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        subd Scratch_b6     ; d = d - memory contents
        std  VehicleSpeed_W ; store d (a&b) into memory
        cli                 ; enable interrupts (clear)
.Return rts                 ; return from subroutine

MM_LD24E   subroutine
        ldd  #$060a         ; load d (a&b) with value -00000110 00001010-
        tst  BitFlags84
        bpl  L1868          ; branch if plus (hi bit 0)
        deca                ; decrement a
L1868:  std  sciModeControl ; store d (a&b) into memory
        ldd  #$bef6         ; load d (a&b) with value -10111110 11110110-
        std  ioPort1ddr     ; store d (a&b) into memory
        sei                 ; disable interrupts (set)
        ldd  timerControl1  ; load d (a&b) with memory contents
        ora  #$e0           ;  -11100000-
        andb #$fb           ;  -11111011-
        orb  #$c0           ;  -11000000-
        std  timerControl1  ; store d (a&b) into memory
        cli                 ; enable interrupts (clear)
        clra                ; a = 0
        sta  Z83_PortAcontrol
        sta  Z83_PortBcontrol
        sta  Z83_PortCDataDirection
        sta  Z83_PortCcontrol
        rts                 ; return from subroutine

ADConversionOutputTable:
    dw  RawBattTemp
    dw  RawO2Sensor
    dw  ChargeTempVolts
    dw  CoolantTemp
    dw  TpsVolts
    dw  RawKnockSensor
    dw  RawBattVoltage
    dw  BarometricPressure
    dw  TpsVolts
ADConversionPatternTable:
    db  $00,$20,$10,$30,$40 ; ". .0@"

TestASDRelay    subroutine
        sei                 ; * disable interrupts - called at d319
        lda  Z83_PortAdata  ; load a with memory contents
        ora  #PortA_ASD     ; * turn on ASD relay
        sta  Z83_PortAdata
        ldb  Z83_PortADataDirection    ; load b with memory contents
        stb  Scratch_b6     ; store b into memory
        orb  #$02           ;  -00000010-
        stb  Z83_PortADataDirection    ; store b into memory
        jsr  TimeWaste      ; call subroutine
        jsr  TimeWaste      ; call subroutine
        andb #$fd           ;  -11111101-
        stb  Z83_PortADataDirection    ; store b into memory
        jsr  TimeWaste      ; call subroutine
        jsr  TimeWaste      ; call subroutine
        lda  Z83_PortAdata  ; load a with memory contents
        orb  #$02           ;  -00000010-
        stb  Z83_PortADataDirection    ; store b into memory
        ldb  Z83_PortA_copy ; load b with memory contents
        stb  Z83_PortAdata  ; store b into memory
        jsr  TimeWaste      ; call subroutine
        jsr  TimeWaste      ; call subroutine
        ldb  Scratch_b6     ; load b with memory contents
        stb  Z83_PortADataDirection    ; store b into memory
        cli                 ; enable interrupts (clear)
        bita #$02           ;  -00000010-
        bne  TimeWaste      ; branch if not equal (not zero)
        ldb  #$61           ; load b with value -01100001-
        lda  L00C9          ; load a with memory contents
        cmpa #$60           ; compare a with value -01100000-
        bls  L1201          ; branch if lower or same
        ldb  #$70           ; load b with value -01110000-
        cmpa #$69           ; compare a with value -01101001-
        beq  L1201          ; branch if equal (zero)
        clrb                ; b = 0
        cmpa #$72           ; compare a with value -01110010-
        bhs  L1201          ; branch if greater or equal
        inca                ; increment a
        tab                 ; b = a
L1201:  stb  L00C9          ; store b into memory
TimeWaste:
        rts                 ; return from subroutine

;  ______________________
; |                      \
; |   Serial Comm        |
; \______________________|

MM_SendRomSerial   subroutine
        lda  BitFlags84     ; load a with memory contents
        bpl  L1202          ; * jump if high speed serial
        ldd  sciTransceiveCtrl    ; load d (a&b) with memory contents
        cmpb #$fa           ; -11111010-, check if byte received?
        bne  SendRamSerial  ; branch if not equal (not zero)
        ldx  #RomBegin      ; load index with value
        lda  Counter_MainLoop1 ; load a with memory contents
        lsra
        bcc  L1203          ; branch if greater or equal
        inx                 ; increment index (x=x+1)
L1203:  ldb  $00,x          ; load b with memory at index + value
        jmp  SendByteSerial

SendRamSerial:
    ; MOD:
    ; Send some massaged values for the convenience of ChEM:
        cmpb #$f0           ; New value: send single byte RPMs
        bne  .f1
        ldd  RPM_W          ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        tab                 ; b = a
        jmp  .sendb

.f1     cmpb #$f1           ; New value: send single byte Speed
        bne  .stock
        ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        tab                 ; b = a
        jmp  .sendb

.stock  ldx  #$00           ; load index with value
        abx                 ; add b to index
        ldb  $00,x          ; load b with memory at index + value
.sendb  lda  BitFlags84     ; load a with memory contents
        anda #$fb           ; AND a with value -11111011-
        cpx  #$00fe         ;  -00000000 11111110-
        bne  L1204          ; branch if not equal (not zero)
        ldb  RPM_W          ; load b with memory contents
        ora  #$04           ;  -00000100-
L1204:  sta  BitFlags84     ; store a into memory
        bra  SendByteSerial ; branch

L1202:  lda  Counter_MainLoop1    ; load a with memory contents
        anda #$3f           ; AND a with value -00111111-
        cmpa #$07           ; compare a with value -00000111-
        bne  L1205          ; branch if not equal (not zero)
        jsr  TestASDRelay   ; call subroutine
L1205:  lda  Counter_MainLoop1    ; load a with memory contents
        bita #$07           ;  -00000111-
        bne  L1206          ; branch if not equal (not zero)
        ldb  L00C9          ; load b with memory contents
        beq  HandleSerial   ; branch if equal (zero)
        lda  sciTransceiveCtrl    ; * check serial port
        bpl  DiagnosticsOff ; * jump if no new byte received
        lda  sciRxData      ; * get received byte
        cmpa #$f0           ; compare a with value -11110000-
        bne  DiagnosticsOff ; branch if not equal (not zero)
        cmpb #$70           ;  -01110000-
        bne  L1207          ; branch if not equal (not zero)
        jmp  L1208

HandleSerial:
        lda  BitFlags84     ; load a with memory contents
        anda #$fb           ; * turn off diagnostic mode flag
        ldb  sciTransceiveCtrl    ; * check if byte received
        bpl  NoByteWaiting  ; * jump if no byte waiting
        ldb  sciRxData      ; * get the received byte
        cmpb #$12           ; * check if control for high speed
        bne  NotCode12      ; * jump if not
        ldb  #$05           ; * set high speed mode
        stb  sciModeControl ; store b into memory
        ora  #$80           ; * set high speed bit flag
NotCode12:
        cmpb #$f0           ; * check if control for diagnostic mode
        bne  NoByteWaiting  ; * jump if not
        ora  #$04           ; * set diagnostic mode bit flag
NoByteWaiting:
        sta  BitFlags84     ; store a into memory
        lda  sciRxData      ; load a with memory contents
        lda  RawO2Sensor    ; load a with memory contents
        ldb  #$ff           ; load b with value -11111111-
        cmpa O2MIDH_O2HighLimitForMidLevelDetermination    ; compare a with memory contents (data is 1c)
        bhi  L1209          ; branch if higher
        cmpa O2MIDL_O2LowLimitForMidLevelDetermination    ; compare a with memory contents (data is 12)
        bhi  L1210          ; branch if higher
        andb #$a0           ;  -10100000-
L1209:  andb #$b1           ;  -10110001-
L1210:  ldx  #CylinderRetard1 - 1         ; load index with value
L1211:  inx                 ; increment index (x=x+1)
        cpx  #MaxRetard     ;  -00000000 11010010-
        beq  SendByteSerial ; branch if equal (zero)
        lda  $00,x          ; load a with memory at index + value
        anda #$3f           ; AND a with value -00111111-
        beq  L1211          ; branch if equal (zero)
        andb #$0f           ;  -00001111-
        orb  #$80           ;  -10000000-
        bra  SendByteSerial ; branch

DiagnosticsOff:
        lda  BitFlags84     ; load a with memory contents
        anda #$fb           ; AND a with value -11111011-
        sta  BitFlags84     ; store a into memory
SendByteSerial:
        stb  sciTxData      ; store b into memory
L1206:  rts                 ; return from subroutine

L1207:  lda  BitFlags84     ; load a with memory contents
        anda #$fb           ; AND a with value -11111011-
        sta  BitFlags84     ; store a into memory
        ldx  #ADConversionOutputTable - 2     ; load index with value
        andb #$1f           ;  -00011111-
        lslb
        abx                 ; add b to index
        ldx  $00,x
        lda  $00,x          ; load a with memory at index + value
        lsrb                ; logical shift right b
        cmpb #$09           ;  -00001001-
        bne  L1212          ; branch if not equal (not zero)
        suba LowestSessionTPS
        bcc  L1212          ; branch if greater or equal
        clra                ; a = 0
L1212:  cmpb #$06           ;  -00000110-
        bne  L1213          ; branch if not equal (not zero)
        clrb                ; b = 0
        stb  Scratch_b6     ; store b into memory
        stb  Scratch_b7     ; store b into memory
        ldx  #CylinderRetard1         ; load index with value
L1215:  lda  $00,x          ; load a with memory at index + value
        anda #$3f           ; AND a with value -00111111-
        incb
        cmpa Scratch_b7     ; compare a with memory contents
        bls  L1214          ; branch if lower or same
        sta  Scratch_b7     ; store a into memory
        stb  Scratch_b6     ; store b into memory
L1214:  inx                 ; increment index (x=x+1)
        cpx  #MaxRetard     ;  -00000000 11010010-
        blo  L1215          ; branch if lower
        ldx  #ADConversionPatternTable    ; load index with value
        ldb  Scratch_b6     ; load b with memory contents
        abx                 ; add b to index
        ldb  Scratch_b7     ; load b with memory contents
        lda  #$b4           ; load a with value -10110100-
        mul                 ; multiply (d=a*b)
        cmpa #$09           ; compare a with value -00001001-
        bls  L1216          ; branch if lower or same
        lda  #$09           ; load a with value -00001001-
L1216:  ldb  $00,x          ; load b with memory at index + value
        aba                 ; a=a+b
        tab                 ; b = a
        bra  SendByteSerial    ; branch

L1213:  cmpb #$04           ;  -00000100-
        bne  L1217          ; branch if not equal (not zero)
        ldb  #$2e           ; load b with value -00101110-
        mul                 ; multiply (d=a*b)
        adca #$ec           ;  -11101100-
        bcs  L1218          ; branch if lower
        clra                ; a = 0
        bra  L1218          ; branch

L1217:  cmpb #$07           ;  -00000111-
        bne  L1219          ; branch if not equal (not zero)
        ldb  #$fd           ; load b with value -11111101-
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
        lsra
        lsra
        lsra
        lsra
        bra  L1220          ; branch

L1219:  cmpb #$08           ;  -00001000-
        bne  L1221          ; branch if not equal (not zero)
        ldb  #$3b           ; load b with value -00111011-
        suba MapValue
        bcc  L1222          ; branch if greater or equal
        nega                ; negate a (set high bit)
        ldb  #$1d           ; load b with value -00011101-
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
        cmpa #$09           ; compare a with value -00001001-
        bhi  L1218          ; branch if higher
        ora  #$a0           ;  -10100000-
        tab                 ; b = a
        jmp  SendByteSerial

L1221:  cmpb #$12           ;  -00010010-
        beq  L1223          ; branch if equal (zero)
        cmpb #$11           ;  -00010001-
        beq  L1224          ; branch if equal (zero)
        ldb  #$32           ; load b with value -00110010-
L1222:  mul                 ; multiply (d=a*b)
L1220:  adca #$00           ;  -00000000-
L1218:  sei                 ; disable interrupts (set)
        sta  Scratch_bb     ; store a into memory
        ldd  #$199a         ; load d (a&b) with value -00011001 10011010-
        jsr  mul16bit       ; call subroutine
        ldb  #$06           ; load b with value -00000110-
        mul                 ; multiply (d=a*b)
        addb Scratch_bb     ; b=b+memory contents
        cli                 ; enable interrupts (clear)
        jmp  SendByteSerial

L1208:  lda  BitFlags84     ; load a with memory contents
        ora  #$04           ;  -00000100-
        sta  BitFlags84     ; store a into memory
L1223:  lda  Counter_MainLoop1    ; load a with memory contents
        bita #$7f           ;  -01111111-
        beq  L1225          ; branch if equal (zero)
L1227:  rts                 ; return from subroutine

L1225:  ldd  RPM_W          ; load d (a&b) with memory contents
        bita #$e0           ;  -11100000-
        beq  L1226          ; branch if equal (zero)
        ldd  #$ffff         ; load d (a&b) with value -11111111 11111111-
L1226:  asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        ldb  #$cd           ; load b with value -11001101-
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
L1228:  cmpa #$63           ; compare a with value -01100011-
        blo  L1218          ; branch if lower
        lda  #$63           ; load a with value -01100011-
        bra  L1218          ; branch

L1224:  lda  Counter_MainLoop1    ; load a with memory contents
        bita #$1f           ;  -00011111-
        bne  L1227          ; branch if not equal (not zero)
        ldd  VehicleSpeed_W    ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        bra  L1228          ; branch

MM_KnockDetect subroutine
        clra                             ; a = 0
        ldb  RPM_W                       ; load b with memory contents
        cmpb KNKRPM_KnockRetardMinRPM ; compare b with memory contents (data is 10)
        blo  .2                          ; branch if lower
        ldx  #RETMAX_KnockRetardMaxRetardFromMAP ; load index with value
        deca                             ; decrement a
        ldb  BitFlags80                  ; load b with memory contents
        andb #b80_BadMAP | b80_BadTPS        ;  -11000000-
        bne  .1                          ; branch if not equal (not zero)
        lda  MapValue                    ; load a with memory contents
        ldb  TpsVolts                    ; load b with memory contents
        subb LowestSessionTPS            ; b = b-memory contents
        bcs  .1                          ; branch if lower
        cmpb THRKNK_KnockRetardThrottleLimit               ; compare b with memory contents (data is 53)
        blo  .1                          ; branch if lower
        ldb  RPM_W                       ; load b with memory contents
        cmpb KLMRPM_KnockRetardMaxRPMforRate        ; compare b with memory contents (data is 10)
        bhs  .1                          ; branch if greater or equal
        lda  KNKLIM_KnockRetardLimitDegrees    ; load a with memory contents (data is 04)
        bra  .2                          ; branch

.1      jsr  LookupTable                 ; call subroutine
.2      sta  MaxRetard                   ; store a into memory
        lda  RPM_W                       ; load a with memory contents
        ldx  #DLTKNK_KnockSensorVoltageFromKnock    ; load index with value
        jsr  LookupTable                 ; call subroutine
        sta  KnockLookup                 ; store a into memory
        rts                              ; return from subroutine

MM_WorkWithPowerLossLight subroutine
        lda  BitFlags82     ; load a with memory contents
        bpl  .2             ; branch if b82_EngineNotRunning clear 
        lda  Counter_EngineRunningTime          ; load a with memory contents
        bne  .4             ; branch if not equal (not zero)
.1      ldb  #$20           ; load b with value -00100000-
        sei                 ; disable interrupts (set)
        orb  ioPort1data    ; * turn on power loss light
        bra  .5             ; branch

		; check if fuel is shut off for whatever reason
.2      lda  BitFlags84     ; load a with memory contents
        bita #$08           ;  -00001000-
        beq  .3             ; branch if equal (zero)
        ; turn on CE light if off idle
        lda  BitFlags81     ; load a with memory contents
        bmi  .1             ; branch if minus(hi bit 1)

.3      lda  BitFlags80     ; load a with memory contents
        bita L1236          ;  (data is db)
        bne  .1             ; branch if not equal (not zero)
        lda  L004F          ; load a with memory contents
        bita #$20           ;  -00100000-
        bne  .1             ; branch if not equal (not zero)

.4      ldb  #$df           ; load b with value -11011111-
        sei                 ; disable interrupts (set)
        andb ioPort1data    ; * turn off power loss light

.5      stb  ioPort1data    ; store b into memory
        cli                 ; enable interrupts (clear)
        rts                 ; return from subroutine

MM_DetermineDistributorSync_v subroutine
; 89: DETERMINE_IF_WINDOWEDBLADE_IN_SYNC_SENSOR_ON_DISTRIBUTER_ROUT
        lda  BitFlags85     ; load a with memory contents
        ldb  BitFlags82     ; load b with memory contents
        bpl  .2             ; branch if b82_EngineNotRunning clear
        bitb #$04           ;  -00000100-
        bne  .2             ; branch if not equal (not zero)
        ldb  ioPort2data    ; load b with memory contents
        lsrb                ; * check ?distance reference?
        bcs  .3             ; branch if lower
        ldb  Z83_PortCdata  ; load b with memory contents
        bmi  .1             ; * branch if dist sync signal
        ora  #$01           ;  -00000001-
        bra  .4             ; branch

.1      bita #$01           ;  -00000001-
        beq  .Return        ; branch if equal (zero)
        ora  #$02           ;  -00000010-
        bra  .4             ; branch

.2      anda #$fd           ; AND a with value -11111101-
.3      anda #$fe           ; AND a with value -11111110-
.4      sta  BitFlags85     ; store a into memory
.Return
        rts                 ; return from subroutine

;  ______________________
; |                      \
; |   NonMaskable Int    |
; \______________________|

NMIInterrupt subroutine
        sts  L0070          ; * save stack location
        lds  #$6000         ; middle of nowhere to avoid writing any RAM
        lda  LowestSessionTPS    ; load a with memory contents
        sta  LowestHistoricalTPS ; store a into memory
        lda  #$83           ; * turn off internal RAM,
        sta  ramControl     ; * set standby power OK flag
        ldd  #$060a         ; * 7812 baud, enable Tx and Rx
        std  sciModeControl ; store d (a&b) into memory
        ldd  #$ff32         ; load d (a&b) with value -11111111 00110010-
        ; $ff = PortA_CruiseVent | PortA_PurgeEGR       | PortA_CruiseVac | PortA_ACclutch |
        ;       PortA_FAN        | PortA_Wastegate | PortA_ASD       | PortA_BaroRead
        sta  Z83_PortAdata  ; * turn off baro,asd,wastegate,fan,ac,cruise sol,egr,cruise vent
        sta  Z83_PortADataDirection
L1238:  decb                ; * this is just a short pause
        beq  L1237          ; branch if equal (zero)
        jmp  L1238

JustJump:
        jmp  NMIInterrupt

L1237:  lda  #$c3           ; * turn on internal RAM,
        sta  ramControl     ; * set standby power OK flag
        lda  Z83_PortA_copy ; load a with memory contents
        sta  Z83_PortAdata  ; transfer buffer to port
        lds  L0070          ; * restore stack location
        tst  L0070
        bne  JumpToReset    ; * if stack not empty, reset
        rti                 ; return from interrupt

JumpToReset:
        jmp  ResetRoutine

MM_DealWithColdLoadEnrichment   subroutine
        ldb  BitFlags82     ; load b with memory contents
        bmi  L1874          ; branch if b82_EngineNotRunning set
        ldb  BitFlags84     ; load b with memory contents
        bitb #$02           ;  -00000010-
        bne  L1875          ; branch if not equal (not zero)
        ldb  BitFlags81     ; load b with memory contents
        bitb #b81_bit4      ;  -00010000-
        bne  L1876          ; branch if not equal (not zero)
        ldb  HOTHLD_CurveAHotHoldTime          ; load b with memory contents (data is 15)
        lda  CoolantTemp    ; load a with memory contents
        cmpa TMPHLD_TempSwitchPointForCurveAholdAndDecayTimes   ; compare a with memory contents (data is b2)
        blo  L1879          ; branch if lower
        ldb  CLDHLD_CurveAColdHoldTime  ; load b with memory contents (data is 80)
L1879:  stb  Scratch_b6     ; store b into memory
        ldd  Counter_EngineRunningTime  ; load d (a&b) with memory contents
        bita #$c0           ;  -11000000-
        bne  L1876          ; branch if not equal (not zero)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        cmpa Scratch_b6     ; compare a with memory contents
        blo  L1881          ; branch if lower
L1876:  lda  #$ff           ; load a with value -11111111-
        sta  ColdLoadEnrichmentValue    ; store a into memory
        lda  BitFlags84     ; load a with memory contents
        ora  #$02           ;  -00000010-
        sta  BitFlags84     ; store a into memory
L1875:  ldb  DCYTMH_CurveAHotDecayTime  ; load b with memory contents (data is 04)
        lda  CoolantTemp    ; load a with memory contents
        cmpa TMPHLD_TempSwitchPointForCurveAholdAndDecayTimes ; compare a with memory contents (data is b2)
        blo  L1883          ; branch if lower
        ldb  DCYTMC_CurveAColdDecayTime          ; load b with memory contents (data is 04)
        bra  L1883          ; branch

L1881:  cmpa CCURV_HoldTimeForCurveC          ; compare a with memory contents (data is 0a)
        blo  L1874          ; branch if lower
        ldb  DCYCC_DecayRateForCurveC          ; load b with memory contents (data is 0d)
L1883:  lda  InverseRPM_B          ; load a with memory contents
        bne  L1874          ; branch if not equal (not zero)
        lda  Counter_MainLoop1    ; load a with memory contents
        bita #$1f           ;  -00011111-
        bne  L1874          ; branch if not equal (not zero)
        lda  ColdLoadEnrichmentValue          ; load a with memory contents
        sba                 ; subtract (a=a-b)
        bcc  L1887          ; branch if greater or equal
        clra                ; a = 0
L1887:  sta  ColdLoadEnrichmentValue          ; store a into memory
L1874:  rts                 ; return from subroutine

MM_ToggleSerialClock   subroutine
        lda  ioPort2data    ; load a with memory contents
        xora #$04           ; XOR a with value -00000100-
        sta  ioPort2data    ; store a into memory
        rts                 ; return from subroutine

LookupTable subroutine
        ldb  $00,x          ; * load b with number of control points
        stb  Scratch_b9     ; store b into memory
        inx                 ; increment index (x=x+1)
        cmpa $00,x          ; * check if below first control point
        bhi  .KeepScanning  ; branch if higher
.ReturnTableValue
        lda  $01,x          ; load a with memory at index + value
        tab                 ; b = a
        rts                 ; return from subroutine

.KeepScanning
        ldb  #$04           ; * 4 bytes per table entry
.NextTableEntry
        dec  Scratch_b9     ; * decrement control points remaining
        beq  .ReturnTableValue    ; * if no more points, return highest value
        cmpa $04,x          ; * check if below next control point
        blo  .FoundTableEntry    ; * if so, interpolate value
        abx                 ; * set index to next control point
        bra  .NextTableEntry    ; branch

.FoundTableEntry
        suba $00,x          ; * determine how much above control point
        sta  Scratch_b8     ; store a into memory
        ldd  $02,x          ; * get slope
        psha                ; push a onto stack
        lda  Scratch_b8     ; load a with memory contents
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
        sta  Scratch_b9     ; store a into memory
        lda  Scratch_b8     ; load a with memory contents
        pulb                ; pull b from stack
        mul                 ; multiply (d=a*b)
        addb Scratch_b9     ; b=b+memory contents
        addb $01,x          ; * add the Y value to the sloped value
        tba                 ; a = b
        rts                 ; return from subroutine

mul16bit subroutine
    ; multiplies Scratch_bb (byte) by d, leaves result in d
        psha                ; push a onto stack
        lda  Scratch_bb     ; load a with memory contents
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
        sta  Scratch_bb + 1 ; store a into memory
        lda  Scratch_bb     ; load a with memory contents
        pulb                ; pull b from stack
        mul                 ; multiply (d=a*b)
        addb Scratch_bb + 1 ; b=b+memory contents
        bcc  .Return        ; branch if greater or equal
        inca                ; increment a
.Return rts                 ; return from subroutine

Lookup5ByteTable subroutine
        ldb  #$05           ; * 5 bytes per table entry
        cmpa $00,x          ; compare a with memory at index + value
        bhi  .ChecktableEntry    ; branch if higher
        ldd  $01,x          ; * get 16-bit value from table
        rts                 ; return from subroutine

.NextTableEntry
        abx                 ; * set index to next control point
.ChecktableEntry
        cmpa $05,x          ; * check if below next control point
        bhi  .NextTableEntry    ; branch if higher
        suba $00,x
        sta  Scratch_bb     ; store a into memory
        ldd  $03,x          ; * get slope value from table
        jsr  mul16bit       ; * interpolate value
        addd $01,x          ; * add back the original Y
        rts                 ; * return with 16 bit value in d

; 16BitScaleByB
; Scale x by b, return result in d
; if a is 1: zero 2nd word first which means return only
; the increase/decrease, not the total scaled value

ScaleXbySignedB_ReturnD  subroutine
        pshb                ; push b onto stack
        lsra
        ldd  $00,x
        bcs  .1             ; branch if lower
        clra                ; a = 0
        clrb                ; b = 0
.1      std  $02,x          ; store d into index plus offset

        lda  $01,x          ; load a with memory at index + value
        pulb                ; pull b from stack
        pshb                ; push b onto stack
        mul                 ; multiply (d=a*b)
        adca $03,x
        bcc  .2             ; branch if greater or equal
        inc  $02,x
.2      sta  $03,x
        pulb                ; pull b from stack
        lda  $00,x          ; load a with memory at index + value
        mul                 ; multiply (d=a*b)
        addd $02,x
        bcc  .Return        ; branch if greater or equal
        ldd  #$ffff         ; load d (a&b) with value -11111111 11111111-
.Return rts                 ; return from subroutine

MM_HandleDistributorOverflow   subroutine
    ; on entry, d is loaded with counter
        subd ElapsedTimeSinceLastDistSync_W ; d = d - memory contents
        sta  Scratch_bb     ; store a into memory
        lda  Counter_DistTimerOverflow      ; load a with memory contents
        lsrd                ; shift right (d=d/2)
        xorb Scratch_bb
        bpl  .1             ; branch if plus (hi bit 0)
        lda  Counter_DistTimerOverflow      ; load a with memory contents
        inca                ; increment a
        beq  .1             ; branch if equal (zero)
        sta  Counter_DistTimerOverflow      ; store a into memory
.1      lda  Counter_DistTimerOverflow      ; load a with memory contents
        lsra
        cmpa InverseRPM_B     ; compare a with memory contents
        blo  .Return        ; branch if lower
        sta  InverseRPM_B     ; store a into memory
.Return rts                 ; return from subroutine

; Determine Magnitude of Transient
; *****************************************
; usually called after loading x with Block_MAP
; which would make $03 as Factor_LeanoutOrEnrichTime_MAP
;                  $02 as MapValue
; once called after loading x with Block_TPS
;                  $03 as Factor_LeanoutOrEnrichTime_TPS
;                  $02 as TpsVolts
L1346   subroutine
        lda  $03,x          ; load a with Factor_LeanoutOrEnrichTime_TPS
        bne  .1             ; branch if not equal (not zero)
        clrb                ; b = 0
        lda  $02,x          ; load a with memory at index + value
        bra  .2             ; branch

.1      sta  Scratch_bb     ; store a into memory
        ldd  $00,x
        jsr  mul16bit       ; call subroutine
        std  FuelPulsewidth_scratch1    ; store d (a&b) into memory
        ldd  $02,x
        mul                 ; multiply (d=a*b)
        subd FuelPulsewidth_scratch1    ; d = d - memory contents
        addd $00,x
.2      std  $00,x          ; store d into index plus offset
        rts                 ; return from subroutine

MM_DetermineEngineRPMs subroutine
    ; This routine converts the time between EPPs (InverseRPM) into RPMs
    ; Result returned in d
        ldx  InverseRPM_W
        ldd  #$e4e2         ; load d (a&b) with value -11100100 11100010-
        std  Scratch_b6     ; store d (a&b) into memory
        lda  #$0c           ; load a with value -00001100-
        ldb  InverseRPM_B   ; load b with memory contents
        beq  L1163          ; branch if equal (zero)
        ldx  InverseRPM_B
        lda  #$04           ; load a with value -00000100-
L1163:  stx  Scratch_b8     ; store index into memory
        sta  Scratch_ba     ; store a into memory
        ldx  #Scratch_b6    ; load index with value
        ldd  $02,x
        bne  .7             ; branch if not equal (not zero)
        coma                ; complement (toggle all bits)
        comb                ; complement (toggle all bits)
        bra  .1

.7      bmi  .5             ; branch if minus(hi bit 1)
.6      inc  $04,x
        asld                ; shift left (d=d*2)
        bpl  .6             ; branch if plus (hi bit 0)
        std  $02,x          ; store d into index plus offset
.5      ldd  Scratch_b6     ; load d (a&b) with memory contents
        clr  $00,x
        clr  $01,x
.4      subd $02,x
        bcc  .3             ; branch if greater or equal
        addd $02,x
        clc                 ; clear carry
        bra  .2             ; branch

.3      sec                 ; set carry
.2      rol  $01,x
        rol  $00,x
        lsr  $02,x
        ror  $03,x
        dec  $04,x
        bne  .4             ; branch if not equal (not zero)
        ldd  $00,x
.1      rts                 ; return from subroutine

;  ______________________
; |                      \
; |   O2 Feedback        |
; \______________________|
; ********************************************************
; O2 feedback is important for part throttle fueling and
; overall emissions and economy.
; Closed Loop = using O2 feedback
; Open Loop   = not using O2 feedback, usually for a cold
;               motor or at full throttle or maxed out
;               part throttle enrichment.
;
; The system will not switch into closed loop mode until a
; timer has expired, since on a cold motor the heater in
; the O2 sensor needs time to bring the sensor to
; its operating temperature.
; ********************************************************

MM_CheckIfO2Valid subroutine
        lda  BitFlags81     ; load a with memory contents  bit0 = FullThrottle
        ldb  L1246          ; load b with memory contents (data is 85)
        addb LowestSessionTPS    ; b=b+memory contents
        bcs  L1247          ; branch if lower
        cmpb #$d0           ;  -11010000-
        blo  L1248          ; branch if lower
L1247:  ldb  #$d0           ; load b with value -11010000-
L1248:  lsra
        bcc  L1249          ; branch if b81_FullThrottle is clear
        subb L1250          ; subtract memory contents from b (data is 02)
L1249:  cmpb TpsVolts
        bhs  L1251          ; branch if greater or equal
        rola
        bra  L1252          ; branch

L1251:  lsla                ; logical shift left a
        sta  BitFlags81     ; store a into memory
        ldb  MapValue       ; load b with memory contents
        cmpb OverboostConstant1    ; compare b with memory contents (data is aa)
        bhs  L1253          ; branch if greater or equal
        ldb  L00CA          ; load b with memory contents
        cmpb L1254          ; compare b with memory contents (data is 2b)
        bhs  L1253          ; branch if greater or equal
        bita #$10           ;  -00010000-
        bne  L1255          ; branch if not equal (not zero)
        lda  CoolantTemp    ; load a with memory contents
        cmpa CLTEMP_O2TempToStartClosedLoopTimer    ; compare a with memory contents (data is 96)
        blo  L1253          ; branch if lower
        lda  Timer_CLTIME_ClosedLoopTimer ; load a with memory contents
        cmpa O2TIME_O2DelayTimeToClosedLoop ; compare a with memory contents (data is 15)
        bhs  CheckIfO2SensorDataIsValid   ; branch if greater or equal
        ldb  BitFlags82     ; load b with memory contents
        bmi  L1253          ; branch if b82_EngineNotRunning set
        ldb  Counter_MainLoop1    ; load b with memory contents
        bitb #$1f           ;  -00011111-
        bne  L1253          ; branch if not equal (not zero)
        inca                ; increment a
        sta  Timer_CLTIME_ClosedLoopTimer ; store a into memory
        bra  L1253          ; branch

CheckIfO2SensorDataIsValid:
        lda  RawO2Sensor    ; load a with memory contents
        cmpa O2TLMH_O2HighLimitDelayedClosedLoop    ; compare a with memory contents (data is 1f)
        bhs  L1257          ; branch if greater or equal
        cmpa O2TLML_O2LowLimitDelayedClosedLoop    ; compare a with memory contents (data is 0f)
        bhs  L1253          ; branch if greater or equal
        ; O2 sensor has now traveled outside of the .35 to .55 volt range, which means
        ; it is likely returning real data.
L1257:  clra                ; a = 0
        sta  L00B1          ; store a into memory
        sta  Timer_CLTIME_ClosedLoopTimer ; store a into memory
        lda  BitFlags81     ; load a with memory contents
        ora  #b81_bit4      ;  -00010000-
L1252:  sta  BitFlags81     ; store a into memory
L1253:  clra                ; a = 0
        sta  SecondaryO2ControllerKicks_Signed          ; store a into memory
        rts                 ; return from subroutine

L1255:  clrb                ; b = 0
        lda  RawO2Sensor    ; load a with memory contents
        cmpa O2MIDH_O2HighLimitForMidLevelDetermination    ; compare a with memory contents (data is 1c)
        bhs  L1258          ; branch if greater or equal
        cmpa O2MIDL_O2LowLimitForMidLevelDetermination    ; compare a with memory contents (data is 12)
        blo  L1258          ; branch if lower
        ldb  Timer_CLTIME_ClosedLoopTimer ; load b with memory contents
        cmpb #$7f           ;  -01111111-
        bls  L1259          ; branch if lower or same
        jmp  L1260

L1259:  incb
L1258:  stb  Timer_CLTIME_ClosedLoopTimer ; store b into memory
        clrb                ; b = 0
        cmpa O2HSWP_O2HighLimitForRampSwitching ; compare a with memory contents (data is 19)
        bhs  O2Signal_AboveToggle          ; branch if RICH
        cmpa O2LSWP_O2LowLimitForRampSwitching  ; compare a with memory contents (data is 18)
        blo  O2Signal_BelowToggle          ; branch if LEAN
        ; return immediately if stoich (18)
        rts                 ; return from subroutine

O2Signal_AboveToggle:
        decb                ; b was cleared, so is now $ff
O2Signal_BelowToggle:
        lda  BitFlags84     ; load a with memory contents
        bita #b84_bit4      ;  -00010000-
        bne  L1263          ; branch if not equal (not zero)
        clra                ; a = 0
        xorb BitFlags81
        andb #b81_O2Rich      ;  -00000100-
        beq  L1264          ; branch if equal (zero)
        xorb BitFlags81
        stb  BitFlags81     ; store b into memory
        clrb                ; b = 0
        lda  Timer_OutOfClosedLoopMode          ; load a with memory contents
        stb  Timer_OutOfClosedLoopMode          ; store b into memory
        stb  L00CB          ; store b into memory
        ldb  #$3f           ; load b with value -00111111-
        sei                 ; disable interrupts (set)
        andb CylinderRetard3
        stb  CylinderRetard3  ; store b into memory
        cli                 ; enable interrupts (clear)
        bra  L1265          ; branch

L1264:  ldb  Timer_OutOfClosedLoopMode    ; load b with memory contents
        beq  L1263          ; branch if equal (zero)
L1265:  sta  Scratch_b6     ; store a into memory
        ldx  #PRIRMP_PrimaryO2Ramp ; load index with value
        jsr  DetermineRampCellFromRPM          ; call subroutine
        ldb  Timer_OutOfClosedLoopMode  ; load b with memory contents
        addb $00,x          ; add memory at index + value to b
        sba                 ; subtract (a=a-b)
        bcc  L1267          ; branch if greater or equal
        adda $00,x
        bpl  L1268          ; branch if plus (hi bit 0)
        clra                ; a = 0
L1268:  clrb                ; b = 0
        stb  Timer_OutOfClosedLoopMode          ; store b into memory
        bra  L1269          ; branch

L1267:  lda  $00,x          ; load a with memory at index + value
        tab                 ; b = a
        addb Timer_OutOfClosedLoopMode     ; b=b+memory contents
        stb  Timer_OutOfClosedLoopMode     ; store b into memory
L1269:  ldb  Scratch_b6     ; load b with memory contents
        beq  L1270          ; branch if equal (zero)
        cba
        bcs  L1270          ; branch if lower
        tba                 ; a = b
L1270:  ldb  BitFlags81     ; load b with memory contents
        bitb #b81_O2Rich      ;  -00000100-
        beq  L1271          ; branch if equal (zero)
        nega                ; negate a (set high bit)
L1271:  jsr  L1272          ; call subroutine
        clra                ; a = 0
        sta  L00B1          ; store a into memory
L1275:  lda  Counter_MainLoop1    ; load a with memory contents
        bne  L1273          ; branch if not equal (not zero)
        inc  L00B1
L1273:  rts                 ; return from subroutine

L1263:  lda  L00B1          ; load a with memory contents
        cmpa L1274          ; compare a with memory contents (data is fe)
        blo  L1275          ; branch if lower
        lda  CylinderRetard3  ; load a with memory contents
        anda #$c0           ; AND a with value -11000000-
        cmpa #$c0           ; compare a with value -11000000-
        bne  L1276          ; branch if not equal (not zero)
        lda  #b80_bit5      ; load a with value -00100000-
        sei                 ; disable interrupts (set)
        ora  BitFlags80     ; OR a with memory contents
        sta  BitFlags80     ; store a into memory
        bra  L1277          ; branch

L1276:  lda  #$40           ; load a with value -01000000-
        sei                 ; disable interrupts (set)
        adda CylinderRetard3
        sta  CylinderRetard3  ; store a into memory
L1277:  cli                 ; enable interrupts (clear)
        ldx  #FaultCode1    ; load index with value
        ldb  #$08           ; load b with value -00001000-
        lda  BitFlags81     ; load a with memory contents
        bita #b81_O2Rich      ;  -00000100-
        beq  L1278          ; branch if equal (zero)
        lsrb                ; logical shift right b
L1278:  jsr  SetNonCriticalFaultCode    ; call subroutine

        ldx  #AutoCalCell0 ; load index with value
.loop   sta  $00,x
        inx                 ; increment index (x=x+1)
        cpx  #AutoCalCellEnd
        bne  .loop          ; branch if not equal (not zero)

L1260:  clra                ; a = 0
        sta  L00B1          ; store a into memory
        sta  EPPsSinceCalChange           ; store a into memory
        sta  Timer_CLTIME_ClosedLoopTimer ; store a into memory
        ldb  BitFlags81     ; load b with memory contents
        andb #<~b81_bit4    ; clear bit -11101111-
        stb  BitFlags81     ; store b into memory
        rts                 ; return from subroutine

; *****************************************

MM_HandleO2Feedback_v    subroutine
        lda  BitFlags84     ; load a with memory contents
        bita #$10           ;  -00010000-
        beq  WorkWithSECRMP ; branch if equal (zero)
L1280:  clra                ; a = 0
        sta  L00CB          ; store a into memory
        rts                 ; return from subroutine

WorkWithSECRMP:
        ldb  BitFlags81     ; load b with memory contents
        bitb #b81_bit4      ;  -00010000-
        beq  L1280          ; branch if equal (zero)
        ldx  #SECRMP_SecondaryO2Ramp    ; load index with value
        jsr  DetermineRampCellFromRPM          ; call subroutine
        sei                 ; disable interrupts (set)
        lda  L00CB          ; load a with memory contents
        sba                 ; subtract (a=a-b)
        bcs  L1281          ; branch if lower
        sta  L00CB          ; store a into memory
        cli                 ; enable interrupts (clear)
        lda  $00,x          ; load a with memory at index + value
L1272:  ldb  O2LIMH_O2RangeLimitHigh    ; load b with memory contents (data is 7f)
        adda SecondaryO2ControllerKicks_Signed
        bvc  L1282          ; branch if overflow
        lda  SecondaryO2ControllerKicks_Signed          ; load a with memory contents
        bpl  L1283          ; branch if plus (hi bit 0)
        ldb  O2LIML_O2RangeLimitLow    ; load b with memory contents (data is b3)
        tba                 ; a = b
L1282:  cba
        bge  L1283          ; branch if greater or equal
        ldb  O2LIML_O2RangeLimitLow    ; load b with memory contents (data is b3)
        cba
        bge  L1284          ; branch if greater or equal
L1283:  tba                 ; a = b
L1284:  ldb  BitFlags80     ; load b with memory contents
        bitb #b80_bit5      ;  -00100000-
        beq  L1285          ; branch if equal (zero)
        tsta                ; test a
        bmi  L1281          ; branch if minus(hi bit 1)
L1285:  sta  SecondaryO2ControllerKicks_Signed          ; store a into memory
L1281:  cli                 ; enable interrupts (clear)
        rts                 ; return from subroutine

; *****************************************
; Process O2 Sensor Feedback
; *****************************************
; First, x is loaded with the address of one of the ramps, either
; Primary or Secondary, then this routine is called.
; *****************************************
DetermineRampCellFromRPM subroutine
        lda  BitFlags84     ; load a with memory contents
        bita #$40           ;  -01000000-     FALL TO IDLE
        bne  L1286          ; branch if not equal (not zero)
        lda  BitFlags81     ; load a with memory contents
        bmi  L1286          ; branch if b81_OffIdle is set
        ; idle: jump directly to the idle cell offset
        ldb  #$16           ; load b with value -00010110-
        bra  L1287          ; branch

L1286:  clrb                ; b = 0
        lda  RPM_W          ; load a with memory contents
L1289:  cmpa $00,x          ; compare a with "Lower RPM boundary"
        blo  L1288          ; branch if lower
        incb
        inx                 ; increment index (x=x+1)
        cmpb #$02           ;  -00000010-
        blo  L1289          ; branch if lower
        bra  L1290          ; branch

L1288:  tstb
        bne  L1291          ; branch if not equal (not zero)
        inx                 ; increment index (x=x+1) "Upper RPM boundary"
L1291:  inx                 ; increment index (x=x+1) "Lower MAP boundary"
L1290:  lda  MapValue            ; load a with memory contents
L1293:  cmpa $00,x          ; compare a with memory at index + value
        blo  L1292          ; branch if lower
        addb #$03           ;  -00000011-
        inx                 ; increment index (x=x+1) "Upper MAP boundary"
        cmpb #$06           ;  -00000110-
        blo  L1293          ; branch if lower
        bra  L1294          ; branch

L1292:  cmpb #$03           ;  -00000011-
        bhs  L1295          ; branch if greater or equal
        inx                 ; increment index (x=x+1)
L1295:  inx                 ; increment index (x=x+1)
L1294:  lslb
        lda  BitFlags81     ; load a with memory contents
L1287:  bita #b81_O2Rich      ;  -00000100-
        beq  L1296          ; branch if equal (zero)
        incb
L1296:  abx                 ; add b to index
        lda  $14,x          ; load a with memory at index + value "Limits"
        tab                 ; b = a
        rts                 ; return from subroutine
        ; at return, x points to the kick, a and b are the limits?

; *****************************************

MM_DetermineAdaptiveMemoryCell   subroutine
    ; Determine which AutoCal cell is currently active:
        clra                ; a = 0
        sta  Scratch_b6     ; store a into memory
        ldb  CurrentAutoCalCell   ; load b with memory contents
        lda  BitFlags81     ; load a with memory contents
        bmi  .findcell      ; * branch if off idle
        lda  TargetIdleSpeed    ; load a with memory contents
        adda HYTRPM_AutocalHysterisisForIDLOFS    ;  (data is 02)
        cmpb #$0c           ; cell 12: already at idle
        bne  L1896          ; branch if not equal (not zero)
        adda IDLOFS_AutocalRPMDeltaForIdleRPMforIdleCell    ;  (data is 01)
L1896:  cmpa RPM_W          ; compare a with memory contents
        blo  .findcell      ; branch if lower
        ldb  #$0c           ; cell 12: idle
        bra  L1897          ; branch

.findcell
        ldx  #MP1_AutocalMapCellBoundary1    ; load index with value

.cellLoop
        lda  $00,x          ; load a with memory at index + value
        cmpb Scratch_b6
        bhs  L1898          ; branch if greater or equal
        adda DLTMAP_AutocalMapHysteresisAtMAPCellBoundaries    ;  (data is 01)
L1898:  cmpa MapValue            ; compare a with memory contents
        bhs  L1899          ; branch if greater or equal
        inx                 ; increment index (x=x+1)
        inc  Scratch_b6
        inc  Scratch_b6
        cpx  #MapCellBoundaryEnd  ; one byte above the top value
        bne  .cellLoop      ; branch if not equal (not zero)

L1899:  lda  R1_AutocalRPMCellBoundary    ; load a with memory contents (data is 1d)
        lsrb                ; logical shift right b
        bcs  L1901          ; branch if lower
        adda DLTRPM_AutocalDeltaRPMatRPMBoundaries    ;  (data is 01)
L1901:  ldb  Scratch_b6     ; load b with memory contents
        cmpa RPM_W          ; compare a with memory contents
        bhs  L1897          ; branch if greater or equal
        incb

L1897:  cmpb CurrentAutoCalCell
        beq  .Return        ; branch if equal (zero)
        clra                ; a = 0
        sta  EPPsSinceCalChange  ; store a into memory
        stb  CurrentAutoCalCell   ; store b into memory
.Return rts                 ; return from subroutine

; *****************************************

MM_PerformAutoCalAdjustments   subroutine
        lda  CoolantTemp    ; load a with memory contents
        ldb  BitFlags82     ; load b with memory contents
        bpl  L1297          ; branch if b82_EngineNotRunning clear
        ldb  AMTIMH_AutocalAMInhibitTimeHot    ; load b with memory contents (data is 40)
        cmpa AMTPSW_AutocalTempPointToTurnOnSelectedAMTimer    ; compare a with memory contents (data is cd)
        bhs  L1300          ; branch if greater or equal
        ldb  AMTIMC_AutocalAMInhibitTimeCold   ; load b with memory contents (data is 0c)
L1300:  stb  Timer_AutoCalInhibit       ; store b into memory
        rts                 ; return from subroutine

L1297:  cmpa AMTEMP_AutocalStartingTempToSelectHotOrColdAMtimer ; compare a with memory contents (data is cd)
        blo  L1303          ; branch if lower
        ldb  Timer_AutoCalInhibit ; load b with memory contents
        beq  L1304          ; branch if equal (zero)
        lda  Counter_MainLoop1     ; load a with memory contents
        bne  L1303          ; branch if not equal (not zero)
        decb
        stb  Timer_AutoCalInhibit ; store b into memory
L1303:  rts                 ; return from subroutine

L1304:  ldd  Pulsewidth_InjectorSolution       ; load d (a&b) with memory contents
        addd Pulsewidth_PartThrottleEnrichment ; d = d + memory contents
        subd PWMIN_AutocalNoCellUpdateBelowThisValue  ;  (data is 1200)
        bcs  .return        ; branch if lower

        lda  L00CA          ; load a with memory contents
        cmpa L1254          ; compare a with memory contents (data is 2b)
        bhs  .return        ; branch if greater or equal

        ; check if O2 sensor is returning valid data, return if not:
        lda  BitFlags81     ; load a with memory contents
        bita #b81_bit4      ;  -00010000-
        beq  .return        ; branch if equal (zero)

        ; check if at full throttle, return if so:
        lsra
        bcs  .return        ; branch if lower

        clrb                ; b = 0
        anda #$02           ; AND a with value -00000010-
        beq  L1306          ; branch if equal (zero)
        decb                ; b = $ff
L1306:  lda  CALTHR_AutocalO2CtrlDeadbandLimitForUpdatingCells ; load a with memory contents (data is 09)
        stb  Scratch_b6     ; store b into memory
        bmi  L1307          ; branch if minus(hi bit 1)
        nega                ; negate a (set high bit)
L1307:  adda SecondaryO2ControllerKicks_Signed
        bvs  L1308          ; branch if no overflow
        xora Scratch_b6
        bmi  .return        ; branch if minus(hi bit 1)
        
L1308:  ldb  Timer_Leanout  ; load b with memory contents
        bne  .return        ; branch if not equal (not zero)

        ldb  MapValue            ; load b with memory contents
        cmpb MAPLIM_AutocalNoCellUpdateAboveThisMAP    ; compare b with memory contents (data is 5c)
        bhi  .return        ; branch if higher

        ldb  RPM_W          ; load b with memory contents
        cmpb RPMLIM_AutocalNoCellUpdateAboveThisRPM    ; compare b with memory contents (data is 2b)
        bhi  .return        ; branch if higher

        ldb  EPPsSinceCalChange             ; load b with memory contents
        cmpb AutoCalEppsBetweenCalChanges   ; compare b with memory contents (data is 1a)
        blo  .return        ; branch if lower

        ldb  SecondaryO2ControllerKicks_Signed  ; load b with memory contents
        beq  .return        ; branch if equal (zero)

        ; we're about to make a cal change:
        clra                ; a = 0
        sta  EPPsSinceCalChange ; store a into memory
        lda  CELCHG_AutocalBitsChangePerUpdateInCell   ; load a with memory contents (data is 01)
        tstb
        bpl  L1311          ; branch if plus (hi bit 0)
        nega                ; negate a (set high bit)

L1311:  ldx  #AutoCalCell0  ; load index with value
        ldb  CurrentAutoCalCell   ; load b with memory contents
        abx                 ; add b to index
        ldb  AMLIMH_AutocalRangeOfAuthorityHigh ; load b with memory contents (data is 7f)
        adda $00,x
        bvc  L1313          ; branch if overflow
        lda  $00,x          ; load a with memory at index + value
        bpl  L1314          ; branch if plus (hi bit 0)
        ldb  AMLIML_AutocalRangeOfAuthorityLow ; load b with memory contents (data is b3)
        tba                 ; a = b
L1313:  cba
        bge  L1314          ; branch if greater or equal
        ldb  AMLIML_AutocalRangeOfAuthorityLow ; load b with memory contents (data is b3)
        cba
        bge  L1316          ; branch if greater or equal
L1314:  tba                 ; a = b
L1316:  sta  $00,x
.return
        rts                 ; return from subroutine

;  ______________________
; |                      \
; |   Idle Control       |
; \______________________|

MM_DetermineIdleAdjustmentRequirement subroutine
        clra                ; a = 0
        ldx  RPM_W
        cpx  TargetIdleSpeed
        ; If engine speed is below TargetIdle, cancel the delay timer and start controlling immediately
        bls  .1             ; branch if lower or same
        lda  Timer_AISDelay ; load a with memory contents
        beq  .2             ; branch if equal (zero)
        deca                ; decrement a
.1      sta  Timer_AISDelay ; store a into memory
.2      lda  BitFlags81     ; load a with memory contents
        bmi  .4             ; * branch if throttle pressed
        ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        sta  Scratch_b6     ; store a into memory
        lda  MPHREF_SwitchPointBetweenDecelIdle          ; load a with memory contents (data is 06)
        ldb  BitFlags84     ; load b with memory contents
        bitb #$40           ;  -01000000-
        bne  .3             ; branch if not equal (not zero)
        adda #$03           ;  -00000011-
.3      orb  #$40           ;  -01000000-
        cmpa Scratch_b6     ; compare a with memory contents
        blo  .5             ; branch if lower
.4      ldb  BitFlags84     ; load b with memory contents
        andb #$bf           ;  -10111111-
.5      stb  BitFlags84     ; store b into memory
        rts                 ; return from subroutine

; *****************************************

MM_ControlAIS  subroutine
        lda  BitFlags82     ; load a with memory contents
        ora  L004F          ; OR a with memory contents
        ora  BitFlags81     ; OR a with memory contents
        bmi  L1317          ; branch if b81_OffIdle is set
        ldb  InverseRPM_B   ; load b with memory contents
        orb  Timer_AISDelay
        beq  L1318          ; branch if equal (zero)
L1317:  rts                 ; return from subroutine

L1318:  lda  BitFlags80     ; load a with memory contents
        bpl  L1319          ; branch if b80_BadMAP is clear
        clra                ; a = 0
        jmp  L1320

L1319:  lda  L0041          ; load a with memory contents
        lsra
        lsra
        adda DANOM_AisBase  ;  (data is 10)
        bcs  L1321          ; branch if lower
        ldb  BitFlags84     ; load b with memory contents
        bitb #$40           ; * check if car is moving
        beq  L1322          ; branch if equal (zero)
        sta  Scratch_b7     ; store a into memory
        ldx  #TIMEDC_AisDecelTime    ; load index with value
        lda  Counter_EngineRunningTime          ; load a with memory contents
        jsr  LookupTable    ; call subroutine
        lda  Scratch_b7     ; load a with memory contents
        aba                 ; a=a+b
        bcs  L1321          ; branch if lower
        sta  Scratch_b7     ; store a into memory
        ldx  #TEMPDC_AisDecelTemp    ; load index with value
        lda  CoolantTemp    ; load a with memory contents
        jsr  LookupTable    ; call subroutine
        ldb  Scratch_b7     ; load b with memory contents
        aba                 ; a=a+b
        bcs  L1321          ; branch if lower
        ldb  Z83_PortCdata  ; load b with memory contents
        bitb #$04           ; * check if a/c clutch input
        bne  L1322          ; branch if not equal (not zero)
        adda DAACF_AisAcFactor    ;  (data is 09)
        bcc  L1322          ; branch if greater or equal
L1321:  lda  #$ff           ; load a with value -11111111-
L1322:  ldb  InverseRPM_W   ; load b with memory contents
        mul                 ; multiply (d=a*b)
        asld                ; shift left (d=d*2)
        bcs  L1323          ; branch if lower
        sta  Scratch_b6     ; store a into memory
        ldx  #MAPFLR_AisAdditionToControlMap    ; load index with value
        ldd  RPM_W          ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        jsr  LookupTable    ; call subroutine
        sta  Scratch_b7     ; store a into memory
        ldx  #MPTEMP_AisTempTermForMapOffset    ; load index with value
        lda  CoolantTemp    ; load a with memory contents
        jsr  LookupTable    ; call subroutine
        ldb  Scratch_b7     ; load b with memory contents
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
        sta  Scratch_b7     ; store a into memory
        ldx  #BARFLR_AisBarometricFloor    ; load index with value
        lda  BarometricPressure    ; load a with memory contents
        jsr  LookupTable    ; call subroutine
        ldb  Scratch_b7     ; load b with memory contents
        mul                 ; multiply (d=a*b)
        asld                ; shift left (d=d*2)
        bcs  L1323          ; branch if lower
        tstb
        bpl  L1324          ; branch if plus (hi bit 0)
        inca                ; increment a
        beq  L1323          ; branch if equal (zero)
L1324:  adda Scratch_b6
        bcc  L1325          ; branch if greater or equal
L1323:  lda  #$ff           ; load a with value -11111111-
L1325:  ldb  BitFlags84     ; load b with memory contents
        bitb #$40           ;  -01000000-
        bne  L1326          ; branch if not equal (not zero)
        ldb  TRANSM_Transmission    ; load b with memory contents (data is 00)
        beq  L1327          ; * branch if manual transmission
        ldb  Z83_PortCdata  ; load b with memory contents
        bitb #$08           ; * check park/neutral switch
        beq  L1328          ; branch if equal (zero)
L1327:  ldb  Counter_EngineRunningTime          ; load b with memory contents
        cmpb DCALTM_AisDacalInhibitAfterStart    ; compare b with memory contents (data is 15)
        blo  L1328          ; branch if lower
        ldb  Z83_PortCdata  ; load b with memory contents
        andb #$04           ; * check if a/c clutch input
        beq  L1328          ; branch if equal (zero)
        ldb  CoolantTemp    ; load b with memory contents
        cmpb DATHOT_AisCalEngineTemp    ; compare b with memory contents (data is cd)
        bls  L1328          ; branch if lower or same
        ldb  Counter_MainLoop1    ; load b with memory contents
        bitb L1329          ;  (data is 7e)
        bne  L1330          ; branch if not equal (not zero)
        ldb  TargetIdleSpeed    ; load b with memory contents
        subb RPM_W          ; b = b-memory contents
        bcc  L1331          ; branch if greater or equal
        negb
L1331:  cmpb DADRPM_AisDeltaRpmFromIdle    ; compare b with memory contents (data is 02)
        bhi  L1330          ; branch if higher
        ldb  L0041          ; load b with memory contents
        cmpa MapValue            ; compare a with memory contents
        bhi  L1332          ; branch if higher
        incb
        bne  L1333          ; branch if not equal (not zero)
L1332:  tstb
        beq  L1330          ; branch if equal (zero)
        decb
L1333:  stb  L0041          ; store b into memory
L1330:  ldd  RPM_W          ; load d (a&b) with memory contents
        subd TargetIdleSpeed    ; d = d - memory contents
        bcc  L1334          ; branch if greater or equal
        coma                ; complement (toggle all bits)
        comb                ; complement (toggle all bits)
        addd #ioPort2ddr    ;  -00000000 00000001-
L1334:  asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        bcs  L1328          ; branch if lower
        asld                ; shift left (d=d*2)
        bcs  L1328          ; branch if lower
        cmpa #$0a           ; compare a with value -00001010-
        bhs  L1328          ; branch if greater or equal
        lda  L0042          ; load a with memory contents
        cmpa L00E2          ; compare a with memory contents
        bne  L1328          ; branch if not equal (not zero)
        lda  L00E2          ; load a with memory contents
        cmpa CALPOS_AisPositionGuess    ; compare a with memory contents (data is 1a)
        beq  L1328          ; branch if equal (zero)
        bhi  L1335          ; branch if higher
        inca                ; increment a
        inca                ; increment a
L1335:  deca                ; decrement a
        sta  L00E2          ; store a into memory
        sta  L0042          ; store a into memory
L1328:  rts                 ; return from subroutine

L1326:  ldb  CoolantTemp    ; load b with memory contents
        cmpb L1336          ; compare b with memory contents (data is cd)
        blo  L1320          ; branch if lower
        ldb  RPM_W          ; load b with memory contents
        subb TargetIdleSpeed    ; b = b-memory contents
        bcs  L1320          ; branch if lower
        cmpb L1337          ; compare b with memory contents (data is 00)
        blo  L1338          ; branch if lower
L1320:  ldb  Counter_L00B3  ; load b with memory contents
        cmpb DCLFRQ_AisDecelPulseTime    ; compare b with memory contents (data is 3f)
        blo  L1338          ; branch if lower
        ldb  L0060          ; load b with memory contents
        andb #$f7           ;  -11110111-
        stb  L0060          ; store b into memory
        cmpa DCLMN2_NoAISControlIfMapBelowThis  ; compare a with memory contents (data is 0d)
        bhs  L1339          ; branch if greater or equal
        lda  DCLMN2_NoAISControlIfMapBelowThis  ; load a with memory contents (data is 0d)
L1339:  ldx  #DAISOP_AisDecelOpenSteps    ; load index with value
        suba MapValue        
        bcc  L1340          ; branch if greater or equal
        nega                ; negate a (set high bit)
        ldx  #DAISCL_AisDecelCloseSteps    ; load index with value
L1340:  jsr  LookupTable    ; call subroutine
        jsr  L1341          ; call subroutine
        cmpa L0042          ; compare a with memory contents
        beq  L1338          ; branch if equal (zero)
        clra                ; a = 0
        sta  Counter_L00B3  ; store a into memory
L1338:  rts                 ; return from subroutine

MM_LDAB5   subroutine
        ldb  #$80           ; load b with value -10000000-
        lda  L00E2          ; load a with memory contents
        suba L0042
        bcc  L1908          ; branch if greater or equal
        clrb                ; b = 0
        nega                ; negate a (set high bit)
L1908:  xorb L0060
        lslb
        ldb  L0060          ; load b with memory contents
        bcc  L1909          ; branch if greater or equal
        xorb #$80           ;  -10000000-
        bra  L1910          ; branch

L1909:  cmpa #$04           ; compare a with value -00000100-
        blo  L1910          ; branch if lower
        bitb #$03           ;  -00000011-
        beq  L1911          ; branch if equal (zero)
        decb
        bra  L1912          ; branch

L1910:  orb  #$03           ;  -00000011-
L1912:  andb #$df           ;  -11011111-
        bra  L1913          ; branch

L1911:  orb  #$20           ;  -00100000-
L1913:  stb  L0060          ; store b into memory
        rts                 ; return from subroutine

MM_L1600   subroutine
        lda  BitFlags84     ; load a with memory contents
        bita #$40           ;  -01000000-
        bne  L1914          ; branch if not equal (not zero)
        lda  BitFlags80     ; load a with memory contents
        bmi  L1914          ; branch if b80_BadMAP is set
        lda  L00E2          ; load a with memory contents
        ora  L0042          ; OR a with memory contents
        bne  L1914          ; branch if not equal (not zero)
        ldb  Counter_MainLoop1    ; load b with memory contents
        bitb #$7e           ;  -01111110-
        bne  L1914          ; branch if not equal (not zero)
        adda #$02           ;  -00000010-
        sta  L00E2          ; store a into memory
        sta  L0042          ; store a into memory
        lda  L0060          ; load a with memory contents
        bita #$10           ;  -00010000-
        bne  L1914          ; branch if not equal (not zero)
        lda  #$10           ; load a with value -00010000-
        ora  BitFlags85     ; OR a with memory contents
        sta  BitFlags85     ; store a into memory
        lda  #$3f           ; load a with value -00111111-
        jsr  SetFaultCode   ; call subroutine
        bcc  L1914          ; branch if greater or equal
        lda  L0060          ; load a with memory contents
        ora  #$10           ;  -00010000-
        sta  L0060          ; store a into memory
L1914:  rts                 ; return from subroutine

;  ______________________
; |                      \
; |   Fueling            |
; \______________________|

MM_DetermineFuelTweaks subroutine
        lda  BarometricPressure
        ldx  #PWADD_FuelAddBaroComp
        jsr  LookupTable                ; result is in b
        lda  InverseRPM_W               ; load a with memory contents
        tst  InverseRPM_B
        beq  L1342                      ; branch if equal (zero)
        lda  #$ff                       ; load a with value -11111111-
        ; a contains the factor, if low RPMs it is 1.0, higher RPMs will be a lower factor
L1342:  mul                             ; multiply (d=a*b)
        lsrd                            ; shift right (d=d/2)
        lsrd                            ; shift right (d=d/2)
        lsrd                            ; shift right (d=d/2)
        lsrd                            ; shift right (d=d/2)
        lsrd                            ; shift right (d=d/2)
        std  Scratch_b6                 ; store d (a&b) into memory

        ldx  #FuelLeanoutFactorOnThrottle  ; load index with value
        lda  BitFlags81                 ; load a with memory contents
        bmi  L1343                      ; branch if b81_OffIdle is set
        ldx  #FuelLeanoutFactorOffThrottle ; load index with value
L1343:  ldd  RPM_W                      ; load d (a&b) with memory contents
        asld                            ; shift left (d=d*2)
        jsr  LookupTable                ; call subroutine
        clra                            ; a = 0
        tstb
        bpl  L1344                      ; branch if plus (hi bit 0)
        deca                            ; decrement a
L1344:  asld                            ; shift left (d=d*2)
        asld                            ; shift left (d=d*2)
        asld                            ; shift left (d=d*2)
        addd Scratch_b6                 ; d = d + memory contents

        std  LeanoutFactor_W            ; store d (a&b) into memory

        ldx  #BATOFF_FuelBatteryOffset  ; load index with value
        lda  RawBattVoltage             ; load a with memory contents
        jsr  LookupTable                ; call subroutine
        sta  FuelPulsewidthAddedForBatteryVoltage ; store a into memory

        ldb  MapValue                   ; load b with memory contents
        cmpb MapAverageValue
        bhs  L1345                      ; branch if greater or equal
        ldb  Counter_MainLoop1          ; load b with memory contents
        bitb #$07                       ;  -00000111-
        bne  L1345                      ; branch if not equal (not zero)
        ldx  #Block_MAP                 ; load index with value
        sei                             ; disable interrupts (set)
        jsr  L1346                      ; call subroutine
        cli                             ; enable interrupts (clear)
L1345:  ldd  Pulsewidth_InjectorSolution       ; load d (a&b) with memory contents
        addd Pulsewidth_PartThrottleEnrichment ; d = d + memory contents
        subd PWMIN_AutocalNoCellUpdateBelowThisValue  ; (data is 1000)
        lda  BitFlags84                 ; load a with memory contents
        bcs  L1347                      ; branch if lower
        bita #$20                       ;  -00100000-
        bne  L1347                      ; branch if not equal (not zero)
        anda #$ef                       ; AND a with value -11101111-
        bra  L1348                      ; branch

L1347:  ora  #$10                       ;  -00010000-
L1348:  sta  BitFlags84                 ; store a into memory
        rts                             ; return from subroutine

; *****************************************

MM_DetermineStartingFuel_v subroutine
        lda  BitFlags82                 ; load a with memory contents
        bita #b82_DecayingFuelStartToRun ; test bit -00001000-
        beq  .1                         ; branch if not still processing starting decay (zero)
        ldx  #SFDCAY_StartDecayIntoRun  ; load index with value
        lda  CoolantTemp                ; load a with memory contents
        jsr  LookupTable                ; call subroutine
        sta  FuelSubtractedFromStartToRun ; store a into memory
.1      lda  BitFlags82                 ; load a with memory contents
        bpl  .Return                    ; branch if b82_EngineNotRunning clear

        ; IF full throttle while cranking, set fuel to zero
        ; This is to crank away flooding.
        lda  BitFlags81                 ; load a with memory contents
        lsra
        ldd  #0                         ; load d (a&b) with value -00000000 00000000-
        bcs  .4                         ; branch if b81_FullThrottle is set

        jsr  CalculateHeatSoakAdjustment ; call subroutine
        ldx  #STRTFL_StartingFuelFromEngineTemp    ; load index with value
        ldb  BitFlags81                 ; load b with memory contents
        bmi  .2                         ; branch if throttle pressed (off idle) (hi bit 1)

        cmpa STRLMT_StartingFuelColdestTemp ; compare a with memory contents (data is 63)
        bhs  .2                         ; branch if greater or equal
        lda  STRLMT_StartingFuelColdestTemp ; load a with memory contents (data is 63)

.2      jsr  LookupTable                ; call subroutine
        ; square the result:
        mul                             ; multiply (d=a*b)
        std  Scratch_b6                 ; store d (a&b) into memory

        lda  BarometricPressure         ; load a with memory contents
        ldx  #STFBAR_StartFuelBaro      ; load index with value
        jsr  LookupTable                ; call subroutine
        sta  Scratch_ba                 ; store a into memory
        lda  CoolantTemp                ; load a with memory contents
        ldx  #STFTMP_StartFuelTemperature ; load index with value
        jsr  LookupTable                ; call subroutine
        lda  Scratch_ba                 ; load a with memory contents
        mul                             ; multiply (d=a*b)
        adca #$00                       ;  -00000000-
        beq  .3                         ; branch if equal (zero)
        tab                             ; b = a
        negb
        clra                            ; a = 0
        ldx  #Scratch_b6                ; load index with value
        jsr  ScaleXbySignedB_ReturnD    ; call subroutine
        std  Scratch_b6                 ; store d (a&b) into memory
.3      lda  Counter_EPPs               ; load a with memory contents
        ldx  #EPPMUL_StartFuelPerEPP    ; load index with value
        jsr  LookupTable                ; call subroutine
        clra                            ; a = 0
        ldx  #Scratch_b6                ; load index with value
        jsr  ScaleXbySignedB_ReturnD    ; call subroutine
.4      std  Scratch_b6                 ; store d (a&b) into memory
        ldb  FuelPulsewidthAddedForBatteryVoltage ; load b with memory contents
        clra                            ; a = 0
        asld                            ; shift left (d=d*2)
        addd Scratch_b6                 ; d = d + memory contents
        cmpa #$bf                       ; compare a with value -10111111-
        bls  .5                         ; branch if lower or same
; MAX INJECTOR PULSEWIDTH
        ldd  #$bfff                     ; load d (a&b) with value -10111111 11111111-
.5      std  Pulsewidth_InjectorSolution ; store d (a&b) into memory
.Return rts

; *****************************************

MM_CalculateFinalFuelingValue_v subroutine
        clra                ; a = 0
        sta  Scratch_b6     ; store a into memory

        ldb  BitFlags81     ; load b with memory contents
        lsrb                ; logical shift right b
        bcs  L1355          ; branch if b81_FullThrottle is set
        ldb  L00CA          ; load b with memory contents
        cmpb L1254          ; compare b with memory contents (data is 2b)
        bhs  L1355          ; branch if greater or equal
        jsr  CalculateHeatSoakAdjustment    ; call subroutine
        sta  Scratch_b7     ; store a into memory
        ldb  BitFlags84     ; load b with memory contents
        bitb #$02           ;  -00000010-
        bne  L1356          ; branch if not equal (not zero)
        ldx  #CURVEC_ColdEnrichmentFuelCurveC    ; load index with value
        jsr  LookupTable    ; call subroutine
        ldb  ColdLoadEnrichmentValue    ; load b with memory contents
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
        sta  Scratch_b6     ; store a into memory
L1356:  lda  Scratch_b7     ; load a with memory contents
        ldx  #CURVEA_ColdEnrichmentFuelCurveA    ; load index with value
        jsr  LookupTable    ; call subroutine
        ldb  BitFlags84     ; load b with memory contents
        bitb #$02           ;  -00000010-
        beq  L1357          ; branch if equal (zero)
        ldb  ColdLoadEnrichmentValue          ; load b with memory contents
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
L1357:  adda Scratch_b6
        bcc  L1358          ; branch if greater or equal
        lda  #$ff           ; load a with value -11111111-
L1358:  sta  Scratch_b6     ; store a into memory
L1355:  lda  CoolantTemp    ; load a with memory contents
        ldx  #CURVEB_ColdEnrichmentFuelCurveB    ; load index with value
        jsr  LookupTable    ; call subroutine
        clra                ; a = 0
        addb Scratch_b6     ; b=b+memory contents
        rola
        asld                ; shift left (d=d*2)
        std  Scratch_b6     ; store d (a&b) into memory
        lda  BitFlags81     ; load a with memory contents
        lsra
        bcs  L1359          ; branch if b81_FullThrottle is set
        lda  L00CA          ; load a with memory contents
        suba L1254          ;  (data is 2b)
        bcc  L1359          ; branch if greater or equal
        ldx  #HMPMUL_LoadEnrichment2    ; load index with value
        lda  MapValue            ; load a with memory contents
        jsr  LookupTable    ; call subroutine
        sta  Scratch_ba     ; store a into memory
        ldx  #HIGHMP_LoadEnrichment    ; load index with value
        lda  CoolantTemp    ; load a with memory contents
        jsr  LookupTable    ; call subroutine
        lda  Scratch_ba     ; load a with memory contents
        mul                 ; multiply (d=a*b)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        addd Scratch_b6     ; d = d + memory contents
        std  Scratch_b6     ; store d (a&b) into memory
L1359:
        lda  BarometricPressure    ; load a with memory contents
        ldx  #BAROCP_BaroCompWarmup    ; load index with value
        jsr  LookupTable    ; call subroutine
        clra                ; a = 0
        ldx  #Scratch_b6    ; load index with value
        jsr  ScaleXbySignedB_ReturnD          ; call subroutine
        inca                ; increment a
        std  Scratch_b6     ; store d (a&b) into memory
        deca                ; decrement a
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        stb  L00E9          ; store b into memory
        
        ldx  #AIRTMP_FuelChargeTempFactor  ; load index with value
        lda  ChargeTempVolts    ; load a with memory contents
        jsr  LookupTable    ; call subroutine
        stb  Scratch_ba     ; store b into memory

        ldx  #PEFTBL_PumpingEfficiency ; load index with value

        ldd  RPM_W          ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        jsr  LookupTable    ; call subroutine
        ldx  #Scratch_b6    ; load index with value
        clra                ; a = 0
        jsr  ScaleXbySignedB_ReturnD   ; call subroutine
        std  Scratch_b6     ; store d (a&b) into memory
        ldb  Timer_Leanout  ; load b with memory contents
        beq  L1360          ; branch if equal (zero)
        clra                ; a = 0
        jsr  ScaleXbySignedB_ReturnD   ; call subroutine
        std  Scratch_b6     ; store d (a&b) into memory
L1360:  ldb  Scratch_ba     ; load b with memory contents
        beq  L1361          ; branch if equal (zero)
        lda  #$01           ; load a with value -00000001-
        jsr  ScaleXbySignedB_ReturnD   ; call subroutine
        ; d contains only the amount to add/subtract, not the total scaled value
        std  Scratch_b6     ; store d (a&b) into memory
L1361:  ldb  CurrentAutoCalCell   ; load b with memory contents
        ldx  #AutoCalCell0 ; load index with value
        abx                 ; add b to index
        ldb  $00,x          ; load b with memory at index + value
        lda  BitFlags81     ; load a with memory contents
        lsra
        bcs  L1362          ; branch if b81_FullThrottle is set
        lda  L00CA          ; load a with memory contents
        cmpa L1254          ; compare a with memory contents (data is 2b)
        blo  L1363          ; branch if lower
L1362:  cmpb L1364          ; compare b with memory contents (data is 00)
        bgt  L1363          ; branch if greater
        ldb  L1364          ; load b with memory contents (data is 00)
L1363:  clra                ; a = 0
        tstb
        bmi  L1365          ; branch if minus(hi bit 1)
        inca                ; increment a
L1365:  asrb
        ldx  #Scratch_b6    ; load index with value
        jsr  ScaleXbySignedB_ReturnD          ; call subroutine
        std  Scratch_b6     ; store d (a&b) into memory
        ldb  BitFlags81     ; load b with memory contents
        bmi  L1366          ; branch if b81_OffIdle is set
        ldb  BitFlags80     ; load b with memory contents
        bitb #b80_BadCoolantTemp      ;  -00000010-
        beq  L1366          ; branch if equal (zero)
        clra                ; a = 0
        sta  SecondaryO2ControllerKicks_Signed          ; store a into memory
        ldb  L1367          ; load b with memory contents (data is 1a)
        lda  #$01           ; load a with value -00000001-
        bra  L1368          ; branch

L1366:  lda  L00C9          ; load a with memory contents
        cmpa #$70           ; compare a with value -01110000-
        bne  L1369          ; branch if not equal (not zero)
        lda  BitFlags84     ; load a with memory contents
        bita #$04           ;  -00000100-
        beq  L1369          ; branch if equal (zero)
        ldb  L1370          ; load b with memory contents (data is 1a)
        lda  #$01           ; load a with value -00000001-
        bra  L1368          ; branch

L1369:  clra                ; a = 0
        ldb  SecondaryO2ControllerKicks_Signed          ; load b with memory contents
        bmi  L1371          ; branch if minus(hi bit 1)
        inca                ; increment a
L1371:  asrb
L1368:  jsr  ScaleXbySignedB_ReturnD          ; call subroutine
        std  Counter_MainLoop2   ; store d (a&b) into memory
        rts                 ; return from subroutine

; *****************************************
; On a carb car, when you press the throttle the 'accelerator pump' squirts some extra
; fuel in. This is simulated with the 'transient' calculations.

MM_CalculateTransientFuelAddition   subroutine
        lda  BitFlags82     ; load a with memory contents
        bita #b82_DecayingFuelStartToRun      ;  -00001000-
        bne  L1372          ; branch if still decaying start to run (not zero)

        clra                ; a = 0
        clrb                ; b = 0
        std  Scratch_b6     ; store d (a&b) into memory
        lda  BitFlags81     ; load a with memory contents
        bmi  L1373          ; branch if b81_OffIdle is set
        lda  TpsVolts            ; load a with memory contents
        sta  TpsAverageValue  ; store a into memory
L1378:  ldd  Scratch_b6     ; load d (a&b) with memory contents
        bra  L1374          ; branch

L1373:  ldx  #POSTRG_TransientThrottleTriggerLevel    ; load index with value ($04)
        ldb  TpsVolts            ; load b with memory contents
        subb TpsAverageValue  ; b = b-memory contents
        ; determine how much above the previous
        ; if lower than the previous, save new value
        bcs  L1375          ; branch if lower
        ; see if throttle increase is above the trigger level
        subb $00,x
        ; nope
        bcs  L1376          ; branch if lower
        ; see if it's below Max delta throttle
        cmpb $01,x          ; THRMAX_TransientMaxDeltaThrottle
        ; yep
        blo  L1377          ; branch if lower
        ; otherwise, use the Max value
        ldb  $01,x          ; load b with THRMAX_TransientMaxDeltaThrottle
L1377:  stb  Scratch_ba     ; save b for later
        lda  CoolantTemp    ; load a with memory contents
        inx                 ; set x to table POSSLP_TransientThrottleEnrich
        inx
        jsr  LookupTable    ; call subroutine
        ldb  Scratch_ba     ; load b with memory contents
        mul                 ; multiply (d=a*b)
        std  Scratch_b6     ; store d (a&b) into memory
        bra  L1376          ; branch

L1375:  ldb  TpsVolts            ; load b with memory contents
        stb  TpsAverageValue  ; store b into memory

        ; Next section is similar to previous, except uses MapValue instead of TPS
L1376:  lda  MapValue            ; load a with memory contents
        suba MapAverageValue
        bls  L1378          ; branch if lower or same
        suba MAPTRG_TransientMapTrigger    ;  (data is 05)
        bcs  L1378          ; branch if lower
        cmpa MAPMAX_TransientMaxDeltaMap    ; compare a with memory contents (data is ff)
        blo  L1379          ; branch if lower
        lda  MAPMAX_TransientMaxDeltaMap    ; load a with memory contents (data is ff)
L1379:  sta  Scratch_ba     ; store a into memory
        lda  CoolantTemp    ; load a with memory contents
        ldx  #AESLOP_TransientEnrichMap    ; load index with value
        jsr  LookupTable    ; call subroutine
        lda  Scratch_ba     ; load a with memory contents
        mul                 ; multiply (d=a*b)
        addd Scratch_b6     ; d = d + memory contents
L1374:  asld                ; shift left (d=d*2)
        bcs  L1380          ; branch if high bit set by doubling
        asld                ; shift left (d=d*2)
        bcs  L1380          ; branch if high bit set by doubling
        asld                ; shift left (d=d*2)
        bcc  L1381          ; branch if high bit NOT set by doubling
L1380:  ldd  #$ffff         ; load d (a&b) with value -11111111 11111111-
L1381:  sei                 ; disable interrupts (set)
        std  Pulsewidth_PartThrottleEnrichment    ; store d (a&b) into memory
L1372:  ldd  Pulsewidth_PartThrottleEnrichment    ; load d (a&b) with memory contents
        cli                 ; enable interrupts (clear)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        tsta                ; test a
        beq  L1382          ; branch if equal (zero)
        ldb  #$ff           ; load b with value -11111111-
L1382:  stb  L00C8          ; apparently this value is never really used except for serial comm
        rts                 ; return from subroutine
;

; *****************************************
; handle knock retard?

HandleKnockRetard   subroutine
        lda  RPM_W          ; load a with memory contents
        cmpa #$47           ; compare a with value -01000111-
        blo  L1915          ; branch if lower
        lda  BitFlags82     ; load a with memory contents
        bita #b82_bit2      ;  -00000100-
        beq  L1916          ; branch if b82_bit2 clear
        lda  icrHi          ; load a with memory contents
        suba ElapsedTimeSinceLastDistSync_W
        ldb  #$03           ; load b with value -00000011-
        mul                 ; multiply (d=a*b)
        lda  BitFlags82     ; load a with memory contents
        cmpb InverseRPM_W
        bhs  L1917          ; branch if greater or equal
        ora  #b82_bit6      ;  -01000000-
        bra  L1918          ; branch

L1917:  anda #<~b82_bit6    ; clear bit -10111111-
        bra  L1919          ; branch

L1915:  lda  BitFlags82     ; load a with memory contents
        bita #b82_bit2      ;  -00000100-
        bne  L1920          ; branch if b82_bit2 set
L1916:  andb #$fd           ;  -11111101-
        stb  timerCSR       ; store b into memory
        ldb  BitFlags85     ; load b with memory contents
        bitb #$02           ;  -00000010-
        beq  L1920          ; branch if equal (zero)
        ora  #b82_bit2      ;  -00000100-
        sta  BitFlags82     ; store a into memory
        ldd  CylinderRetard1          ; load d (a&b) with memory contents
        ora  #$c0           ;  -11000000-
        orb  #$40           ;  -01000000-
        andb #$7f           ;  -01111111-
        std  CylinderRetard1          ; store d (a&b) into memory
        bra  L1921          ; branch

L1920:  ldb  Scratch_bc     ; load b with memory contents
        lsrb                ; logical shift right b
        sta  Scratch_bb     ; store a into memory
        anda #$bf           ; AND a with value -10111111-
        aba                 ; a=a+b
        xorb Scratch_bb
        lslb
        bpl  L1922          ; branch if plus (hi bit 0)
        ldb  Scratch_bc     ; load b with memory contents
        bpl  L1919          ; branch if plus (hi bit 0)
L1918:  ora  #b82_bit2      ;  -00000100-
        sta  BitFlags82     ; store a into memory
        ldd  CylinderRetard1 ; load d (a&b) with memory contents
        anda #$3f           ; AND a with value -00111111-
        andb #$3f           ;  -00111111-
        orb  #$80           ;  -10000000-
        std  CylinderRetard1 ; store d (a&b) into memory
        lda  BitFlags82     ; load a with memory contents
        lsra
        bcs  L1923          ; branch if b82_bit0 set
        ldb  timerCSR       ; load b with memory contents
        orb  #$02           ;  -00000010-
        stb  timerCSR       ; store b into memory
L1925:  rti                 ; return from interrupt

L1923:  lsla                ; logical shift left a
        sta  BitFlags82     ; clears b82_bit0
        lda  #$e9           ; load a with value -11101001-
        sta  timerControl1    ; store a into memory
        ldd  counterHi      ; load d (a&b) with memory contents
        addd #ioPort3CSR    ;  -00000000 00001111-
        std  ocr3Hi         ; store d (a&b) into memory
        lds  #$00ff         ;  -00000000 11111111-
        jmp  L1924

L1922:  ldb  CylinderRetard1          ; load b with memory contents
        bitb #$c0           ;  -11000000-
        bne  L1919          ; branch if not equal (not zero)
        anda #<~b82_bit2    ;$fb ; AND a with value -11111011-
        sta  BitFlags82     ; store a into memory
        lda  BitFlags7f     ; load a with memory contents
        ora  #$08           ;  -00001000-
        sta  BitFlags7f     ; store a into memory
        rti                 ; return from interrupt

L1919:  sta  BitFlags82     ; store a into memory
        ldd  CylinderRetard1          ; load d (a&b) with memory contents
        adda #$40           ;  -01000000-
        addb #$40           ;  -01000000-
        std  CylinderRetard1          ; store d (a&b) into memory
L1921:  lda  BitFlags82     ; load a with memory contents
        lsra
        bcs  L1923          ; branch b82_bit0 set
        bita #$02           ; will be b82_bit2 -00000010-
        beq  L1925          ; branch if equal (zero)
        lda  ioPort1data    ; load a with memory contents
        anda #$7f           ; AND a with value -01111111-
        sta  ioPort1data    ; * turn off knock kill
        ldb  Z85_ADConverter1    ; load b with memory contents
        ldb  Z85_ADConverter1    ; load b with memory contents
        lda  Z83_PortBdata    ; load a with memory contents
        ora  #$20           ;  -00100000-
        sta  Z83_PortBdata    ; * turn on tach out
        lda  timerCSR       ; load a with memory contents
        anda #$fd           ; AND a with value -11111101-
        sta  timerCSR       ; store a into memory
        lda  icrHi          ; load a with memory contents
        lda  Counter_EPPs    ; load a with memory contents
        inca                ; increment a
        beq  L1926          ; branch if equal (zero)
        sta  Counter_EPPs    ; store a into memory
L1926:  ldx  #CylinderRetard1 ; load index with value
        ldb  $01,x          ; load b with memory at index + value
        rolb                ; rotate b left through carry
        rolb                ; rotate b left through carry
        rolb                ; rotate b left through carry
        andb #$03           ;  -00000011-
        abx                 ; add b to index
        lda  $00,x          ; load a with memory at index + value
        tab                 ; b = a
        anda #$c0           ; AND a with value -11000000-
        andb #$3f           ;  -00111111-
        sta  FuelBaselineResult    ; store a into memory
        lda  BitFlags7f     ; load a with memory contents
        bmi  L1927          ; branch if minus(hi bit 1)
        clra                ; a = 0
        bra  L1928          ; branch

L1927:  lda  BarometricPressure    ; load a with memory contents
        adda KNKOFS_KnockRetardOffset    ;  (data is 2f)
        cmpa MapValue            ; compare a with memory contents
        lda  RTRDRT_KnockRetardRateNoBoost    ; load a with memory contents (data is 06)
        bcc  L1928          ; branch if greater or equal
        lda  BitFlags7f     ; load a with memory contents
        ora  #$10           ;  -00010000-
        sta  BitFlags7f     ; store a into memory
        lda  L00C5          ; load a with memory contents
        inca                ; increment a
        beq  L1929          ; branch if equal (zero)
        sta  L00C5          ; store a into memory
L1929:  lda  RTRDRT2_KnockRetardRateInBoost    ; load a with memory contents (data is 03)
L1928:  aba                 ; a=a+b
        bcs  L1930          ; branch if lower
        cmpa #$40           ; compare a with value -01000000-
        blo  L1931          ; branch if lower
L1930:  lda  #$3f           ; load a with value -00111111-
L1931:  ldb  MaxRetard      ; load b with memory contents
        cba
        blo  L1932          ; branch if lower
        tba                 ; a = b
L1932:  ora  FuelBaselineResult    ; OR a with memory contents
        sta  $00,x
        jsr  CheckMapSensor       ; call subroutine
        ldb  BitFlags85     ; load b with memory contents
        bmi  BaroReadIfRequired    ; branch if minus(hi bit 1)
        adda L0091
        rora
        sta  MapValue       ; store a into memory
        lda  BitFlags82     ; load a with memory contents
        bmi  L1383          ; branch if b82_EngineNotRunning set
        lda  BitFlags80     ; load a with memory contents
        bpl  L1383          ; branch if b80_BadMAP is clear
        jsr  LC7A2          ; call subroutine
        jsr  L1832          ; call subroutine
        sta  MapValue       ; store a into memory
        bra  L1383          ; branch

BaroReadIfRequired:
        ldb  BitFlags82     ; load b with memory contents
        bitb #b82_bit4      ;  -00010000-
        beq  L1383          ; branch if equal (zero)
        andb #<~b82_bit4    ; $ef           ;  -11101111-
        stb  BitFlags82     ; store b into memory
        cmpa BAROHI_BaroReadMaxValid    ; compare a with memory contents (data is 56)
        bhs  L1384          ; branch if greater or equal
        cmpa BAROLO_BaroReadMinValid    ; compare a with memory contents (data is 2e)
        bhs  L1385          ; branch if greater or equal
L1384:  lda  BARDFT_BaroReadDefault    ; load a with memory contents (data is 54)
L1385:  sta  BarometricPressure    ; store a into memory
L1383:  inc  L00CB
        lda  BitFlags82     ; load a with memory contents
        bpl  L1386          ; branch if b82_EngineNotRunning clear 
        ldb  ADScanIndex    ; load b with memory contents
        jsr  ADLookup       ; call subroutine
        jmp  L1387

L1386:  lda  Z85_ADConverterKnockSensor    ; load a with memory contents
        lda  CylinderRetard1          ; load a with memory contents
        bita #$40           ;  -01000000-
        bne  CalculateInjectorPulsewidthForEngineRunning    ; branch if not equal (not zero)
        ldb  #$10           ; load b with value -00010000-
L1388:  decb
        bne  L1388          ; branch if not equal (not zero)
        jmp  L1389

;  ______________________
; |                      \
; |   Running Fuel       |
; \______________________|

CalculateInjectorPulsewidthForEngineRunning    subroutine
        clra                ; a = 0
        clrb                ; b = 0
        std  FuelPulsewidth_scratch1  ; zero scratch variable

        lda  MapValue                      ; load a with memory contents
        ldx  #WOTTBL_FuelFullThrottle    ; load index with value
        jsr  Lookup5ByteTable         ; call subroutine
        std  FuelBaselineResult       ; store d (a&b) into memory
        ; Baseline result should be used for Full Throttle, or
        ; as the maximum for part throttle.

        lda  MapValue                      ; load a with memory contents
        ldx  #MAPIDL_FuelNoThrottle        ; load index with value

        ; check if at idle:
        ldb  BitFlags81               ; load b with memory contents
        bpl  L1390                    ; At idle, jump

        ; check if at full throttle:
        lsrb                ; logical shift right b
        bcs  L1391          ; At full throttle, jump

        ; fallthrough: part throttle:
        ; (I'd LOVE to know what L00CA is for - gla)
        ldb  L00CA          ; load b with memory contents
        cmpb L1254          ; compare b with memory contents (data is 2b)
        blo  L1392          ; branch if lower

        lda  RPM_W          ; load a with memory contents
        cmpa L1393          ; compare a with memory contents (data is 3a)
        blo  L1392          ; branch if lower

L1391:  clra                ; a = 0
        sta  SecondaryO2ControllerKicks_Signed          ; store a into memory
        bra  L1394          ; branch

L1392:  lda  MapValue            ; load a with memory contents
        cmpa OverboostConstant1    ; compare a with memory contents (data is aa)
        blo  L1395          ; branch if lower
        tba                 ; a = b
        bra  L1396          ; branch

L1395:  aba                 ; a=a+b
        bcc  L1397          ; branch if greater or equal
        lda  #$ff           ; load a with value -11111111-
L1397:  suba OverboostConstant1    ;  (data is aa)
        bls  L1398          ; branch if lower or same
L1396:  clrb                ; b = 0
        stb  SecondaryO2ControllerKicks_Signed          ; store b into memory
        sta  Scratch_bb     ; store a into memory
        ldd  FuelPulsewidthFunctionUnknown ;  (data is 1eb2)
        jsr  mul16bit       ; call subroutine
        std  FuelPulsewidth_scratch1 ; store d (a&b) into memory
L1398:  lda  MapValue            ; load a with memory contents
        ldx  #MAPTBL_FuelPartThrottle ; load index with value

L1390:  jsr  Lookup5ByteTable    ; call subroutine
        addd FuelPulsewidth_scratch1    ; d = d + memory contents
        std  FuelPulsewidth_scratch1    ; store d (a&b) into memory
        subd FuelBaselineResult    ; d = d - memory contents
        ldd  FuelPulsewidth_scratch1    ; load d (a&b) with memory contents
        ; IF scratch is higher than Baseline, use Baseline (it's the maximum allowed)
        bcs  L1399          ; branch if lower

L1394:  ldd  FuelBaselineResult    ; We also get sent directly here at full throttle.
L1399:  asld                ; shift left (d=d*2)
        std  Scratch_bb     ; store d (a&b) into memory
        ldd  LeanoutFactor_W  ; load d (a&b) with memory contents
        addd Scratch_bb     ; d = d + memory contents
        bmi  L1400          ; branch if minus(hi bit 1)
        std  Scratch_bb     ; store d (a&b) into memory

L1400:  ldx  #FuelPulsewidth_scratch1   ; load index with value
        ldd  Counter_MainLoop2          ; load d (a&b) with memory contents
        std  FuelPulsewidth_scratch1    ; store d (a&b) into memory
        ldb  Scratch_bc     ; load b with memory contents
        clra                ; a = 0
        jsr  ScaleXbySignedB_ReturnD    ; call subroutine
        std  FuelPulsewidth_scratch1    ; store d (a&b) into memory
        lda  Scratch_bb     ; load a with memory contents
        ldb  Counter_MainLoop3          ; load b with memory contents
        mul                 ; multiply (d=a*b)
        addd FuelPulsewidth_scratch1    ; d = d + memory contents
        std  FuelPulsewidth_scratch1    ; store d (a&b) into memory
        ldb  Counter_MainLoop2          ; load b with memory contents
        lda  Scratch_bb     ; load a with memory contents
        mul                 ; multiply (d=a*b)
        tsta                ; test a
        bne  L1401          ; branch if not equal (not zero)
        addb FuelPulsewidth_scratch1    ; b=b+memory contents
        bcc  L1402          ; branch if greater or equal
L1401:  ldb  #$ff           ; load b with value -11111111-
L1402:  tba                 ; a = b
        ldb  L00BE          ; load b with memory contents
        std  Scratch_bb     ; store d (a&b) into memory
        lda  BitFlags82     ; load a with memory contents
        bita #b82_DecayingFuelStartToRun      ;  -00001000-
        bne  L1389          ; branch if not equal (not zero)
        ldd  Scratch_bb     ; load d (a&b) with memory contents
        std  Pulsewidth_InjectorSolution     ; store d (a&b) into memory
        addd Pulsewidth_PartThrottleEnrichment    ; d = d + memory contents
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        addd FuelMonitorOutputTimer_W    ; d = d + memory contents
        bcs  L1389          ; branch if lower
        std  FuelMonitorOutputTimer_W    ; store d (a&b) into memory
L1389:  ldb  ADScanIndex    ; load b with memory contents
        jsr  ADLookup       ; call subroutine
        ldx  #RawBattVoltage    ; load index with value
        jsr  RegulateVoltage    ; call subroutine
L1387:  jsr  L1403          ; call subroutine
        lda  BitFlags82     ; load a with memory contents
        bita #b82_SparkStartMode      ;  -00000010-
        beq  L1404          ; branch if spark running mode (zero)
        ; spark starting mode:
        lda  #$19           ; load a with value -00011001-
        sta  timerCSR       ; store a into memory
        ldd  #counterLo     ; load d (a&b) with value -00000000 00001010-
        addd counterHi      ; d = d + memory contents
        std  ocrHi          ; store d (a&b) into memory
        mul                 ; multiply (d=a*b)

        ; check if injector check circuit is functioning
        ldb  ioPort1data    ; load b with memory contents
        bitb #$40           ; * check injector
        beq  L1405          ; branch if equal (zero)
        andb #$06           ;  -00000110-
        bne  L1405          ; branch if not equal (not zero)
        ; if we get here, there is no injector check signal, even though
        ; either or both injectors should be on.
        lda  BitFlags83     ; load a with memory contents
        ora  #$10           ;  -00010000- INJECTOR FAULT
        sta  BitFlags83     ; store a into memory

L1405:  ldd  #$1b58         ; load d (a&b) with value -00011011 01011000-
        addd icrHi          ; d = d + memory contents
        cmpa timerCSR       ; compare a with memory contents
        std  ocrHi          ; store d (a&b) into memory
L1404:  rti                 ; return from interrupt
;

L1403   subroutine
        lda  CylinderRetard1          ; load a with memory contents
        lsla                ; logical shift left a
        bmi  L1406          ; branch if minus(hi bit 1)
        rts                 ; return from subroutine

L1406:  ldx  #ocr3Hi        ; load index with value
        lda  #$fb           ; load a with value -11111011-
        bcc  L1407          ; branch if greater or equal
        lda  MapValue            ; load a with memory contents
        tab                 ; b = a
        subb L00E5          ; b = b-memory contents
        bcc  L1408          ; branch if greater or equal
        cmpb #$80           ;  -10000000-
        bhs  L1409          ; branch if greater or equal
        ldb  #$80           ; load b with value -10000000-
        bra  L1409          ; branch

L1408:  cmpb #$7f           ;  -01111111-
        blo  L1409          ; branch if lower
        ldb  #$7f           ; load b with value -01111111-
L1409:  stb  Delta_MAP    ; store b into memory
        sta  L00E5          ; store a into memory
        dex                 ; decrement index (x=x-1)
        dex                 ; decrement index (x=x-1)
        lda  #$fd           ; load a with value -11111101-
L1407:  sta  FuelPulsewidth_scratch1    ; store a into memory
        anda timerControl1    ; logical and a with memory contents
        jsr  L1410          ; call subroutine
        lda  BitFlags84     ; load a with memory contents
        bita #$28           ;  -00101000-
        beq  L1411          ; branch if equal (zero)
        clra                ; a = 0
        clrb                ; b = 0
        std  FuelMonitorOutputTimer_W    ; store d (a&b) into memory
        lda  #$f9           ; load a with value -11111001-
        anda timerControl1    ; logical and a with memory contents

L1592:  ldx  #ocr2Hi        ; load index with value
        jsr  L1410          ; call subroutine
        inx                 ; increment index (x=x+1)
        inx                 ; increment index (x=x+1)
        bra  L1412          ; branch

L1410:  sta  timerControl1    ; store a into memory
L1412:  ldd  counterHi      ; load d (a&b) with memory contents
        addd #ioPort3CSR    ;  -00000000 00001111-
        std  $00,x          ; store d into index plus offset
        rts                 ; return from subroutine

L1411:  ldd  ocrHi          ; load d (a&b) with memory contents
        subd counterHi      ; d = d - memory contents
        cmpa #$02           ; compare a with value -00000010-
        blo  L1413          ; branch if lower
        lda  ioPort2data    ; load a with memory contents
        bita #$02           ;  -00000010-
        beq  L1413          ; branch if Dwell is off (zero)
        lda  BitFlags82     ; load a with memory contents
        bita #b82_SparkStartMode ;  -00000010-
        bne  L1413          ; branch if spark starting mode (not zero)
        ; spark running mode:
        lda  timerControl1  ; load a with memory contents
        anda #$df           ; AND a with value -11011111-
        ldb  ioPort2data    ; load b with memory contents
        andb #$fd           ;  -11111101-
        sta  timerControl1  ; store a into memory
        stb  ioPort2data    ; Turn OFF Dwell (make a spark)
L1413:  lda  FuelPulsewidth_scratch1    ; load a with memory contents
        coma                ; complement (toggle all bits)
        ora  timerControl1  ; OR a with memory contents
        jsr  L1410          ; call subroutine
        lda  RawBattVoltage ; load a with memory contents
        cmpa #$91           ; compare a with value -10010001-
        blo  L1414          ; branch if lower

        lda  ioPort1data    ; load a with memory contents
        bita #$40           ; * check injector
        beq  L1414          ; branch if equal (zero)
        ; work with injectors 1-2
        lda  #$04           ; load a with value -00000100-
        ldb  FuelPulsewidth_scratch1    ; load b with memory contents
        cmpb #$fd           ;  -11111101-
        beq  L1415          ; branch if equal (zero)
        ; shift... work with injectors 3-4
        lsla                ; logical shift left a
L1415:  ora  BitFlags83     ; OR a with memory contents
        sta  BitFlags83     ; store a into memory

L1414:  lda  BitFlags82     ; load a with memory contents
        bita #b82_SparkStartMode ;  -00000010-
        bne  L1416          ; branch if spark starting mode (not zero)
        ; spark running mode:
        ldb  ioPort2data    ; load b with memory contents
        orb  #$02           ;  -00000010-
        stb  ioPort2data    ; * turn ON Dwell (stop spark)
        ldb  timerControl1  ; load b with memory contents
        orb  #$20           ;  -00100000-
        stb  timerControl1  ; store b into memory
L1416:  clra                ; a = 0
        ldb  FuelPulsewidthAddedForBatteryVoltage          ; load b with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        addd Scratch_bb     ; d = d + memory contents
        bcc  L1417          ; branch if greater or equal
        ldd  #$ffff         ; load d (a&b) with value -11111111 11111111-
L1417:  tsta                ; test a
        bne  L1418          ; branch if not equal (not zero)
        ldb  #$ff           ; load b with value -11111111-
L1418:  std  Scratch_bb     ; store d (a&b) into memory
        lda  BitFlags82     ; load a with memory contents
        bmi  L1419          ; branch if b82_EngineNotRunning set
        bita #$08           ;  -00001000-
        bne  L1420          ; branch if not equal (not zero)
L1426:  ldd  Scratch_bb     ; load d (a&b) with memory contents
        addd Pulsewidth_PartThrottleEnrichment    ; d = d + memory contents
        bcc  L1421          ; branch if greater or equal
        ldd  #$ffff         ; load d (a&b) with value -11111111 11111111-
L1421:  tsta                ; test a
        bne  L1422          ; branch if not equal (not zero)
        ldb  #$ff           ; load b with value -11111111-
L1422:  cpx  #ocr2Hi        ;  -00000000 00011010-
        bne  L1423          ; branch if not equal (not zero)
        addd ocr2Hi         ; d = d + memory contents
        std  ocr2Hi         ; store d (a&b) into memory
        lda  timerControl1  ; load a with memory contents
        anda #$fd           ; AND a with value -11111101-
        bra  L1424          ; branch

L1423:  addd ocr3Hi         ; d = d + memory contents
        std  ocr3Hi         ; store d (a&b) into memory
        lda  timerControl1  ; load a with memory contents
        anda #$fb           ; AND a with value -11111011-
L1424:  sta  timerControl1  ; store a into memory
        rts                 ; return from subroutine

L1420:  ldd  Pulsewidth_InjectorSolution          ; load d (a&b) with memory contents
        suba FuelSubtractedFromStartToRun
        bcc  L1425          ; branch if greater or equal
        clra                ; a = 0
L1425:  std  Pulsewidth_InjectorSolution          ; store d (a&b) into memory
        ldd  Scratch_bb     ; load d (a&b) with memory contents
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        subd Pulsewidth_InjectorSolution ; d = d - memory contents
        bcs  L1419          ; branch if lower
        lda  BitFlags82     ; load a with memory contents
        anda #<~b82_DecayingFuelStartToRun    ; $f7 clear bit -11110111-
        sta  BitFlags82     ; store a into memory
        lda  Bits_WhichInjector ; load a with memory contents
        anda #$fc           ; AND a with value -11111100-
        sta  Bits_WhichInjector ; store a into memory
        clra                ; a = 0
        sta  FuelSubtractedFromStartToRun ; store a into memory
        sta  Counter_EPPs   ; store a into memory
        clrb                ; b = 0
        std  Pulsewidth_PartThrottleEnrichment    ; store d (a&b) into memory
        lda  timerControl2  ; load a with memory contents
        anda #$cf           ; AND a with value -11001111-
        sta  timerControl2  ; store a into memory
        bra  L1426          ; branch

L1419:  ldd  Pulsewidth_InjectorSolution ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        cmpa #$02           ; compare a with value -00000010-
        bhs  L1427          ; branch if greater or equal
        ldd  #$0200         ; load d (a&b) with value -00000010 00000000-
L1427:  std  Pulsewidth_PartThrottleEnrichment    ; store d (a&b) into memory
        stx  FuelBaselineResult    ; store index into memory
        cpx  #ocr2Hi        ;  -00000000 00011010-
        beq  L1428          ; branch if equal (zero)
        ldx  #Bits_WhichInjector ; load index with value
        bra  L1429          ; branch

L1428:  ldx  #BitFlags7f    ; load index with value
L1429:  ldb  $00,x          ; load b with memory at index + value
        andb #$fc           ;  -11111100-
        stb  $00,x
        lda  Pulsewidth_InjectorSolution  ; load a with memory contents
        lsla                ; logical shift left a
        rolb                ; rotate b left through carry
        lsla                ; logical shift left a
        rolb                ; rotate b left through carry
        andb #$03           ;  -00000011-
        orb  $00,x
        stb  $00,x
        lda  BitFlags82     ; load a with memory contents
        bita #b82_SparkStartMode ;  -00000010-
        beq  L1430          ; branch if spark running mode (zero)
        ; spark starting mode:
        ; * Check injector circuits:
        lda  ioPort1data    ; Get the port data
        ora  #$f9           ;  -11111001-
        coma                ; complement a (toggle all bits)
        beq  L1430          ; branch if both injectors on (zero)
        ldb  #$05           ; load b with value -00000101-

L1431:  lda  ioPort1data    ; load a with memory contents
        lsla                ; logical shift left a
        bmi  L1430          ; branch if minus(hi bit 1) (an injector circuit is activated)

        decb
        bne  L1431          ; branch if not equal (not zero)
        lda  #$04           ; load a with value -00000100- (work with injectors 1-2)
        ldb  FuelPulsewidth_scratch1    ; load b with memory contents
        cmpb #$fd           ;  -11111101-
        beq  L1432          ; branch if equal (zero)
        lsla                ; logical shift left a (work with injectors 3-4)

L1432:  ora  BitFlags83     ; OR a with memory contents
        sta  BitFlags83     ; store a into memory
        ; mark injector circuit bad (return is on but neither should be on)
L1430:  lda  $00,x          ; load a with memory at index + value
        bita #$03           ;  -00000011-
        bne  L1433          ; branch if not equal (not zero)
        lda  FuelPulsewidth_scratch1    ; load a with memory contents
        anda timerControl1  ; logical and a with memory contents
        sta  timerControl1  ; store a into memory

L1433:  ldx  FuelBaselineResult
        lda  timerStatus    ; load a with memory contents
        ldd  Pulsewidth_PartThrottleEnrichment    ; load d (a&b) with memory contents
        bmi  L1434          ; branch if minus(hi bit 1)
        addd $00,x
        std  $00,x          ; store d into index plus offset
        subd counterHi      ; d = d - memory contents
        bpl  L1435          ; branch if plus (hi bit 0)
        lda  timerStatus    ; load a with memory contents
        ldb  FuelPulsewidth_scratch1    ; load b with memory contents
        jsr  L1100          ; call subroutine
        bra  L1435          ; branch

L1434:  addd $00,x
        std  $00,x          ; store d into index plus offset
L1435:  lda  #$fb           ; load a with value -11111011-
        sta  timerControl2  ; store a into memory  Enable timer overflow interrupt
        rts                 ; return from subroutine

;  ______________________
; |                      \
; |   Interrupt Handler  |
; \______________________|
;
; This interrupt is fired by:
;   * Distributor sync pulse
;   * Speed/distance sensor pulse
;   * timer overflow for either speed or distributor
;   If a timer overflows, the associated overflow counter is incremented
;   If a hardware event occurs, use it to calculate speed or RPMs

InterruptFromDistributorSyncAndSpeedSensor  subroutine

        lda  timerStatus    ; load a with memory contents
        bpl  L1436          ; branch if plus (hi bit 0)
        ; about bit 7 of the TimerStatus register:
        ;   ICF2 is set to indicate that a proper level transition has occurred; it is cleared by
        ;   reading the TSR (with ICF2 set) and the input capture register 2 high byte ($1e), or during reset.
        ; So, we jump to HandleSpeedSensorInterrupt if the cause of this interrupt is a speed/distance sensor pulse or timer overflow
        jmp  HandleSpeedSensorInterrupt

        ; see if distributor sync is what brought us here
L1436:  ldb  Z83_PortCdata  ; load b with memory contents
        andb #$80           ; * check if dist sync
        stb  Scratch_bc     ; store b into memory
        lda  timerCSR       ; load a with memory contents
        ldd  icrHi          ; load d (a&b) with memory contents
        jsr  MM_HandleDistributorOverflow          ; call subroutine
        lda  timerCSR       ; load a with memory contents
        tab                 ; b = a
        lsra
        xora ioPort2data
        lsra
        bcc  L1439          ; branch if greater or equal
        rti                 ; return from interrupt

L1439:  lda  L00AF          ; load a with memory contents
        inca                ; increment a
        beq  L1440          ; branch if equal (zero)
        sta  L00AF          ; store a into memory
L1440:  clra                ; a = 0
        sta  LowestHistoricalTPS ; store a into memory
        orb  #$18           ;  -00011000-
        stb  timerCSR       ; store b into memory
        lda  ErrorCodes4    ; load a with memory contents
        anda #$7f           ; turn off bit 7 -01111111-
        sta  ErrorCodes4    ; store a into memory
        bitb #$02           ;  -00000010-
        beq  L1441          ; branch if equal (zero)
        jmp  HandleKnockRetard

L1441:  ; begin an A/D conversion
        ldb  Z85_ADConverter1 ; load b with memory contents
        ldb  Z85_ADConverter1 ; load b with memory contents

        ; turn off tach output
        ldb  Z83_PortBdata  ; load b with memory contents
        andb #$df           ;  -11011111-
        stb  Z83_PortBdata  ; * turn off tach out

        lda  Counter_DistTimerOverflow  ; load a with memory contents
        lsra
        sta  InverseRPM_B   ; store a into memory

        ; Get the input capture register
        ldd  icrHi          ; load d (a&b) with memory contents
        subd ElapsedTimeSinceLastDistSync_W ; d = d - memory contents
        std  InverseRPM_W   ; store d (a&b) into memory

        ; get the input capture register again
        ldd  icrHi          ; load d (a&b) with memory contents
        std  ElapsedTimeSinceLastDistSync_W ; store d (a&b) into memory
        clra                ; a = 0
        sta  Counter_DistTimerOverflow      ; store a into memory

        lda  BitFlags82     ; load a with memory contents
        bita #b82_bit2      ;  -00000100-
        bne  L1442          ; branch if not equal (not zero)

        ; restart the timer
        ldb  timerCSR       ; load b with memory contents
        orb  #$02           ;  -00000010-
        stb  timerCSR       ; store b into memory
        rti                 ; return from interrupt

L1442:  lda  #$ff           ; load a with value -11111111-
        ldx  InverseRPM_B
        cpx  TransitionRPMsImmediateSwitchToRun    ;  (data is 0092)
        blo  L1443          ; branch if lower
        ldd  SparkTransitionStartToRun    ; data is 00c3
;       mod, GLA, remove lookup
;        ldx  #Table09_FromCoolantTemp    ; load index with value
;        lda  CoolantTemp   ; load a with memory contents
;        jsr  Lookup5ByteTable    ; call subroutine
        subd InverseRPM_B          ; d = d - memory contents
        bcc  L1444          ; branch if greater or equal
        lda  BitFlags82     ; load a with memory contents
        bita #b82_SparkStartMode ;  -00000010-
        beq  L1445          ; branch if spark running mode (zero)
        clra                ; a = 0
        sta  Counter_ConsecutiveEPPs ; store a into memory
L1445:
        ldd  SparkTransitionRunToStart        ; data is 011b
;       mod, GLA, remove lookup
;        ldx  #Table10_FromCoolantTemp    ; load index with value
;        lda  CoolantTemp   ; load a with memory contents
;        jsr  Lookup5ByteTable    ; call subroutine
        subd InverseRPM_B          ; d = d - memory contents
        bcc  L1446          ; branch if greater or equal
L1450:  lda  BitFlags82     ; Transition Run to Start
        ora  #b82_SparkStartMode ; set to spark starting mode -00000010-
        bra  L1447          ; branch

L1444:  lda  Counter_ConsecutiveEPPs ; load a with memory contents
        inca                ; increment a
        beq  L1448          ; branch if equal (zero)

    ; we are jumped here with a=$ff if engine speed is above TransitionRPMsImmediateSwitchToRun
    ; this allows the NumConsecutiveEPPs test to be skipped if the engine fires on first EPP
L1443:  sta  Counter_ConsecutiveEPPs ; store a into memory
        ldx  #SRTIME_StartRunHotNumConsecEPPs    ; load index with value
        ldb  SRTMTP_StartRunTemperatureSwitchPoint    ; load b with memory contents (data is b0)
        cmpb CoolantTemp
        blo  L1449          ; branch if lower
        inx                 ; increment index (x=x+1)
L1449:  cmpa $00,x          ; compare a with memory at index + value
        blo  L1450          ; turn on BitFlags82 bit 1 (spark starting)

        ; Transition from start to run
L1448:  lda  BitFlags82     ; load a with memory contents
        bita #b82_SparkStartMode ;  -00000010-
        beq  L1446          ; branch if spark running mode (zero)
        anda #<~b82_SparkStartMode ; $fd clear bit -11111101-

L1447:  sta  BitFlags82     ; store a into memory
L1446:  lda  #$1a           ; load a with value -00011010-
        sta  timerCSR       ; store a into memory
        jsr  CheckMapSensor       ; call subroutine
        ldb  BitFlags85     ; load b with memory contents
        bmi  L1452          ; branch if minus(hi bit 1)
        sta  L0091          ; store a into memory
L1452:  lda  BitFlags82     ; load a with memory contents
        bita #b82_SparkStartMode ;  -00000010-
        beq  L1453          ; branch if spark running mode (zero)

        ; starting:
        ldb  ADScanIndex    ; load b with memory contents
        jsr  ADLookup       ; call subroutine
        ldd  #$1b00         ; load d (a&b) with value -00011011 00000000-
        addd icrHi          ; d = d + memory contents
        std  Scratch_bc     ; store d (a&b) into memory
        lda  #$1a           ; load a with value -00011010-
        jmp  L1454

        ; running:
L1453:  ldd  InverseRPM_W          ; load d (a&b) with memory contents
        cmpa #$2a           ; compare a with value -00101010-
        bhs  L1455          ; branch if greater or equal
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        coma                ; complement (toggle all bits)
        comb                ; complement (toggle all bits)
        addd InverseRPM_W   ; d = d + memory contents
        lsrd                ; shift right (d=d/2)
        std  L00C1          ; store d (a&b) into memory
        lsrd                ; shift right (d=d/2)
        cmpa #$06           ; compare a with value -00000110-
        blo  L1456          ; branch if lower
        addd L00C1          ; d = d + memory contents
        suba #$0c           ;  -00001100-
        bra  L1456          ; branch

L1455:  suba #$1c           ;  -00011100-
L1456:  std  L00C1          ; store d (a&b) into memory
        ldx  #CylinderRetard1         ; load index with value
        ldb  $00,x          ; load b with memory at index + value
        rolb                ; rotate b left through carry
        rolb                ; rotate b left through carry
        rolb                ; rotate b left through carry
        andb #$03           ;  -00000011-
        abx                 ; add b to index
        lda  $00,x          ; load a with memory at index + value
        anda #$3f           ; AND a with value -00111111-
        adda OverallAdvance
        sta  Scratch_bb     ; store a into memory
        ldb  BitFlags80     ; load b with memory contents
        bitb #b80_BadCoolantTemp      ;  -00000010-
        beq  L1457          ; branch if equal (zero)
        ldb  BitFlags81     ; load b with memory contents
        bmi  L1457          ; branch if b81_OffIdle is set
        ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        cmpa MPHREF_SwitchPointBetweenDecelIdle          ; compare a with memory contents (data is 06)
        bhs  L1457          ; branch if greater or equal
        lda  L1459          ; load a with memory contents (data is 6e)
        sta  Scratch_bb     ; store a into memory
L1457:  lda  Z85_ADConverterKnockSensor    ; load a with memory contents
        sta  RawKnockSensor ; store a into memory
        ldb  BitFlags7f     ; load b with memory contents
        andb #$7f           ;  -01111111-
        cmpa KnockLookup    ; compare a with memory contents
        blo  L1460          ; branch if lower
        orb  #$80           ; * turn on "knock" flag
L1460:  stb  BitFlags7f     ; store b into memory
        lda  BitFlags82     ; load a with memory contents
        ora  BitFlags81     ; OR a with memory contents
        bmi  L1461          ; branch if b81_OffIdle is set and b82_EngineNotRunning is set
        ldb  BitFlags84     ; load b with memory contents
        andb #$40           ;  -01000000-
        orb  Timer_AISDelay
        bne  L1461          ; branch if not equal (not zero)
        ldx  #IdleBytes1    ; load index with value
        lda  #$40           ; load a with value -01000000-
        sta  Scratch_bc     ; store a into memory
        ldd  RPM_W          ; load d (a&b) with memory contents
        subd TargetIdleSpeed    ; d = d - memory contents
        bcc  L1462          ; branch if greater or equal
        asl  Scratch_bc
        ldx  #IdleBytes2    ; load index with value
        ldd  TargetIdleSpeed    ; load d (a&b) with memory contents
        subd RPM_W          ; d = d - memory contents
L1462:  lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        subd SubtractedFromRPMDuringIdleRoutine    ;  (data is 001e)
        lda  $00,x          ; load a with memory at index + value
        bcc  L1463          ; branch if greater or equal
        addb AddedBackToRPMDuringIdleRoutine    ;  (data is 1e)
        bmi  L1461          ; branch if minus(hi bit 1)
        lda  $01,x          ; load a with memory at index + value
        mul                 ; multiply (d=a*b)
        adca $02,x
L1463:  ldb  Scratch_bc     ; load b with memory contents
        bpl  L1464          ; branch if plus (hi bit 0)
        nega                ; negate a (set high bit)
L1464:  ldb  BitFlags80     ; load b with memory contents
        bitb #b80_BadMAP | b80_BadCoolantTemp ;  -10000010-
        bne  L1465          ; branch if not equal (not zero)
        ldx  InverseRPM_B
        cpx  ComparedWithInverseRPM_BDuringIdleRoutine    ;  (data is 00d0)
        bhs  L1465          ; branch if greater or equal
        tab                 ; b = a
        addb Scratch_bb     ; b=b+memory contents
        stb  Scratch_bb     ; store b into memory
        ldb  BitFlags84     ; load b with memory contents
        bitb #$04           ;  -00000100-
        beq  L1465          ; branch if equal (zero)
        ldb  L00C9          ; load b with memory contents
        cmpb #$70           ;  -01110000-
        bne  L1465          ; branch if not equal (not zero)
        ldb  L1466          ; load b with memory contents (data is 6d)
        stb  Scratch_bb     ; store b into memory
L1465:  adda L00B5
        bvc  L1467          ; branch if overflow
        ldb  L00E2          ; load b with memory contents
        cmpb L0042
        bne  L1468          ; branch if not equal (not zero)
        lda  aByteUsedDuringIdleRoutine    ; load a with memory contents (data is 01)
        beq  L1468          ; branch if equal (zero)
        ldb  Scratch_bc     ; load b with memory contents
        bmi  L1469          ; branch if minus(hi bit 1)
        nega                ; negate a (set high bit)
L1469:  jsr  L1341          ; call subroutine
L1468:  clra                ; a = 0
L1467:  sta  L00B5          ; store a into memory
L1461:  lda  #$76           ; load a with value -01110110-
        cmpa Scratch_bb     ; compare a with memory contents
        bhi  L1470          ; branch if higher
        sta  Scratch_bb     ; store a into memory
L1470:  ldd  InverseRPM_W          ; load d (a&b) with memory contents
        tst  InverseRPM_B
        beq  L1471          ; branch if equal (zero)
        lsrd                ; shift right (d=d/2)
        ora  #$80           ;  -10000000-
        jsr  mul16bit       ; call subroutine
        asld                ; shift left (d=d*2)
        bcc  L1472          ; branch if greater or equal
        ldd  #$ffff         ; load d (a&b) with value -11111111 11111111-
        bra  L1472          ; branch

L1471:  jsr  mul16bit       ; call subroutine
L1472:  addd ElapsedTimeSinceLastDistSync_W ; d = d + memory contents
        std  Scratch_bc     ; store d (a&b) into memory
        addd L00C1          ; d = d + memory contents
        subd InverseRPM_W   ; d = d - memory contents
        tst  InverseRPM_B
        bne  L1473          ; branch if not equal (not zero)
        cmpa timerCSR       ; compare a with memory contents
        std  ocrHi          ; store d (a&b) into memory
        ldb  counterHi      ; load b with memory contents
        incb
        sba                 ; subtract (a=a-b)
        bpl  L1474          ; branch if plus (hi bit 0)
L1473:  lda  #$1b           ; load a with value -00011011-
L1454:  sta  L00BE          ; store a into memory
        ldd  #counterLo     ; load d (a&b) with value -00000000 00001010-
        addd counterHi      ; d = d + memory contents
        std  ocrHi          ; store d (a&b) into memory
        mul                 ; multiply (d=a*b)
        ldd  Scratch_bc     ; load d (a&b) with memory contents
        cmpa timerCSR       ; compare a with memory contents
        std  ocrHi          ; store d (a&b) into memory
        lda  L00BE          ; load a with memory contents
        sta  timerCSR       ; store a into memory
L1474:  lda  ioPort1data    ; load a with memory contents
        ora  #$80           ;  -10000000-
        sta  ioPort1data    ; * turn on knock kill
        lda  BitFlags82     ; load a with memory contents
        bita #b82_SparkStartMode ;  -00000010-
        bne  L1475          ; branch if spark starting mode (not zero)

        ; spark running:
        ldb  ADScanIndex    ; load b with memory contents
        jsr  ADLookup       ; call subroutine
        ldx  #RawBattVoltage    ; load index with value
        jsr  RegulateVoltage    ; call subroutine

L1475:  ldx  InverseRPM_B
        cpx  RNFPRD_FuelTransitionStartToRun    ;  (data is 00a7)
        blo  L1476          ; branch if above transition point
        cpx  STFPRD_FuelTransitionRunToStart    ;  (data is 0104)
        blo  L1477          ; branch if above transition point (slow but still above transition)
        ; otherwise, below run/start transition, switch back to fuel start mode:
        lda  BitFlags82     ; load a with memory contents
        ; bit 7, Engine Not Running, bit 3, still decaying fuel
        ora  #b82_EngineNotRunning | b82_DecayingFuelStartToRun    ; $88           ;  -10001000-
        sta  BitFlags82     ; store a into memory
        bra  L1477          ; branch

        ; fuel run mode:
L1476:  lda  BitFlags82     ; load a with memory contents
        lsla                ; logical shift left a
        bcc  L1477          ; branch if b82_EngineNotRunning clear (already running)
        lsra                ; clear b82_EngineNotRunning (set engine running)
        ora  #b82_FuelRunMode      ; set fuel running mode -00100000-
        ; bit 7 has been cleared, Engine Running, bit 5 $20 is set for Fuel Running
        sta  BitFlags82     ; store a into memory
        ldx  Counter_EngineRunningTime
        clra                ; a = 0
        clrb                ; b = 0
        std  Counter_EngineRunningTime          ; store d (a&b) into memory
        cpx  #$14           ;  -00000000 00010100-
        ldb  BitFlags80     ; load b with memory contents
        bcc  L1478          ; branch if greater or equal
        andb #<~b80_BadMAP  ; clear bit -01111111-
        stb  BitFlags80     ; store b into memory
L1478:  bpl  L1479          ; branch if plus (hi bit 0)
        ldb  #$20           ; load b with value -00100000-
        ldx  #FaultCode4    ; load index with value
        jsr  SetNonCriticalFaultCode    ; call subroutine
L1479:  lda  BarometricPressure    ; load a with memory contents
        cmpa BAROHI_BaroReadMaxValid    ; compare a with memory contents (data is 56)
        bhs  L1480          ; branch if greater or equal
        cmpa BAROLO_BaroReadMinValid    ; compare a with memory contents (data is 2e)
        bhs  L1477          ; branch if greater or equal
L1480:  lda  BARDFT_BaroReadDefault    ; load a with memory contents (data is 54)
        sta  BarometricPressure    ; store a into memory
L1477:  lda  MapValue            ; load a with memory contents
        cmpa MapAverageValue    ; compare a with memory contents
        blo  L1481          ; branch if lower
        ldx  #Block_MAP     ; load index with value
        jsr  L1346          ; call subroutine
L1481:  clra                ; a = 0
        ldb  MapAverageValue          ; load b with memory contents
        subb MapValue            ; b = b-memory contents
        bls  L1482          ; branch if lower or same
        stb  Scratch_bb     ; store b into memory
        ldb  RPM_W          ; load b with memory contents
        cmpb L1483          ; compare b with memory contents (data is 14)
        blo  L1482          ; branch if lower
        ldb  Counter_EngineRunningTime          ; load b with memory contents
        cmpb L1484          ; compare b with memory contents (data is 02)
        blo  L1482          ; branch if lower
        ldb  Scratch_bb     ; load b with memory contents
        lda  Timer_Leanout  ; load a with memory contents
        beq  L1485          ; branch if equal (zero)
        addb L1486          ;  (data is 02)
L1485:  clra                ; a = 0
        subb L1487          ; subtract memory contents from b (data is 03)
        bcs  L1482          ; branch if lower
        lda  L1488          ; load a with memory contents (data is ed)
        cmpb L1489          ; compare b with memory contents (data is 08)
        bhs  L1482          ; branch if greater or equal
        lda  Timer_Leanout          ; load a with memory contents
        beq  L1490          ; branch if equal (zero)
        inca                ; increment a
L1482:  sta  Timer_Leanout          ; store a into memory
L1490:  ldx  #Block_TPS     ; load index with value
        jsr  L1346          ; call subroutine
        ldb  EPPsSinceCalChange     ; load b with memory contents
        incb
        beq  L1491          ; branch if equal (zero)
        stb  EPPsSinceCalChange ; store b into memory
L1491:  rti                 ; return from interrupt
;

;  ______________________
; |                      \
; |   Idle Control       |
; \______________________|

MM_DetermineAISStartPosition   subroutine
        lda  L0060          ; load a with memory contents
        bita #$08           ;  -00001000-
        bne  L1492          ; branch if not equal (not zero)
        ldb  Counter_MainLoop1    ; load b with memory contents
        andb #$7e           ;  -01111110-
        bne  L1492          ; branch if not equal (not zero)
        anda #$fb           ; AND a with value -11111011-
        sta  L0060          ; store a into memory
L1492:  ldb  L004F          ; load b with memory contents
        orb  BitFlags82
        bpl  L1493          ; branch if Engine Running (hi bit 0)
        lda  L00E2          ; load a with memory contents
        cmpa L0042          ; compare a with memory contents
        bne  L1494          ; branch if not equal (not zero)
        ldb  L0060          ; load b with memory contents
        bitb #$40           ;  -01000000-
        bne  L1494          ; branch if not equal (not zero)
        ldb  L004F          ; load b with memory contents
        andb #$7f           ;  -01111111-
        stb  L004F          ; store b into memory
L1494:  lda  CoolantTemp    ; load a with memory contents
        ldx  #AISTRT_AisStartPosition    ; load index with value
        jsr  LookupTable    ; call subroutine
        jmp  L1495

L1493:  ldb  BitFlags80     ; load b with memory contents
        bpl  L1496          ; branch if b80_BadMAP is clear
        lda  L1497          ; load a with memory contents (data is 0c)
        sta  L00E2          ; store a into memory
        lda  BitFlags85     ; load a with memory contents
        anda #$f7           ; AND a with value -11110111-
        sta  BitFlags85     ; store a into memory
        rts                 ; return from subroutine

L1496:  clra                ; a = 0
        sta  Scratch_b7     ; store a into memory
        lda  Z83_PortCdata  ; load a with memory contents
        anda #$04           ; * check a/c clutch input
        sta  Scratch_b6     ; store a into memory
        ldb  BitFlags85     ; load b with memory contents
        andb #$04           ;  -00000100-
        xorb Scratch_b6
        beq  L1498          ; branch if equal (zero)
        lda  Counter_EngineRunningTime          ; load a with memory contents
        cmpa ACCTIM_ACDelayFromStartRun          ; compare a with memory contents (data is 04)
        blo  L1499          ; branch if lower
        xorb BitFlags85
        stb  BitFlags85     ; store b into memory
        bitb #$04           ;  -00000100-
        beq  L1500          ; branch if equal (zero)
        lda  BitFlags81     ; load a with memory contents
        bpl  L1501          ; branch if b81_OffIdle is clear
        ldx  #ACCTBL_AisACOffIdleDelta    ; load index with value
        ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        jsr  LookupTable    ; call subroutine
        ldb  L1502          ; load b with memory contents (data is f0)
        mul                 ; multiply (d=a*b)
        tab                 ; b = a
        lda  L00E2          ; load a with memory contents
        sba                 ; subtract (a=a-b)
        bcc  L1503          ; branch if greater or equal
        clra                ; a = 0
L1503:  sta  L00E2          ; store a into memory
        bra  L1504          ; branch

L1501:  lda  L1505          ; load a with memory contents (data is f8)
        jsr  L1506          ; call subroutine

L1504:  clra                ; a = 0
        sta  L00E4          ; store a into memory
        bra  L1507          ; branch

L1500:  lda  L1508          ; load a with memory contents (data is 16)
        sta  L00E4          ; store a into memory
        lda  BitFlags81     ; load a with memory contents
        bmi  L1509          ; branch if b81_OffIdle is set
        lda  L1510          ; load a with memory contents (data is 12)
        jsr  L1506          ; call subroutine
L1509:  lda  L00E4          ; load a with memory contents
        beq  L1511          ; branch if equal (zero)
L1514:  deca                ; decrement a
L1511:  sta  L00E4          ; store a into memory
L1507:  clra                ; a = 0
        sta  Counter_L00B3  ; store a into memory
L1499:  lda  BitFlags85     ; load a with memory contents
        ora  #$08           ;  -00001000-
        bra  L1512          ; branch

L1498:  tsta                ; test a
        bne  L1499          ; branch if not equal (not zero)
        lda  L00E4          ; load a with memory contents
        cmpa #$01           ; compare a with value -00000001-
        blo  L1513          ; branch if lower
        bne  L1514          ; branch if not equal (not zero)
        lda  BitFlags81     ; load a with memory contents
        bmi  L1515          ; branch if b81_OffIdle is set
        lda  L1516          ; load a with memory contents (data is 07)
        jsr  L1517          ; call subroutine
L1515:  clra                ; a = 0
        sta  L00E4          ; store a into memory
        sta  Counter_L00B3  ; store a into memory
L1513:  lda  BitFlags85     ; load a with memory contents
        anda #$f7           ; AND a with value -11110111-
L1512:  sta  BitFlags85     ; store a into memory
        lda  Scratch_b7     ; load a with memory contents
        beq  L1518          ; branch if equal (zero)
L1522:  rts                 ; return from subroutine

L1518:  ldb  BitFlags81     ; load b with memory contents
        bpl  L1519          ; branch if b81_OffIdle is clear
        lda  L004F          ; load a with memory contents
        anda #$ef           ; AND a with value -11101111-
        sta  L004F          ; store a into memory
        ldb  Z83_PortCdata  ; load b with memory contents
        bitb #$10           ; * check brake switch
        beq  L1520          ; branch if equal (zero)
        ldx  #BRKRPM_AisBrakeOverride    ; load index with value
        lda  TpsVolts            ; load a with memory contents
        suba LowestSessionTPS
        jsr  LookupTable    ; call subroutine
        cmpb RPM_W
        blo  L1519          ; branch if lower
L1520:  jmp  L1521

L1519:  ldb  BitFlags85     ; load b with memory contents
        bitb #$20           ;  -00100000-
        bne  L1520          ; branch if not equal (not zero)
        lda  BitFlags84     ; load a with memory contents
        anda #$40           ; AND a with value -01000000-
        ora  Timer_AISDelay          ; OR a with memory contents
        bne  L1522          ; branch if not equal (not zero)
        lda  L00E2          ; load a with memory contents
        cmpa #$04           ; compare a with value -00000100-
        blo  L1523          ; branch if lower
        andb #$ef           ;  -11101111-
        stb  BitFlags85     ; store b into memory
L1523:  lda  L004F          ; load a with memory contents
        anda #$10           ; AND a with value -00010000-
        beq  L1524          ; branch if equal (zero)
        ldx  #DRPNK_AisParkNeutralKick    ; load index with value
        bita #$08           ;  -00001000-
        beq  L1525          ; branch if equal (zero)
        ldx  #PNDRK_AisNeutralDriveKick    ; load index with value
        ldd  TargetIdleSpeed    ; load d (a&b) with memory contents
        subd RPM_W          ; d = d - memory contents
        bra  L1526          ; branch

L1525:  ldd  RPM_W          ; load d (a&b) with memory contents
        subd TargetIdleSpeed    ; d = d - memory contents
L1526:  bcs  L1524          ; branch if lower
        subd PNDLT_DeltaRPMEngineChangesToInitPNDriveKicks          ;  (data is 00c8)
        bcs  L1524          ; branch if lower
        lda  CoolantTemp    ; load a with memory contents
        jsr  LookupTable    ; call subroutine
        jsr  L1341          ; call subroutine
        lda  L004F          ; load a with memory contents
        anda #$ef           ; AND a with value -11101111-
        sta  L004F          ; store a into memory
        bra  L1528          ; branch

L1524:  ldb  L1529          ; load b with memory contents (data is 3f)
        cmpb Counter_L00B3
        bhs  L1530          ; branch if greater or equal
        lda  L0060          ; load a with memory contents
        anda #$f7           ; AND a with value -11110111-
        sta  L0060          ; store a into memory
        ldx  #FWDAIS_AisPeriodForward    ; load index with value
        ldd  TargetIdleSpeed    ; load d (a&b) with memory contents
        subd RPM_W          ; d = d - memory contents
        bcc  L1531          ; branch if greater or equal
        ldx  #REVAIS_AisPeriodReverse    ; load index with value
        ldd  RPM_W          ; load d (a&b) with memory contents
        subd TargetIdleSpeed    ; d = d - memory contents
L1531:  lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        tst  TRANSM_Transmission    ;  (data is 00)
        beq  L1532          ; branch if equal (zero)
        psha                ; push a onto stack
        lda  L004F          ; load a with memory contents
        bita #$08           ;  -00001000-
        pula                ; pull a from stack
        bne  L1533          ; branch if not equal (not zero)
L1532:  lsrd                ; shift right (d=d/2)
L1533:  tsta                ; test a
        beq  L1534          ; branch if equal (zero)
        ldb  #$ff           ; load b with value -11111111-
L1534:  tba                 ; a = b
        cmpa $01,x          ; compare a with memory at index + value
        blo  L1530          ; branch if lower
        jsr  LookupTable    ; call subroutine
        beq  L1530          ; branch if equal (zero)
        jsr  L1341          ; call subroutine
L1528:  clra                ; a = 0
        sta  Counter_L00B3  ; store a into memory
L1530:  rts                 ; return from subroutine

L1521:  lda  L0060          ; load a with memory contents
        ora  #$08           ;  -00001000-
        sta  L0060          ; store a into memory
        lda  CoolantTemp    ; load a with memory contents
        ldx  #CLDPOS_AisOffIdlePositionCold    ; load index with value
        jsr  LookupTable    ; call subroutine
        sta  Scratch_b6     ; store a into memory
        lda  BarometricPressure    ; load a with memory contents
        ldx  #BARAIS_AisBaroCompensation    ; load index with value
        jsr  LookupTable    ; call subroutine
        lda  Scratch_b6     ; load a with memory contents
        mul                 ; multiply (d=a*b)
        asld                ; shift left (d=d*2)
        bcs  L1535          ; branch if lower
        asld                ; shift left (d=d*2)
        bcs  L1535          ; branch if lower
        sta  Scratch_b6     ; store a into memory
        lda  RPM_W          ; load a with memory contents
        ldx  #RPMPOS_AisOffIdlePositionRpm    ; load index with value
        jsr  LookupTable    ; call subroutine
        cmpa Scratch_b6     ; compare a with memory contents
        blo  L1536          ; branch if lower
        sta  Scratch_b6     ; store a into memory
L1536:  lda  TpsVolts            ; load a with memory contents
        suba LowestSessionTPS
        bcc  L1537          ; branch if greater or equal
        clra                ; a = 0
L1537:  ldx  #THRPOS_AisOffIdlePositionThrottle    ; load index with value
        jsr  LookupTable    ; call subroutine
        cmpa Scratch_b6     ; compare a with memory contents
        blo  L1538          ; branch if lower
        sta  Scratch_b6     ; store a into memory
L1538:  clra                ; a = 0
        ldb  BitFlags85     ; load b with memory contents
        bitb #$04           ;  -00000100-
        bne  L1539          ; branch if not equal (not zero)
        lda  ACPOS_AcPosition    ; load a with memory contents (data is 17)
        bitb #$20           ;  -00100000-
        bne  L1539          ; branch if not equal (not zero)
        ldb  CoolantTemp    ; load b with memory contents
        cmpb ACTMP_AisTempSwap    ; compare b with memory contents (data is a6)
        blo  L1539          ; branch if lower
        ldx  #ACCTBL_AisACOffIdleDelta    ; load index with value
        ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        jsr  LookupTable    ; call subroutine
        ldb  BitFlags85     ; load b with memory contents
        bitb #$08           ;  -00001000-
        beq  L1539          ; branch if equal (zero)
        ldb  L1540          ; load b with memory contents (data is 80)
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
L1539:  ldb  Scratch_b6     ; load b with memory contents
        aba                 ; a=a+b
        bcs  L1535          ; branch if lower
        ldb  L004F          ; load b with memory contents
        bitb #$08           ;  -00001000-
        bne  L1495          ; branch if not equal (not zero)
        adda PNPOS_ParkNeutralPosition    ;  (data is 00)
        bcs  L1535          ; branch if lower
L1495:  ldb  L0060          ; load b with memory contents
        bitb #$10           ;  -00010000-
        bne  L1541          ; branch if not equal (not zero)
        ldb  BitFlags85     ; load b with memory contents
        bitb #$10           ;  -00010000-
        bne  L1541          ; branch if not equal (not zero)
        adda CALPOS_AisPositionGuess    ;  (data is 1a)
        bcs  L1535          ; branch if lower
L1541:  cmpa L1542          ; compare a with memory contents (data is dc)
        blo  L1543          ; branch if lower
L1535:  lda  L1542          ; load a with memory contents (data is dc)
L1543:  ldb  BitFlags85     ; load b with memory contents
        bitb #$20           ;  -00100000-
        beq  L1544          ; branch if equal (zero)
        andb #$df           ;  -11011111-
        stb  BitFlags85     ; store b into memory
        bra  L1545          ; branch

L1544:  cmpa L00E2          ; compare a with memory contents
        bhs  L1545          ; branch if greater or equal
        ldb  Counter_MainLoop1    ; load b with memory contents
        bitb CALRAT_AisCalibrateRate    ;  (data is 0e)
        bne  L1546          ; branch if not equal (not zero)
        lda  L00E2          ; load a with memory contents
        deca                ; decrement a
L1545:  sta  L00E2          ; store a into memory
L1546:  rts                 ; return from subroutine

; *****************************************

L1506   subroutine
        ldb  L0060          ; load b with memory contents
        bitb #$04           ;  -00000100-
        beq  L1547          ; branch if equal (zero)
        andb #$fb           ;  -11111011-
        stb  L0060          ; store b into memory
        lda  L1505          ; load a with memory contents (data is f8)
        ldb  BitFlags85     ; load b with memory contents
        bitb #$04           ;  -00000100-
        beq  L1548          ; branch if equal (zero)
        lda  L1510          ; load a with memory contents (data is 12)
        ldb  L00E4          ; load b with memory contents
        bne  L1548          ; branch if not equal (not zero)
        adda L1516          ;  (data is 07)
L1548:  nega                ; negate a (set high bit)
        clrb                ; b = 0
        stb  L00E4          ; store b into memory
        bra  L1517          ; branch

L1547:  orb  #$0c           ;  -00001100-
        stb  L0060          ; store b into memory
L1517:  ldb  #$ff           ; load b with value -11111111-
        stb  Scratch_b7     ; store b into memory
L1341:  tsta                ; test a
        bmi  L1549          ; branch if minus(hi bit 1)
        ldb  L1542          ; load b with memory contents (data is dc)
        adda L00E2
        bcs  L1550          ; branch if lower
        cba
        blo  L1551          ; branch if lower
L1550:  tba                 ; a = b
        bra  L1551          ; branch

L1549:  adda L00E2
        bcs  L1551          ; branch if lower
        clra                ; a = 0
L1551:  sta  L00E2          ; store a into memory
        rts                 ; return from subroutine

; *****************************************

MM_DetermineTargetIdleSpeed    subroutine
        ldb  L004F          ; load b with memory contents
        bitb #$10           ;  -00010000-
        bne  L1552          ; branch if not equal (not zero)
        ldb  BitFlags84     ; load b with memory contents
        bitb #$04           ;  -00000100-
        beq  L1553          ; branch if equal (zero)
        lda  #$32           ; load a with value -00110010-
        ldb  L00C9          ; load b with memory contents
        cmpb #$70           ;  -01110000-
        beq  L1554          ; branch if equal (zero)
        lda  #$bc           ; load a with value -10111100-
        bra  L1554          ; branch

L1553:  lda  #$7d           ; load a with value -01111101-
        ldb  BitFlags80     ; load b with memory contents
        bitb #b80_BadCoolantTemp      ;  -00000010-
        bne  L1554          ; branch if not equal (not zero)
        ldx  #AISCLD_AisIdleSpeedDrive    ; load index with value
        ldb  Z83_PortCdata  ; load b with memory contents
        bitb #$08           ; * check park/neutral
        bne  L1555          ; branch if not equal (not zero)
        ldx  #AISPN_AisIdleSpeed    ; load index with value
L1555:  lda  CoolantTemp    ; load a with memory contents
        jsr  LookupTable    ; call subroutine

        ldb  STTIME_AdditionalStartupIdleTime    ; load b with memory contents (data is 0b)
        cmpb Counter_EngineRunningTime
        blo  L1554          ; branch if lower
        adda DRIDHT_AdditionalStartupIdle    ;  (data is 0d)
        bcc  L1554          ; branch if greater or equal
        lda  #$ff           ; load a with value -11111111-
L1554:
        clrb                ; b = 0
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        lsrd                ; shift right (d=d/2)
        std  TargetIdleSpeed    ; store d (a&b) into memory
L1552:  rts                 ; return from subroutine

; *****************************************

LE6E8   subroutine
        lda  RawBattVoltage ; load a with memory contents
        cmpa BTVAIS_AisMinVoltageToControlIdle ; (data is 71)
        blo  L1934          ; branch if lower
MoveAISStepper:
        ldb  L0042          ; load b with memory contents
        lda  L0060          ; load a with memory contents
        bita #$40           ;  -01000000-
        beq  Operate_AIS_Motor    ; branch if equal (zero)
        lda  #$ff           ; load a with value -11111111-
        tstb
        bne  L1935          ; branch if not equal (not zero)
        lda  L0060          ; load a with memory contents
        anda #$bf           ; AND a with value -10111111-
        sta  L0060          ; store a into memory
        bra  L1934          ; branch

AISTEP_AisMotorBitPatterns:
        db  $06 ;  0 1 1 0
        db  $0a ;  1 0 1 0
        db  $09 ;  1 0 0 1
        db  $05 ;  0 1 0 1

Operate_AIS_Motor:
        lda  #$01           ; load a with value -00000001-
        cmpb L00E2
        beq  L1934          ; branch if equal (zero)
        bcs  L1935          ; branch if lower
        nega                ; negate a (set high bit)
L1935:  sta  Scratch_b6     ; store a into memory
        ldx  #AISTEP_AisMotorBitPatterns    ; load index with value
        ldb  AISMotorPositionStorage        ; load b with memory contents
        addb Scratch_b6     ; b=b+memory contents
        andb #$03           ;  -00000011-
        stb  AISMotorPositionStorage        ; store b into memory
        abx                 ; add b to index
        ldb  L0042          ; load b with memory contents
        addb Scratch_b6     ; b=b+memory contents
        stb  L0042          ; store b into memory
        sei                 ; disable interrupts (set)
        lda  Z83_PortBdata  ; load a with memory contents
        anda #$f0           ; AND a with value -11110000-
        ora  $00,x
        sta  Z83_PortBdata
L1934:  lda  counterHi      ; load a with memory contents
        sta  L00E3          ; store a into memory
        ldd  #$ff6f         ; load d (a&b) with value -11111111 01101111-
        std  Z83_PortADataDirection
        lda  Z83_PortBdata  ; load a with memory contents
        sta  Scratch_b9     ; store a into memory
        lda  Z83_PortA_copy ; load a with memory contents
        ldb  BitFlags82     ; load b with memory contents
        bitb #b82_bit0      ;  -00000001-
        bne  L1936          ; branch if not equal (not zero)
        anda #<~PortA_Wastegate ; $fb ; test bit -11111011-
        ldb  Z83_PortAdata  ; load b with memory contents
        andb #PortA_Wastegate   ; $04 ; * wastegate
        aba                 ; a=a+b
L1936:  sta  Z83_PortAdata
        clra                ; a = 0
        ldb  RawBattVoltage ; load b with memory contents
        cmpb #$a2           ;  -10100010-
        bhi  L1937          ; branch if higher
        cmpb #$61           ;  -01100001-
        bhs  L1938          ; branch if greater or equal
        cmpb #$51           ;  -01010001-
        blo  L1937          ; branch if lower
        ldb  #$61           ; load b with value -01100001-
L1938:  lda  #$2d           ; load a with value -00101101-
        mul                 ; multiply (d=a*b)
        nega                ; negate a (set high bit)
        adda #$1d           ;  -00011101-
L1937:  sta  Scratch_ba     ; store a into memory
        clra                ; a = 0
        ldb  RawBattVoltage ; load b with memory contents
        cmpb #$a2           ;  -10100010-
        bhs  L1939          ; branch if greater or equal
        deca                ; decrement a
L1939:  ldb  #$60           ; load b with value -01100000-
        std  Z83_PortADataDirection
        abx                 ; add b to index
        ldb  Scratch_ba     ; load b with memory contents
        ldx  #BriefDelayTable  ; load index with value
        abx                 ; add b to index
        jmp  $00,x          ; indexed jump

BriefDelayTable:
        nop                 ; no operation
        nop                 ; no operation
        nop                 ; no operation
        nop                 ; no operation
        nop                 ; no operation
        nop                 ; no operation
        nop                 ; no operation
        nop                 ; no operation
        nop                 ; no operation
        nop                 ; no operation
        nop                 ; no operation
        nop                 ; no operation
        ldb  Scratch_b9     ; load b with memory contents
        tba                 ; a = b
        xorb Z83_PortBdata
        andb #$0f           ;  -00001111-
        beq  L1556          ; branch if equal (zero)
        cli                 ; enable interrupts (clear)
        lda  RawBattVoltage ; load a with memory contents
        cmpa #$ba           ; compare a with value -10111010-
        blo  L1557          ; branch if lower
        lda  #$15           ; load a with value -00010101-
        jsr  L1042          ; call subroutine
        bra  L1558          ; branch

RefLoc302:
    db  $00,$40,$00,$10,$08,$04,$02,$01 ; ".@......"
    db  $0f,$12,$11,$10,$0e,$0d,$0a,$0c ; "........"
L1556:  anda #$6f           ; AND a with value -01101111-
        ora  #$60           ;  -01100000-
        sta  Z83_PortBDataDirection
        cli                 ; enable interrupts (clear)
L1558:  lda  BitFlags82     ; load a with memory contents
        lsra
        bcs  L1557          ; branch if b82_bit0 clear
        ldx  #RefLoc302     ; load index with value
        ldb  Counter_MainLoop1    ; load b with memory contents
        andb #$07           ;  -00000111-
        abx                 ; add b to index
        sei                 ; disable interrupts (set)
        lda  BitFlags7f     ; load a with memory contents
        anda #$04           ; AND a with value -00000100-
        ldb  Z83_PortA_copy ; load b with memory contents
        andb #<~PortA_Wastegate    ; $fb ; * wastegate off
        aba                 ; a=a+b
        xora Z83_PortAdata
        cli                 ; enable interrupts (clear)
        bita $00,x
        beq  L1557          ; branch if equal (zero)
        lda  $08,x          ; load a with memory at index + value
        cmpa #$0d           ; compare a with value -00001101-
        bne  L1559          ; branch if not equal (not zero)
        jsr  SetFaultCode   ; call subroutine
        bcc  L1557          ; branch if greater or equal
        ldb  #b80_WastegateLimpinMode ; load b with value -00010000-
        sei                 ; disable interrupts (set)
        orb  BitFlags80
        stb  BitFlags80     ; store b into memory
        cli                 ; enable interrupts (clear)
        rts                 ; return from subroutine

; switch on A/C if required and if the engine has been running for 2.6 seconds:
L1559:  cmpa #$0e           ; compare a with value -00001110-
        bne  L1560          ; branch if not equal (not zero)
        ldb  BitFlags82     ; load b with memory contents
        bmi  L1557          ; branch if b82_EngineNotRunning set
        ldb  Z83_PortCdata  ; load b with memory contents
        bitb #$04           ; * check a/c clutch input
        beq  L1557          ; branch if equal (zero)
        ldb  Counter_EngineRunningTime          ; load b with memory contents
        beq  L1557          ; branch if equal (zero)
L1560:  jsr  L1042          ; call subroutine
L1557:  rts                 ; return from subroutine

;  ______________________
; |                      \
; |   Reset Routine      |
; \______________________|

ResetRoutine    subroutine
        sei                 ; disable interrupts (set)
        ldd  #$ff00         ; load d (a&b) with value -11111111 00000000-
        std  Z83_PortAdata  ; * turn off baro,asd,wastegate,fan,ac,cruise sol,egr,cruise vent
        lda  Z85_ADConverter5    ; load a with memory contents
        lds  #$00ff         ; * reset stack to top of memory
        ldd  #$be04         ; load d (a&b) with value -10111110 00000100-
        std  ioPort1ddr     ; store d (a&b) into memory
        ldd  #$fd6f         ; load d (a&b) with value -11111101 01101111-
        std  Z83_PortADataDirection
        ldd  #$0814         ; load d (a&b) with value -00001000 00010100-
        sta  ioPort1data    ; turn on cts bias, off: injectors, voltage reg, power loss light, knock kill
L1561:  jsr  MM_ToggleSerialClock    ; call subroutine
        decb
        bne  L1561          ; branch if not equal (not zero)
        lda  Z85_ADConverter1    ; load a with memory contents
        cmpa #$0f           ; compare a with value -00001111-
        bls  L1562          ; branch if lower or same
        cmpa #$40           ; compare a with value -01000000-
        bhs  L1563          ; branch if greater or equal
        ; at this point, b is zero
        stb  ioPort1data    ; store b into memory
L1563:  lda  ramControl     ; load a with memory contents
        bmi  L1564          ; * branch if standby power has NOT been lost

L1562:  ldd  #$150a         ; load d (a&b) with value -00010101 00001010-
        std  sciModeControl ; store d (a&b) into memory
        lda  #$55           ; load a with value -01010101-
        sta  ocrHi          ; store a into memory
L1569:  ldd  sciTransceiveCtrl    ; load d (a&b) with memory contents
        lda  ocrHi          ; load a with memory contents
        sta  sciTxData      ; store a into memory
        ldx  #$01aa         ; load index with value
L1566:  cpx  #timerCSR      ;  -00000000 00001000-
        bne  L1565          ; branch if not equal (not zero)
        ldb  Z85_ADConverter5    ; load b with memory contents
L1565:  dex                 ; decrement index (x=x-1)
        bne  L1566          ; branch if not equal (not zero)
        ldb  Z85_ADConverter1    ; load b with memory contents
        cmpb #$40           ;  -01000000-
        bhs  L1567          ; branch if greater or equal
        clr  ioPort1data    ; turn off injectors, cts bias, voltage reg, power loss light, knock kill
L1567:  tst  ramControl
        bpl  L1568          ; branch if plus (hi bit 0)
        tstb
        bpl  L1564          ; branch if plus (hi bit 0)
L1568:  ldb  sciTransceiveCtrl    ; load b with memory contents
        bpl  L1564          ; branch if plus (hi bit 0)
        ldb  sciRxData      ; load b with memory contents
        cba
        bne  L1569          ; branch if not equal (not zero)
        ldx  #ioPort1ddr    ; load index with value
        ldb  #$fe           ; load b with value -11111110-
L1572:  tba                 ; a = b
L1570:  ldb  sciTransceiveCtrl    ; load b with memory contents
        bpl  L1570          ; branch if plus (hi bit 0)
        ldb  sciRxData      ; load b with memory contents
        stb  sciTxData      ; store b into memory
        cmpa #$fe           ; compare a with value -11111110-
        bne  L1571          ; branch if not equal (not zero)
        abx                 ; add b to index
        stx  ocrHi          ; store index into memory
        inca                ; increment a
        tab                 ; b = a
L1571:  cmpa #$ff           ; compare a with value -11111111-
        beq  L1572          ; branch if equal (zero)
        stb  $00,x
        inx                 ; increment index (x=x+1)
        deca                ; decrement a
        bne  L1570          ; branch if not equal (not zero)
        ldx  ocrHi          ; $0b
        jmp  $00,x          ; indexed jump

L1564:  lda  #$40           ; load a with value -01000000-
        sta  L0060          ; store a into memory
        ldx  #L0060         ; load index with value
        lda  LowestSessionTPS    ; load a with memory contents
        cmpa LowestHistoricalTPS          ; compare a with memory contents
        bne  L1573          ; branch if not equal (not zero)
        lda  ramControl     ; load a with memory contents
        bmi  L1574          ; * branch if standby power has NOT been lost
L1573:  lda  L1575          ; load a with memory contents (data is 5c)
        ldb  #$80           ; load b with value -10000000-
        std  LowestSessionTPS    ; store d (a&b) into memory
        sta  LowestHistoricalTPS          ; store a into memory
        stb  L0043          ; store b into memory
        stb  L0044          ; store b into memory
        lda  #$df           ; load a with value -11011111-
        sta  L0042          ; store a into memory
        ldx  #LowestHistoricalTPS         ; load index with value
L1574:  jsr  MM_ToggleSerialClock    ; call subroutine
        clrb                ; b = 0
L1576:  inx                 ; increment index (x=x+1)
        stb  $00,x
        cpx  #$00ff         ;  -00000000 11111111-
        bne  L1576          ; branch if not equal (not zero)
        stb  Z83_PortCDataDirection    ; store b into memory
        lda  ramControl     ; load a with memory contents
        bmi  L1577          ; * branch if standby power has NOT been lost
        lda  ErrorCodes4    ; load a with memory contents
        ora  #$c0           ;  -11000000-
        sta  ErrorCodes4    ; store a into memory
L1577:  lda  #$c0           ; * enable bits 6 and 7 (ram/standby)
        sta  ramControl     ; * ensure ram is configured properly
        lda  L0060          ; load a with memory contents
        xora #$40           ; XOR a with value -01000000-
        sta  L0060          ; store a into memory
        stb  Z83_PortAcontrol    ; store b into memory
        stb  Z83_PortBcontrol    ; store b into memory
        stb  Z83_PortCcontrol    ; store b into memory
        ldb  AISMotorPositionStorage        ; load b with memory contents
        ldx  #AISTEP_AisMotorBitPatterns    ; load index with value
        abx                 ; add b to index
        lda  $00,x          ; load a with memory at index + value
        sta  Z83_PortBdata
        lda  L004F          ; load a with memory contents
        ora  #$80           ;  -10000000-
        anda #$83           ; AND a with value -10000011-
        ldb  Z83_PortAdata  ; load b with memory contents
        bitb #PortA_ASD     ; * ASD relay
        bne  L1578          ; branch if not equal (not zero)
        ora  #$04           ;  -00000100-
L1578:  sta  L004F          ; store a into memory
        ldb  #$ff           ; load b with value -11111111-
        stb  Z83_PortADataDirection    ; store b into memory
        stb  Z83_PortA_copy ; store b into memory
        jsr  MM_ToggleSerialClock  ; call subroutine
        jsr  CheckMapSensor        ; call subroutine
        sta  BarometricPressure    ; store a into memory
        sta  MapValue       ; store a into memory
        sta  L0091          ; store a into memory
        sta  MapAverageValue       ; store a into memory
        sta  L0070          ; store a into memory
        ldd  #$408a         ; load d (a&b) with value -01000000 10001010-
        sta  CylinderRetard1          ; store a into memory
        ; b is Engine Not Running, spark starting mode, fuel starting mode, start decay on
        ; b is b82_EngineNotRunning | b82_bit4 | b82_bit2
        stb  BitFlags82     ; store b into memory
        ldd  #$10e9         ; load d (a&b) with value -00010000 11101001-
        sta  BitFlags84     ; store a into memory
        stb  timerControl1  ; store b into memory
        ldd  counterHi      ; load d (a&b) with memory contents
        addd #ioPort3CSR    ;  -00000000 00001111-
        std  ocrHi          ; store d (a&b) into memory
        std  ocr2Hi         ; store d (a&b) into memory
        std  ocr3Hi         ; store d (a&b) into memory
        ldd  #$f3f6         ; load d (a&b) with value -11110011 11110110-
        sta  timerControl2  ; store a into memory
        stb  ioPort2ddr     ; store b into memory
        ; Check if this is the first time here since battery power lost:
        lda  L004F          ; load a with memory contents
        tab                 ; b = a
        andb #$03           ;  -00000011-
        cmpb #$02           ;  -00000010-
        bne  L1579          ; branch if not equal (not zero)
        ; first time, run the cold boot.
        jmp  ResetStartup

L1579:  inca                ; increment a
        sta  L004F          ; store a into memory
L1924:  lda  Z85_ADConverter2    ; load a with memory contents
        ldd  #$e2a2         ; load d (a&b) with value -11100010 10100010-
        sta  Z22_Timer6840_1
        stb  Z22_Timer6840  ; store b into memory
        inca                ; increment a
        sta  Z22_Timer6840_1
        stb  Z22_Timer6840  ; store b into memory
        ldd  #$04ff         ; load d (a&b) with value -00000100 11111111-
        sta  BitFlags81     ; store a into memory sets b81_O2Rich
        sta  L00CD          ; store a into memory
        lda  #b80_BadMAP    ; load a with value -10000000-
        sta  BitFlags80     ; store a into memory
        lda  BitFlags85     ; load a with memory contents
        ora  #$04           ;  -00000100-
        sta  BitFlags85     ; store a into memory
        lda  BitFlags83     ; load a with memory contents
        ora  #$02           ;  -00000010-
        sta  BitFlags83     ; store a into memory
        ; b is still $ff
        stb  Counter_DistTimerOverflow          ; store b into memory
        stb  InverseRPM_B          ; store b into memory
        stb  ColdLoadEnrichmentValue          ; store b into memory
        stb  Timer_Over5PSIBoost    ; store b into memory
        stb  BoostGoal      ; store b into memory
L1580:  decb
        bne  L1580          ; branch if not equal (not zero)
        lda  Z85_ADConverterCoolantTemp    ; load a with memory contents
        sta  RawBattTemp    ; store a into memory
        stb  ChEMreserved   ; after the countdown timer, b == 0
L1581:  decb
        bne  L1581          ; branch if not equal (not zero)
        lda  Z85_ADConverter4    ; load a with memory contents
        sta  TpsVolts            ; store a into memory
        sta  TpsAverageValue  ; store a into memory
        ldb  LowestSessionTPS    ; load b with memory contents
        addb L1582          ;  (data is 03)
        cmpb L1575          ; compare b with memory contents (data is 5c)
        blo  L1583          ; branch if lower
        adda #$06           ;  -00000110-
        ldb  L1575          ; load b with memory contents (data is 5c)
        cba
        bhs  L1583          ; branch if greater or equal
        tab                 ; b = a
L1583:  stb  LowestSessionTPS    ; store b into memory
        stb  LowestHistoricalTPS          ; store b into memory
        clrb                ; b = 0
L1584:  decb
        bne  L1584          ; branch if not equal (not zero)
        lda  Z85_ADConverter5    ; load a with memory contents
        sta  ChargeTempVolts    ; store a into memory
        ldb  #$19           ; load b with value -00011001-
L1585:  decb
        bne  L1585          ; branch if not equal (not zero)
        lda  Z85_ADConverterCoolantTemp    ; load a with memory contents
        sta  RawCoolantTemp ; store a into memory
        ldx  #ScaleCoolantTempCold_TMPTL1    ; load index with value
        ldb  ioPort1data    ; load b with memory contents
        bitb #$08           ; * check coolant scaling bias
        bne  L1586          ; branch if not equal (not zero)
        ldx  #ScaleCoolantTempHot_TMPTL2    ; load index with value
L1586:  jsr  LookupTable    ; call subroutine
        sta  CoolantTemp    ; store a into memory
        cmpa #$a6           ; compare a with value -10100110-
        bhs  L1587          ; branch if greater or equal
        ldb  RawBattTemp    ; load b with memory contents
        cmpb #$99           ;  -10011001-
        blo  L1587          ; branch if lower
        ldb  #$40           ; load b with value -01000000-
        stb  L00D6          ; store b into memory
L1587:  lda  CoolantTemp    ; load a with memory contents
        cmpa L1588          ; compare a with memory contents (data is 63)
        blo  L1589          ; branch if lower
        cmpa L1590          ; compare a with memory contents (data is e4)
        bhs  L1589          ; branch if greater or equal
        inc  FanOnCount
L1589:  jsr  LC74C          ; call subroutine
        ldd  #$060a         ; load d (a&b) with value -00000110 00001010-
        std  sciModeControl ; store d (a&b) into memory
        ldd  #$13bd         ; load d (a&b) with value -00010011 10111101-
        sta  timerCSR       ; store a into memory
        stb  Z83_PortAdata  ; store b into memory
        stb  Z83_PortA_copy ; store b into memory
        lda  ioPort1data    ; load a with memory contents
        ora  #$20           ;  -00100000-
        sta  ioPort1data    ; * turn on power loss light
        jsr  MM_ToggleSerialClock    ; call subroutine
        ; this is... at key-on
        ldx  #$091e         ; * Pause... 0x091e cycles (2334)
L1591:  dex                 ; decrement index (x=x-1)
        bne  L1591          ; branch if not equal (not zero)
        jsr  MM_ToggleSerialClock    ; call subroutine
        ldx  #PRSHOT_StartPrimeFuelShot    ; load index with value
        lda  CoolantTemp    ; load a with memory contents
        jsr  LookupTable    ; call subroutine
        ldb  #$fa           ; load b with value -11111010-
        mul                 ; multiply (d=a*b)
        std  Pulsewidth_InjectorSolution          ; store d (a&b) into memory
        lda  #$06           ; load a with value -00000110-
        ora  timerControl1  ; OR a with memory contents
        jsr  L1592          ; call subroutine
        lda  #$fd           ; load a with value -11111101-
        sta  FuelPulsewidth_scratch1    ; store a into memory
        ldx  #ocr2Hi        ; load index with value
        jsr  L1419          ; call subroutine
        lda  #$fb           ; load a with value -11111011-
        sta  FuelPulsewidth_scratch1    ; store a into memory
        ldx  #ocr3Hi        ; load index with value
        jsr  L1419          ; call subroutine
        lda  timerStatus    ; load a with memory contents
        sta  ocrHi          ; store a into memory
        lda  icrHi          ; load a with memory contents
        lda  icr2Hi         ; load a with memory contents
        ldb  counterHi      ; load b with memory contents
        jmp  L1593

;  ______________________
; |                      \
; |   Main Loop          |
; \______________________|

; *****************************************
; This is the tight eleven millisecond loop, also called MajorMainLoop.
;
; Note: many times, functionality is broken up so that the time between
; calls to "MM_CriticalTiming_ADScan" doesn't exceed a maximum. Not sure exactly what
; the max is, but be aware of this if developing new functionality.
; _v indicates verified functionality
; *****************************************
ElevenMillisecondLoop:
        jsr  MM_SendRomSerial
        jsr  MM_DetermineDistributorSync_v
        jsr  MM_CheckThrottle_v
        jsr  MM_SubtractOne32ndFromSpeed_v  ; if cruise is set
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_ToggleSerialClock
        jsr  MM_HandleCruiseControl_v
        lda  Counter_MainLoop1
        lsra
        bcc  First44msLoop   ; branch if carry clear (even number)
        jmp  TwentyTwoMillisecondLoop

First44msLoop:
        lsra
        bcc  Second44msLoop  ; branch if carry clear (even number)
        jsr  MM_DetermineCoolantTemp_v
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_DetermineChargeTempVolts_v
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_CalcAdvance_v
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_CalculateFinalFuelingValue_v
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_HandleO2Feedback_v
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_ControlBatteryVoltage
        jsr  MM_TestCoolantTemp
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_TestIfBatteryChargingProperly
        jsr  MM_DetermineStartingFuel_v
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_L1597
        jmp  BackTo11msLoop

Second44msLoop:
        jsr  MM_CriticalTiming_ADScan    ; call subroutine
        jsr  MM_DetermineBaroRead
        jsr  MM_PerformAutoCalAdjustments
        jsr  MM_DetermineTransient3
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_CheckIfO2Valid
        jsr  MM_CheckIfFanIsNeeded
        jsr  MM_DealWithColdLoadEnrichment
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_IncrementSlowCounter
        jsr  MM_DetermineFuelTweaks
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_DetermineTargetIdleSpeed
        jsr  MM_ConfirmO2GoodAndAdjustKnockRetard
        jsr  MM_KnockDetect
        jsr  MM_EngineFanRoutine
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_DealWith_ASD_Relay
     .if UseEGRPurge > 0
        jsr  MM_HandleCanisterPurge_v
     .else
        jsr  MM_SpeedSensitiveAero
     .endif
        jsr  MM_DetermineAdaptiveMemoryCell
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_TestForFaultyMAP
        jsr  MM_WorkWithPowerLossLight
        jsr  MM_CriticalTiming_ADScan
    .if UseDetonationMod > 0
        jsr  MM_DetonationIndicatorMod
    .endif
        jsr  MM_CalcBoostGoal
        jsr  MM_DealWith_AC_Cutout
        jsr  MM_DetermineTransient1
        jsr  HandlePartThrottleEnrichment_MM
        bra  BackTo11msLoop     ; branch

TwentyTwoMillisecondLoop:
        jsr  MM_L1600           ; call subroutine
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_DetermineAISStartPosition
        jsr  MM_ControlAIS
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_LDAB5
        jsr  MM_CalculateTransientFuelAddition
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_LD24E
        jsr  MM_DetermineEngineRPMs
        std  RPM_W
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_CheckForFuelShutoffConditions
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_DealWithThrottlePositionSensor
        jsr  MM_DetermineIdleAdjustmentRequirement
        sei                 ; disable interrupts (set)
        ldd  counterHi      ; load d (a&b) with memory contents
        jsr  MM_HandleDistributorOverflow
        cli                 ; enable interrupts (clear)
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_ControlBoost
        jsr  MM_CriticalTiming_ADScan
        sei                 ; disable interrupts (set)
        ldd  counterHi      ; load d (a&b) with memory contents
        jsr  MM_L1602       ; call subroutine
        cli                 ; enable interrupts (clear)

BackTo11msLoop:
        jsr  MM_CriticalTiming_ADScan
        jsr  MM_L1603       ; call subroutine
        lda  L00AE          ; load a with memory contents
        suba #$15           ;  -00010101-
L1604:  cmpa counterHi      ; compare a with memory contents
        bpl  L1604          ; branch if plus (hi bit 0)
        jsr  MM_CriticalTiming_ADScan    ; call subroutine
        lda  L00AE          ; load a with memory contents
        suba #$10           ;  -00010000-
L1605:  cmpa counterHi      ; compare a with memory contents
        bpl  L1605          ; branch if plus (hi bit 0)
        jsr  MM_CriticalTiming_ADScan    ; call subroutine
        lda  L00AE          ; load a with memory contents
        suba #$08           ;  -00001000-
L1606:  cmpa counterHi      ; compare a with memory contents
        bpl  L1606          ; branch if plus (hi bit 0)
        jsr  MM_CriticalTiming_ADScan    ; call subroutine
        sei                 ; disable interrupts (set)
        lda  L00AE          ; load a with memory contents
        ldb  counterHi      ; load b with memory contents
        cba
        bmi  L1593          ; branch if minus(hi bit 1)
        tab                 ; b = a
L1593:  addb #$2b           ;  -00101011-
        stb  L00AE          ; store b into memory
        cli                 ; enable interrupts (clear)
L1607:  cmpa counterHi      ; compare a with memory contents
        bpl  L1607          ; branch if plus (hi bit 0)
        jmp  ElevenMillisecondLoop

;  ______________________
; |                      \
; |   Diagnostic Mode    |
; \______________________|

HandleOtherTimerInterrupts   subroutine
        ldd  #$4268         ; load d (a&b) with value -01000010 01101000-
        addd ocr3Hi         ; d = d + memory contents
        std  ocr3Hi         ; store d (a&b) into memory
        jsr  MM_ToggleSerialClock    ; call subroutine
        cli                 ; enable interrupts (clear)
        ldx  L00C1
        inx                 ; increment index (x=x+1)
        beq  L1608          ; branch if equal (zero)
        stx  L00C1          ; store index into memory
        lda  Z83_PortBdata  ; load a with memory contents
        xora #$60           ; XOR a with value -01100000-
        sta  Z83_PortBdata  ; * toggle tach and fuel monitor
L1608:  ldb  #$ff           ; load b with value -11111111-
        stb  Z83_PortA_copy ; store b into memory
        lda  L00C2          ; load a with memory contents
        bmi  L1609          ; branch if minus(hi bit 1)
        lda  ioPort1data    ; load a with memory contents
        anda #$ef           ; AND a with value -11101111-
        sta  ioPort1data    ; * turn off voltage reg

L1609:  cpx  #$003f         ;  -00000000 00111111-
        blo  L1610          ; branch if lower
        cpx  #Timer_CLTIME_ClosedLoopTimer ;  -00000000 10110000-
        bne  L1611          ; branch if not equal (not zero)
        ldb  #$df           ; load b with value -11011111-
        andb ioPort1data
        stb  ioPort1data    ; * turn off power loss light

L1611:  lda  L00C2          ; load a with memory contents
        cmpa #$03           ; compare a with value -00000011-
        bne  L1612          ; branch if not equal (not zero)
        ldd  sciTransceiveCtrl    ; load d (a&b) with memory contents
        bpl  L1613          ; * branch if hi bit 0, which means the receive buffer is empty
        ; Serial Port Receive buffer has a value, which is now in b
        cmpb #$f0           ;  -11110000-
        beq  L1614          ; branch if equal (ignore $f0 input)

L1613:  lda  BitFlags81     ; load a with memory contents
        bmi  L1614          ; branch if b81_OffIdle is set
        ldx  Scratch_bb
        ldb  $00,x          ; load b with memory at index + value
        stb  sciTxData      ; store b into memory
        cmpb #$55           ;  -01010101-
        bne  L1615          ; branch if not equal (not zero)
        ora  #b81_OffIdle   ; set bit -10000000-
        ldb  Z83_PortCdata  ; load b with memory contents
        andb #$9c           ;  -10011100-
        stb  Scratch_b8     ; store b into memory
        sta  BitFlags81     ; store a into memory
        rts                 ; return from subroutine

L1615:  inx                 ; increment index (x=x+1)
        stx  Scratch_bb     ; store index into memory
L1614:  rts                 ; return from subroutine

L1612:  ldb  L004F          ; load b with memory contents
        andb #$04           ;  -00000100-
        bne  L1610          ; branch if not equal (not zero)
        ldb  BitFlags81     ; load b with memory contents
        bpl  L1610          ; branch if b81_OffIdle is clear
        bita #$3f           ;  -00111111-
        bne  L1616          ; branch if not equal (not zero)
        ldb  Z83_PortAdata  ; load b with memory contents
        orb  #PortA_ASD     ; * ASD relay
        stb  Z83_PortAdata  ; store b into memory
        ldb  #$ff           ; load b with value -11111111-
        stb  Z83_PortADataDirection    ; store b into memory
        ldb  #$08           ; load b with value -00001000-
L1617:  decb
        bne  L1617          ; branch if not equal (not zero)
        stb  Z83_PortADataDirection    ; store b into memory
        ldb  #$08           ; load b with value -00001000-
L1618:  decb
        bne  L1618          ; branch if not equal (not zero)
        ldb  Z83_PortAdata  ; load b with memory contents
        bitb #PortA_ASD     ; * ASD relay
        bne  L1616          ; branch if not equal (not zero)
        lda  L00C9          ; load a with memory contents
        inca                ; increment a
        cmpa #$0c           ; compare a with value -00001100-
        blo  L1619          ; branch if lower
        lda  #$01           ; load a with value -00000001-
L1619:  sta  L00C9          ; store a into memory
        ldb  #$02           ; load b with value -00000010-
        stb  L00C1          ; store b into memory
L1616:  lda  L00C9          ; load a with memory contents
        bne  L1620          ; branch if not equal (not zero)
L1610:  jsr  MoveAISStepper ; call subroutine
        lda  BitFlags81     ; load a with memory contents
        bpl  L1622          ; branch if b81_OffIdle is clear 
        ldb  L00C2          ; load b with memory contents
        bitb #$07           ;  -00000111-
        bne  L1622          ; branch if not equal (not zero)
        lda  Z83_PortCdata  ; load a with memory contents
        anda #$9c           ; AND a with value -10011100-
        cmpa Scratch_b8     ; compare a with memory contents
        beq  L1622          ; branch if equal (zero)
        sta  Scratch_b8     ; store a into memory
        cmpa sciTransceiveCtrl    ; compare a with memory contents
        clra                ; a = 0
        tst  L00C8
        bne  L1623          ; branch if not equal (not zero)
        lda  #$88           ; load a with value -10001000-
L1623:  sta  sciTxData      ; store a into memory
        sta  L00C8          ; store a into memory
        lda  BitFlags81     ; load a with memory contents
        lsla                ; logical shift left a
        bpl  L1622          ; branch if b81_LowTpsUpdated is clear (hi bit 0)
        lda  #$20           ; load a with value -00100000-
        xora ioPort1data
        sta  ioPort1data    ; * toggle power loss light
L1622:  rts                 ; return from subroutine

L1620:  ldb  #$df           ; load b with value -11011111-
        andb ioPort1data
        stb  ioPort1data    ; * turn off power loss light
        ldb  BitFlags81     ; load b with memory contents
        orb  #b81_bit5      ;  -00100000-
        stb  BitFlags81     ; store b into memory
        cpx  #$4b00         ;  -01001011 00000000-
        bhs  L1610          ; branch if greater or equal
        ldb  sciTransceiveCtrl    ; load b with memory contents
        bitb #$c0           ;  -11000000-
        beq  L1624          ; branch if equal (zero)
        ldb  sciRxData      ; load b with memory contents
        cmpb #$f0           ;  -11110000-
        bne  L1624          ; branch if not equal (not zero)
        jmp  L1625

L1624:  tab                 ; b = a
        cmpb #$0a           ;  -00001010-
        blo  L1626          ; branch if lower
        lda  #$10           ; load a with value -00010000-
        cmpb #$0b           ;  -00001011-
        bne  L1626          ; branch if not equal (not zero)
        inca                ; increment a
L1626:  sta  sciTxData      ; store a into memory
        tba                 ; a = b
        ldb  #<~PortA_ASD   ; $fd ; load b with value -11111101-
        cmpa #$06           ; compare a with value -00000110-
        beq  DiagnosticCode05to0a          ; branch if equal (zero)
        cmpa #$02           ; compare a with value -00000010-
        bhi  L1628          ; branch if higher
        andb Z83_PortA_copy
        stb  Z83_PortA_copy ; store b into memory

    ; Diagnostic routine switch:
L1628:  ldb  L00C2          ; load b with memory contents
        cmpa #$03           ; compare a with value -00000011-
        beq  DiagnosticCode03          ; branch if equal (zero)
        cmpa #$04           ; compare a with value -00000100-
        beq  DiagnosticCode04          ; branch if equal (zero)
        ldb  #$ef           ; load b with value -11101111-
        cmpa #$05           ; compare a with value -00000101-
        beq  DiagnosticCode05to0a          ; branch if equal (zero)
        ldb  #$bf           ; load b with value -10111111-
        cmpa #$07           ; compare a with value -00000111-
        beq  DiagnosticCode05to0a          ; branch if equal (zero)
        ldb  #$5f           ; load b with value -01011111-
        cmpa #$08           ; compare a with value -00001000-
        beq  DiagnosticCode05to0a          ; branch if equal (zero)
        ldb  #$fb           ; load b with value -11111011-
        cmpa #$09           ; compare a with value -00001001-
        beq  DiagnosticCode05to0a          ; branch if equal (zero)
        ldb  #$fe           ; load b with value -11111110-
        cmpa #$0a           ; compare a with value -00001010-
        beq  DiagnosticCode05to0a          ; branch if equal (zero)
        cmpa #$0b           ; compare a with value -00001011-
        bne  L1631          ; branch if not equal (not zero)
        lda  L00C2          ; load a with memory contents
        bpl  L1632          ; branch if plus (hi bit 0)
        lda  ioPort1data    ; load a with memory contents
        ora  #$10           ;  -00010000-
        sta  ioPort1data    ; * turn on voltage reg
        jmp  MoveAISStepper

L1631:  ldb  L00C2          ; load b with memory contents
        lslb
        bne  L1632          ; branch if not equal (not zero)
        cmpa #$01           ; compare a with value -00000001-
        beq  L1633          ; branch if equal (zero)
        lda  #$02           ; load a with value -00000010-
        ora  timerControl1  ; OR a with memory contents
        sta  timerControl1  ; store a into memory
        ldd  counterHi      ; load d (a&b) with memory contents
        addd #counterLo     ;  -00000000 00001010-
        std  ocr2Hi         ; store d (a&b) into memory
        lda  ioPort1data    ; load a with memory contents
        ora  #$04           ;  -00000100-
        sta  ioPort1data    ; * turn on injector bank 3-4
        lda  timerStatus    ; load a with memory contents
        ldd  ocr2Hi         ; load d (a&b) with memory contents
        std  ocr2Hi         ; store d (a&b) into memory
        ldd  timerControl1  ; load d (a&b) with memory contents
        anda #$fd           ; AND a with value -11111101-
        orb  #$10           ;  -00010000-
        std  timerControl1  ; store d (a&b) into memory
        ldd  L1634          ;  (data is 07d0)
        addd ocr2Hi         ; d = d + memory contents
        std  ocr2Hi         ; store d (a&b) into memory
        jmp  MoveAISStepper

DiagnosticCode05to0a:
        clra                ; a = 0
        sta  CylinderRetard2  ; store a into memory
        lda  L00C2          ; load a with memory contents
        bmi  L1632          ; branch if minus(hi bit 1)
L1635:  andb Z83_PortA_copy
        stb  Z83_PortA_copy ; store b into memory
        jmp  MoveAISStepper

DiagnosticCode04:
        lslb
        bne  L1632          ; branch if not equal (not zero)
        ldb  #$f7           ; load b with value -11110111-
        bra  L1635          ; branch

DiagnosticCode03:
        lda  #$20           ; load a with value -00100000-
        cmpb #$20           ;  -00100000-
        beq  L1636          ; branch if equal (zero)
        cmpb #$45           ;  -01000101-
        beq  L1637          ; branch if equal (zero)
        jmp  MoveAISStepper

LED16:  addd #ioPort3CSR    ;  -00000000 00001111-
        std  ocrHi          ; store d (a&b) into memory
L1633:  lda  timerCSR       ; load a with memory contents
        xora #$01           ; XOR a with value -00000001-
        sta  timerCSR       ; store a into memory
        lsra
        ldd  counterHi      ; load d (a&b) with memory contents
        bcc  LED16          ; branch if greater or equal
        addd #$1b60         ;  -00011011 01100000-
        std  ocrHi          ; store d (a&b) into memory
        jmp  MoveAISStepper

L1637:  nega                ; negate a (set high bit)
L1636:  adda L0042
        sta  L00E2          ; store a into memory
L1632:  jmp  MoveAISStepper

L1625:  tab                 ; b = a
        cmpa #$09           ; compare a with value -00001001-
        bne  L1638          ; branch if not equal (not zero)
        ldb  #$ff           ; load b with value -11111111-
        lda  Z83_PortCdata  ; load a with memory contents
        anda #$43           ; AND a with value -01000011-
        bita #$40           ;  -01000000-
        beq  L1639          ; branch if equal (zero)
        clrb                ; b = 0
        cmpa #$41           ; compare a with value -01000001-
        beq  L1639          ; branch if equal (zero)
        incb
        cmpa #$43           ; compare a with value -01000011-
        beq  L1639          ; branch if equal (zero)
        cmpa #$40           ; compare a with value -01000000-
        beq  L1632          ; branch if equal (zero)
        ldb  #$10           ; load b with value -00010000-
L1639:  stb  sciTxData      ; store b into memory
        jmp  MoveAISStepper

L1638:  cmpa #$08           ; compare a with value -00001000-
        blo  L1640          ; branch if lower
        ldb  #$ff           ; load b with value -11111111-
        cmpa #$08           ; compare a with value -00001000-
        bhi  L1639          ; branch if higher
        clrb                ; b = 0
L1640:  ldx  #Z85_ADConverter1    ; load index with value
        abx                 ; add b to index
        lda  $00,x          ; load a with memory at index + value
        cmpb #$04           ;  -00000100-
        bne  L1641          ; branch if not equal (not zero)
        clr  CoolantTemp    ; zero byte at memory location
        jsr  GetCoolantTempQuick    ; call subroutine
        ldb  #$2e           ; load b with value -00101110-
        mul                 ; multiply (d=a*b)
        adca #$ec           ;  -11101100-
        bcs  L1642          ; branch if lower
        clra                ; a = 0
        bra  L1642          ; branch

L1641:  cmpb #$07           ;  -00000111-
        bne  L1643          ; branch if not equal (not zero)
        ldb  #$fd           ; load b with value -11111101-
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
        lsra
        lsra
        lsra
        lsra
        adca #$00           ;  -00000000-
        bra  L1642          ; branch

L1643:  cmpb #$06           ;  -00000110-
        bne  L1644          ; branch if not equal (not zero)
        ldb  CylinderRetard2  ; load b with memory contents
        bne  L1645          ; branch if not equal (not zero)
        incb
        stb  CylinderRetard2          ; store b into memory
        tba                 ; a = b
L1645:  ldb  #$12           ; load b with value -00010010-
L1646:  decb
        bne  L1646          ; branch if not equal (not zero)
        ldb  ioPort1data    ; load b with memory contents
        orb  #$80           ;  -10000000-
        stb  ioPort1data    ; * turn on knock kill
        andb #$7f           ;  -01111111-
        stb  ioPort1data    ; * turn off knock kill
        ldb  CylinderRetard2  ; load b with memory contents
        cba
        bhs  L1647          ; branch if greater or equal
        tba                 ; a = b
L1647:  tab                 ; b = a
        bra  L1648          ; branch

L1644:  clrb                ; b = 0
L1648:  stb  CylinderRetard2          ; store b into memory
        ldb  #$32           ; load b with value -00110010-
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
L1642:  sta  Scratch_bb     ; store a into memory
        ldd  #$199a         ; load d (a&b) with value -00011001 10011010-
        jsr  mul16bit       ; call subroutine
        ldb  #$06           ; load b with value -00000110-
        mul                 ; multiply (d=a*b)
        addb Scratch_bb     ; b=b+memory contents
        bra  L1639          ; branch

ADLookup:
        ldx  #ADTableOffsets83    ; load index with value
        abx                 ; add b to index
        ldb  $00,x          ; load b with memory at index + value
        ldx  #Z85_ADConverter1    ; load index with value
        abx                 ; add b to index
        tab                 ; b = a
        lda  $00,x          ; load a with memory at index + value
        rts                 ; return from subroutine

MM_CriticalTiming_ADScan subroutine
    ; Control output pulse to the Navigator:
        lda  Timer_NavigatorFuelPulsewidth    ; load a with memory contents
        adda #$08           ;  -00001000-
        cmpa counterHi      ; compare a with memory contents
        bpl  .2             ; branch if plus (hi bit 0)
        sei                 ; disable interrupts (set)
        ldd  FuelMonitorOutputTimer_W ; load d (a&b) with memory contents
        subd FuelMonitorScaling       ;  (data is 028e)
        bcs  .1             ; branch if lower
        std  FuelMonitorOutputTimer_W ; store d (a&b) into memory
        lda  Z83_PortBdata  ; load a with memory contents
        xora #$40           ; XOR a with value -01000000-
        sta  Z83_PortBdata  ; * Toggle fuel monitor output
.1      cli                 ; enable interrupts (clear)
        lda  counterHi      ; load a with memory contents
        sta  Timer_NavigatorFuelPulsewidth    ; store a into memory

.2      lda  #$0f           ; load a with value -00001111-
        ldb  L0060          ; load b with memory contents
        bitb #$40           ;  -01000000-
        bne  L1651          ; branch if not equal (not zero)
        lda  #$1e           ; load a with value -00011110-
        bitb #$20           ;  -00100000-
        beq  L1651          ; branch if equal (zero)
        lda  #$1e           ; load a with value -00011110-
L1651:  adda L00E3
        cmpa counterHi      ; compare a with memory contents
        bpl  L1652          ; branch if plus (hi bit 0)
        jsr  LE6E8          ; call subroutine
L1652:  lda  Z83_PortBdata  ; load a with memory contents
        bpl  L1653          ; * branch if a/d conversion is done?
        sei                 ; disable interrupts (set)
        ldb  ADScanIndex    ; load b with memory contents
        tba                 ; a = b
        incb
        cmpb #$0b           ; * Number of entries in AD table (11)
        bls  L1654          ; branch if lower or same
        clrb                ; b = 0
L1654:  stb  ADScanIndex    ; store b into memory
        jsr  ADLookup       ; call subroutine
        cli                 ; enable interrupts (clear)
        ldx  #ADTableOffsets83    ; load index with value
        abx                 ; add b to index
        ldb  $00,x          ; load b with memory at index + value
        ldx  #CoolantTemp   ; load index with value
        abx                 ; add b to index
        cmpb #$07           ;  -00000111-
        bne  L1655          ; branch if not equal (not zero)
        jmp  DetermineVoltage

L1655:  sta  $00,x
L1653:  rts                 ; return from subroutine

HandlePartThrottleEnrichment_MM subroutine
        lda  BitFlags82     ; load a with memory contents
        bmi  L1656          ; branch if b82_EngineNotRunning set

        ldd  RPM_W          ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        ldx  #RPMMAP_FuelPartThrottleEnrichment    ; load index with value
        jsr  LookupTable    ; call subroutine
        lda  L00CA          ; load a with memory contents
        cmpa L1254          ; compare a with memory contents (data is 2b)
        blo  L1657          ; branch if lower
        subb L1658          ; subtract memory contents from b (data is 03)
        bcc  L1657          ; branch if greater or equal
        clrb                ; b = 0
L1657:  lda  Counter_EPPs    ; load a with memory contents
        cmpb MapValue        
        blo  L1659          ; branch if lower (not in boost?)
        cmpa L1660          ; compare a with memory contents (data is 64)
        blo  L1656          ; branch if lower
        ldb  L1660          ; load b with memory contents (data is 64)
        lda  L00CA          ; load a with memory contents
        beq  L1661          ; branch if equal (zero)
        deca                ; decrement a
        bra  L1662          ; branch

L1659:  cmpa L1663          ; compare a with memory contents (data is 3c)
        blo  L1656          ; branch if lower
        ldb  L1663          ; load b with memory contents (data is 3c)
        lda  L00CA          ; load a with memory contents
        cmpa L1254          ; compare a with memory contents (data is 2b)
        bhs  L1661          ; branch if greater or equal
        inca                ; increment a
L1662:  sta  L00CA          ; store a into memory
L1661:  sei                 ; disable interrupts (set)
        lda  Counter_EPPs    ; load a with memory contents
        sba                 ; subtract (a=a-b)
        sta  Counter_EPPs    ; store a into memory
        cli                 ; enable interrupts (clear)
L1656:  rts                 ; return from subroutine

MM_CheckForFuelShutoffConditions subroutine
        lda  BitFlags82     ; load a with memory contents
        bmi  L1664          ; branch if b82_EngineNotRunning set
        lda  BarometricPressure       ; load a with memory contents
        ldb  BitFlags80     ; load b with memory contents
        bitb #b80_WastegateLimpinMode ;  -00010000-
        beq  L1665          ; branch if not set
        adda LMPSHF_OverboostShutoffDuringLimpinMode          ;  (data is 73)
        cmpa MaximumAllowableMAP    ; compare a with memory contents (data is fc)
        blo  L1667          ; branch if lower
        lda  MaximumAllowableMAP    ; load a with memory contents (data is fc)
L1667:  cmpa MapValue            ; compare a with memory contents
        bls  L1668          ; branch if lower or same
        bra  L1664          ; branch

;      a is preloaded with baro. Add shutoff to baro,
;      if it's more than ff cap it at ff. Then compare
;      a with MapValue, and if it is above, no overboost
L1665:  adda MPSHOF_OverboostShutoff    ;  (data is ff)
        bcc  L1669          ; branch if not above ff
        lda  #$ff           ; otherwise, cap at ff
L1669:  cmpa MapValue            ; compare a with memory contents
OverboostShutoffControl_OBCTL:  
    chem 6 n * X 0 256 in Y 0 256 out OBCTL Overboost_shutdown_feature Enabled 22 Disabled 20 Unused 00 Unused 00 Unused 00 Unused 00 Unused 00 Unused 00
    chem2 byte OBCTL choice Enabled 22 choice Disabled 20 "Overboost shutdown feature. Enabled 22 Disabled 20."
        bhi  L1670          ; branch if higher (no overboost)
	; otherwise, POSSIBLY overboosting:
        adda SHFDLT_OverboostShutoffDelayTime          ;  (data is 1a)
        bcc  L1672          ; branch if greater or equal
        lda  #$ff           ; load a with value -11111111-
L1672:  cmpa MapValue            ; compare a with memory contents
        blo  L1668          ; branch if lower
        ldb  Timer_FuelShutoff ; load b with memory contents
        beq  L1668          ; branch if equal (zero)
        decb
        stb  Timer_FuelShutoff ; store b into memory
        bra  L1664          ; branch

L1668:  ldb  MapValue            ; load b with memory contents
        cmpb MaximumAllowableMAP    ; compare b with memory contents (data is fc)
        bhs  L1673          ; branch if greater or equal
        lda  #$07           ; load a with value -00000111-
        jsr  L1042          ; call subroutine
        bra  L1673          ; branch

L1670:  lda  TRNTM_OverboostShutoffTime ; load a with memory contents (data is ff)
        sta  Timer_FuelShutoff ; store a into memory
L1664:  lda  InverseRPM_B          ; load a with memory contents
        bne  L1675          ; branch if not equal (not zero)
        ldx  #REVLMT_OverRevFuelShutoff     ; load index with value
        lda  BitFlags80     ; load a with memory contents
        bpl  L1676          ; branch if b80_BadMAP is clear
        ldb  THRTHR_25pctValueForThrForOverRevShutoff ; load b with memory contents (data is 21)
        addb LowestSessionTPS    ; b=b+memory contents
        cmpb TpsVolts
        blo  L1678          ; branch if lower
L1676:  ldx  #REVLMT3_OverRevFuelShutoff     ; load index with value
        bita #b80_BadTPS      ;  -01000000-
        beq  L1679          ; branch if equal (zero)
        ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        cmpa RVLSPD_VehicleSpeedForRevLimit ; compare a with memory contents (data is 14)
        bhs  L1681          ; branch if greater or equal
        bra  L1682          ; branch

L1679:  ldb  BitFlags81     ; load b with memory contents
        bmi  L1681          ; branch if b81_OffIdle is set(hi bit 1)
L1682:  ldb  BarometricPressure    ; load b with memory contents
        lsrb                ; logical shift right b
        tba                 ; a = b
        lsrb                ; logical shift right b
        aba                 ; a=a+b
        cmpa MapValue            ; compare a with memory contents
        blo  L1678          ; branch if lower

L1681:  ldx  #REVLMT5_OverRevFuelShutoff     ; load index with value
L1678:  ldd  InverseRPM_W          ; load d (a&b) with memory contents
        subd $00,x
        bcc  L1675          ; branch if greater or equal
        ldd  InverseRPM_W          ; load d (a&b) with memory contents
        subd $02,x
        bcs  L1673          ; branch if lower
        ldb  BitFlags84     ; load b with memory contents
        bitb #$08           ;  -00001000-
        beq  L1683          ; branch if equal (zero)
L1673:  ldb  BitFlags84     ; load b with memory contents
;       throw an overboost code
        orb  #$08           ;  -00001000-
        bra  L1684          ; branch

L1675:  ldb  BitFlags84     ; load b with memory contents
;       unthrow the overboost code
        andb #$f7           ;  -11110111-
L1684:  stb  BitFlags84     ; store b into memory
L1683:

    .if UseStagingMod > 0
StagingMod:
        ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        cmpa RVLSPD_Staging ; compare a with memory contents
        bhs  .isok          ; 03/10/07 - always disable above speed

        ldx  #REVLMTX_StagingModFuelShutoff
        ldd  InverseRPM_W   ; load d (a&b) with memory contents
        subd $00,x
        bcc  .isok          ; branch if greater or equal
        ldd  InverseRPM_W   ; load d (a&b) with memory contents
        subd $02,x
        bcs  .cutit         ; branch if lower
        ldb  BitFlags84     ; load b with memory contents
        bitb #$08           ;  -00001000-
        beq  .donehere      ; branch if equal (zero)
.cutit: ldb  BitFlags84     ; load b with memory contents
;       turn off fuel
        orb  #$08           ;  -00001000-
        bra  .setit         ; branch

.isok:  ldb  BitFlags84     ; load b with memory contents
;       turn on fuel
        andb #$f7           ;  -11110111-
.setit: stb  BitFlags84     ; store b into memory

    .endif
    
.donehere
        rts    

MM_L1603 subroutine
        lda  Counter_L00B3  ; load a with memory contents
        inca                ; increment a
        beq  L1685          ; branch if equal (zero)
        sta  Counter_L00B3  ; store a into memory
L1685:  lda  InverseRPM_B   ; load a with memory contents
        cmpa #$04           ; compare a with value -00000100-
        blo  L1686          ; branch if lower
        clrb                ; b = 0
        lda  #b82_EngineNotRunning | b82_DecayingFuelStartToRun | b82_SparkStartMode ; -10001010-
        sei                 ; disable interrupts (set)
        ora  BitFlags82     ; OR a with memory contents
        bita #b82_FuelRunMode      ;  -00100000-
        beq  L1687          ; branch if equal (zero) Fuel Starting Mode:
        anda #<~b82_FuelRunMode    ; $df clear bit -11011111-
        stb  Counter_EPPs   ; store b into memory
L1687:  sta  BitFlags82     ; store a into memory
        cli                 ; enable interrupts (clear)
        lda  BitFlags81     ; load a with memory contents
        anda #<~b81_bit4    ; clear bit -11101111-
        sta  BitFlags81     ; store a into memory
        stb  WastegateControlDutyCycle  ; store b into memory
        stb  Timer_CLTIME_ClosedLoopTimer ; store b into memory
L1686:  inc  Counter_MainLoop1
        bne  L1688          ; branch if not equal (not zero)
        ldb  L004F          ; load b with memory contents
        andb #$fc           ;  -11111100-
        stb  L004F          ; store b into memory
        sei                 ; disable interrupts (set)
        lda  Counter_EngineRunningTime          ; load a with memory contents
        inca                ; increment a
        beq  .over1         ; branch if equal (zero)
        sta  Counter_EngineRunningTime          ; store a into memory
.over1
        cli                 ; enable interrupts (clear)
        lda  BitFlags82     ; load a with memory contents
        bmi  L1688          ; branch if b82_EngineNotRunning clear
    ; This counter is supposed to be incremented every 2.6 seconds when engine is running
        lda  VerySlowCounter2point6seconds  ; load a with memory contents
        inca                ; increment a
        beq  L1688          ; branch if equal (zero)
        sta  VerySlowCounter2point6seconds  ; store a into memory
L1688:  rts                 ; return from subroutine

MM_L1597 subroutine
        ldb  L004F          ; load b with memory contents
        andb #$08           ;  -00001000-
        stb  Scratch_b6     ; store b into memory
        ldb  Z83_PortCdata  ; load b with memory contents
        andb #$08           ; * check park/neutral
        xorb Scratch_b6
        beq  L1690          ; branch if equal (zero)
        xorb L004F
        stb  L004F          ; store b into memory
        lda  BitFlags84     ; load a with memory contents
        lsla                ; logical shift left a
        ora  BitFlags80     ; OR a with memory contents
        bmi  L1690          ; branch if minus(hi bit 1)
        orb  #$10           ;  -00010000-
        stb  L004F          ; store b into memory
L1690:  rts                 ; return from subroutine

;  ______________________
; |                      \
; |   Calculate Spark    |
; \______________________|

MM_CalcAdvance_v subroutine
        clrb                ; b = 0
        clra                ; a = 0
        std  Scratch_b6     ; store d (a&b) into memory
        lda  MapValue            ; load a with memory contents
        ldb  CoolantTemp    ; load b with memory contents
        cmpb TMPSWP_AdvanceEngineTempSwPoint    ; compare b with memory contents (data is a6)
        bhs  .WarmEngine    ; branch if greater or equal

.ColdEngine
        ldx  #CLDMAP_AdvanceFromMapCold    ; load index with value
        jsr  LookupTable    ; call subroutine

        ldb  BitFlags81     ; load b with memory contents
        bmi  .1             ; branch if b81_OffIdle is set 

        ; if returned value is less than CLDLMT, use CLDLMT
        cmpa CLDLMT_LimitForColdMAPAtClosedThrottle    ; compare a with memory contents (data is 14)
        blo  .1             ; branch if lower
        lda  CLDLMT_LimitForColdMAPAtClosedThrottle    ; load a with memory contents (data is 14)
.1      sta  Scratch_b6     ; store advance value in scratch location

        ; **********
    .if AutomaticTransmission > 0

        ; check if off idle:
        ldb  BitFlags81     ; load b with memory contents
        bmi  .2             ; branch if b81_OffIdle is set(hi bit 1)

; mod, GLA, remove stuff for auto trans
L1695:
        lda  RPM_W          ; load a with memory contents
        cmpa PNRPM_AdvanceParkNeutralIdleRPMSwitch    ; compare a with memory contents (data is 01)
        bhi  .2             ; branch if higher
        lda  Z83_PortCdata  ; load a with memory contents
        bita #$08           ; * check park/neutral
        bne  .2             ; branch if not equal (not zero)
        lda  CoolantTemp    ; load a with memory contents
        ldx  #PNIDLE_AdvanceParkNeutralIdle    ; load index with value
        jsr  LookupTable    ; call subroutine
        adda Scratch_b6
        sta  Scratch_b6     ; store a into memory
.2      jmp  L1694

;       we get here if the engine is warm (above TMPSWP)
.WarmEngine
        ldb  BitFlags81     ; load b with memory contents
        bpl  L1695          ; branch if b81_OffIdle is clear

        ; **********
    .else
        ; **********
        ; manual transmission

.2      jmp  L1694

;       we get here if the engine is warm (above TMPSWP)
.WarmEngine
        ldb  BitFlags81     ; load b with memory contents
        bpl  .2             ; branch if b81_OffIdle is clear 

    .endif
        ; **********

; continuation here: transmission specific stuff is done

        ; we get here at part OR full throttle, but not at idle:
        ldx  #WOTMAP_AdvanceFromMapWarmFull    ; load index with value
        jsr  LookupTable    ; call subroutine
        sta  Scratch_b6     ; store a into memory

        ldb  BitFlags81     ; load b with memory contents
        lsrb                ; logical shift right b
        bcs  L1696          ; branch if b81_FullThrottle is set

        ; we get here at part throttle
        lda  L00CA          ; load a with memory contents
        cmpa L1254          ; compare a with memory contents (data is 2b)
        blo  L1697          ; branch if lower
        ldb  RPM_W          ; load b with memory contents
        cmpb L1393          ; compare b with memory contents (data is 3a)
        bhs  L1696          ; branch if greater or equal

L1697:  ldb  MapValue            ; load b with memory contents
        cmpb OverboostConstant1    ; compare b with memory contents (data is aa)
        bhs  L1698          ; branch if greater or equal
        aba                 ; a=a+b
        bcc  L1699          ; branch if greater or equal
        lda  #$ff           ; load a with value -11111111-
L1699:  suba OverboostConstant1    ;  (data is aa)
        bls  L1700          ; branch if lower or same
L1698:  ldb  aMaxMap1       ; load b with memory contents (data is 1f)
        mul                 ; multiply (d=a*b)
        adca #$00           ;  -00000000-
        sta  Scratch_b7     ; store a into memory

L1700:  lda  MapValue            ; load a with memory contents
        ldx  #HOTMAP_AdvanceFromMapWarmPart    ; load index with value
        jsr  LookupTable    ; call subroutine
        ldb  aFurtherAdvanceCalcIfLT80    ; load b with memory contents (data is ff)
        bmi  L1701          ; branch if minus(hi bit 1)
        adda Scratch_b7
        cmpa Scratch_b6     ; compare a with memory contents
        bge  L1696          ; branch if greater or equal
        bra  L1702          ; branch

L1701:  suba Scratch_b7
        cmpa Scratch_b6     ; compare a with memory contents
        ble  L1696          ; branch if less or equal
L1702:  sta  Scratch_b6     ; store a into memory

L1696:  ldx  #zAdvanceFromRpmsWarmAlways    ; load index with value
        ldd  RPM_W          ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        jsr  LookupTable    ; call subroutine
        sta  Scratch_b7     ; store a into memory

        ldx  #RetardFactorFromBarometricPressure    ; load index with value
        lda  BarometricPressure    ; load a with memory contents
        jsr  LookupTable    ; call subroutine
        sta  Scratch_ba     ; store a into memory

        ldx  #zRetardFromRpmsForAltitude    ; load index with value
        ldd  RPM_W          ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        jsr  LookupTable    ; call subroutine
        ldb  Scratch_ba     ; load b with memory contents

        mul                 ; multiply (d=a*b)

        tab                 ; b = a
        lda  Scratch_b7     ; load a with memory contents
        sba                 ; subtract (a=a-b)
        cmpa MinimumAdvance ; compare a with memory contents (data is 06)
        bpl  L1703          ; branch if plus (hi bit 0)
        lda  MinimumAdvance    ; load a with memory contents (data is 06)

L1703:  cmpa Scratch_b6     ; compare a with memory contents
        bgt  L1694          ; branch if greater
        sta  Scratch_b6     ; store a into memory

L1694:  ldx  #GOVNER_AdvanceFromRpms    ; load index with value
        ldd  RPM_W          ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        jsr  LookupTable    ; call subroutine
        addb Scratch_b6     ; b=b+memory contents
        bpl  L1704          ; branch if plus (hi bit 0)
        clrb                ; b = 0
L1704:  stb  Scratch_b6     ; store b into memory
        lda  MapValue            ; load a with memory contents
        cmpa RetardFromTPSAboveThisMAP    ; compare a with memory contents (data is 41 (62))
        bls  L1705          ; branch if lower or same
        lda  RPM_W          ; load a with memory contents
        cmpa RetardFromTPSAboveThisRPM    ; compare a with memory contents (data is 27)
        bls  L1705          ; branch if lower or same
        ldb  TpsAverageValue  ; load b with memory contents
        cmpb RetardFromTPSAboveThisTPS    ; compare b with memory contents (data is 50)
        bls  L1705          ; branch if lower or same
        lda  TpsVolts            ; load a with memory contents
        sba                 ; subtract (a=a-b)
        ; a now is the Delta-TPS
        bcs  L1705          ; branch if lower (accelerating)
        ldx  #RetardFromTPS ; load index with value
        jsr  LookupTable    ; call subroutine
        lda  Scratch_b6     ; load a with memory contents
        sba                 ; subtract (a=a-b)
        bpl  L1706          ; branch if plus (hi bit 0)
        clra                ; a = 0
L1706:  sta  Scratch_b6     ; store a into memory

L1705:  ldx  #SPKLMT_AdvanceAllowedFromRpms    ; load index with value
        ldd  RPM_W          ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        jsr  LookupTable    ; call subroutine
        cmpb Scratch_b6
        blo  L1707          ; branch if lower
        ldb  Scratch_b6     ; load b with memory contents

L1707:  negb
        addb #$a6           ;  -10100110-
        lda  #$5b           ; load a with value -01011011-
        mul                 ; multiply (d=a*b)
        asld                ; shift left (d=d*2)
        tstb
        bpl  L1708          ; branch if plus (hi bit 0)
        inca                ; increment a
L1708:  sta  OverallAdvance ; store a into memory
        rts                 ; return from subroutine

MM_DealWithThrottlePositionSensor    subroutine
        ldb  BitFlags80     ; load b with memory contents
        bitb #b80_BadTPS    ;  -01000000-
        bne  .TPSAlreadyMarkedAsBad          ; branch if not equal (not zero)
        ; Get the A/D result for TPS:
        lda  RawTPS         ; load a with memory contents
        cmpa THRLOW_MinValidTPS    ; compare a with memory contents (data is 08)
        blo  .TPSOutOfRange ; branch if lower
        cmpa THRHIG_MaxValidTPS    ; compare a with memory contents (data is f0)
        blo  .TPSValueGood          ; branch if lower

.TPSOutOfRange
        lda  Counter_MainLoop1    ; load a with memory contents
        bita #$06           ;  -00000110-
        ; only set the error if some time has elapsed with an invalid reading
        ; This is to handle bad conversions or a dirty TPS wiper.
        bne  L1711          ; branch if not equal (not zero)
        lda  #$16           ; load a with value -00010110-
        jsr  SetFaultCode   ; call subroutine
        bcc  L1711          ; branch if greater or equal
        ldb  #b80_BadTPS      ; load b with value -01000000-
        sei                 ; disable interrupts (set)
        orb  BitFlags80
        stb  BitFlags80     ; store b into memory
        cli                 ; enable interrupts (clear)
.TPSAlreadyMarkedAsBad
        lda  #$40           ; load a with value -01000000-
        sta  LowestSessionTPS    ; store a into memory
        ldb  LowestHistoricalTPS ; load b with memory contents
        beq  L1712          ; branch if equal (zero)
        sta  LowestHistoricalTPS ; store a into memory
L1712:  ldb  BitFlags82     ; load b with memory contents
        bmi  .TPSValueGood  ; branch if b82_EngineNotRunning set
        
        ldb  MapValue            ; load b with memory contents
        cmpb MPTHR_MapSwitchPointForWOTorIdle  ; compare b with memory contents (data is 68)
        blo  .TPSValueGood  ; branch if lower
        adda L1246          ;  (data is 85)
        
    ; We get here with either the legit or a simulated TPS value
.TPSValueGood
        sta  TpsVolts            ; store a into memory
L1711:  ldb  BitFlags81     ; load b with memory contents
        bmi  L1714          ; branch if b81_OffIdle is set 
        ldb  Timer_AISDelay          ; load b with memory contents
        bne  L1714          ; branch if not equal (not zero)
        ldb  BitFlags82     ; load b with memory contents
        bpl  L1715          ; branch if b82_EngineNotRunning clear (Engine Running)
        ; we get here if Engine Not Running:
        ldb  RawBattVoltage ; load b with memory contents
        cmpb #$b2           ;  -10110010-
        blo  L1714          ; branch if lower
        bra  L1716          ; branch

L1715:  ldb  Counter_EngineRunningTime
        beq  L1714          ; branch if zero (engine just started or not running)
L1716:  cmpa LowestSessionTPS  ; compare a with memory contents
        bhs  L1714          ; branch if greater or equal
        lda  L00CD          ; load a with memory contents
        deca                ; decrement a
        bne  L1717          ; branch if not equal (not zero)
        ldb  LowestSessionTPS   ; load b with memory contents
        cmpb #$10           ;  -00010000-
        bhs  L1718          ; branch if greater or equal
        ldb  L1575          ; load b with memory contents (data is 5c)
L1718:  decb
        stb  LowestSessionTPS       ; store b into memory
        lda  LowestHistoricalTPS    ; load a with memory contents
        beq  L1719          ; branch if equal (zero)
        stb  LowestHistoricalTPS    ; store b into memory
L1719:  lda  BitFlags81     ; load a with memory contents
        anda #<~b81_LowTpsUpdated    ; AND a with value -10111111-
        sta  BitFlags81     ; store a into memory
L1714:  lda  L1720          ; load a with memory contents (data is 10)
L1717:  sta  L00CD          ; store a into memory
        lda  L1721          ; load a with memory contents (data is 03)
        ldb  BitFlags81     ; load b with memory contents
        bpl  L1722          ; branch if b81_OffIdle is clear
        lda  ValueAboveLowestTPSToConsiderOffIdle ; load a with memory contents (data is 01)
L1722:  adda LowestSessionTPS
        cmpa TpsVolts            ; compare a with memory contents
        bhs  L1724          ; branch if greater or equal
        tstb
        bmi  L1725          ; branch if throttle marked as off idle(hi bit 1)
        bitb #$40           ;  -01000000-
        bne  L1726          ; branch if LowestHistoricalTPS was just updated (not zero)
        lda  LowestSessionTPS    ; load a with memory contents
        inca                ; increment a
        cmpa TpsVolts            ; compare a with memory contents
        bhs  L1726          ; branch if greater or equal
        sta  LowestSessionTPS    ; store a into memory
        tst  LowestHistoricalTPS
        beq  L1727          ; branch if equal (zero)
        sta  LowestHistoricalTPS ; store a into memory
L1727:  orb  #$40           ;  -01000000-
L1726:  clra                ; a = 0
        sta  Timer_Leanout  ; store a into memory
        lda  MapValue            ; load a with memory contents
        suba TestedAgainstMAPDuringTPSProcessing    ;  (data is 07)
        bcc  L1728          ; branch if greater or equal
        clra                ; a = 0
L1728:  sta  MapAverageValue          ; store a into memory
L1725:  orb  #$80           ;  -10000000-
        bra  L1729          ; branch

L1724:  tstb
        bpl  L1730          ; branch if plus (hi bit 0)
        lda  BitFlags85     ; load a with memory contents
        bita #$04           ;  -00000100-
        bne  L1731          ; branch if not equal (not zero)
        ora  #$20           ;  -00100000-
        sta  BitFlags85     ; store a into memory
L1731:  ldx  #DLYTIM_AisDelayAfterThrottleCloses_moving_stopped  ; load index with value
        stb  Scratch_b6     ; store b into memory
        ldd  VehicleSpeed_W ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        ldb  Scratch_b6     ; load b with memory contents
        cmpa MPHREF_SwitchPointBetweenDecelIdle          ; compare a with memory contents (data is 06)
        bhs  L1732          ; Use MOVING value
        lda  Z83_PortCdata  ; load a with memory contents
        bita #$08           ; * check park/neutral
        bne  L1732          ; Use MOVING value
        inx                 ; Use STOPPED value
L1732:  lda  $00,x          ; load a with memory at index + value
        sta  Timer_AISDelay          ; store a into memory
L1730:  andb #$7f           ;  -01111111-
L1729:  bitb #$08           ;  -00001000-
        bne  L1733          ; branch if not equal (not zero)
        lda  Z83_PortCdata  ; load a with memory contents
        anda #$10           ; * check brake switch
        beq  L1733          ; branch if equal (zero)
        lda  RPM_W          ; load a with memory contents
        cmpa L1734          ; compare a with memory contents (data is 18)
        blo  L1733          ; branch if lower
        lda  MapValue            ; load a with memory contents
        cmpa AHighMAPvalueDuringTPSProcessing          ; compare a with memory contents (data is 5e)
        bhs  L1733          ; branch if greater or equal
        lda  LowestSessionTPS    ; load a with memory contents
        adda L1736          ;  (data is 04)
        cmpa L1575          ; compare a with memory contents (data is 5c)
        blo  L1737          ; branch if lower
        lda  L1575          ; load a with memory contents (data is 5c)
L1737:  sta  LowestSessionTPS    ; store a into memory
        tst  LowestHistoricalTPS
        beq  L1738          ; branch if equal (zero)
        sta  LowestHistoricalTPS          ; store a into memory
L1738:  orb  #$08           ;  -00001000-
L1733:  lda  LowestSessionTPS    ; load a with memory contents
        adda #$30           ;  -00110000-
        cmpa TpsVolts            ; compare a with memory contents
        bhs  L1739          ; branch if greater or equal
        andb #<~b81_bit3    ; clear bit -11110111-
L1739:  stb  BitFlags81     ; store b into memory
        rts                 ; return from subroutine

MM_EngineFanRoutine    subroutine
        ldd  VehicleSpeed_W    ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        sta  Scratch_b6     ; store a into memory
        ldb  Z83_PortA_copy ; load b with memory contents
        lda  BarometricPressure    ; load a with memory contents
        adda FANMAP_FanMap  ;  (data is 11)
        bcs  L1740          ; branch if lower
        bitb #PortA_FAN     ;  -00001000-
        beq  L1741          ; branch if equal (zero)
        adda #$04           ;  -00000100-
        bcs  L1740          ; branch if lower
L1741:  cmpa MapValue       ; compare a with memory contents
        bhs  L1740          ; branch if greater or equal
        lda  RawBattTemp    ; load a with memory contents
        bitb #PortA_FAN     ;  -00001000-
        bne  L1742          ; branch if not equal (not zero)
        deca                ; decrement a
        deca                ; decrement a
L1742:  cmpa FANBAT_FanBat  ; compare a with memory contents (data is 9a)
        blo  L1743          ; branch if lower
L1740:  lda  CoolantTemp    ; load a with memory contents
        cmpa #$d2           ; compare a with value -11010010-
        blo  L1744          ; branch if lower
        lda  L00D6          ; load a with memory contents
        beq  L1744          ; branch if equal (zero)
        tst  Counter_MainLoop1
        bne  L1745          ; branch if not equal (not zero)
        deca                ; decrement a
        sta  L00D6          ; store a into memory
L1745:  clra                ; a = 0
        bitb #$08           ;  -00001000-
        bne  L1746          ; branch if not equal (not zero)
        lda  #$08           ; load a with value -00001000-
L1746:  cmpa Scratch_b6     ; compare a with memory contents
        bhs  L1743          ; branch if greater or equal
L1744:  lda  BitFlags82     ; load a with memory contents
        bita #b82_EngineNotRunning      ;  -10000000-
        bne  L1747          ; branch if Engine Not Running (not zero)

        lda  BitFlags85     ; load a with memory contents
        bita #$08           ;  -00001000-
        beq  L1743          ; branch if equal (zero)
L1747:  lda  BitFlags80     ; load a with memory contents
        bita #b80_BadCoolantTemp      ;  -00000010-
        bne  L1743          ; branch if not equal (not zero)
        ldx  #FANTMP_FanOnOver45MPH    ; load index with value
        lda  Scratch_b6     ; load a with memory contents
        cmpa #$5a           ; compare a with value -01011010-
        bhs  L1748          ; branch if greater or equal
        cmpa #$46           ; compare a with value -01000110-
        lda  BitFlags84     ; load a with memory contents
        blo  L1749          ; branch if lower
        bita #$01           ;  -00000001-
        bne  L1748          ; branch if not equal (not zero)
L1749:  inx                 ; increment index (x=x+1)
        inx                 ; increment index (x=x+1)
        anda #$fe           ; AND a with value -11111110-
        bra  L1750          ; branch

L1748:  lda  BitFlags84     ; load a with memory contents
        ora  #$01           ;  -00000001-
L1750:  sta  BitFlags84     ; store a into memory
        bitb #PortA_FAN     ; test bit -00001000-
        bne  L1751          ; branch if not equal (not zero) (Fan Off)
        inx                 ; increment index (x=x+1)
L1751:  lda  $00,x          ; load a with memory at index + value
        cmpa CoolantTemp    ; compare a with memory contents
        bhs  L1752          ; branch if greater or equal
L1743:  lda  #$17           ; load a with value -00010111-
        sta  L00E7          ; store a into memory
        andb #<~PortA_FAN   ; clear bit -11110111- Turn Fan ON
        bra  L1753          ; branch

L1752:  lda  L00E7          ; load a with memory contents
        beq  L1754          ; branch if equal (zero)
        deca                ; decrement a
        sta  L00E7          ; store a into memory
        rts                 ; return from subroutine

L1754:  orb  #PortA_FAN     ; set bit -00001000-  Turn Fan OFF
L1753:  stb  Z83_PortA_copy ; store b into memory
        rts                 ; return from subroutine

;  ______________________
; |                      \
; |   Boost Control      |
; \______________________|

MM_CalcBoostGoal   subroutine
        ; MOD - gla
        ; Simplify Boost Goal calculation

        ldx  #BoostAllowedFromRPM_WSGRPM    ; load index with value
        ldd  RPM_W          ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        jsr  LookupTable    ; call subroutine
        sta  Scratch_ba     ; store a into memory

        ldx  #WSGCLT_BoostAllowedFromCoolantTemp    ; load index with value
        lda  CoolantTemp    ; load a with memory contents
        jsr  LookupAndStoreLowestInBA    ; call subroutine

        ldb  Z83_PortCdata  ; load b with memory contents
        andb #$08           ; * check park/neutral
        beq  .NotAutomatic  ; branch if equal (zero)
        ldx  #BoostAllowedFromSpeed_MXSBST    ; load index with value
        ldd  VehicleSpeed_W    ; load d (a&b) with memory contents
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        asld                ; shift left (d=d*2)
        jsr  LookupAndStoreLowestInBA    ; call subroutine
.NotAutomatic

        lda  BitFlags81     ; load a with memory contents
        lsra                ; load carry flag with "full throttle" bit
        bcs  .CalcDone      ; keep this boost goal if full throttle
        lda  BoostAllowedMaxPartThrottle    ; load a with memory contents (data is 45)
        jsr  StoreLowestInBA    ; call subroutine
.CalcDone
        lda  Scratch_ba     ; load a with memory contents
        adda BarometricPressure
        bcc  .StoreValue    ; branch if greater or equal
        lda  #$ff           ; load a with value -11111111-
.StoreValue
        sta  BoostGoal      ; store a into memory
        rts                 ; return from subroutine

LookupAndStoreLowestInBA    subroutine
        jsr  LookupTable    ; call subroutine
StoreLowestInBA:
        cmpa Scratch_ba     ; compare a with memory contents
        bhs  .Return        ; branch if greater or equal
        sta  Scratch_ba     ; store a into memory
.Return rts                 ; return from subroutine

; OLD boost goal calculation
;        ldx  #BoostAllowedFromRPM_WSGRPM    ; load index with value
;        ldd  RPM_W          ; load d (a&b) with memory contents
;        asld                ; shift left (d=d*2)
;        jsr  LookupTable    ; call subroutine
;        sta  Scratch_ba     ; store a into memory
;        ldx  #BoostAllowedFromTime_WSGTIM    ; load index with value
;        lda  BoostTimer     ; load a with memory contents
;        jsr  LookupAndStoreLowestInBA    ; call subroutine
;        ldx  #WSGCLT_BoostAllowedFromCoolantTemp    ; load index with value
;        lda  CoolantTemp    ; load a with memory contents
;        jsr  LookupAndStoreLowestInBA    ; call subroutine
;        ldx  #BoostAllowedFromChargeTemp_WSGCHG    ; load index with value
;        lda  ChargeTempVolts    ; load a with memory contents
;        jsr  LookupAndStoreLowestInBA    ; call subroutine
;        lda  BitFlags81     ; load a with memory contents
;        lsra
;        bcs  L1755          ; branch if lower
;        lda  BoostAllowedMaxPartThrottle    ; load a with memory contents (data is 45)
;        jsr  StoreLowestInBA    ; call subroutine
;L1755:  ldb  Z83_PortCdata    ; load b with memory contents
;        andb #$08           ; * check park/neutral
;        beq  L1756          ; branch if equal (zero)
;        ldx  #BoostAllowedFromSpeed_MXSBST    ; load index with value
;        ldd  VehicleSpeed_W    ; load d (a&b) with memory contents
;        asld                ; shift left (d=d*2)
;        asld                ; shift left (d=d*2)
;        asld                ; shift left (d=d*2)
;        jsr  LookupAndStoreLowestInBA    ; call subroutine
;L1756:  ldx  #BoostTimeSinceLastKnock_WSGKNK    ; load index with value
;        lda  Timer_Over5PSIBoost    ; load a with memory contents
;        jsr  LookupTable    ; call subroutine
;        lda  Scratch_ba     ; load a with memory contents
;        sba                 ; subtract (a=a-b)
;        bcc  KeepThisBoostGoal          ; branch if greater or equal
;        clra                ; a = 0

; *****************************************

MM_ControlBoost subroutine
        ldb  Counter_MainLoop1    ; load b with memory contents
        incb
        lda  BitFlags82     ; load a with memory contents
        bmi  L1760          ; branch if b82_EngineNotRunning set
        tstb
        bne  L1761          ; branch if not equal (not zero)
        lda  Timer_Over5PSIBoost    ; load a with memory contents
        inca                ; increment a
        beq  L1761          ; branch if equal (zero)
        sta  Timer_Over5PSIBoost    ; store a into memory
L1761:  lda  MapValue            ; load a with memory contents
        suba BarometricPressure
        bcs  L1760          ; branch if lower
        cmpa MAPOFS_BoostMapOffsetForDCWOT    ; compare a with memory contents (data is 17)
        bhs  L1762          ; branch if greater or equal
L1760:  clra                ; a = 0
        sta  WastegateControlDutyCycle  ; store a into memory
        sta  L00DA          ; store a into memory
        sta  L00C5          ; store a into memory
        bitb #$1f           ;  -00011111-
        bne  L1763          ; branch if not equal (not zero)
        lda  BoostTimer     ; load a with memory contents
        beq  L1763          ; branch if equal (zero)
        deca                ; decrement a
        sta  BoostTimer     ; store a into memory
L1763:  rts                 ; return from subroutine

L1762:  cmpa KNOKMP_BoostKNKCNTinc    ; compare a with memory contents (data is 17)
        bhs  L1764          ; branch if greater or equal
        clra                ; a = 0
        sta  L00C5          ; store a into memory
L1764:  lda  L00C5          ; load a with memory contents
        cmpa BoostMaxKNKBeforeReset_KNKMAX    ; compare a with memory contents (data is 10)
        blo  L1765          ; branch if lower
        clra                ; a = 0
        sta  Timer_Over5PSIBoost    ; store a into memory
L1765:  tstb
        bne  L1766          ; branch if not equal (not zero)
        clra                ; a = 0
        sta  L00C5          ; store a into memory
L1766:  bitb #$1f           ;  -00011111-
        bne  L1767          ; branch if not equal (not zero)
        ldb  RPM_W          ; load b with memory contents
        cmpb RPMBST_NoBOOSTIincWhenBelowThis          ; compare b with memory contents (data is 26)
        blo  L1767          ; branch if lower
        lda  BoostTimer     ; load a with memory contents
        inca                ; increment a
        beq  L1767          ; branch if equal (zero)
        sta  BoostTimer     ; store a into memory
L1767:  lda  Delta_MAP      ; load a with memory contents
        sta  Scratch_b7     ; store a into memory
        bpl  L1769          ; branch if plus (hi bit 0)
        nega                ; negate a (set high bit)
L1769:  sta  Scratch_b6     ; store a into memory
        tst  Scratch_b7
        bmi  ControlWastegate    ; branch if minus(hi bit 1)
        lda  RPM_W          ; load a with memory contents
        ldx  #RATLIM_BoostOverboostPrevention    ; load index with value
        jsr  LookupTable    ; call subroutine
        lda  Scratch_b6     ; load a with memory contents
        cba
        blo  ControlWastegate    ; branch if lower
        ldb  PRTMAX_MaxDCForPartThrottleInc ; load b with memory contents (data is c8)
        stb  WastegateControlDutyCycle      ; store b into memory
        rts                 ; return from subroutine

ControlWastegate:
        clrb                ; b = 0
        suba MPDEL_HysteresisForBoostIncreasingCondition          ;  (data is 00)
        bls  L1772          ; branch if lower or same
        sta  Scratch_ba     ; store a into memory
        lda  BoostGoal      ; load a with memory contents
        suba MapValue
        adda #$80           ;  -10000000-
        ldx  #RESFAC_BoostMapCorrectionResponseFactor    ; load index with value
        jsr  LookupTable    ; call subroutine
        lda  Scratch_ba     ; load a with memory contents
        mul                 ; multiply (d=a*b)
        tsta                ; test a
        beq  L1773          ; branch if equal (zero)
        ldb  #$ff           ; load b with value -11111111-
L1773:  tst  Scratch_b7
        bmi  L1774          ; branch if minus(hi bit 1)
        lda  UPRLIM_UpperLimitOnMPCORR    ; load a with memory contents (data is 0b)
        bra  L1775          ; branch

L1774:  lda  LWRLIM_LowerLimitOnMPCORR    ; load a with memory contents (data is 15)
L1775:  cba
        bhs  L1772          ; branch if greater or equal
        tab                 ; b = a
L1772:  stb  L00E8          ; store b into memory
        lda  BoostGoal      ; load a with memory contents
        tst  Scratch_b7
        bpl  L1776          ; branch if plus (hi bit 0)
        aba                 ; a=a+b
        bcc  L1777          ; branch if greater or equal
        lda  #$ff           ; load a with value -11111111-
        bra  L1777          ; branch

L1776:  sba                 ; subtract (a=a-b)
        bcc  L1777          ; branch if greater or equal
        clra                ; a = 0
L1777:  suba BarometricPressure
        bcc  L1778          ; branch if greater or equal
        clra                ; a = 0
L1778:  ldx  #DCWOT_BoostWotDutyCycle    ; load index with value
        jsr  LookupTable    ; call subroutine
        sta  Scratch_ba     ; store a into memory
        lda  BitFlags81     ; load a with memory contents
        bita #b81_FullThrottle      ;  -00000001-
        beq  L1779          ; branch if equal (zero)
        lda  RPM_W          ; load a with memory contents
        ldx  #DCWADJ_BoostWotDutyCycleAdj    ; load index with value
        jsr  LookupTable    ; call subroutine
        bmi  L1780          ; branch if minus(hi bit 1)
        adda Scratch_ba
        bcs  L1781          ; branch if lower
        cmpa #$c8           ; compare a with value -11001000-
        bls  L1782          ; branch if lower or same
L1781:  lda  #$c8           ; load a with value -11001000-
        bra  L1782          ; branch

L1780:  adda Scratch_ba
        bcs  L1782          ; branch if lower
        clra                ; a = 0
L1782:  sta  Scratch_ba     ; store a into memory
L1779:  sei                 ; disable interrupts (set)
        lda  BitFlags7f     ; load a with memory contents
        bita #$10           ;  -00010000-
        beq  L1783          ; branch if equal (zero)
        anda #$ef           ; AND a with value -11101111-
        sta  BitFlags7f     ; store a into memory
        cli                 ; enable interrupts (clear)
        ldb  L00DA          ; load b with memory contents
        addb KDCINC_DCincreaseStepWhenKnockPresent          ;  (data is 04)
        bcc  L1785          ; branch if greater or equal
        ldb  #$ff           ; load b with value -11111111-
L1785:  lda  KDCMAX_MaxDCwhenKnockPresent          ; load a with memory contents (data is 50)
        cba
        blo  L1787          ; branch if lower
        lda  KDCMIN_MinDCWhenIncreasingDueToKnock          ; load a with memory contents (data is 0c)
        cba
        blo  L1789          ; branch if lower
L1787:  tab                 ; b = a
        bra  L1789          ; branch

L1783:  cli                 ; enable interrupts (clear)
        ldx  #CylinderRetard1 - 1         ; load index with value
L1792:  inx                 ; increment index (x=x+1)
        cpx  #MaxRetard     ;  -00000000 11010010-
        beq  L1790          ; branch if equal (zero)
        lda  $00,x          ; load a with memory at index + value
        anda #$3f           ; AND a with value -00111111-
        cmpa BSOLCO_BoostSolenoidCutout          ; compare a with memory contents (data is 07)
        blo  L1792          ; branch if lower
        ldb  Counter_MainLoop1    ; load b with memory contents
        lsrb                ; logical shift right b
        bitb DECTIM_TimeScaleForDCDECK          ;  (data is 03)
        bne  L1794          ; branch if not equal (not zero)
        ldb  L00DA          ; load b with memory contents
        subb DCDECK_DCdecrementWhenSparkStillDerated          ; subtract memory contents from b (data is 02)
        bcc  L1789          ; branch if greater or equal
L1790:  clrb                ; b = 0
L1789:  stb  L00DA          ; store b into memory
L1794:  lda  BoostGoal      ; load a with memory contents
        ldb  BarometricPressure    ; load b with memory contents
        addb MAPPT_MapPointForWastegateControlCells          ;  (data is 62)
        bcc  L1797          ; branch if greater or equal
        ldb  #$ff           ; load b with value -11111111-
L1797:  ldx  #L0043         ; load index with value
        cba
        bcc  L1798          ; branch if greater or equal
        inx                 ; increment index (x=x+1)
L1798:  ldb  $00,x          ; load b with memory at index + value
        suba MapValue
        bhi  L1799          ; branch if higher
        nega                ; negate a (set high bit)
        sta  Scratch_b7     ; store a into memory
        lda  L004F          ; load a with memory contents
        bita #$40           ;  -01000000-
        bne  L1800          ; branch if not equal (not zero)
        ora  #$40           ;  -01000000-
        bra  L1801          ; branch

L1800:  lda  MPDEL1_IncCounterWhenDeltaMAPLessThanThis          ; load a with memory contents (data is 03)
        cmpa Scratch_b6     ; compare a with memory contents
        blo  L1803          ; branch if lower
        lda  Counter_WastegateControl ; load a with memory contents
        inca                ; increment a
        cmpa INCPT_AdaptiveIncAfterThisTime ; compare a with memory contents (data is 0a)
        blo  L1805          ; branch if lower
        lda  Scratch_b7     ; load a with memory contents
        cmpa MPROOF_NoAdaptiveWhenMapIsThisCloseToGoal ; compare a with memory contents (data is 02)
        blo  L1803          ; branch if lower
        cmpb INCLIM_UpperLimitOnDCCORX ; compare b with memory contents (data is b0)
        bhs  L1803          ; branch if greater or equal
        incb
        bra  L1808          ; branch

L1799:  sta  Scratch_b7     ; store a into memory
        lda  L004F          ; load a with memory contents
        bita #$40           ;  -01000000-
        beq  L1809          ; branch if equal (zero)
        anda #$bf           ; AND a with value -10111111-
L1801:  sta  L004F          ; store a into memory
        bra  L1803          ; branch

L1809:  lda  L00DA          ; load a with memory contents
        bne  L1803          ; branch if not equal (not zero)
        lda  BitFlags81     ; load a with memory contents
        lsra
        bcc  L1803          ; branch if b81_FullThrottle is clear
        lda  RPM_W          ; load a with memory contents
        cmpa L1810          ; compare a with memory contents (data is 2f)
        blo  L1803          ; branch if lower
        cmpa L1811          ; compare a with memory contents (data is ff)
        bhi  L1803          ; branch if higher
        lda  L1812          ; load a with memory contents (data is 02)
        cmpa Scratch_b6     ; compare a with memory contents
        blo  L1803          ; branch if lower
        lda  Counter_WastegateControl ; load a with memory contents
        inca                ; increment a
        cmpa L1813          ; compare a with memory contents (data is 0a)
        blo  L1805          ; branch if lower
        lda  Scratch_b7     ; load a with memory contents
        cmpa L1814          ; compare a with memory contents (data is 00)
        blo  L1803          ; branch if lower
        cmpb L1815          ; compare b with memory contents (data is 62)
        bls  L1803          ; branch if lower or same
        decb
L1808:  stb  $00,x
L1803:  clra                ; a = 0
L1805:  sta  Counter_WastegateControl          ; store a into memory
        lda  Scratch_ba     ; load a with memory contents
        subb #$80           ;  -10000000-
        bmi  L1816          ; branch if minus(hi bit 1)
        aba                 ; a=a+b
        bcs  L1817          ; branch if lower
        cmpa #$c8           ; compare a with value -11001000-
        bls  L1818          ; branch if lower or same
L1817:  lda  #$c8           ; load a with value -11001000-
        bra  L1818          ; branch

L1816:  negb
        sba                 ; subtract (a=a-b)
        bcc  L1818          ; branch if greater or equal
        clra                ; a = 0
L1818:  adda L00DA
        bcc  L1819          ; branch if greater or equal
        lda  #$ff           ; load a with value -11111111-
L1819:  sta  WastegateControlDutyCycle  ; store a into memory
        rts                 ; return from subroutine
;

; Another Interrupt Handler
IRQ1strobe3 subroutine                  ; ok, not really a sub, but an interrupt service routine

    ; handles duty cycle related operations: Wastegate, Charging

        ldb  BitFlags7f                 ; load b with memory contents
        lda  WastegateControlDutyCycle  ; load a with memory contents
        beq  .1                         ; branch if equal (zero)
        cmpa #$c8                       ; compare a with value (c8 = 200 = 100% duty cycle)
        bhs  .2                         ; branch if greater or equal
        bitb #$04                       ;  -00000100-
        bne  .3                         ; branch if not equal (not zero)
.1      orb  #$04                       ;  -00000100-
        nega                            ; negate a (set high bit)
        adda #$c8                       ;  -11001000-
        bra  .4                         ; branch

.2      lda  #$64                       ; load a with value -01100100-
.3      andb #$fb                       ;  -11111011-
.4      stb  BitFlags7f                 ; store b into memory
        andb #$04                       ;  -00000100-
        sta  Scratch_bb                 ; store a into memory
        lda  Z83_PortAdata              ; load a with memory contents
        anda #<~PortA_Wastegate         ; * $fb = clear bit, wastegate ON
        aba                             ; a=a+b
        sta  Z83_PortAdata
        ldb  Z83_PortADataDirection     ; load b with memory contents
        orb  #$04                       ;  -00000100-
        stb  Z83_PortADataDirection     ; store b into memory
        ldb  #$0b                       ; load b with value -00001011-
.5      decb
        bne  .5                         ; branch if not equal (not zero)
        lda  Scratch_bb                 ; load a with memory contents
        std  Z22_Timer6840_4
        ldb  RawBattVoltage             ; load b with memory contents
        cmpb #$a2                       ;  -10100010-
        blo  .Return                    ; branch if lower
        ldb  Z83_PortADataDirection     ; load b with memory contents
        andb #$fb                       ;  -11111011-
        stb  Z83_PortADataDirection     ; store b into memory
.Return
        rti                             ; return from interrupt

;  ______________________
; |                      \
; |   Modifications      |
; \______________________|

    .if UseDetonationMod > 0

MM_DetonationIndicatorMod subroutine
        lda  BitFlags82
        bmi  .Return        ; don't do this if Engine Not Running
        lda  CoolantTemp
        cmpa #DetonationModMinTemp
        blo  .Return        ; don't do this if cold motor (otherwise a LOT of false positives)

        lda  CylinderRetard4
        anda #$3f
        bne  .LightItUp     ; not zero, turn on power loss light
        lda  CylinderRetard1
        anda #$3f
        bne  .LightItUp     ; not zero, turn on power loss light
        lda  CylinderRetard2
        anda #$3f
        bne  .LightItUp     ; not zero, turn on power loss light
        lda  CylinderRetard3
        anda #$3f
        bne  .LightItUp     ; not zero, turn on power loss light
        lda  ChEMreserved
        bita #$20
        beq  .Return
        ldb  #$FF
        sei
        andb ioPort1data
        bra  .Store
.LightItUp
        ldb  #$20
        sei
        orb  ioPort1data
.Store
        stb  ioPort1data
        stb  ChEMreserved
        cli
.Return
        rts
    .endif

    org $ffaa
ActualChecksumFixup:
    db  $00

; Signature
    org $ffb0
    db $00, $00, "Blueberry 60", $00, $00
    db "1C3BA64A5HG134666", $00
    db "Feb 16 07 GLA"

;  ______________________
; |                      \
; |   Vectors            |
; \______________________|

    org $fff0
IRQ2sioV:
    dw  ResetRoutine
IRQ2timerOverV:
    dw  ResetRoutine
IRQ2timerOutV:
    dw  IRQtOut
IRQ2timerInV:
    dw  InterruptFromDistributorSyncAndSpeedSensor
IRQ1strobe3V:
    dw  IRQ1strobe3
SWIVector:
    dw  ResetRoutine
NMIVector:
    dw  NMIInterrupt
ResetVector:
    dw  ResetRoutine

; Quick 6803 reference:
;   a  Accumulator A (8-bit)
;   b  Accumulator B (8-bit)
;   d  A and B combined (16-bit, A hi, B lo)
;   p  Condition Code Register (8-bit, CCR)
;   pc Program Counter (16-bit)
;   s  Stack Pointer (16-bit)
;   x  Index Register (16-bit)
;
; 32 bytes of RAM from 0040 to 005F are kept alive by standby
;    battery power.
;
;   0000 to 001f - Internal registers
;   0040 to 00ff - 192 bytes of internal RAM (U4 variant)
;   fff0 to ffff - Interrupt vectors
;   fff0 to fff1 - IRQ2 serial I/O interrupt vector
;   fff2 to fff3 - IRQ2 timer overflow vector
;   fff4 to fff5 - IRQ2 timer output compare vector
;   fff6 to fff7 - IRQ2 timer input capture vector
;   fff8 to fff9 - IRQ1 interrupt strobe 3 vector
;   fffa to fffb - SWI instruction interrupt vector
;   fffc to fffd - Non-maskable interrupt vector
;   fffe to ffff - Reset vector
